app.controller('editorCtrl', function ($scope,$http,$location) {

	$scope.valLevelName = "level0";

	var rendererWindow = new Engine.WebGLRenderer( 1280, 720 );
	Engine.conf.MAX_POINT_LIGHTS = 10;
	document.body.appendChild(rendererWindow.domElement);
	var engine = new Engine(rendererWindow.domElement);
	var scene1 = new Engine.Scene();
	var scene2 = new Engine.Scene();
	var camera = new Engine.PerspectiveCamera( 45, null, 0.1, 600, null, [0,30,47]);
	var lightOmni = new Engine.PointLight( 0xffffff, 1.0 , 150, [10,20,10] );
	var lightAmbient = new Engine.AmbientLight( 0x222222 );
	scene1.add( camera.lookAt(new Engine.Vector3(0,0,0)) );
	scene1.add( lightOmni );
	scene1.add( lightAmbient );
	scene2.add( camera );
	var editor = new MapEditor(scene1,scene2);

	engine.handleMousedown = function(event){
		event.stopPropagation();
		if(!editor.ready) return false;

		switch(event.button){
			case 0 : editor.addOrRemoveItem();
			break;
			case 1 : editor.symmetry();
			break;
			case 2 : editor.rotation();
			break;
		}
		editor.update();
		return false;
	};

	engine.handleWheel = function(event){
		if(!editor.ready) return;
		if(event.wheelDelta>0||event.deltaY>0)
			editor.next();
		else
			editor.prev();
		editor.update();
	};

	engine.handleMouseMove = function(event) {
		if(!editor.ready) return;
		editor.position(event);
		if(Engine.keys.SUP){
			editor.deleteTiles();
			editor.addOrRemoveItem();
		}
		editor.update();
	};

	engine.handleKeyDown = function(){
		if(!editor.ready) return;
		editor.update();
	};

	engine.animate = function(){
		lightOmni.position = camera.position;
		if(Engine.keys.UP)         { editor.moveFront(); }
		else if(Engine.keys.DOWN)  { editor.moveBack();  }
		if(Engine.keys.LEFT)       { editor.moveleft();  }
		else if(Engine.keys.RIGHT) { editor.moveRight(); }
	};

	engine.render = function() {
		rendererWindow.clearColorEnable = true;
		rendererWindow.renderBasic( scene1,null );
		rendererWindow.clearColorEnable = false;
		rendererWindow.renderBasic( scene2,null );
		scene1._lightsNeedUpdate = true;
	}


    $scope.loadSkin = function(cb){
    	engine.stop();
    	var request = $http.get('/data/skin/'+$scope.valSkin+".json");
    	request.success(function(data, status) {
    		data.id = $scope.valSkin;
    		editor.loadSkin(data);
    		engine.start();
    		if(cb)cb();
 		});
    	
    };

    $scope.saveLevel = function(){
    	
    	var json = editor.toJsonPackage($scope.valLevelName,$scope.valTimeout||0,$scope.valTimetraget||0,$scope.valScoretraget||0,$scope.valSkin);
    	if($scope.valTitle)json.title = $scope.valTitle;
    	var request = $http.post('/editor/save',json);
    	request.success(function(data, status) {
    		console.log('save success');
 		});

    };

    $scope.loadLevel = function(){

    	var request = $http.post('/editor/load',{id:$scope.valLevelName});
    	request.success(function(data, status) {
    		data = JSON.parse(data);
    		console.log('load success');
    		
    		if(data.skinId != $scope.valSkin){
    			$scope.valSkin = data.skinId;
    			$scope.loadSkin(function(){
    				editor.buildMap(data);
    			})
    		}else{
    			editor.buildMap(data);
    		}
	    	$scope.valTimeout = data.timeout;
	    	$scope.valTimetraget = data.timetarget;
	    	$scope.valScoretraget = data.scoreTarget;
	    	$scope.valTitle = data.title || null;
 		});

    };


    $scope.switchItem = function(val){

    	editor.currentLevelId = $scope.valLevelId||1;
		editor.currentLightColor = parseInt("0x"+ ($scope.valColorLight||'ffffff'));
		editor.currentLightRadius = parseFloat($scope.valRadiusLight||5);

    	editor.switchMode(val);
    	editor.update();
    };

});