var app = angular.module('angularEditor', []);;app.controller('editorCtrl', function ($scope,$http,$location) {

	$scope.valLevelName = "level0";

	var rendererWindow = new Engine.WebGLRenderer( 1280, 720 );
	Engine.conf.MAX_POINT_LIGHTS = 10;
	document.body.appendChild(rendererWindow.domElement);
	var engine = new Engine(rendererWindow.domElement);
	var scene1 = new Engine.Scene();
	var scene2 = new Engine.Scene();
	var camera = new Engine.PerspectiveCamera( 45, null, 0.1, 600, null, [0,30,47]);
	var lightOmni = new Engine.PointLight( 0xffffff, 1.0 , 150, [10,20,10] );
	var lightAmbient = new Engine.AmbientLight( 0x222222 );
	scene1.add( camera.lookAt(new Engine.Vector3(0,0,0)) );
	scene1.add( lightOmni );
	scene1.add( lightAmbient );
	scene2.add( camera );
	var editor = new MapEditor(scene1,scene2);

	engine.handleMousedown = function(event){
		event.stopPropagation();
		if(!editor.ready) return false;

		switch(event.button){
			case 0 : editor.addOrRemoveItem();
			break;
			case 1 : editor.symmetry();
			break;
			case 2 : editor.rotation();
			break;
		}
		editor.update();
		return false;
	};

	engine.handleWheel = function(event){
		if(!editor.ready) return;
		if(event.wheelDelta>0||event.deltaY>0)
			editor.next();
		else
			editor.prev();
		editor.update();
	};

	engine.handleMouseMove = function(event) {
		if(!editor.ready) return;
		editor.position(event);
		if(Engine.keys.SUP){
			editor.deleteTiles();
			editor.addOrRemoveItem();
		}
		editor.update();
	};

	engine.handleKeyDown = function(){
		if(!editor.ready) return;
		editor.update();
	};

	engine.animate = function(){
		lightOmni.position = camera.position;
		if(Engine.keys.UP)         { editor.moveFront(); }
		else if(Engine.keys.DOWN)  { editor.moveBack();  }
		if(Engine.keys.LEFT)       { editor.moveleft();  }
		else if(Engine.keys.RIGHT) { editor.moveRight(); }
	};

	engine.render = function() {
		rendererWindow.clearColorEnable = true;
		rendererWindow.renderBasic( scene1,null );
		rendererWindow.clearColorEnable = false;
		rendererWindow.renderBasic( scene2,null );
		scene1._lightsNeedUpdate = true;
	}


    $scope.loadSkin = function(cb){
    	engine.stop();
    	var request = $http.get('/data/skin/'+$scope.valSkin+".json");
    	request.success(function(data, status) {
    		data.id = $scope.valSkin;
    		editor.loadSkin(data);
    		engine.start();
    		if(cb)cb();
 		});
    	
    };

    $scope.saveLevel = function(){
    	
    	var json = editor.toJsonPackage($scope.valLevelName,$scope.valTimeout||0,$scope.valTimetraget||0,$scope.valScoretraget||0,$scope.valSkin);
    	if($scope.valTitle)json.title = $scope.valTitle;
    	var request = $http.post('/editor/save',json);
    	request.success(function(data, status) {
    		console.log('save success');
 		});

    };

    $scope.loadLevel = function(){

    	var request = $http.post('/editor/load',{id:$scope.valLevelName});
    	request.success(function(data, status) {
    		data = JSON.parse(data);
    		console.log('load success');
    		
    		if(data.skinId != $scope.valSkin){
    			$scope.valSkin = data.skinId;
    			$scope.loadSkin(function(){
    				editor.buildMap(data);
    			})
    		}else{
    			editor.buildMap(data);
    		}
	    	$scope.valTimeout = data.timeout;
	    	$scope.valTimetraget = data.timetarget;
	    	$scope.valScoretraget = data.scoreTarget;
	    	$scope.valTitle = data.title || null;
 		});

    };


    $scope.switchItem = function(val){

    	editor.currentLevelId = $scope.valLevelId||1;
		editor.currentLightColor = parseInt("0x"+ ($scope.valColorLight||'ffffff'));
		editor.currentLightRadius = parseFloat($scope.valRadiusLight||5);

    	editor.switchMode(val);
    	editor.update();
    };

});;MapEditor = function (scene1,scene2 ) {

	this.scene1 = scene1;
	this.scene2 = scene2;
	this.skin;
	this.tileSize;
	this.meshTiles = [];
	this.meshElements = [];
	this.meshMobs = [];
	this.meshDoor;
	this.meshOil = [];
	this.meshDestructor;
	this.meshPlayer;
	this.meshtargetPlayer;
	this.meshtargetMob;
	this.meshTilesSelect = [];
	this.meshElementsSelect = [];
	this.meshMobsSelect = [];
	this.meshDoorSelect;
	this.meshOilSelect = [];
	this.meshDestructorSelect;
	this.meshPlayerSelect;
	this.meshTargetPlayerSelect;
	this.meshTargetMobSelect;
	this.N = 0;
	this.M = 0;
	this.currentDelete;
	this.currentPosition;
	this.currentIndex;
	this.currentRotation;
	this.previousRotation;
	this.currentSymmetry;
	this.currentN;
	this.currentM;
	this.currentDelay = 0;
	this.currentLightColor = 0xffffff;
	this.currentLightRadius = 5;

	this.meshLight;
	this.meshLightSelect;
	this.lights = [];



};

MapEditor.prototype = {

	constructor: MapEditor,

	loadSkin: function (skin) {

		this.cleanMap();

		this.skin = skin;

		this.tileSize =  this.skin.level.tileSize;
		this.N = skin.nWidth||25;
		this.M = skin.nHeight||25;

		this.getModels();

		this.currentDelete = false;
		this.currentPosition = new Engine.Vector3(0,0,0);

		this.currentIndex = 0;

		this.currentRotation = 0;
		this.previousRotation = null;

		this.currentSymmetry = false;

		this.currentN = Math.floor(this.N/2);
		this.currentM = Math.floor(this.M/2);
		
		this.editTitle = true;
		this.editElement = false;
		this.editMob = false;
		this.editDoor = false;
		this.editDestructor = false;
		this.editPlayer = false;
		this.editTargetPlayer = false;
		this.editBeginPlayer = false;
		this.editTargetMob = false;
		this.editOil = false;
		this.editLight = false;

		this.switchMode(1);

		this.ready = true;

	},

	getModels: function ( ) {

		var mapLevel = new Engine.Texture(  this.skin.level.tilesMap);
		var mapPlayer = new Engine.Texture(this.skin.player.material.colorMap);
		var mapMob = new Engine.Texture(this.skin.mobs[0].material.colorMap);

		var materialLevel = new Engine.MeshPhongMaterial( { map: mapLevel } );
		var materialLevelSel = new Engine.MeshPhongMaterial( {  emissive: 0x00ff00, opacity: 0.9, transparent: true, map: mapLevel } );

		var materialPlayer = new Engine.MeshPhongMaterial( { map: mapPlayer } );
		var materialPlayerSel = new Engine.MeshPhongMaterial( {  emissive: 0x00ff00, opacity: 0.9, transparent: true, map: mapPlayer } );

		var materialMob = new Engine.MeshPhongMaterial( { map: mapMob } );
		var materialMobSel = new Engine.MeshPhongMaterial( {  emissive: 0x00ff00, opacity: 0.9, transparent: true, map: mapMob } );


		this.lightSelect = new Engine.PointLight( 0xffffff, 1.0 , 5, [0,4,0] );
		this.scene1.add( this.lightSelect );

		
		var tiles = this.skin.level.tiles,i=0;
		var geo,mesh0,mesh1;
		for(i=0;i<tiles.length;i++) {
			geo = new Engine.ObjGeometry( tiles[i] ) ;
			mesh0 = new Engine.Mesh( geo, materialLevel ).addClone(0,-1000,0 );
			mesh1 = new Engine.Mesh( geo, materialLevelSel );
			this.meshTiles.push( mesh0 );
			this.meshTilesSelect.push( mesh1 );
			this.scene1.add( mesh0 );
		}

		var elements = this.skin.level.elements;
		for(i=0;i<elements.length;i++) {
			geo = new Engine.ObjGeometry( elements[i] ) ;
			mesh0 = new Engine.Mesh( geo, materialLevel ).addClone(0,-1000,0 );
			mesh1 = new Engine.Mesh( geo, materialLevelSel );
			this.meshElements.push( mesh0 );
			this.meshElementsSelect.push( mesh1 );
			this.scene1.add( mesh0 );
		}

		var oils = this.skin.oils;
		for(i=0;i<oils.length;i++) {
			var geoOil = new Engine.ObjGeometry( oils[i].block ) ;
			this.meshOil[i] = new Engine.Mesh( geoOil, materialPlayer ).addClone(0,-1000,0 );	
			this.meshOilSelect[i] = new Engine.Mesh( geoOil, materialPlayerSel );
			this.scene1.add( this.meshOil[i] );
		} 

		var door = "assets/obj/doors/doorA.obj";
		var geodoor = new Engine.ObjGeometry(door ) ;
		var mesh2 = new Engine.Mesh( geodoor, materialLevel ).addClone(0,-1000,0 );
		var mesh3 = new Engine.Mesh( geodoor, materialLevelSel );	
		this.meshDoor =  mesh2;
		this.meshDoorSelect =  mesh3;
		this.scene1.add( mesh2 );



		mobs = this.skin.mobs;
		for(i=0;i<mobs.length;i++) {
			geo = new Engine.ObjGeometry( mobs[i].geometry ) ;
			mesh0 = new Engine.Mesh( geo, materialMob ).addClone(0,-1000,0 );
			mesh1 = new Engine.Mesh( geo, materialMobSel );
			this.meshMobs.push( mesh0 );
			this.meshMobsSelect.push( mesh1 );
			this.scene1.add( mesh0 );
		}

		var player = this.skin.player;
		var geoplayer = new Engine.ObjGeometry( player.geometryBody ) ;
		this.meshPlayer = new Engine.Mesh( geoplayer, materialPlayer ).addClone(0,-1000,0 );
		this.meshPlayerSelect = new Engine.Mesh( geoplayer, materialPlayerSel );
		this.scene1.add( this.meshPlayer );

		var destruc = '/assets/obj/mobs/destructor.obj';
		var geoDest = new Engine.ObjGeometry( destruc ) ;
		this.meshDestructor  = new Engine.Mesh( geoDest, materialLevel ).addClone(0,-1000,0 );	
		this.meshDestructorSelect = new Engine.Mesh( geoDest, materialLevelSel );
		this.scene1.add( this.meshDestructor  );

		var targetPlayer = '/assets/obj/editor/flagPlayer.obj';
		var geoTargetPlayer = new Engine.ObjGeometry( targetPlayer ) ;
		this.meshTargetPlayer = new Engine.Mesh( geoTargetPlayer, materialLevel ).addClone(0,-1000,0 );	
		this.meshTargetPlayerSelect = new Engine.Mesh( geoTargetPlayer, materialLevelSel );
		this.scene1.add( this.meshTargetPlayer );

		var beginPlayer = '/assets/obj/editor/flagMob.obj';
		var geoBeginPlayer = new Engine.ObjGeometry( beginPlayer ) ;
		this.meshBeginPlayer = new Engine.Mesh( geoBeginPlayer, materialLevel ).addClone(0,-1000,0 );	
		this.meshBeginPlayerSelect = new Engine.Mesh( geoBeginPlayer, materialLevelSel );
		this.scene1.add( this.meshBeginPlayer );

		var targetMob = '/assets/obj/editor/flagMob.obj';
		var geoTargetMob = new Engine.ObjGeometry( targetMob ) ;
		this.meshTargetMob = new Engine.Mesh( geoTargetMob, materialLevel ).addClone(0,-1000,0 );	
		this.meshTargetMobSelect = new Engine.Mesh( geoTargetMob, materialLevelSel );
		this.scene1.add( this.meshTargetMob );

		var geoLight = new Engine.SphereGeometry(0.3, 10, 10);
		this.meshLight = new Engine.Mesh( geoLight,  new Engine.MeshBasicMaterial( { color: 0xffffff, lighting: false } ) ).addClone(0,-1000,0 );
		this.meshLightSelect = new Engine.Mesh( geoLight, new Engine.MeshBasicMaterial( { color: 0x008800, lighting: false } ) );
		this.scene1.add( this.meshLight );



		var materialBackground = new Engine.ParticleSystemMaterial( {color: 0x00ff00} );
		var geoBackground = new Engine.PlaneGeometry( 100, 100, 10, 10 );
		var meshBackground1 = new Engine.Mesh( geoBackground, materialBackground );
		meshBackground1.rotateX(-Math.PI/2);
		meshBackground1.position = new Engine.Vector3(50,0,50);
		this.scene1.add( meshBackground1 );
	},




	deleteCloneInBlock: function(mesh,d){

		var meshs = mesh.clones;
		for(var i=0,il=meshs.length;i<il;i++ ){
			if( Math.abs(this.currentN*d+d/2 - meshs[i].elements[12])<1  && Math.abs(this.currentM*d+d/2 - meshs[i].elements[14])<1  ){
				meshs.splice(i,1);
				break;
			}
		}
	},

	deleteCloneInAbsolute: function(mesh){

		var meshs = mesh.clones;
		for(var i=0,il=meshs.length;i<il;i++ ){
			if( Math.abs(this.currentPosition.x - meshs[i].elements[12])<0.5  && Math.abs(this.currentPosition.z - meshs[i].elements[14])<0.5  ){
				meshs.splice(i,1);
				break;
			}
		}
	},

	deleteLight: function(){
		var l;
		for(var i=0,il=this.lights.length;i<il;i++ ){
			l=this.lights[i];
			if(Math.abs(this.currentPosition.x-l.position.x)<1&&Math.abs(this.currentPosition.z-l.position.z)<1){
				this.scene1.remove(l);
				break;
			}
		}
	},

	updateTilesInScene: function(d){
		for(var i=0,il=this.meshTiles.length;i<il;i++ ){
			this.deleteCloneInBlock(this.meshTiles[i],d);
		}
		if(!this.currentDelete)
			this.meshTiles[this.currentIndex].addClone(this.currentN*d+d/2,0,this.currentM*d+d/2,0,this.currentRotation,0,0,0,this.currentSymmetry?-1:0);
	},

	updateElementsInScene: function(){
		this.deleteCloneInAbsolute(this.meshElements[this.currentIndex]);

		if(!this.currentDelete)
			this.meshElements[this.currentIndex].addClone(this.currentPosition.x,this.currentPosition.y,this.currentPosition.z,0,this.currentRotation,0,0,0,this.currentSymmetry?-1:0);
	},

	updateMobsInScene: function(){
		this.deleteCloneInAbsolute(this.meshMobs[this.currentIndex]);

		if(!this.currentDelete)
			this.meshMobs[this.currentIndex].addClone(this.currentPosition.x,this.currentPosition.y,this.currentPosition.z,0,this.currentRotation);
	},

	updateDoorInScene: function(){
		this.deleteCloneInAbsolute(this.meshDoor);

		if(!this.currentDelete){
			this.meshDoor.addClone(this.currentPosition.x,this.currentPosition.y,this.currentPosition.z,0,this.currentRotation);
			this.meshDoor.getClone().levelId = this.currentLevelId;
		}
	},

	updateDestructorInScene: function(d){
		this.deleteCloneInBlock(this.meshDestructor,d);

		if(!this.currentDelete)
			this.meshDestructor.addClone(this.currentN*d+d/2,0,this.currentM*d+d/2,0,this.currentRotation);
	},

	updatePlayerInScene: function(){
		this.deleteCloneInAbsolute(this.meshPlayer);

		if(!this.currentDelete)
			this.meshPlayer.addClone(this.currentPosition.x,this.currentPosition.y,this.currentPosition.z,0,this.currentRotation);
			this.currentDelete = false;
	},


	updateTargetPlayerInScene: function(){
		this.deleteCloneInAbsolute(this.meshTargetPlayer);

		if(!this.currentDelete)
			this.meshTargetPlayer.addClone(this.currentPosition.x,this.currentPosition.y,this.currentPosition.z,0,this.currentRotation);
	},

	updateBeginPlayerInScene: function(){
		this.deleteCloneInAbsolute(this.meshBeginPlayer);

		if(!this.currentDelete)
			this.meshBeginPlayer.addClone(this.currentPosition.x,this.currentPosition.y,this.currentPosition.z,0,this.currentRotation);
	},

	updateTargetMobInScene: function(){
		this.deleteCloneInAbsolute(this.meshTargetMob);

		if(!this.currentDelete)
			this.meshTargetMob.addClone(this.currentPosition.x,this.currentPosition.y,this.currentPosition.z,0,this.currentRotation);
	},

	updateLightInScene: function(){
		this.deleteCloneInAbsolute(this.meshLight);
		this.deleteLight()
		if(!this.currentDelete){
			this.meshLight.addClone(this.currentPosition.x,0.5,this.currentPosition.z);
			var newLight = this.lightSelect.clone();
			this.lights.push(newLight);
			this.scene1.add(newLight);
		}
	},


	updateOilInScene: function(){

		this.deleteCloneInAbsolute(this.meshOil[this.currentIndex]);

		if(!this.currentDelete)
			this.meshOil[this.currentIndex].addClone(this.currentPosition.x,this.currentPosition.y,this.currentPosition.z,0);
	},


	addOrRemoveItem: function() {

		if(this.currentN>=this.N || this.currentN<0 ) return;

		var d = this.tileSize, h = d/2;

		if(this.editElement)
			this.updateElementsInScene();
		else if (this.editMob)
			this.updateMobsInScene();
		else if (this.editTitle)
			this.updateTilesInScene(d,h);
		else if (this.editDoor)
			this.updateDoorInScene(d,h);
		else if (this.editDestructor)
			this.updateDestructorInScene(d,h);
		else if (this.editPlayer)
			this.updatePlayerInScene();
		else if (this.editTargetPlayer)
			this.updateTargetPlayerInScene();
		else if (this.editBeginPlayer)
			this.updateBeginPlayerInScene();
		else if (this.editTargetMob)
			this.updateTargetMobInScene();
		else if (this.editLight)
			this.updateLightInScene();
		else if (this.editOil)
			this.updateOilInScene();

		this.currentDelete = false;

	},

	update: function() {

		var position = this.currentPosition.clone();position.y = 0;	
		
		if(this.editTitle  || this.editDestructor ){
			position.x = ( this.currentN ) * this.tileSize + this.tileSize/2;
			position.z = ( this.currentM ) * this.tileSize + this.tileSize/2;
		}

		var mesh;

		if(this.editElement){
			mesh = this.meshElementsSelect[this.currentIndex];
		}else if( this.editMob ){
			mesh = this.meshMobsSelect[this.currentIndex];
		}else if( this.editTitle ){
			mesh = this.meshTilesSelect[this.currentIndex];
		}else if( this.editDoor ){
			mesh = this.meshDoorSelect;
		}else if( this.editDestructor ){
			mesh = this.meshDestructorSelect;
		}else if( this.editPlayer ){
			mesh = this.meshPlayerSelect;
		}else if( this.editTargetPlayer ){
			mesh = this.meshTargetPlayerSelect;
		}else if( this.editBeginPlayer ){
			mesh = this.meshBeginPlayerSelect;
		}else if( this.editTargetMob ){
			mesh = this.meshTargetMobSelect;
		}else if( this.editLight ){
			mesh = this.meshLightSelect;
		}else if( this.editOil ){
			mesh = this.meshOilSelect[ this.currentIndex ];
		}

		mesh.position = position;
		mesh.scale.x = this.currentSymmetry?-1:1;
		mesh.rotation.y = this.currentRotation;

	},

	next: function() {

		this.currentSymmetry = false;

		if(this.editElement){
			this.scene2.remove( this.meshElementsSelect[this.currentIndex] );
		}else if(this.editMob){
			this.scene2.remove( this.meshMobsSelect[this.currentIndex] );
		} else if(this.editTitle){
			this.scene2.remove( this.meshTilesSelect[this.currentIndex] );
		}else if(this.editDoor){
			this.scene2.remove( this.meshDoorSelect );
		}else if(this.editOil){
			this.scene2.remove( this.meshOilSelect[this.currentIndex] );
		}

		this.currentIndex++;

		if(this.editElement){	
			if(this.currentIndex>this.meshElements.length-1){
				this.currentIndex = 0;	
			}
			this.scene2.add( this.meshElementsSelect[this.currentIndex]  );

		}else if(this.editMob){

			if(this.currentIndex>this.meshMobs.length-1){
				this.currentIndex = 0;	
			}
			this.scene2.add( this.meshMobsSelect[this.currentIndex]  );

		}else if(this.editTitle){
			if(this.currentIndex>this.meshTiles.length-1){
				this.currentIndex = 0;	
			}
			this.scene2.add( this.meshTilesSelect[this.currentIndex]  );

		}else if(this.editDoor){
			this.currentIndex = 0;	
			this.scene2.add( this.meshDoorSelect  );
		}else if(this.editOil){
			if(this.currentIndex>this.meshOil.length-1){
				this.currentIndex = 0;	
			}
			this.scene2.add( this.meshOilSelect[this.currentIndex]  );
		}
		
	},

	prev: function() {

		this.currentSymmetry = false;

		if (this.editElement){
			this.scene2.remove( this.meshElementsSelect[this.currentIndex] );
		} else if(this.editMob){
			this.scene2.remove( this.meshMobsSelect[this.currentIndex] );
		} else if(this.editTitle){
			this.scene2.remove( this.meshTilesSelect[this.currentIndex] );
		} else if(this.editDoor){
			this.scene2.remove( this.meshDoorSelect );
		}else if(this.editOil){
			this.scene2.remove( this.meshOilSelect[this.currentIndex] );
		}
		this.currentIndex-=1;

		if (this.currentIndex<0){
			if(this.editElement){
				this.currentIndex = this.meshElements.length-1;	
			}else if(this.editMob){
				this.currentIndex = this.meshMobs.length-1;		
			}else if(this.editTitle){
				this.currentIndex = this.meshTiles.length-1;
			}else if(this.editDoor){
				this.currentIndex = 0;
			}else if(this.editOil){
				this.currentIndex = this.meshOilSelect.length-1;
			}
		}

		if (this.editElement){
			this.scene2.add( this.meshElementsSelect[this.currentIndex] );
		}else if(this.editMob){
			this.scene2.add( this.meshMobsSelect[this.currentIndex] );
		} else if(this.editTitle) {
			this.scene2.add( this.meshTilesSelect[this.currentIndex] );
		} else if(this.editDoor) {
			this.scene2.add( this.meshDoorSelect );
		} else if(this.editOil) {
			this.scene2.add( this.meshOilSelect[this.currentIndex] );
		}

	},

	rotation: function(dir) {
		this.previousRotation = this.currentRotation;
		if(!dir){
			this.currentRotation+=Math.PI/2;
			if(this.currentRotation>Math.PI*2) {
				this.currentRotation = Math.PI/2;
			}
		}else{
			this.currentRotation-=Math.PI/2;
			if(this.currentRotation<0) {
				this.currentRotation = Math.PI*3;
			}
		}
	},

	symmetry: function() {
		this.currentSymmetry = !this.currentSymmetry;
	},

	deleteTiles: function() {
		this.currentDelete = true;
	},

	switchMode: function(mode){
	

		if(this.editElement){
			this.scene2.remove( this.meshElementsSelect[this.currentIndex] );
		}else if(this.editMob){
			this.scene2.remove( this.meshMobsSelect[this.currentIndex] );
		}else if(this.editTitle){
			this.scene2.remove( this.meshTilesSelect[this.currentIndex] );
		}else if(this.editDoor){
			this.scene2.remove( this.meshDoorSelect );
		}else if(this.editDestructor){
			this.scene2.remove( this.meshDestructor );
		}else if(this.editPlayer){
			this.scene2.remove( this.meshPlayerSelect );
		}else if(this.editTargetPlayer){
			this.scene2.remove( this.meshTargetPlayerSelect );
		}else if(this.editBeginPlayer){
			this.scene2.remove( this.meshTargetBeginSelect );
		}else if(this.editTargetMob){
			this.scene2.remove( this.meshTargetMobSelect );
		}else if(this.editOil){
			this.scene2.remove( this.meshOil[ this.currentIndex ] );
		}else if(this.editLight){
			this.scene2.remove( this.meshLightSelect );
		}

		this.currentIndex = 0;

		this.editMob = false;
		this.editElement = false;
		this.editTitle = false;
		this.editDoor = false;
		this.editDestructor = false;
		this.editPlayer = false;
		this.editTargetPlayer = false;
		this.editBeginPlayer = false;
		this.editTargetMob = false;
		this.editOil = false;
		this.editLight = false;
		
		if( mode==2 ){
			this.editElement = true;
			this.scene2.add( this.meshElementsSelect[this.currentIndex]  );
		}else if( mode == 3){
			this.editMob = true;
			this.scene2.add( this.meshMobsSelect[this.currentIndex]  );
		}else if( mode == 1){
			this.scene2.add( this.meshTilesSelect[this.currentIndex]  );
			this.editTitle = true;
		}else if( mode == 4){
			this.editDoor = true;
			this.scene2.add( this.meshDoorSelect  );
		}else if( mode == 5){
			this.editDestructor = true;
			this.scene2.add( this.meshDestructorSelect );
		}else if( mode == 6){
			this.editPlayer = true;
			this.scene2.add( this.meshPlayerSelect );
		}else if( mode == 7){
			this.editTargetPlayer = true;
			this.scene2.add( this.meshTargetPlayerSelect );
		}else if( mode == 8){
			this.editBeginPlayer = true;
			this.scene2.add( this.meshBeginPlayerSelect );
		}else if( mode == 9){
			this.editOil = true;
			this.scene2.add( this.meshOilSelect[ this.currentIndex ] );
		}else if( mode == 10){
			this.editLight = true;
			this.scene2.add( this.meshLightSelect );
			this.lightSelect.color.setHex(this.currentLightColor);
			this.lightSelect.distance = this.currentLightRadius;
		}
	},

	position: function(event) {
		var surface = {x:0,y:0,z:0,dx:0,dy:1,dz:0};

		var direction = this.scene1.camera.vectorClick(event.clientX,event.clientY);

		var vector = this.scene1.camera.position;

		this.currentPosition = Math.intersectionLinePlane( vector, direction, surface );

		var nm = this.positionRealToIndex(this.currentPosition.x,this.currentPosition.z);

		this.currentN = nm[0];
		this.currentM = nm[1];

		if(this.currentN>this.N)this.currentN=this.N;
		else if(this.currentN<0)this.currentN=0;

		if(this.currentM>this.M)this.currentM=this.M;
		else if(this.currentM<0)this.currentM=0;

		this.updateLightPosition();

	},

	updateLightPosition : function(){
		if(this.editLight){
			this.lightSelect.position.x =  this.currentPosition.x;
			this.lightSelect.position.y =  this.lightSelect.distance/2;
			this.lightSelect.position.z =  this.currentPosition.z;
		}else{

			this.lightSelect.distance = 0.1;
		}

	},

	moveleft: function() {
		this.scene1.camera.position.x-=0.6;
	},
	moveRight: function() {
		this.scene1.camera.position.x+=0.6;
	},
	moveFront: function() {
		this.scene1.camera.position.z-=0.6;
	},
	moveBack: function() {
		this.scene1.camera.position.z+=0.6;
	},


	toJsonPackage: function(name,timeout,timeTarget,scoreTarget,skinId) {

		var h = this.tileSize/2;
 		var tile;
 		var ele;
 		var model = [];
 		var door = [];
 		var spawnPlayer = [];
 		var destructor = [];
 		var targetPlayer;
 		var beginPlayer;
 		var lights = [];
 		var targetMob;
 		var oil;
		var i = 0;
		var k=0,kl=0;
		var v=0,vl=0;

		for(k=0,kl=this.meshTiles.length;k<kl;k++ ){
			var meshTile = this.meshTiles[k];

			for(v=1,vl=meshTile.clones.length;v<vl;v++){
				
				var clone = meshTile.clones[v];

				var newTile = {
					id: k,
					pos: clone.getPosition().toArray(),
					rot: clone.getRotationY()/Math.PI,
					sym: clone.elements[10]/clone.elements[0]==1?false:true,
					ele: [],
					mob: [],
					wea: []
				};
				newTile.pos.splice(1,1);
				model.push(newTile);
			}
		}

		for(k=0,kl=this.meshElements.length;k<kl;k++ ){
			var meshElement = this.meshElements[k];
			for(v=1,vl=meshElement.clones.length;v<vl;v++){
				var clone = meshElement.clones[v];
				var pos = clone.getPosition();
				for(var p=0,pl=model.length;p<pl;p++){
					var tile = model[p];
					if(pos.x>tile.pos[0]-h && pos.x<tile.pos[0]+h && pos.z>tile.pos[1]-h && pos.z<tile.pos[1]+h){
						tile.ele.push({
							id: k,
							pos: [pos.x,pos.z],
							rot: clone.getRotationY()/Math.PI,
							sym: clone.elements[10]/clone.elements[0]==1?false:true,
						});
					}
				}
			}
		}

		for(k=0,kl=this.meshMobs.length;k<kl;k++ ){
			var meshMob = this.meshMobs[k];
			for(v=1,vl=meshMob.clones.length;v<vl;v++){
				var clone = meshMob.clones[v];
				var pos = clone.getPosition();
				for(var p=0,pl=model.length;p<pl;p++){
					var tile = model[p];
					if(pos.x>tile.pos[0]-h && pos.x<tile.pos[0]+h && pos.z>tile.pos[1]-h && pos.z<tile.pos[1]+h){
						tile.mob.push({
							id: k,
							pos: [pos.x,pos.z],
							rot: clone.getRotationY()/Math.PI
						});
					}
				}
			}
		}

		for(k=0,kl=this.meshOil.length;k<kl;k++ ){
			var meshOil = this.meshOil[k];
			for(v=1,vl=meshOil.clones.length;v<vl;v++){
				var clone = meshOil.clones[v];
				var pos = clone.getPosition();
				for(var p=0,pl=model.length;p<pl;p++){
					var tile = model[p];
					if(pos.x>tile.pos[0]-h && pos.x<tile.pos[0]+h && pos.z>tile.pos[1]-h && pos.z<tile.pos[1]+h){
						tile.wea.push({
							id: k,
							pos: [pos.x,pos.z]
						});
					}
				}
			}
		}

		var clone;
		


		var meshDoor = this.meshDoor.clones;
		door;
		for(var v=1,vl=meshDoor.length;v<vl;v++ ){
			clone = meshDoor[v];
			door[v-1] = {
				pos:clone.getPosition().toArray(),
				rot:clone.getRotationY()/Math.PI,
				levelId:clone.levelId
			};
			door[v-1].pos.splice(1,1);
		}
	
		
		var meshDestruc = this.meshDestructor.clones;
		for(k=1,kl=meshDestruc.length;k<kl;k++ ){
			clone = meshDestruc[k];
			destructor[k-1] = clone.getPosition().toArray();
			destructor[k-1].splice(1,1);
		}
		
		var meshPlayer = this.meshPlayer.clones;
		for(k=1,kl=meshPlayer.length;k<kl;k++ ){
			clone = meshPlayer[k];
			spawnPlayer[k-1] = clone.getPosition().toArray();
			spawnPlayer[k-1].splice(1,1);
		}


		var meshTargetPlayer = this.meshTargetPlayer.clones;
		for(k=1,kl=meshTargetPlayer.length;k<kl;k++ ){
			clone = meshTargetPlayer[k];
			targetPlayer = clone.getPosition().toArray();
			targetPlayer.splice(1,1);
		}


		var meshBeginPlayer = this.meshBeginPlayer.clones;
		for(k=1,kl=meshBeginPlayer.length;k<kl;k++ ){
			clone = meshBeginPlayer[k];
			beginPlayer = clone.getPosition().toArray();
			beginPlayer.splice(1,1);
		}

		
		if(this.meshTargetMob.clones[1]){
			targetMob = this.meshTargetMob.clones[1].getPosition().toArray();
			targetMob.splice(1,1);
		}


		for(var i=0,il=this.lights.length;i<il;i++ ){
			lights.push({
				pos:this.lights[i].position.toArray(),
				color:this.lights[i].color.getHex(),
				radius:this.lights[i].distance,
			});
		}



		var json = {
			name:name,
			model: model,
			doors: door,
			spawnPlayer: spawnPlayer,
			targetMob: targetMob,
			targetPlayer: targetPlayer,
			beginPlayer: beginPlayer,
			destructor: destructor,
			timeout: parseInt(timeout),
			timeTarget : parseInt(timeTarget),
			scoreTarget : parseInt(scoreTarget),
			lights:lights,
			skinId: skinId
		}

		console.log(json)

		return json;
	},

	cleanMap : function(){

		for(var k=0,kl=this.meshTiles.length;k<kl;k++ ){
			this.meshTiles[k].removeAllClones();
			this.meshTiles[k].addClone(0,-1000,0 );
		}
		for(var k=0,kl=this.meshElements.length;k<kl;k++ ){
			this.meshElements[k].removeAllClones();
			this.meshElements[k].addClone(0,-1000,0 );
		}
		for(var k=0,kl=this.meshMobs.length;k<kl;k++ ){
			this.meshMobs[k].removeAllClones();
			this.meshMobs[k].addClone(0,-1000,0 );
		}
		for(var k=0,kl=this.meshOil.length;k<kl;k++ ){
			this.meshOil[k].removeAllClones();
			this.meshOil[k].addClone(0,-1000,0 );
		}


		for(var i=0,il=this.lights.length;i<il;i++ ){
			this.scene1.remove(this.lights[i])
		}
		this.lights = [];


		if(this.meshDestructor) this.meshDestructor.removeAllClones();
		if(this.meshPlayer) this.meshPlayer.removeAllClones();
		if(this.meshTargetPlayer)this.meshTargetPlayer.removeAllClones();
		if(this.meshTargetMob)this.meshTargetMob.removeAllClones();
		if(this.meshLight) this.meshLight.removeAllClones();
		if(this.meshDestructor)this.meshDestructor.addClone(0,-1000,0 );
		if(this.meshPlayer)this.meshPlayer.addClone(0,-1000,0 );
		if(this.meshTargetPlayer)this.meshTargetPlayer.addClone(0,-1000,0 );
		if(this.meshTargetMob)this.meshTargetMob.addClone(0,-1000,0 );
		if(this.meshLight)this.meshLight.addClone(0,-1000,0 );
	},


	buildMap: function( data ) {

		var models = data.model;
		
		this.cleanMap(); 

		var d = this.tileSize;

		for(i=0, il=models.length; i<il;i++){

			var model = models[i];

			this.meshTiles[model.id].addClone(model.pos[0],0,model.pos[1],0,model.rot*Math.PI,0,0,0,model.sym?-1:1);
			var u = this.meshTiles[model.id].clones[this.meshTiles[model.id].clones.length-1].getRotationY()

			for(k=0, kl=model.ele.length; k<kl;k++){
				debugger;
				var element = model.ele[k];
				this.meshElements[element.id].addClone(element.pos[0],0,element.pos[1],0,element.rot*Math.PI,0,0,0,element.sym?-1:1);	
			}

			for(k=0, kl=model.mob.length; k<kl;k++){
				mob = model.mob[k];
				this.meshMobs[mob.id].addClone(mob.pos[0],0,mob.pos[1],0,mob.rot*Math.PI);	
			}

			for(k=0, kl=model.wea.length; k<kl;k++){
				wea = model.wea[k];
				this.meshOil[wea.id].addClone(wea.pos[0],0,wea.pos[1]);	
			}
		}

		if(data.doors){
			var doors = data.doors;
			var meshDoor = this.meshDoor;
			for(k=0,kl=doors.length;k<kl;k++ ){
				var dataDoor = doors[k];
				meshDoor.addClone(dataDoor.pos[0],0,dataDoor.pos[1],null,dataDoor.rot*Math.PI);
				meshDoor.getClone().levelId = dataDoor.levelId;
			}

		}

		if(data.destructor){
			var destructor = data.destructor;
			var meshDestructor = this.meshDestructor;
			for(k=0,kl=destructor.length;k<kl;k++ ){
				meshDestructor.addClone(destructor[k][0],0,destructor[k][1]);
			}
		}
		
		if(data.spawnPlayer){
			var spawnPlayer = data.spawnPlayer;
			var meshPlayer = this.meshPlayer;
			for(k=0,kl=spawnPlayer.length;k<kl;k++ ){
				meshPlayer.addClone(spawnPlayer[k][0],0,spawnPlayer[k][1]);
			}
		}

		if(data.targetPlayer){
			var targetPlayer = data.targetPlayer;
			var meshTargetPlayer = this.meshTargetPlayer;
			for(k=0,kl=targetPlayer.length;k<kl;k++ ){
				meshTargetPlayer.addClone(targetPlayer[k][0],0,targetPlayer[k][1]);
			}
		}

		if(data.beginPlayer){
			var beginPlayer = data.beginPlayer;
			var meshBeginPlayer = this.meshBeginPlayer;
			for(k=0,kl=beginPlayer.length;k<kl;k++ ){
				meshBeginPlayer.addClone(beginPlayer[0],0,beginPlayer[1]);
			}
		}

		if(data.targetMob){
			var targetMob = data.targetMob;
			var meshTargetMob = this.meshTargetMob;
			for(k=0,kl=targetMob.length;k<kl;k++ ){
				meshTargetMob.addClone(targetMob[k][0],0,targetMob[k][1]);
			}
		}

		for(var i=0,il=data.lights.length;i<il;i++ ){
			var l = data.lights[i];
			var newLight = new Engine.PointLight( l.color, 1.0 , l.radius, l.pos );
			this.meshLight.addClone(l.pos[0],0.5,l.pos[2]);
			this.lights.push(newLight);
			this.scene1.add(newLight);
		}


	},

	positionRealToIndex : function(v1,v2){
		var n = Math.floor(v1/this.tileSize);
		var m = Math.floor(v2/this.tileSize);
		return [n,m];
	}

};