
Mob = function(dataLocal,dataGlobal,material,geometry){

    /******************************************************/
	/**********************MODEL***************************/
	/******************************************************/

	/*****************variable deplacement*****************/
	this.m_position;
	this.m_direction;
	this.m_dirVector;
	this.m_speed;
    this.m_pointTarget;
    this.m_path;
    this.m_goTo;
    this.m_targetReached;
	/*****************variable d'affichage*****************/
	this.m_geometry;
	this.m_material;
	this.m_scene;
	/*****************variable d'etat**********************/
	this.m_currentState;
	this.m_hp;
	this.m_isHurt;
	this.m_follow;
	/*****************variable d'animation*****************/
	this.m_animations;
	/*****************variable timing**********************/
	this.m_interval;
	this.m_counter;
	/*****************variable divers**********************/
	this.m_player;
	this.m_level;
	this.m_pathManager;
	this.m_awakeDistance;
	this.m_attackDistance;
	this.m_errormin;
	this.m_radius;
	this.m_hurtInterval;
	this.m_hurtTimer;

	this.m_scoreValue;

	this.m_hasShadow = true;
	
    /******************************************************/
	this.c_initModel(dataLocal, dataGlobal,material,geometry);

	Mob.count ++;
};

/**********************************************************************************/
/**********************CONTROLLER**************************************************/
/**********************************************************************************/

Mob.count = 0;

Mob.prototype.c_initModel = function(dataLocal,dataGlobal,material,geometry){
	this.m_position = new Engine.Vector3( dataLocal.pos[0],0,dataLocal.pos[1] );
	this.m_direction = dataLocal.rot*Math.PI;
	this.m_goTo = dataLocal.target; //new Engine.Vector3(1,0,15) ;//dataLocal.goTo;
	this.m_speed = dataGlobal.speed;
	this.m_currentState = 0;
	this.m_hp = dataGlobal.hp;
	this.m_animations = dataGlobal.animations;
	this.m_deleted = false;
	this.m_material = material;
	this.m_geometry = geometry;
	this.m_awakeDistance = dataGlobal.awakeRange;
	this.m_errormin = 0.1;
	this.m_pointTarget = new Engine.Vector3();
	this.m_dirVector = new Engine.Vector3();
	this.m_attackDistance = dataGlobal.reach;
	this.m_radius = dataGlobal.size;
	this.m_damage = dataGlobal.damage;
	this.m_gameManager = dataLocal.gm;

	this.m_scoreValue = dataGlobal.value;

	this.m_interval = dataGlobal.reactivity;
	this.m_hurtInterval = 200;
	this.m_counter = 0;
	this.m_follow = false;
	this.m_ctnFinish = 0;
	this.m_ctnBlood = 0;

	this.m_mesh = new Engine.MeshMorph( this.m_geometry, this.m_material, this.m_hasShadow );

	//this.m_helper = new Engine.Mesh( new Engine.CubeGeometry( 0.5, 0.5, 0.5 ), this.m_material );

};



Mob.prototype.c_update = function(t){

	if( this.m_currentState != this.DEAD ){

		var distanceToPlayer = this.m_player.m_position.distanceTo(this.m_position, this.m_player.m_position);

		if(distanceToPlayer<this.m_attackDistance){
			this.c_actionAttack(t);
		}else if(distanceToPlayer<this.m_awakeDistance){
			
			this.c_detectionOfPlayer();

			if(!this.m_follow){
				if(this.m_goTo){
					this.c_actionGoToTarget(t); 
				}else{
					this.c_actionWait();
				}
				
			}else{
				if(!this.m_isHurt ){
					this.c_actionGoToPlayer(t);
				}
			}

		}else{
			if(this.m_goTo){
				this.c_actionGoToTarget(t);
			}else{
				this.c_actionWait();
			}
		}

		if(this.m_player.m_win){

			this.c_dead();
		}
	}else{

		if(this.m_blood.progress>0.95){
			this.m_deleted = true;
		}

		this.m_ctnFinish += t/350;

		this.m_blood.progress = -Math.exp(-this.m_ctnFinish*3)+1;
		//this.m_blood.material.opacity = (1-(this.m_ctnFinish))*2-1;
		//this.m_blood.material.opacity = 1-this.m_ctnFinish//(1-(this.m_ctnFinish))*2-1;
		this.m_blood.material.opacity = Math.cos(this.m_ctnFinish*Math.PI/2)

	}


	if(this.m_counter > this.m_interval){
		this.m_counter = 0;
	}else{
		this.m_counter +=t; 
	}

	this.c_checkDestructionTiles();

	this.c_updateMesh(t);

};



Mob.prototype.c_actionGoToPlayer = function(t){
	this.c_updatePath(t,this.m_player.m_position);
	if(this.m_path&&this.m_path.length)
		this.c_followPath(t);
	this.c_updateDirection(this.m_dirVector.xzToAngle(),t);
	this.m_currentState = this.WALK;
};


Mob.prototype.c_actionAttack = function(t){
	this.m_dirVector.copy(this.m_player.m_position).sub(this.m_position);
	this.c_updateDirection(this.m_dirVector.xzToAngle(),t);
	this.m_player.c_hurt(this.m_damage);
	this.m_currentState = this.SHOOT;
};

Mob.prototype.c_actionGoToTarget = function(t){
	if(this.m_targetReached)return;

	if( this.m_position.distanceTo(this.m_goTo) < this.m_radius) {
		this.m_gameManager.c_MobTakeTarget(this);
		this.m_targetReached = true;
	}

	this.c_updatePath(t,this.m_goTo,1000);
	if(this.m_path&&this.m_path.length)
		this.c_followPath(t);
	this.c_updateDirection(this.m_dirVector.xzToAngle(),t);
	this.m_currentState = this.WALK;
};

Mob.prototype.c_actionWait = function(t){
	this.m_follow = false;
	this.m_currentState = this.WAIT;
};



Mob.prototype.c_updateDirection = function(angle,t){

    angle = Math.to2KPI(angle);

    this.m_direction += Math.subAngle(angle, this.m_direction)*t*0.01;

    this.m_direction = Math.to2KPI(this.m_direction);

};

Mob.prototype.c_detectionOfPlayer = function(){
	if(this.m_counter > this.m_interval && !this.m_follow){
		this.m_dirVector.copy(this.m_player.m_position).sub(this.m_position);
		var e = Math.abs(Math.subAngle(this.m_direction,this.m_dirVector.xzToAngle()));
		if(this.m_pathManager.c_isDirectPath(this.m_position, this.m_player.m_position) && e<Math.PI/2){
			sm.play('assets/sound/monstre0.mp3',1);
			this.m_follow = true;
		}
	}
};




Mob.prototype.c_updatePath = function(t,target,interation){


	if(this.m_counter > this.m_interval){
		this.m_path = this.m_pathManager.findPath(this.m_position, target, interation);
		/*var path = this.m_path
		var m_helper = this.m_helper;
		m_helper.removeAllClones();
		var sizeb = this.m_level.m_blockSize;
		for(var i=0, il= path.length; i<il; i++){
			m_helper.addClone(path[i].x*sizeb,0,path[i].y*sizeb);
		}*/
	}

};

Mob.prototype.c_followPath = function(t){

	var b = this.m_level.m_blockSize;
	var tar = this.m_pointTarget;
	var pos = this.m_position;
	var min = this.m_errormin;
	var dir = this.m_dirVector;
	var a;
	
	if(Math.abs(tar.x-pos.x)<min && Math.abs(tar.z-pos.z)<min ){
		this.m_path.shift();
	}

	if(this.m_path.length){
		tar.x = this.m_path[0].x*b;
		tar.z = this.m_path[0].y*b;	
	}else{
		tar.x = this.m_player.m_position.x*b;
		tar.z = this.m_player.m_position.y*b;		
	}


	dir.set(tar.x,0,tar.z).sub(pos).normalize();

	pos.x += dir.x*this.m_speed;
	pos.z += dir.z*this.m_speed;

};





Mob.prototype.c_dead = function(){
	if(this.m_currentState != this.DEAD){
		this.m_currentState = this.DEAD;
		var self = this;
		this.m_blood.visible = true;
		self.m_mesh.visible = false;
		this.m_blood.position = self.m_position.clone();
		this.m_blood.rotation.y = Math.random()*Math.PI*2;
		this.m_blood.position.y = 1;
		this.m_blood.keyA.index = 0;
		this.m_blood.keyB.index = 1;
		this.m_blood.progress =0;
		this.m_blood.scale.x  = 1;
		this.m_blood.scale.z  = 1;
	}
};



Mob.prototype.c_checkDestructionTiles = function(){
	var p = this.m_position;
	if(this.m_level.c_positionToTile(p.x,p.z).destructor){
		this.m_deleted = true;
	}
};




/**********************************************************************************/
/**********************CONTROLLER GESTION DES ANIMATIONS***************************/
/**********************************************************************************/

Mob.prototype.c_updateMesh = function(t){

	this.m_mesh.position = this.m_position;
	this.m_mesh.rotation.y = -this.m_direction;

	var d = (1-this.m_mesh.scale.x)*0.005;
	this.m_mesh.scale.x += d*t;
	this.m_mesh.scale.z += d*t;	

		
	switch(this.m_currentState){

		case this.WAIT :
				this.m_mesh.setAnimation(this.m_animations.wait,1000,true);
		break;
		case this.WALK :
				this.m_mesh.setAnimation(this.m_animations.walk,500,true);
		break;
		case this.SHOOT :
				this.m_mesh.setAnimation(this.m_animations.shoot,500,true);
		break;
		case this.BEGIN :
				this.m_mesh.setAnimation(this.m_animations.wait,2000,true);
		break;
		case this.DEAD :
				this.m_mesh.setAnimation(this.m_animations.wait,2000,true);
		break;
	}

	this.m_mesh.update(t);
};




Mob.prototype.c_setPlayer = function(player){

	this.m_player = player;

};

Mob.prototype.c_setPathManager = function(pathManager){

	this.m_pathManager = pathManager;

};

Mob.prototype.c_setLevel = function(level){

	this.m_level = level;

	this.m_pathManager.c_setLevel(this.m_level);

	this.m_level.c_addDynamicCollisionBlock(this);

};


Mob.prototype.c_setBlood = function(bloodGeo,bloodMap){
	this.m_blood = new Engine.MeshMorph( bloodGeo,  new Engine.MeshPhongMaterial( { map: bloodMap, transparent : true  } ) );
	this.m_blood.visible = false;
	this.m_blood.rotation.y = Math.random()*Math.PI*2;
	this.m_blood.position.y = 1;
	this.m_blood.keyA.index = 0;
	this.m_blood.keyB.index = 1;

};





/**********************************************************************************/
/**********************CONTROLLER PRIMAIRE*****************************************/
/**********************************************************************************/

Mob.prototype.c_getPosition = function() {
	return this.m_position;
};

Mob.prototype.c_setPosition = function(position) {
	this.m_position = position;
};

Mob.prototype.add = function(scene) {
	scene.add(this.m_mesh);
	scene.add(this.m_blood);
	this.m_scene = scene; 
};

Mob.prototype.remove = function(scene) {

	this.m_level.c_removeDynamicCollisionBlock(this);
	scene.remove(this.m_mesh);
	scene.remove(this.m_blood);
	this.m_scene = null;

	Mob.count -=1;
};

Mob.prototype.WAIT = 1;
Mob.prototype.WALK = 2;
Mob.prototype.SHOOT = 3;
Mob.prototype.DEAD = 9;








