
PathFinding = function(){
	this.m_level;
    this.m_nodes;
    this.m_path;
    this.m_width;
    this.m_height;
    this.m_directPath;
};


PathFinding.prototype.c_setLevel = function(level){

	this.m_level = level;
    this.m_blockSize = level.m_blockSize;

    this.m_N = this.m_level.m_N * this.m_level.m_blockFactor;
    this.m_M = this.m_level.m_M * this.m_level.m_blockFactor;

    this.c_buildNodes();
    
};

PathFinding.prototype.c_init = function(){
    
};

PathFinding.prototype.c_manhattan = function(dx, dy){
     return dx + dy;
};
PathFinding.prototype.c_euclidean = function(dx, dy){
       return Math.sqrt(dx * dx + dy * dy);
};
PathFinding.prototype.c_euclidean = function(dx, dy){
        return Math.max(dx, dy);
};

PathFinding.prototype.c_buildNodes = function(){
        var width = this.m_N,
        height =  this.m_M,
        i = width, j,
        nodes = new Array(width),row;
        while (i--) {
            nodes[i] = new Array(height);
            j = height;
            while (j--) {
                nodes[i][j] = { x:i,y:j};
            }
        }
        this.m_nodes = nodes;  
};

PathFinding.prototype.c_updateWalkable = function(){
        var i = this.m_N; 
        var j;
       
        while (i--) {
            j = this.m_M;
            while (j--) {
                this.m_nodes[i][j].walkable =  !this.m_level.c_getBlockByIndex(i,j);
            }
        }

};

PathFinding.prototype.c_clearNode = function(){
        var node;
        var i = this.m_N; 
        var j = this.m_M;
        while (i--) {
            j = this.m_M;
            while (j--) {
                node = this.m_nodes[i][j];
                node.opened = false;
                node.closed = false;
                node.parent = null;
                node.h = 0;
            }
        }

};

PathFinding.prototype.getNodeAt = function(x,y){
    var n = Math.floor(x/this.m_blockSize);
    var m = Math.floor(y/this.m_blockSize);
    return this.m_nodes[n][m];
};


PathFinding.prototype.getNeighbors = function(node){

        var x = node.x,
        y = node.y,
        neighbors = [],
        nodes = this.m_nodes;


    if (this.isWalkableAt(x, y - 1)) {
        neighbors.push(nodes[x][y - 1]);
    }

    if (this.isWalkableAt(x + 1, y)) {
        neighbors.push(nodes[x + 1][y]);
    }

    if (this.isWalkableAt(x, y + 1)) {
        neighbors.push(nodes[x][y + 1]);
    }

    if (this.isWalkableAt(x - 1, y)) {
        neighbors.push(nodes[x - 1][y]);
    }

    return neighbors;
};


PathFinding.prototype.isWalkableAt = function(x, y) {

    if(this.m_nodes[x] && this.m_nodes[x][y] && (this.m_nodes[x][y].walkable || this.m_nodes[x][y]==this.endNode) ){
    //if(this.m_nodes[x][y].walkable || this.m_nodes[x][y]==this.endNode ){
        return true;
    }else{
        return false;
    }
};


PathFinding.prototype.createPath = function(nodeEnd){

    var parent = nodeEnd.parent;
    var path = [];

    path.push({x:nodeEnd.x,y:nodeEnd.y});

    while(parent){
        path.push({x:parent.x,y:parent.y});
        parent = parent.parent;
    }
    path.reverse();
    path = this.smoothPath(path);
    this.m_path = path;
    return path;
};

PathFinding.prototype.smoothPath = function(path){
    
    var smoothpath = [];
    var currentPoint;
    var nextPoint;
    for(var i=0,il=path.length-1;i<il;i++){
        currentPoint = path[i];
        nextPoint = path[i+1];
        smoothpath[i]={
            x: currentPoint.x+0.5+(nextPoint.x-currentPoint.x)*0.5 ,
            y: currentPoint.y+0.5+(nextPoint.y-currentPoint.y)*0.5
        };
    }
    
    return smoothpath;

};




PathFinding.prototype.c_isDirectPath = function(positionStart, positionEnd){
    var startNode = this.getNodeAt(positionStart.x, positionStart.z);
    var endNode = this.getNodeAt(positionEnd.x, positionEnd.z);

    var x = startNode.x;
    var y = startNode.y;
    var dx = endNode.x - x;
    var dy = endNode.y - y;
    var i = 10;
    while(i--){
        if(!this.isWalkableAt(x+dx,y+dy)){
            return false;
        }
        if(dx!==0){
            dx-=dx/Math.abs(dx);
        }
        if(dy!==0){
            dy-=dy/Math.abs(dy);
        }

        if(dx===0 && dy===0){
            break;
        }
    }

    return true;
};





PathFinding.prototype.findPath = function(positionStart, positionEnd, interation) {

    this.c_clearNode();

    interation = !interation ? 10:interation;
 
    var openList = new Heap(function(nodeA, nodeB) {return nodeA.f - nodeB.f;});

    var startNode = this.getNodeAt(positionStart.x, positionStart.z);
    var endNode = this.getNodeAt(positionEnd.x, positionEnd.z);
    this.endNode = endNode;

    var endX = endNode.x;
    var endY = endNode.y;

    var heuristic = this.c_euclidean;
    var abs = Math.abs, SQRT2 = Math.SQRT2;
    var node, neighbors, neighbor, i, l, x, y, ng;

    // set the `g` and `f` value of the start node to be 0
    startNode.g = 0;
    startNode.f = 0;

    // push the start node into the open list
    openList.push(startNode);
    startNode.opened = true;
    var ctn = 0;
    // while the open list is not empty
    while (!openList.empty() && ctn<interation) {
      
        ctn++;
        // pop the position of node which has the minimum `f` value.
        node = openList.pop();
        node.closed = true;

        // if reached the end position, construct the path and return it
        if (node === endNode) {
            var path = this.createPath(node);
           // this.displayInConsole(startNode,endNode);
            return path;
        }

        // get neigbours of the current node
        neighbors = this.getNeighbors(node);

        var ar = neighbors.length;
        for (i = 0, l = neighbors.length; i < l; ++i) {
            neighbor = neighbors[i];

            if (neighbor.closed && neighbor != endNode) {
                continue;
            }

            x = neighbor.x;
            y = neighbor.y;

            // get the distance between current node and the neighbor
            // and calculate the next g score
            ng = 1;

            // check if the neighbor has not been inspected yet, or
            // can be reached with smaller cost from the current node
            if (!neighbor.opened || ng < neighbor.g) {
                neighbor.g = ng * ar/9; //the trace magic
                neighbor.h = neighbor.h || heuristic(abs(x - endX), abs(y - endY));
                neighbor.f = neighbor.g + neighbor.h;
                neighbor.parent = node;

                if (!neighbor.opened) {
                    //openList.push(neighbor);
                    openList.push(neighbor);
                    neighbor.opened = true;
                } else {
                    // the neighbor can be reached with smaller cost.
                    // Since its f value has been updated, we have to
                    // update its position in the open list

                    //openList.updateItem(neighbor);
                    openList.updateItem(neighbor);
                }
            }
        } // end for each neighbor
    } // end while not open list empty

    // fail to find the path
    //console.log("path fail");
    return [];
};
