//////////////////////////////////
// Lib Engine
///////////////////////////////////

	Engine = function(node){

		if(typeof performance != 'undefined' && performance.now){
			this.now = function(){ return performance.now();};
		}else if ( typeof performance != 'undefined' && performance.webkitNow ) {
			this.now = function(){ return performance.webkitNow();};
		}else{
			this.now = function(){return (new Date()).getTime()};
		}
		this.lastTime = this.now();

		//this.constructorEventManager.call(this);
		this.constructorLoopManager.call(this);

		this.initEvents(this,node);

	};

	Engine.pendingRequest = 0;


//////////////////////////////////
// Main Loop
//////////////////////////////////

	( function () {

		var lastTime = 0;
		var vendors = [ 'ms', 'moz', 'webkit', 'o' ];

		for ( var x = 0; x < vendors.length && !self.requestAnimationFrame; ++ x ) {

			self.requestAnimationFrame = self[ vendors[ x ] + 'RequestAnimationFrame' ];
			self.cancelAnimationFrame = self[ vendors[ x ] + 'CancelAnimationFrame' ] || self[ vendors[ x ] + 'CancelRequestAnimationFrame' ];

		}

		if ( self.requestAnimationFrame === undefined && self['setTimeout'] !== undefined ) {

			self.requestAnimationFrame = function ( callback ) {

				var currTime = Date.now(), timeToCall = Math.max( 0, 16 - ( currTime - lastTime ) );
				//var currTime = performance.webkitNow(), timeToCall = Math.max( 0, 16 - ( currTime - lastTime ) );
				var id = self.setTimeout( function() { callback( currTime + timeToCall ); }, timeToCall );
				lastTime = currTime + timeToCall;
				return id;
			};

		}

		if( self.cancelAnimationFrame === undefined && self['clearTimeout'] !== undefined ) {

			self.cancelAnimationFrame = function ( id ) { self.clearTimeout( id ); };

		}

	}() );

	Engine.prototype.constructorLoopManager = function () {

		var self = this;

		var tempoFps = 0;

		var sum = 0.0;
		var sumCtn = 0
		var maxRequest = 1;

		var requestAnimTime = function(){
			self.currentTime = self.now();
			Engine.currentTime = self.currentTime;
			var d = self.currentTime-self.lastTime
			self.lastTime = self.currentTime;
			tempoFps += d;
			
			if(!self.divFps){
				self.divFps = document.createElement('div')
				self.divFps.className = "framerate";
				document.body.appendChild(self.divFps);
				self.divFps.style.color = "#999999";
				self.divFps.textContent = self.fps;
			}else{
				if(tempoFps>90){
					self.fps = (1/(d/1000));
					sum+=self.fps;
					sumCtn++;
					if(sumCtn>9){
						self.divFps.textContent = (sum/sumCtn).toPrecision(2)+' fps';
						sumCtn = 0;
						sum = 0;
					}

					tempoFps = 0;
				}
			}

			return Math.min(d,100);

		}

		this.mainLoop = function() { 

			if(self.render){
				self.render();
			}

			if(self.animate){
				self.animate(requestAnimTime());
			}

			if(self.runing){
				requestAnimationFrame( self.mainLoop );
			}

		};
		
		this.start =  function start(){
			console.log("Engine.pendingRequest"+Engine.pendingRequest)

			if(!Engine.pendingRequest){
				console.log("start")
				this.loading(1);

				if(self.postStart){
					self.postStart();
				}
				self.runing = true;
				self.mainLoop();

			}else{

				maxRequest = Math.max(maxRequest,Engine.pendingRequest);

				this.loading(Engine.pendingRequest/maxRequest);
				
				setTimeout(function(){self.start()},500);

			}

		};

		this.stop =  function stop(){
			Engine.pendingRequest = 0;
			self.runing = false;
		};

		this.loading = function(progress,finish){

		}


	};

	Engine.shaderManager = {}

	Engine.createNode = function(className,text,parent) {

		var div = document.createElement('div');
		div.className = className;
		if(text){
			div.textContent = text;
		}
		if(parent){
			parent.appendChild(div);
		}else{
			document.body.appendChild(div);
		}
		return div;
	}


	Engine.conf = {
		MAX_POINT_LIGHTS : 10,
		MAX_DIRECTION_LIGHTS : 3
	}


Engine.Vector2 = function ( x, y ) {

	this.x = x || 0;
	this.y = y || 0;

};

Engine.Vector2.prototype = {

	constructor: Engine.Vector2,

	set: function ( x, y ) {

		this.x = x;
		this.y = y;

		return this;

	},

	setX: function ( x ) {

		this.x = x;

		return this;

	},

	setY: function ( y ) {

		this.y = y;

		return this;

	},


	setComponent: function ( index, value ) {

		switch ( index ) {

			case 0: this.x = value; break;
			case 1: this.y = value; break;
			default: throw new Error( "index is out of range: " + index );

		}

	},

	getComponent: function ( index ) {

		switch ( index ) {

			case 0: return this.x;
			case 1: return this.y;
			default: throw new Error( "index is out of range: " + index );

		}

	},

	copy: function ( v ) {

		this.x = v.x;
		this.y = v.y;

		return this;

	},

	add: function ( v) {

		this.x += v.x;
		this.y += v.y;

		return this;

	},

	addVectors: function ( a, b ) {

		this.x = a.x + b.x;
		this.y = a.y + b.y;

		return this;

	},

	addScalar: function ( s ) {

		this.x += s;
		this.y += s;

		return this;

	},

	sub: function ( v) {
		
		this.x -= v.x;
		this.y -= v.y;

		return this;

	},

	subVectors: function ( a, b ) {

		this.x = a.x - b.x;
		this.y = a.y - b.y;

		return this;

	},

	multiplyScalar: function ( s ) {

		this.x *= s;
		this.y *= s;

		return this;

	},

	divideScalar: function ( scalar ) {

		if ( scalar !== 0 ) {

			var invScalar = 1 / scalar;

			this.x *= invScalar;
			this.y *= invScalar;

		} else {

			this.x = 0;
			this.y = 0;

		}

		return this;

	},

	min: function ( v ) {

		if ( this.x > v.x ) {

			this.x = v.x;

		}

		if ( this.y > v.y ) {

			this.y = v.y;

		}

		return this;

	},

	max: function ( v ) {

		if ( this.x < v.x ) {

			this.x = v.x;

		}

		if ( this.y < v.y ) {

			this.y = v.y;

		}

		return this;

	},

	clamp: function ( min, max ) {

		if ( this.x < min.x ) {

			this.x = min.x;

		} else if ( this.x > max.x ) {

			this.x = max.x;

		}

		if ( this.y < min.y ) {

			this.y = min.y;

		} else if ( this.y > max.y ) {

			this.y = max.y;

		}

		return this;

	},

	negate: function() {

		return this.multiplyScalar( - 1 );

	},

	dot: function ( v ) {

		return this.x * v.x + this.y * v.y;

	},

	lengthSq: function () {

		return this.x * this.x + this.y * this.y;

	},

	length: function () {

		return Math.sqrt( this.x * this.x + this.y * this.y );

	},

	normalize: function () {

		return this.divideScalar( this.length() );

	},

	distanceTo: function ( v ) {

		return Math.sqrt( this.distanceToSquared( v ) );

	},

	distanceToSquared: function ( v ) {

		var dx = this.x - v.x, dy = this.y - v.y;
		return dx * dx + dy * dy;

	},

	setLength: function ( l ) {

		var oldLength = this.length();

		if ( oldLength !== 0 && l !== oldLength ) {

			this.multiplyScalar( l / oldLength );
		}

		return this;

	},

	lerp: function ( v, alpha ) {

		this.x += ( v.x - this.x ) * alpha;
		this.y += ( v.y - this.y ) * alpha;

		return this;

	},

	equals: function( v ) {

		return ( ( v.x === this.x ) && ( v.y === this.y ) );

	},

	fromArray: function ( array ) {

		this.x = array[ 0 ];
		this.y = array[ 1 ];

		return this;

	},

	toArray: function () {

		return [ this.x, this.y ];

	},

	clone: function () {

		return new Engine.Vector2( this.x, this.y );

	}

};
Engine.Vector3 = function ( x, y, z ) {

	this.x = x || 0;
	this.y = y || 0;
	this.z = z || 0;

};

Engine.Vector3.prototype = {

	constructor: Engine.Vector3,

	set: function ( x, y, z ) {

		this.x = x;
		this.y = y;
		this.z = z;

		return this;

	},

	setX: function ( x ) {

		this.x = x;

		return this;

	},

	setY: function ( y ) {

		this.y = y;

		return this;

	},

	setZ: function ( z ) {

		this.z = z;

		return this;

	},

	setComponent: function ( index, value ) {

		switch ( index ) {

			case 0: this.x = value; break;
			case 1: this.y = value; break;
			case 2: this.z = value; break;
			default: throw new Error( "index is out of range: " + index );

		}

	},

	getComponent: function ( index ) {

		switch ( index ) {

			case 0: return this.x;
			case 1: return this.y;
			case 2: return this.z;
			default: throw new Error( "index is out of range: " + index );

		}

	},

	copy: function ( v ) {

		this.x = v.x;
		this.y = v.y;
		this.z = v.z;

		return this;

	},

	add: function ( v ) {

		this.x += v.x;
		this.y += v.y;
		this.z += v.z;

		return this;

	},

	addScalar: function ( s ) {

		this.x += s;
		this.y += s;
		this.z += s;

		return this;

	},

	addVectors: function ( a, b ) {

		this.x = a.x + b.x;
		this.y = a.y + b.y;
		this.z = a.z + b.z;

		return this;

	},

	sub: function ( v) {

		this.x -= v.x;
		this.y -= v.y;
		this.z -= v.z;

		return this;

	},

	subVectors: function ( a, b ) {

		this.x = a.x - b.x;
		this.y = a.y - b.y;
		this.z = a.z - b.z;

		return this;

	},

	multiply: function ( v ) {

		this.x *= v.x;
		this.y *= v.y;
		this.z *= v.z;

		return this;

	},

	multiplyScalar: function ( scalar ) {

		this.x *= scalar;
		this.y *= scalar;
		this.z *= scalar;

		return this;

	},

	multiplyVectors: function ( a, b ) {

		this.x = a.x * b.x;
		this.y = a.y * b.y;
		this.z = a.z * b.z;

		return this;

	},

	applyMatrix3: function ( m ) {

		var x = this.x;
		var y = this.y;
		var z = this.z;

		var e = m.elements;

		this.x = e[0] * x + e[3] * y + e[6] * z;
		this.y = e[1] * x + e[4] * y + e[7] * z;
		this.z = e[2] * x + e[5] * y + e[8] * z;

		return this;

	},

	applyMatrix4: function ( m ) {

		var x = this.x, y = this.y, z = this.z;

		var e = m.elements;

		this.x = e[0] * x + e[4] * y + e[8]  * z + e[12];
		this.y = e[1] * x + e[5] * y + e[9]  * z + e[13];
		this.z = e[2] * x + e[6] * y + e[10] * z + e[14];

		return this;

	},

	applyMatrix3of4: function ( m ) {

		var x = this.x, y = this.y, z = this.z;

		var e = m.elements;

		this.x = e[0] * x + e[4] * y + e[8]  * z ;
		this.y = e[1] * x + e[5] * y + e[9]  * z ;
		this.z = e[2] * x + e[6] * y + e[10] * z ;

		return this;

	},

	applyProjection: function ( m ) {

		// input: Engine.Matrix4 projection matrix

		var x = this.x, y = this.y, z = this.z;

		var e = m.elements;
		var d = 1 / ( e[3] * x + e[7] * y + e[11] * z + e[15] ); // perspective divide

		this.x = ( e[0] * x + e[4] * y + e[8]  * z + e[12] ) * d;
		this.y = ( e[1] * x + e[5] * y + e[9]  * z + e[13] ) * d;
		this.z = ( e[2] * x + e[6] * y + e[10] * z + e[14] ) * d;

		return this;

	},

	applyQuaternion: function ( q ) {

		var x = this.x;
		var y = this.y;
		var z = this.z;

		var qx = q.x;
		var qy = q.y;
		var qz = q.z;
		var qw = q.w;

		// calculate quat * vector

		var ix =  qw * x + qy * z - qz * y;
		var iy =  qw * y + qz * x - qx * z;
		var iz =  qw * z + qx * y - qy * x;
		var iw = -qx * x - qy * y - qz * z;

		// calculate result * inverse quat

		this.x = ix * qw + iw * -qx + iy * -qz - iz * -qy;
		this.y = iy * qw + iw * -qy + iz * -qx - ix * -qz;
		this.z = iz * qw + iw * -qz + ix * -qy - iy * -qx;

		return this;

	},

	transformDirection: function ( m ) {

		// input: Engine.Matrix4 affine matrix
		// vector interpreted as a direction

		var x = this.x, y = this.y, z = this.z;

		var e = m.elements;

		this.x = e[0] * x + e[4] * y + e[8]  * z;
		this.y = e[1] * x + e[5] * y + e[9]  * z;
		this.z = e[2] * x + e[6] * y + e[10] * z;

		this.normalize();

		return this;

	},

	divide: function ( v ) {

		this.x /= v.x;
		this.y /= v.y;
		this.z /= v.z;

		return this;

	},

	divideScalar: function ( scalar ) {

		if ( scalar !== 0 ) {

			var invScalar = 1 / scalar;

			this.x *= invScalar;
			this.y *= invScalar;
			this.z *= invScalar;

		} else {

			this.x = 0;
			this.y = 0;
			this.z = 0;

		}

		return this;

	},

	min: function ( v ) {

		if ( this.x > v.x ) {

			this.x = v.x;

		}

		if ( this.y > v.y ) {

			this.y = v.y;

		}

		if ( this.z > v.z ) {

			this.z = v.z;

		}

		return this;

	},

	max: function ( v ) {

		if ( this.x < v.x ) {

			this.x = v.x;

		}

		if ( this.y < v.y ) {

			this.y = v.y;

		}

		if ( this.z < v.z ) {

			this.z = v.z;

		}

		return this;

	},

	clamp: function ( min, max ) {

		if ( this.x < min.x ) {

			this.x = min.x;

		} else if ( this.x > max.x ) {

			this.x = max.x;

		}

		if ( this.y < min.y ) {

			this.y = min.y;

		} else if ( this.y > max.y ) {

			this.y = max.y;

		}

		if ( this.z < min.z ) {

			this.z = min.z;

		} else if ( this.z > max.z ) {

			this.z = max.z;

		}

		return this;

	},

	negate: function () {

		return this.multiplyScalar( - 1 );

	},

	dot: function ( v ) {

		return this.x * v.x + this.y * v.y + this.z * v.z;

	},

	lengthSq: function () {

		return this.x * this.x + this.y * this.y + this.z * this.z;

	},

	length: function () {

		return Math.sqrt( this.x * this.x + this.y * this.y + this.z * this.z );

	},

	lengthManhattan: function () {

		return Math.abs( this.x ) + Math.abs( this.y ) + Math.abs( this.z );

	},

	normalize: function () {

		return this.divideScalar( this.length() );

	},

	setLength: function ( l ) {

		var oldLength = this.length();

		if ( oldLength !== 0 && l !== oldLength  ) {

			this.multiplyScalar( l / oldLength );
		}

		return this;

	},

	lerp: function ( v, alpha ) {

		this.x += ( v.x - this.x ) * alpha;
		this.y += ( v.y - this.y ) * alpha;
		this.z += ( v.z - this.z ) * alpha;

		return this;

	},

	cross: function ( v ) {

		var x = this.x, y = this.y, z = this.z;

		this.x = y * v.z - z * v.y;
		this.y = z * v.x - x * v.z;
		this.z = x * v.y - y * v.x;

		return this;

	},

	crossVectors: function ( a, b ) {

		var ax = a.x, ay = a.y, az = a.z;
		var bx = b.x, by = b.y, bz = b.z;

		this.x = ay * bz - az * by;
		this.y = az * bx - ax * bz;
		this.z = ax * by - ay * bx;

		return this;

	},

	angleTo: function ( v ) {

		var theta = this.dot( v ) / ( this.length() * v.length() );

		// clamp, to handle numerical problems

		return Math.acos( Math.clamp( theta, -1, 1 ) );

	},

	distanceTo: function ( v ) {

		return Math.sqrt( this.distanceToSquared( v ) );

	},

	distanceToSquared: function ( v ) {

		var dx = this.x - v.x;
		var dy = this.y - v.y;
		var dz = this.z - v.z;

		return dx * dx + dy * dy + dz * dz;

	},

	setFromMatrixPosition: function ( m ) {

		this.x = m.elements[ 12 ];
		this.y = m.elements[ 13 ];
		this.z = m.elements[ 14 ];

		return this;

	},

	setFromMatrixScale: function ( m ) {

		var sx = this.set( m.elements[ 0 ], m.elements[ 1 ], m.elements[  2 ] ).length();
		var sy = this.set( m.elements[ 4 ], m.elements[ 5 ], m.elements[  6 ] ).length();
		var sz = this.set( m.elements[ 8 ], m.elements[ 9 ], m.elements[ 10 ] ).length();

		this.x = sx;
		this.y = sy;
		this.z = sz;

		return this;
	},

	setFromMatrixColumn: function ( index, matrix ) {

		var offset = index * 4;

		var me = matrix.elements;

		this.x = me[ offset ];
		this.y = me[ offset + 1 ];
		this.z = me[ offset + 2 ];

		return this;

	},

	equals: function ( v ) {

		return ( ( v.x === this.x ) && ( v.y === this.y ) && ( v.z === this.z ) );

	},

	fromArray: function ( array ) {

		this.x = array[ 0 ];
		this.y = array[ 1 ];
		this.z = array[ 2 ];

		return this;

	},

	toArray: function () {

		return [ this.x, this.y, this.z ];

	},

	clone: function () {

		return new Engine.Vector3( this.x, this.y, this.z );

	},


	cartToPolar: function () {

	var polar = {};

	polar.radius = this.length();

	polar.angle1 = Math.asin( this.y / polar.radius );

	if (this.x == 0 && this.z == 0) this.x = 0.001;

	if (this.z < 0)
		polar.angle2 = (2 * Math.PI - Math.acos(this.x / Math.sqrt(Math.pow(this.x, 2) + Math.pow(this.z, 2))));
	else
		polar.angle2 = (Math.acos(this.x / Math.sqrt(Math.pow(this.x, 2) + Math.pow(this.z, 2))));

	return polar;

	},

	polarToCart: function (angle1, angle2, ray) {

		this.x = ray * Math.cos(angle1) * Math.cos(angle2);
		this.z = ray * Math.cos(angle1) * Math.sin(angle2);
		this.y = ray * Math.sin(angle1);
		
		return this;
	},

	xzToAngle: function () {
		if (this.x == 0 && this.z == 0) this.x = 0.001;

		if (this.z < 0)
			return  (2 * Math.PI - Math.acos(this.x / Math.sqrt(Math.pow(this.x, 2) + Math.pow(this.z, 2))));
		else
			return (Math.acos(this.x / Math.sqrt(Math.pow(this.x, 2) + Math.pow(this.z, 2))));
	}

};

Engine.Vector4 = function ( x, y, z, w ) {

	this.x = x || 0;
	this.y = y || 0;
	this.z = z || 0;
	this.w = ( w !== undefined ) ? w : 1;

};

Engine.Vector4.prototype = {

	constructor: Engine.Vector4,

	set: function ( x, y, z, w ) {

		this.x = x;
		this.y = y;
		this.z = z;
		this.w = w;

		return this;

	},

	setX: function ( x ) {

		this.x = x;

		return this;

	},

	setY: function ( y ) {

		this.y = y;

		return this;

	},

	setZ: function ( z ) {

		this.z = z;

		return this;

	},

	setW: function ( w ) {

		this.w = w;

		return this;

	},

	setComponent: function ( index, value ) {

		switch ( index ) {

			case 0: this.x = value; break;
			case 1: this.y = value; break;
			case 2: this.z = value; break;
			case 3: this.w = value; break;
			default: throw new Error( "index is out of range: " + index );

		}

	},

	getComponent: function ( index ) {

		switch ( index ) {

			case 0: return this.x;
			case 1: return this.y;
			case 2: return this.z;
			case 3: return this.w;
			default: throw new Error( "index is out of range: " + index );

		}

	},

	copy: function ( v ) {

		this.x = v.x;
		this.y = v.y;
		this.z = v.z;
		this.w = ( v.w !== undefined ) ? v.w : 1;

		return this;

	},

	add: function ( v, w ) {

		if ( w !== undefined ) {

			console.warn( 'DEPRECATED: Vector4\'s .add() now only accepts one argument. Use .addVectors( a, b ) instead.' );
			return this.addVectors( v, w );

		}

		this.x += v.x;
		this.y += v.y;
		this.z += v.z;
		this.w += v.w;

		return this;

	},

	addScalar: function ( s ) {

		this.x += s;
		this.y += s;
		this.z += s;
		this.w += s;

		return this;

	},

	addVectors: function ( a, b ) {

		this.x = a.x + b.x;
		this.y = a.y + b.y;
		this.z = a.z + b.z;
		this.w = a.w + b.w;

		return this;

	},

	sub: function ( v, w ) {

		if ( w !== undefined ) {

			console.warn( 'DEPRECATED: Vector4\'s .sub() now only accepts one argument. Use .subVectors( a, b ) instead.' );
			return this.subVectors( v, w );

		}

		this.x -= v.x;
		this.y -= v.y;
		this.z -= v.z;
		this.w -= v.w;

		return this;

	},

	subVectors: function ( a, b ) {

		this.x = a.x - b.x;
		this.y = a.y - b.y;
		this.z = a.z - b.z;
		this.w = a.w - b.w;

		return this;

	},

	multiplyScalar: function ( scalar ) {

		this.x *= scalar;
		this.y *= scalar;
		this.z *= scalar;
		this.w *= scalar;

		return this;

	},

	applyMatrix4: function ( m ) {

		var x = this.x;
		var y = this.y;
		var z = this.z;
		var w = this.w;

		var e = m.elements;

		this.x = e[0] * x + e[4] * y + e[8] * z + e[12] * w;
		this.y = e[1] * x + e[5] * y + e[9] * z + e[13] * w;
		this.z = e[2] * x + e[6] * y + e[10] * z + e[14] * w;
		this.w = e[3] * x + e[7] * y + e[11] * z + e[15] * w;

		return this;

	},

	divideScalar: function ( scalar ) {

		if ( scalar !== 0 ) {

			var invScalar = 1 / scalar;

			this.x *= invScalar;
			this.y *= invScalar;
			this.z *= invScalar;
			this.w *= invScalar;

		} else {

			this.x = 0;
			this.y = 0;
			this.z = 0;
			this.w = 1;

		}

		return this;

	},

	setAxisAngleFromQuaternion: function ( q ) {

		// http://www.euclideanspace.com/maths/geometry/rotations/conversions/quaternionToAngle/index.htm

		// q is assumed to be normalized

		this.w = 2 * Math.acos( q.w );

		var s = Math.sqrt( 1 - q.w * q.w );

		if ( s < 0.0001 ) {

			 this.x = 1;
			 this.y = 0;
			 this.z = 0;

		} else {

			 this.x = q.x / s;
			 this.y = q.y / s;
			 this.z = q.z / s;

		}

		return this;

	},

	setAxisAngleFromRotationMatrix: function ( m ) {

		// http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToAngle/index.htm

		// assumes the upper 3x3 of m is a pure rotation matrix (i.e, unscaled)

		var angle, x, y, z,		// variables for result
			epsilon = 0.01,		// margin to allow for rounding errors
			epsilon2 = 0.1,		// margin to distinguish between 0 and 180 degrees

			te = m.elements,

			m11 = te[0], m12 = te[4], m13 = te[8],
			m21 = te[1], m22 = te[5], m23 = te[9],
			m31 = te[2], m32 = te[6], m33 = te[10];

		if ( ( Math.abs( m12 - m21 ) < epsilon )
		  && ( Math.abs( m13 - m31 ) < epsilon )
		  && ( Math.abs( m23 - m32 ) < epsilon ) ) {

			// singularity found
			// first check for identity matrix which must have +1 for all terms
			// in leading diagonal and zero in other terms

			if ( ( Math.abs( m12 + m21 ) < epsilon2 )
			  && ( Math.abs( m13 + m31 ) < epsilon2 )
			  && ( Math.abs( m23 + m32 ) < epsilon2 )
			  && ( Math.abs( m11 + m22 + m33 - 3 ) < epsilon2 ) ) {

				// this singularity is identity matrix so angle = 0

				this.set( 1, 0, 0, 0 );

				return this; // zero angle, arbitrary axis

			}

			// otherwise this singularity is angle = 180

			angle = Math.PI;

			var xx = ( m11 + 1 ) / 2;
			var yy = ( m22 + 1 ) / 2;
			var zz = ( m33 + 1 ) / 2;
			var xy = ( m12 + m21 ) / 4;
			var xz = ( m13 + m31 ) / 4;
			var yz = ( m23 + m32 ) / 4;

			if ( ( xx > yy ) && ( xx > zz ) ) { // m11 is the largest diagonal term

				if ( xx < epsilon ) {

					x = 0;
					y = 0.707106781;
					z = 0.707106781;

				} else {

					x = Math.sqrt( xx );
					y = xy / x;
					z = xz / x;

				}

			} else if ( yy > zz ) { // m22 is the largest diagonal term

				if ( yy < epsilon ) {

					x = 0.707106781;
					y = 0;
					z = 0.707106781;

				} else {

					y = Math.sqrt( yy );
					x = xy / y;
					z = yz / y;

				}

			} else { // m33 is the largest diagonal term so base result on this

				if ( zz < epsilon ) {

					x = 0.707106781;
					y = 0.707106781;
					z = 0;

				} else {

					z = Math.sqrt( zz );
					x = xz / z;
					y = yz / z;

				}

			}

			this.set( x, y, z, angle );

			return this; // return 180 deg rotation

		}

		// as we have reached here there are no singularities so we can handle normally

		var s = Math.sqrt( ( m32 - m23 ) * ( m32 - m23 )
						 + ( m13 - m31 ) * ( m13 - m31 )
						 + ( m21 - m12 ) * ( m21 - m12 ) ); // used to normalize

		if ( Math.abs( s ) < 0.001 ) s = 1;

		// prevent divide by zero, should not happen if matrix is orthogonal and should be
		// caught by singularity test above, but I've left it in just in case

		this.x = ( m32 - m23 ) / s;
		this.y = ( m13 - m31 ) / s;
		this.z = ( m21 - m12 ) / s;
		this.w = Math.acos( ( m11 + m22 + m33 - 1 ) / 2 );

		return this;

	},

	min: function ( v ) {

		if ( this.x > v.x ) {

			this.x = v.x;

		}

		if ( this.y > v.y ) {

			this.y = v.y;

		}

		if ( this.z > v.z ) {

			this.z = v.z;

		}

		if ( this.w > v.w ) {

			this.w = v.w;

		}

		return this;

	},

	max: function ( v ) {

		if ( this.x < v.x ) {

			this.x = v.x;

		}

		if ( this.y < v.y ) {

			this.y = v.y;

		}

		if ( this.z < v.z ) {

			this.z = v.z;

		}

		if ( this.w < v.w ) {

			this.w = v.w;

		}

		return this;

	},

	clamp: function ( min, max ) {

		// This function assumes min < max, if this assumption isn't true it will not operate correctly

		if ( this.x < min.x ) {

			this.x = min.x;

		} else if ( this.x > max.x ) {

			this.x = max.x;

		}

		if ( this.y < min.y ) {

			this.y = min.y;

		} else if ( this.y > max.y ) {

			this.y = max.y;

		}

		if ( this.z < min.z ) {

			this.z = min.z;

		} else if ( this.z > max.z ) {

			this.z = max.z;

		}

		if ( this.w < min.w ) {

			this.w = min.w;

		} else if ( this.w > max.w ) {

			this.w = max.w;

		}

		return this;

	},

	clampScalar: ( function () {

		var min, max;

		return function ( minVal, maxVal ) {

			if ( min === undefined ) {

				min = new Engine.Vector4();
				max = new Engine.Vector4();

			}

			min.set( minVal, minVal, minVal, minVal );
			max.set( maxVal, maxVal, maxVal, maxVal );

			return this.clamp( min, max );

		};

	} )(),

    floor: function () {

        this.x = Math.floor( this.x );
        this.y = Math.floor( this.y );
        this.z = Math.floor( this.z );
        this.w = Math.floor( this.w );

        return this;

    },

    ceil: function () {

        this.x = Math.ceil( this.x );
        this.y = Math.ceil( this.y );
        this.z = Math.ceil( this.z );
        this.w = Math.ceil( this.w );

        return this;

    },

    round: function () {

        this.x = Math.round( this.x );
        this.y = Math.round( this.y );
        this.z = Math.round( this.z );
        this.w = Math.round( this.w );

        return this;

    },

    roundToZero: function () {

        this.x = ( this.x < 0 ) ? Math.ceil( this.x ) : Math.floor( this.x );
        this.y = ( this.y < 0 ) ? Math.ceil( this.y ) : Math.floor( this.y );
        this.z = ( this.z < 0 ) ? Math.ceil( this.z ) : Math.floor( this.z );
        this.w = ( this.w < 0 ) ? Math.ceil( this.w ) : Math.floor( this.w );

        return this;

    },

	negate: function () {

		return this.multiplyScalar( -1 );

	},

	dot: function ( v ) {

		return this.x * v.x + this.y * v.y + this.z * v.z + this.w * v.w;

	},

	lengthSq: function () {

		return this.x * this.x + this.y * this.y + this.z * this.z + this.w * this.w;

	},

	length: function () {

		return Math.sqrt( this.x * this.x + this.y * this.y + this.z * this.z + this.w * this.w );

	},

	lengthManhattan: function () {

		return Math.abs( this.x ) + Math.abs( this.y ) + Math.abs( this.z ) + Math.abs( this.w );

	},

	normalize: function () {

		return this.divideScalar( this.length() );

	},

	setLength: function ( l ) {

		var oldLength = this.length();

		if ( oldLength !== 0 && l !== oldLength ) {

			this.multiplyScalar( l / oldLength );

		}

		return this;

	},

	lerp: function ( v, alpha ) {

		this.x += ( v.x - this.x ) * alpha;
		this.y += ( v.y - this.y ) * alpha;
		this.z += ( v.z - this.z ) * alpha;
		this.w += ( v.w - this.w ) * alpha;

		return this;

	},

	equals: function ( v ) {

		return ( ( v.x === this.x ) && ( v.y === this.y ) && ( v.z === this.z ) && ( v.w === this.w ) );

	},

	fromArray: function ( array ) {

		this.x = array[ 0 ];
		this.y = array[ 1 ];
		this.z = array[ 2 ];
		this.w = array[ 3 ];

		return this;

	},

	toArray: function () {

		return [ this.x, this.y, this.z, this.w ];

	},

	clone: function () {

		return new Engine.Vector4( this.x, this.y, this.z, this.w );

	}

};

Engine.Particle = function ( position, direction, random1, random2, random3) {

	this.x = position.x;
	this.y = position.y;
	this.z = position.z;
	this.dx = direction.x;
	this.dy = direction.y;
	this.dz = direction.z;
	this.t0 = Engine.currentTime;
	this.random1 = random1;
	this.random2 = random2;
	this.random3 = random3;
};
Engine.Box3 = function ( min, max ) {

	this.min = ( min !== undefined ) ? min : new Engine.Vector3( Infinity, Infinity, Infinity );
	this.max = ( max !== undefined ) ? max : new Engine.Vector3( -Infinity, -Infinity, -Infinity );

};

Engine.Box3.prototype = {

	constructor: Engine.Box3,

	set: function ( min, max ) {

		this.min.copy( min );
		this.max.copy( max );

		return this;

	},

	addPoint: function ( point ) {

		if ( point.x < this.min.x ) {

			this.min.x = point.x;

		} else if ( point.x > this.max.x ) {

			this.max.x = point.x;

		}

		if ( point.y < this.min.y ) {

			this.min.y = point.y;

		} else if ( point.y > this.max.y ) {

			this.max.y = point.y;

		}

		if ( point.z < this.min.z ) {

			this.min.z = point.z;

		} else if ( point.z > this.max.z ) {

			this.max.z = point.z;

		}

	},

	setFromPoints: function ( points ) {

		if ( points.length > 0 ) {

			var point = points[ 0 ];

			this.min.copy( point );
			this.max.copy( point );

			for ( var i = 1, il = points.length; i < il; i ++ ) {

				this.addPoint( points[ i ] )

			}

		} else {

			this.makeEmpty();

		}

		return this;

	},

	setFromCenterAndSize: function() {

		var v1 = new Engine.Vector3();

		return function ( center, size ) {

			var halfSize = v1.copy( size ).multiplyScalar( 0.5 );

			this.min.copy( center ).sub( halfSize );
			this.max.copy( center ).add( halfSize );

			return this;

		};

	}(),

	setFromObject: function() {

		// Computes the world-axis-aligned bounding box of an object (including its children),
		// accounting for both the object's, and childrens', world transforms

		var v1 = new Engine.Vector3();

		return function( object ) {

			var scope = this;

			object.updateMatrixWorld( true );

			this.makeEmpty();

			object.traverse( function ( node ) {

				if ( node.geometry !== undefined && node.geometry.vertices !== undefined ) {

					var vertices = node.geometry.vertices;

					for ( var i = 0, il = vertices.length; i < il; i++ ) {

						v1.copy( vertices[ i ] );

						v1.applyMatrix4( node.matrixWorld );

						scope.expandByPoint( v1 );

					}

				}

			} );

			return this;

		};

	}(),

	copy: function ( box ) {

		this.min.copy( box.min );
		this.max.copy( box.max );

		return this;

	},

	makeEmpty: function () {

		this.min.x = this.min.y = this.min.z = Infinity;
		this.max.x = this.max.y = this.max.z = -Infinity;

		return this;

	},

	empty: function () {

		// this is a more robust check for empty than ( volume <= 0 ) because volume can get positive with two negative axes

		return ( this.max.x < this.min.x ) || ( this.max.y < this.min.y ) || ( this.max.z < this.min.z );

	},

	center: function ( optionalTarget ) {

		var result = optionalTarget || new Engine.Vector3();
		return result.addVectors( this.min, this.max ).multiplyScalar( 0.5 );

	},

	size: function ( optionalTarget ) {

		var result = optionalTarget || new Engine.Vector3();
		return result.subVectors( this.max, this.min );

	},

	expandByPoint: function ( point ) {

		this.min.min( point );
		this.max.max( point );

		return this;

	},

	expandByVector: function ( vector ) {

		this.min.sub( vector );
		this.max.add( vector );

		return this;

	},

	expandByScalar: function ( scalar ) {

		this.min.addScalar( -scalar );
		this.max.addScalar( scalar );

		return this;

	},

	containsPoint: function ( point ) {

		if ( point.x < this.min.x || point.x > this.max.x ||
		     point.y < this.min.y || point.y > this.max.y ||
		     point.z < this.min.z || point.z > this.max.z ) {

			return false;

		}

		return true;

	},

	containsBox: function ( box ) {

		if ( ( this.min.x <= box.min.x ) && ( box.max.x <= this.max.x ) &&
			 ( this.min.y <= box.min.y ) && ( box.max.y <= this.max.y ) &&
			 ( this.min.z <= box.min.z ) && ( box.max.z <= this.max.z ) ) {

			return true;

		}

		return false;

	},

	getParameter: function ( point, optionalTarget ) {

		// This can potentially have a divide by zero if the box
		// has a size dimension of 0.

		var result = optionalTarget || new Engine.Vector3();

		return result.set(
			( point.x - this.min.x ) / ( this.max.x - this.min.x ),
			( point.y - this.min.y ) / ( this.max.y - this.min.y ),
			( point.z - this.min.z ) / ( this.max.z - this.min.z )
		);

	},

	isIntersectionBox: function ( box ) {

		// using 6 splitting planes to rule out intersections.

		if ( box.max.x < this.min.x || box.min.x > this.max.x ||
		     box.max.y < this.min.y || box.min.y > this.max.y ||
		     box.max.z < this.min.z || box.min.z > this.max.z ) {

			return false;

		}

		return true;

	},

	clampPoint: function ( point, optionalTarget ) {

		var result = optionalTarget || new Engine.Vector3();
		return result.copy( point ).clamp( this.min, this.max );

	},

	distanceToPoint: function() {

		var v1 = new Engine.Vector3();

		return function ( point ) {

			var clampedPoint = v1.copy( point ).clamp( this.min, this.max );
			return clampedPoint.sub( point ).length();

		};

	}(),

	getBoundingSphere: function() {

		var v1 = new Engine.Vector3();

		return function ( optionalTarget ) {

			var result = optionalTarget || new Engine.Sphere();

			result.center = this.center();
			result.radius = this.size( v1 ).length() * 0.5;

			return result;

		};

	}(),

	intersect: function ( box ) {

		this.min.max( box.min );
		this.max.min( box.max );

		return this;

	},

	union: function ( box ) {

		this.min.min( box.min );
		this.max.max( box.max );

		return this;

	},

	applyMatrix4: function() {

		var points = [
			new Engine.Vector3(),
			new Engine.Vector3(),
			new Engine.Vector3(),
			new Engine.Vector3(),
			new Engine.Vector3(),
			new Engine.Vector3(),
			new Engine.Vector3(),
			new Engine.Vector3()
		];

		return function ( matrix ) {

			// NOTE: I am using a binary pattern to specify all 2^3 combinations below
			points[0].set( this.min.x, this.min.y, this.min.z ).applyMatrix4( matrix ); // 000
			points[1].set( this.min.x, this.min.y, this.max.z ).applyMatrix4( matrix ); // 001
			points[2].set( this.min.x, this.max.y, this.min.z ).applyMatrix4( matrix ); // 010
			points[3].set( this.min.x, this.max.y, this.max.z ).applyMatrix4( matrix ); // 011
			points[4].set( this.max.x, this.min.y, this.min.z ).applyMatrix4( matrix ); // 100
			points[5].set( this.max.x, this.min.y, this.max.z ).applyMatrix4( matrix ); // 101
			points[6].set( this.max.x, this.max.y, this.min.z ).applyMatrix4( matrix ); // 110
			points[7].set( this.max.x, this.max.y, this.max.z ).applyMatrix4( matrix );  // 111

			this.makeEmpty();
			this.setFromPoints( points );

			return this;

		};

	}(),

	translate: function ( offset ) {

		this.min.add( offset );
		this.max.add( offset );

		return this;

	},

	equals: function ( box ) {

		return box.min.equals( this.min ) && box.max.equals( this.max );

	},

	clone: function () {

		return new Engine.Box3().copy( this );

	}

};

Engine.Color = function ( value ) {

	if ( value !== undefined ) this.set( value );

	return this;

};

Engine.Color.prototype = {

	constructor: Engine.Color,

	r: 1, g: 1, b: 1,

	set: function ( value0, value1, value2 ) {

		if ( value0 instanceof Engine.Color ) {

			this.copy( value0 );

		} else if ( typeof value0 === 'number' && !value1 && !value1) {

			this.setHex( value0 );

		} else if (typeof value0 === 'number' && typeof value1 === 'number' && typeof value2 === 'number') { 
			
			this.setRGB(value0,value1,value2);

		}

		return this;

	},

	setHex: function ( hex ) {

		hex = Math.floor( hex );

		this.r = ( hex >> 16 & 255 ) / 255;
		this.g = ( hex >> 8 & 255 ) / 255;
		this.b = ( hex & 255 ) / 255;

		return this;

	},

	setRGB: function ( r, g, b ) {

		this.r = r;
		this.g = g;
		this.b = b;

		return this;

	},


	copy: function ( color ) {

		this.r = color.r;
		this.g = color.g;
		this.b = color.b;

		return this;

	},

	getHex: function () {

		return ( this.r * 255 ) << 16 ^ ( this.g * 255 ) << 8 ^ ( this.b * 255 ) << 0;

	},

	add: function ( color ) {

		this.r += color.r;
		this.g += color.g;
		this.b += color.b;

		return this;

	},

	addColors: function ( color1, color2 ) {

		this.r = color1.r + color2.r;
		this.g = color1.g + color2.g;
		this.b = color1.b + color2.b;

		return this;

	},

	addScalar: function ( s ) {

		this.r += s;
		this.g += s;
		this.b += s;

		return this;

	},

	multiply: function ( color ) {

		this.r *= color.r;
		this.g *= color.g;
		this.b *= color.b;

		return this;

	},

	multiplyScalar: function ( s ) {

		this.r *= s;
		this.g *= s;
		this.b *= s;

		return this;

	},

	lerp: function ( color, alpha ) {

		this.r += ( color.r - this.r ) * alpha;
		this.g += ( color.g - this.g ) * alpha;
		this.b += ( color.b - this.b ) * alpha;

		return this;

	},

	equals: function ( c ) {

		return ( c.r === this.r ) && ( c.g === this.g ) && ( c.b === this.b );

	},

	fromArray: function ( array ) {

		this.r = array[ 0 ];
		this.g = array[ 1 ];
		this.b = array[ 2 ];

		return this;

	},

	toArray: function () {

		return [ this.r, this.g, this.b ];

	},

	clone: function () {

		return new Engine.Color().setRGB( this.r, this.g, this.b );

	}

};
Engine.Face3 = function ( a, b, c) {

	this.a = a;
	this.b = b;
	this.c = c;

};

Engine.Face3.prototype = {

	constructor: Engine.Face3,

	clone: function () {

		var face = new Engine.Face3( this.a, this.b, this.c );
		
		return face;
	},

	addOffset: function (offset) {
		this.a += offset;
		this.b += offset;
		this.c += offset;

		return this;
	}

};
Engine.Matrix4 = function ( n11, n12, n13, n14, n21, n22, n23, n24, n31, n32, n33, n34, n41, n42, n43, n44 ) {

	this.elements = new Float32Array( 16 );

	// TODO: if n11 is undefined, then just set to identity, otherwise copy all other values into matrix
	//   we should not support semi specification of Matrix4, it is just weird.

	var te = this.elements;

	te[0] = ( n11 !== undefined ) ? n11 : 1; te[4] = n12 || 0; te[8] = n13 || 0; te[12] = n14 || 0;
	te[1] = n21 || 0; te[5] = ( n22 !== undefined ) ? n22 : 1; te[9] = n23 || 0; te[13] = n24 || 0;
	te[2] = n31 || 0; te[6] = n32 || 0; te[10] = ( n33 !== undefined ) ? n33 : 1; te[14] = n34 || 0;
	te[3] = n41 || 0; te[7] = n42 || 0; te[11] = n43 || 0; te[15] = ( n44 !== undefined ) ? n44 : 1;

};

Engine.Matrix4.prototype = {

	constructor: Engine.Matrix4,

	set: function ( n11, n12, n13, n14, n21, n22, n23, n24, n31, n32, n33, n34, n41, n42, n43, n44 ) {

		var te = this.elements;

		te[0] = n11; te[4] = n12; te[8] = n13; te[12] = n14;
		te[1] = n21; te[5] = n22; te[9] = n23; te[13] = n24;
		te[2] = n31; te[6] = n32; te[10] = n33; te[14] = n34;
		te[3] = n41; te[7] = n42; te[11] = n43; te[15] = n44;

		return this;

	},

	identity: function () {

		this.set(

			1, 0, 0, 0,
			0, 1, 0, 0,
			0, 0, 1, 0,
			0, 0, 0, 1

		);

		return this;

	},

	copy: function ( m ) {

		this.elements.set( m.elements );

		return this;

	},

	copyPosition: function ( m ) {

		var te = this.elements;
		var me = m.elements;

		te[12] = me[12];
		te[13] = me[13];
		te[14] = me[14];

		return this;

	},

	extractRotation: function () {

		var v1 = new Engine.Vector3();

		return function ( m ) {

			var te = this.elements;
			var me = m.elements;

			var scaleX = 1 / v1.set( me[0], me[1], me[2] ).length();
			var scaleY = 1 / v1.set( me[4], me[5], me[6] ).length();
			var scaleZ = 1 / v1.set( me[8], me[9], me[10] ).length();

			te[0] = me[0] * scaleX;
			te[1] = me[1] * scaleX;
			te[2] = me[2] * scaleX;

			te[4] = me[4] * scaleY;
			te[5] = me[5] * scaleY;
			te[6] = me[6] * scaleY;

			te[8] = me[8] * scaleZ;
			te[9] = me[9] * scaleZ;
			te[10] = me[10] * scaleZ;

			return this;

		};

	}(),

	makeRotationFromEuler: function ( euler ) {

		if ( euler instanceof Engine.Euler === false ) {

			console.error( 'ERROR: Matrix\'s .makeRotationFromEuler() now expects a Euler rotation rather than a Vector3 and order.  Please update your code.' );

		}

		var te = this.elements;

		var x = euler.x, y = euler.y, z = euler.z;
		var a = Math.cos( x ), b = Math.sin( x );
		var c = Math.cos( y ), d = Math.sin( y );
		var e = Math.cos( z ), f = Math.sin( z );

		if ( euler.order === 'XYZ' ) {

			var ae = a * e, af = a * f, be = b * e, bf = b * f;

			te[0] = c * e;
			te[4] = - c * f;
			te[8] = d;

			te[1] = af + be * d;
			te[5] = ae - bf * d;
			te[9] = - b * c;

			te[2] = bf - ae * d;
			te[6] = be + af * d;
			te[10] = a * c;

		} else if ( euler.order === 'YXZ' ) {

			var ce = c * e, cf = c * f, de = d * e, df = d * f;

			te[0] = ce + df * b;
			te[4] = de * b - cf;
			te[8] = a * d;

			te[1] = a * f;
			te[5] = a * e;
			te[9] = - b;

			te[2] = cf * b - de;
			te[6] = df + ce * b;
			te[10] = a * c;

		} else if ( euler.order === 'ZXY' ) {

			var ce = c * e, cf = c * f, de = d * e, df = d * f;

			te[0] = ce - df * b;
			te[4] = - a * f;
			te[8] = de + cf * b;

			te[1] = cf + de * b;
			te[5] = a * e;
			te[9] = df - ce * b;

			te[2] = - a * d;
			te[6] = b;
			te[10] = a * c;

		} else if ( euler.order === 'ZYX' ) {

			var ae = a * e, af = a * f, be = b * e, bf = b * f;

			te[0] = c * e;
			te[4] = be * d - af;
			te[8] = ae * d + bf;

			te[1] = c * f;
			te[5] = bf * d + ae;
			te[9] = af * d - be;

			te[2] = - d;
			te[6] = b * c;
			te[10] = a * c;

		} else if ( euler.order === 'YZX' ) {

			var ac = a * c, ad = a * d, bc = b * c, bd = b * d;

			te[0] = c * e;
			te[4] = bd - ac * f;
			te[8] = bc * f + ad;

			te[1] = f;
			te[5] = a * e;
			te[9] = - b * e;

			te[2] = - d * e;
			te[6] = ad * f + bc;
			te[10] = ac - bd * f;

		} else if ( euler.order === 'XZY' ) {

			var ac = a * c, ad = a * d, bc = b * c, bd = b * d;

			te[0] = c * e;
			te[4] = - f;
			te[8] = d * e;

			te[1] = ac * f + bd;
			te[5] = a * e;
			te[9] = ad * f - bc;

			te[2] = bc * f - ad;
			te[6] = b * e;
			te[10] = bd * f + ac;

		}

		// last column
		te[3] = 0;
		te[7] = 0;
		te[11] = 0;

		// bottom row
		te[12] = 0;
		te[13] = 0;
		te[14] = 0;
		te[15] = 1;

		return this;

	},

	makeRotationFromQuaternion: function ( q ) {

		var te = this.elements;

		var x = q.x, y = q.y, z = q.z, w = q.w;
		var x2 = x + x, y2 = y + y, z2 = z + z;
		var xx = x * x2, xy = x * y2, xz = x * z2;
		var yy = y * y2, yz = y * z2, zz = z * z2;
		var wx = w * x2, wy = w * y2, wz = w * z2;

		te[0] = 1 - ( yy + zz );
		te[4] = xy - wz;
		te[8] = xz + wy;

		te[1] = xy + wz;
		te[5] = 1 - ( xx + zz );
		te[9] = yz - wx;

		te[2] = xz - wy;
		te[6] = yz + wx;
		te[10] = 1 - ( xx + yy );

		// last column
		te[3] = 0;
		te[7] = 0;
		te[11] = 0;

		// bottom row
		te[12] = 0;
		te[13] = 0;
		te[14] = 0;
		te[15] = 1;

		return this;

	},

	lookAt: function() {
		var x = new Engine.Vector3();
		var y = new Engine.Vector3();
		var z = new Engine.Vector3();

		return function ( eye, target, up ) {

			var te = this.elements;

			z.subVectors( eye, target ).normalize();

			if ( z.length() === 0 ) {

				z.z = 1;

			}

			x.crossVectors( up, z ).normalize();

			if ( x.length() === 0 ) {

				z.x += 0.0001;
				x.crossVectors( up, z ).normalize();

			}

			y.crossVectors( z, x );


			te[0] = x.x; te[4] = y.x; te[8] = z.x;
			te[1] = x.y; te[5] = y.y; te[9] = z.y;
			te[2] = x.z; te[6] = y.z; te[10] = z.z;

			return this;

		};

	}(),

	multiply: function ( m ) {

		return this.multiplyMatrices( this, m );

	},

	multiplyMatrices: function ( a, b ) {

		var ae = a.elements;
		var be = b.elements;
		var te = this.elements;

		var a11 = ae[0], a12 = ae[4], a13 = ae[8], a14 = ae[12];
		var a21 = ae[1], a22 = ae[5], a23 = ae[9], a24 = ae[13];
		var a31 = ae[2], a32 = ae[6], a33 = ae[10], a34 = ae[14];
		var a41 = ae[3], a42 = ae[7], a43 = ae[11], a44 = ae[15];

		var b11 = be[0], b12 = be[4], b13 = be[8], b14 = be[12];
		var b21 = be[1], b22 = be[5], b23 = be[9], b24 = be[13];
		var b31 = be[2], b32 = be[6], b33 = be[10], b34 = be[14];
		var b41 = be[3], b42 = be[7], b43 = be[11], b44 = be[15];

		te[0] = a11 * b11 + a12 * b21 + a13 * b31 + a14 * b41;
		te[4] = a11 * b12 + a12 * b22 + a13 * b32 + a14 * b42;
		te[8] = a11 * b13 + a12 * b23 + a13 * b33 + a14 * b43;
		te[12] = a11 * b14 + a12 * b24 + a13 * b34 + a14 * b44;

		te[1] = a21 * b11 + a22 * b21 + a23 * b31 + a24 * b41;
		te[5] = a21 * b12 + a22 * b22 + a23 * b32 + a24 * b42;
		te[9] = a21 * b13 + a22 * b23 + a23 * b33 + a24 * b43;
		te[13] = a21 * b14 + a22 * b24 + a23 * b34 + a24 * b44;

		te[2] = a31 * b11 + a32 * b21 + a33 * b31 + a34 * b41;
		te[6] = a31 * b12 + a32 * b22 + a33 * b32 + a34 * b42;
		te[10] = a31 * b13 + a32 * b23 + a33 * b33 + a34 * b43;
		te[14] = a31 * b14 + a32 * b24 + a33 * b34 + a34 * b44;

		te[3] = a41 * b11 + a42 * b21 + a43 * b31 + a44 * b41;
		te[7] = a41 * b12 + a42 * b22 + a43 * b32 + a44 * b42;
		te[11] = a41 * b13 + a42 * b23 + a43 * b33 + a44 * b43;
		te[15] = a41 * b14 + a42 * b24 + a43 * b34 + a44 * b44;

		return this;

	},

	multiplyToArray: function ( a, b, r ) {

		var te = this.elements;

		this.multiplyMatrices( a, b );

		r[ 0 ] = te[0]; r[ 1 ] = te[1]; r[ 2 ] = te[2]; r[ 3 ] = te[3];
		r[ 4 ] = te[4]; r[ 5 ] = te[5]; r[ 6 ] = te[6]; r[ 7 ] = te[7];
		r[ 8 ]  = te[8]; r[ 9 ]  = te[9]; r[ 10 ] = te[10]; r[ 11 ] = te[11];
		r[ 12 ] = te[12]; r[ 13 ] = te[13]; r[ 14 ] = te[14]; r[ 15 ] = te[15];

		return this;

	},

	multiplyScalar: function ( s ) {

		var te = this.elements;

		te[0] *= s; te[4] *= s; te[8] *= s; te[12] *= s;
		te[1] *= s; te[5] *= s; te[9] *= s; te[13] *= s;
		te[2] *= s; te[6] *= s; te[10] *= s; te[14] *= s;
		te[3] *= s; te[7] *= s; te[11] *= s; te[15] *= s;

		return this;

	},

	multiplyVector3Array: function() {

		var v1 = new Engine.Vector3();

		return function ( a ) {

			for ( var i = 0, il = a.length; i < il; i += 3 ) {

				v1.x = a[ i ];
				v1.y = a[ i + 1 ];
				v1.z = a[ i + 2 ];

				v1.applyProjection( this );

				a[ i ]     = v1.x;
				a[ i + 1 ] = v1.y;
				a[ i + 2 ] = v1.z;

			}

			return a;

		};

	}(),

	determinant: function () {

		var te = this.elements;

		var n11 = te[0], n12 = te[4], n13 = te[8], n14 = te[12];
		var n21 = te[1], n22 = te[5], n23 = te[9], n24 = te[13];
		var n31 = te[2], n32 = te[6], n33 = te[10], n34 = te[14];
		var n41 = te[3], n42 = te[7], n43 = te[11], n44 = te[15];

		return (
			n41 * (
				+n14 * n23 * n32
				-n13 * n24 * n32
				-n14 * n22 * n33
				+n12 * n24 * n33
				+n13 * n22 * n34
				-n12 * n23 * n34
			) +
			n42 * (
				+n11 * n23 * n34
				-n11 * n24 * n33
				+n14 * n21 * n33
				-n13 * n21 * n34
				+n13 * n24 * n31
				-n14 * n23 * n31
			) +
			n43 * (
				+n11 * n24 * n32
				-n11 * n22 * n34
				-n14 * n21 * n32
				+n12 * n21 * n34
				+n14 * n22 * n31
				-n12 * n24 * n31
			) +
			n44 * (
				-n13 * n22 * n31
				-n11 * n23 * n32
				+n11 * n22 * n33
				+n13 * n21 * n32
				-n12 * n21 * n33
				+n12 * n23 * n31
			)

		);

	},

	transpose: function () {

		var te = this.elements;
		var tmp;

		tmp = te[1]; te[1] = te[4]; te[4] = tmp;
		tmp = te[2]; te[2] = te[8]; te[8] = tmp;
		tmp = te[6]; te[6] = te[9]; te[9] = tmp;

		tmp = te[3]; te[3] = te[12]; te[12] = tmp;
		tmp = te[7]; te[7] = te[13]; te[13] = tmp;
		tmp = te[11]; te[11] = te[14]; te[14] = tmp;

		return this;

	},

	flattenToArray: function ( flat ) {

		var te = this.elements;
		flat[ 0 ] = te[0]; flat[ 1 ] = te[1]; flat[ 2 ] = te[2]; flat[ 3 ] = te[3];
		flat[ 4 ] = te[4]; flat[ 5 ] = te[5]; flat[ 6 ] = te[6]; flat[ 7 ] = te[7];
		flat[ 8 ] = te[8]; flat[ 9 ] = te[9]; flat[ 10 ] = te[10]; flat[ 11 ] = te[11];
		flat[ 12 ] = te[12]; flat[ 13 ] = te[13]; flat[ 14 ] = te[14]; flat[ 15 ] = te[15];

		return flat;

	},

	flattenToArrayOffset: function( flat, offset ) {

		var te = this.elements;
		flat[ offset ] = te[0];
		flat[ offset + 1 ] = te[1];
		flat[ offset + 2 ] = te[2];
		flat[ offset + 3 ] = te[3];

		flat[ offset + 4 ] = te[4];
		flat[ offset + 5 ] = te[5];
		flat[ offset + 6 ] = te[6];
		flat[ offset + 7 ] = te[7];

		flat[ offset + 8 ]  = te[8];
		flat[ offset + 9 ]  = te[9];
		flat[ offset + 10 ] = te[10];
		flat[ offset + 11 ] = te[11];

		flat[ offset + 12 ] = te[12];
		flat[ offset + 13 ] = te[13];
		flat[ offset + 14 ] = te[14];
		flat[ offset + 15 ] = te[15];

		return flat;

	},

	setPosition: function ( v ) {

		var te = this.elements;

		te[12] = v.x;
		te[13] = v.y;
		te[14] = v.z;

		return this;

	},

	getPosition: function (  ) {

		var te = this.elements;
		
		return new Engine.Vector3(te[12],te[13],te[14]);

	},

	getInverse: function ( m, throwOnInvertible ) {

		// based on http://www.euclideanspace.com/maths/algebra/matrix/functions/inverse/fourD/index.htm
		var te = this.elements;
		var me = m.elements;

		var n11 = me[0], n12 = me[4], n13 = me[8], n14 = me[12];
		var n21 = me[1], n22 = me[5], n23 = me[9], n24 = me[13];
		var n31 = me[2], n32 = me[6], n33 = me[10], n34 = me[14];
		var n41 = me[3], n42 = me[7], n43 = me[11], n44 = me[15];

		te[0] = n23*n34*n42 - n24*n33*n42 + n24*n32*n43 - n22*n34*n43 - n23*n32*n44 + n22*n33*n44;
		te[4] = n14*n33*n42 - n13*n34*n42 - n14*n32*n43 + n12*n34*n43 + n13*n32*n44 - n12*n33*n44;
		te[8] = n13*n24*n42 - n14*n23*n42 + n14*n22*n43 - n12*n24*n43 - n13*n22*n44 + n12*n23*n44;
		te[12] = n14*n23*n32 - n13*n24*n32 - n14*n22*n33 + n12*n24*n33 + n13*n22*n34 - n12*n23*n34;
		te[1] = n24*n33*n41 - n23*n34*n41 - n24*n31*n43 + n21*n34*n43 + n23*n31*n44 - n21*n33*n44;
		te[5] = n13*n34*n41 - n14*n33*n41 + n14*n31*n43 - n11*n34*n43 - n13*n31*n44 + n11*n33*n44;
		te[9] = n14*n23*n41 - n13*n24*n41 - n14*n21*n43 + n11*n24*n43 + n13*n21*n44 - n11*n23*n44;
		te[13] = n13*n24*n31 - n14*n23*n31 + n14*n21*n33 - n11*n24*n33 - n13*n21*n34 + n11*n23*n34;
		te[2] = n22*n34*n41 - n24*n32*n41 + n24*n31*n42 - n21*n34*n42 - n22*n31*n44 + n21*n32*n44;
		te[6] = n14*n32*n41 - n12*n34*n41 - n14*n31*n42 + n11*n34*n42 + n12*n31*n44 - n11*n32*n44;
		te[10] = n12*n24*n41 - n14*n22*n41 + n14*n21*n42 - n11*n24*n42 - n12*n21*n44 + n11*n22*n44;
		te[14] = n14*n22*n31 - n12*n24*n31 - n14*n21*n32 + n11*n24*n32 + n12*n21*n34 - n11*n22*n34;
		te[3] = n23*n32*n41 - n22*n33*n41 - n23*n31*n42 + n21*n33*n42 + n22*n31*n43 - n21*n32*n43;
		te[7] = n12*n33*n41 - n13*n32*n41 + n13*n31*n42 - n11*n33*n42 - n12*n31*n43 + n11*n32*n43;
		te[11] = n13*n22*n41 - n12*n23*n41 - n13*n21*n42 + n11*n23*n42 + n12*n21*n43 - n11*n22*n43;
		te[15] = n12*n23*n31 - n13*n22*n31 + n13*n21*n32 - n11*n23*n32 - n12*n21*n33 + n11*n22*n33;

		var det = n11 * te[ 0 ] + n21 * te[ 4 ] + n31 * te[ 8 ] + n41 * te[ 12 ];

		if ( det == 0 ) {

			var msg = "Matrix4.getInverse(): can't invert matrix, determinant is 0";

			if ( throwOnInvertible || false ) {

				throw new Error( msg ); 

			} else {

				console.warn( msg );

			}

			this.identity();

			return this;
		}

		this.multiplyScalar( 1 / det );

		return this;

	},

	scale: function ( v ) {

		var te = this.elements;
		var x = v.x, y = v.y, z = v.z;

		te[0] *= x; te[4] *= y; te[8] *= z;
		te[1] *= x; te[5] *= y; te[9] *= z;
		te[2] *= x; te[6] *= y; te[10] *= z;
		te[3] *= x; te[7] *= y; te[11] *= z;

		return this;

	},

	getMaxScaleOnAxis: function () {

		var te = this.elements;

		var scaleXSq = te[0] * te[0] + te[1] * te[1] + te[2] * te[2];
		var scaleYSq = te[4] * te[4] + te[5] * te[5] + te[6] * te[6];
		var scaleZSq = te[8] * te[8] + te[9] * te[9] + te[10] * te[10];

		return Math.sqrt( Math.max( scaleXSq, Math.max( scaleYSq, scaleZSq ) ) );

	},

	makeTranslation: function ( x, y, z ) {

		this.set(

			1, 0, 0, x,
			0, 1, 0, y,
			0, 0, 1, z,
			0, 0, 0, 1

		);

		return this;

	},

	makeRotationX: function ( theta ) {

		var c = Math.cos( theta ), s = Math.sin( theta );

		this.set(

			1, 0,  0, 0,
			0, c, -s, 0,
			0, s,  c, 0,
			0, 0,  0, 1

		);

		return this;

	},

	makeRotationY: function ( theta ) {

		var c = Math.cos( theta ), s = Math.sin( theta );

		this.set(

			 c, 0, s, 0,
			 0, 1, 0, 0,
			-s, 0, c, 0,
			 0, 0, 0, 1

		);

		return this;

	},

	makeRotationZ: function ( theta ) {

		var c = Math.cos( theta ), s = Math.sin( theta );

		this.set(

			c, -s, 0, 0,
			s,  c, 0, 0,
			0,  0, 1, 0,
			0,  0, 0, 1

		);

		return this;

	},

	makeRotationAxis: function ( axis, angle ) {

		// Based on http://www.gamedev.net/reference/articles/article1199.asp

		var c = Math.cos( angle );
		var s = Math.sin( angle );
		var t = 1 - c;
		var x = axis.x, y = axis.y, z = axis.z;
		var tx = t * x, ty = t * y;

		this.set(

			tx * x + c, tx * y - s * z, tx * z + s * y, 0,
			tx * y + s * z, ty * y + c, ty * z - s * x, 0,
			tx * z - s * y, ty * z + s * x, t * z * z + c, 0,
			0, 0, 0, 1

		);

		 return this;

	},


	getRotationY: function ( theta ) {

		var c = this.elements[0];
		var s = this.elements[8];

		if(c==1){
			theta = 0;
		}else if(c==-1){
			theta = Math.PI;
		}else if(s==1){
			theta = Math.PI/2;
		}else if(s==-1){
			theta = 3*Math.PI/2;
		}else{
			debugger;
		}

		return theta;
	},

	makeScale: function ( x, y, z ) {

		this.set(

			x, 0, 0, 0,
			0, y, 0, 0,
			0, 0, z, 0,
			0, 0, 0, 1

		);

		return this;

	},

	compose: function ( position, quaternion, scale ) {

		this.makeRotationFromQuaternion( quaternion );
		this.scale( scale );
		this.setPosition( position );

		return this;

	},

	decompose: function () {

		var vector = new Engine.Vector3();
		var matrix = new Engine.Matrix4();

		return function ( position, quaternion, scale ) {

			var te = this.elements;

			var sx = vector.set( te[0], te[1], te[2] ).length();
			var sy = vector.set( te[4], te[5], te[6] ).length();
			var sz = vector.set( te[8], te[9], te[10] ).length();

			// if determine is negative, we need to invert one scale
			var det = this.determinant();
			if( det < 0 ) {
				sx = -sx;
			}

			position.x = te[12];
			position.y = te[13];
			position.z = te[14];

			// scale the rotation part

			matrix.elements.set( this.elements ); // at this point matrix is incomplete so we can't use .copy()

			var invSX = 1 / sx;
			var invSY = 1 / sy;
			var invSZ = 1 / sz;

			matrix.elements[0] *= invSX;
			matrix.elements[1] *= invSX;
			matrix.elements[2] *= invSX;

			matrix.elements[4] *= invSY;
			matrix.elements[5] *= invSY;
			matrix.elements[6] *= invSY;

			matrix.elements[8] *= invSZ;
			matrix.elements[9] *= invSZ;
			matrix.elements[10] *= invSZ;

			quaternion.setFromRotationMatrix( matrix );

			scale.x = sx;
			scale.y = sy;
			scale.z = sz;

			return this;

		};

	}(),

	makeFrustum: function ( left, right, bottom, top, near, far ) {

		var te = this.elements;
		var x = 2 * near / ( right - left );
		var y = 2 * near / ( top - bottom );

		var a = ( right + left ) / ( right - left );
		var b = ( top + bottom ) / ( top - bottom );
		var c = - ( far + near ) / ( far - near );
		var d = - 2 * far * near / ( far - near );

		te[0] = x;	te[4] = 0;	te[8] = a;	te[12] = 0;
		te[1] = 0;	te[5] = y;	te[9] = b;	te[13] = 0;
		te[2] = 0;	te[6] = 0;	te[10] = c;	te[14] = d;
		te[3] = 0;	te[7] = 0;	te[11] = - 1;	te[15] = 0;

		return this;

	},

	makePerspective: function ( fov, aspect, near, far ) {

		var ymax = near * Math.tan( Math.degToRad( fov * 0.5 ) );
		var ymin = - ymax;
		var xmin = ymin * aspect;
		var xmax = ymax * aspect;

		return this.makeFrustum( xmin, xmax, ymin, ymax, near, far );

	},

	makeOrthographic: function ( left, right, top, bottom, near, far ) {

		var te = this.elements;
		var w = right - left;
		var h = top - bottom;
		var p = far - near;

		var x = ( right + left ) / w;
		var y = ( top + bottom ) / h;
		var z = ( far + near ) / p;

		te[0] = 2 / w;	te[4] = 0;	te[8] = 0;	te[12] = -x;
		te[1] = 0;	te[5] = 2 / h;	te[9] = 0;	te[13] = -y;
		te[2] = 0;	te[6] = 0;	te[10] = -2/p;	te[14] = -z;
		te[3] = 0;	te[7] = 0;	te[11] = 0;	te[15] = 1;

		return this;

	},

	fromArray: function ( array ) {

		this.elements.set( array );

		return this;

	},

	toArray: function () {

		var te = this.elements;

		return [
			te[ 0 ], te[ 1 ], te[ 2 ], te[ 3 ],
			te[ 4 ], te[ 5 ], te[ 6 ], te[ 7 ],
			te[ 8 ], te[ 9 ], te[ 10 ], te[ 11 ],
			te[ 12 ], te[ 13 ], te[ 14 ], te[ 15 ]
		];

	},

	clone: function () {

		var te = this.elements;

		return new Engine.Matrix4(

			te[0], te[4], te[8], te[12],
			te[1], te[5], te[9], te[13],
			te[2], te[6], te[10], te[14],
			te[3], te[7], te[11], te[15]

		);

	}

};
Engine.Quaternion = function ( x, y, z, w ) {

	this._x = x || 0;
	this._y = y || 0;
	this._z = z || 0;
	this._w = ( w !== undefined ) ? w : 1;

};

Engine.Quaternion.prototype = {

	constructor: Engine.Quaternion,

	_x: 0,_y: 0, _z: 0, _w: 0,

	_euler: undefined,

	_updateEuler: function ( callback ) {

		if ( this._euler !== undefined ) {

			this._euler.setFromQuaternion( this, undefined, false );

		}

	},

	get x () {

		return this._x;

	},

	set x ( value ) {

		this._x = value;
		this._updateEuler();

	},

	get y () {

		return this._y;

	},

	set y ( value ) {

		this._y = value;
		this._updateEuler();

	},

	get z () {

		return this._z;

	},

	set z ( value ) {

		this._z = value;
		this._updateEuler();

	},

	get w () {

		return this._w;

	},

	set w ( value ) {

		this._w = value;
		this._updateEuler();

	},

	set: function ( x, y, z, w ) {

		this._x = x;
		this._y = y;
		this._z = z;
		this._w = w;

		this._updateEuler();

		return this;

	},

	copy: function ( quaternion ) {

		this._x = quaternion._x;
		this._y = quaternion._y;
		this._z = quaternion._z;
		this._w = quaternion._w;

		this._updateEuler();

		return this;

	},

	setFromEuler: function ( euler, update ) {

		if ( euler instanceof Engine.Euler === false ) {

			throw new Error( 'ERROR: Quaternion\'s .setFromEuler() now expects a Euler rotation rather than a Vector3 and order.  Please update your code.' );
		}

		// http://www.mathworks.com/matlabcentral/fileexchange/
		// 	20696-function-to-convert-between-dcm-euler-angles-quaternions-and-euler-vectors/
		//	content/SpinCalc.m

		var c1 = Math.cos( euler._x / 2 );
		var c2 = Math.cos( euler._y / 2 );
		var c3 = Math.cos( euler._z / 2 );
		var s1 = Math.sin( euler._x / 2 );
		var s2 = Math.sin( euler._y / 2 );
		var s3 = Math.sin( euler._z / 2 );

		if ( euler.order === 'XYZ' ) {

			this._x = s1 * c2 * c3 + c1 * s2 * s3;
			this._y = c1 * s2 * c3 - s1 * c2 * s3;
			this._z = c1 * c2 * s3 + s1 * s2 * c3;
			this._w = c1 * c2 * c3 - s1 * s2 * s3;

		} else if ( euler.order === 'YXZ' ) {

			this._x = s1 * c2 * c3 + c1 * s2 * s3;
			this._y = c1 * s2 * c3 - s1 * c2 * s3;
			this._z = c1 * c2 * s3 - s1 * s2 * c3;
			this._w = c1 * c2 * c3 + s1 * s2 * s3;

		} else if ( euler.order === 'ZXY' ) {

			this._x = s1 * c2 * c3 - c1 * s2 * s3;
			this._y = c1 * s2 * c3 + s1 * c2 * s3;
			this._z = c1 * c2 * s3 + s1 * s2 * c3;
			this._w = c1 * c2 * c3 - s1 * s2 * s3;

		} else if ( euler.order === 'ZYX' ) {

			this._x = s1 * c2 * c3 - c1 * s2 * s3;
			this._y = c1 * s2 * c3 + s1 * c2 * s3;
			this._z = c1 * c2 * s3 - s1 * s2 * c3;
			this._w = c1 * c2 * c3 + s1 * s2 * s3;

		} else if ( euler.order === 'YZX' ) {

			this._x = s1 * c2 * c3 + c1 * s2 * s3;
			this._y = c1 * s2 * c3 + s1 * c2 * s3;
			this._z = c1 * c2 * s3 - s1 * s2 * c3;
			this._w = c1 * c2 * c3 - s1 * s2 * s3;

		} else if ( euler.order === 'XZY' ) {

			this._x = s1 * c2 * c3 - c1 * s2 * s3;
			this._y = c1 * s2 * c3 - s1 * c2 * s3;
			this._z = c1 * c2 * s3 + s1 * s2 * c3;
			this._w = c1 * c2 * c3 + s1 * s2 * s3;

		}

		if ( update !== false ) this._updateEuler();

		return this;

	},

	setFromAxisAngle: function ( axis, angle ) {

		// from http://www.euclideanspace.com/maths/geometry/rotations/conversions/angleToQuaternion/index.htm
		// axis have to be normalized

		var halfAngle = angle / 2, s = Math.sin( halfAngle );

		this._x = axis.x * s;
		this._y = axis.y * s;
		this._z = axis.z * s;
		this._w = Math.cos( halfAngle );

		this._updateEuler();

		return this;

	},

	setFromRotationMatrix: function ( m ) {

		// http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToQuaternion/index.htm

		// assumes the upper 3x3 of m is a pure rotation matrix (i.e, unscaled)

		var te = m.elements,

			m11 = te[0], m12 = te[4], m13 = te[8],
			m21 = te[1], m22 = te[5], m23 = te[9],
			m31 = te[2], m32 = te[6], m33 = te[10],

			trace = m11 + m22 + m33,
			s;

		if ( trace > 0 ) {

			s = 0.5 / Math.sqrt( trace + 1.0 );

			this._w = 0.25 / s;
			this._x = ( m32 - m23 ) * s;
			this._y = ( m13 - m31 ) * s;
			this._z = ( m21 - m12 ) * s;

		} else if ( m11 > m22 && m11 > m33 ) {

			s = 2.0 * Math.sqrt( 1.0 + m11 - m22 - m33 );

			this._w = (m32 - m23 ) / s;
			this._x = 0.25 * s;
			this._y = (m12 + m21 ) / s;
			this._z = (m13 + m31 ) / s;

		} else if ( m22 > m33 ) {

			s = 2.0 * Math.sqrt( 1.0 + m22 - m11 - m33 );

			this._w = (m13 - m31 ) / s;
			this._x = (m12 + m21 ) / s;
			this._y = 0.25 * s;
			this._z = (m23 + m32 ) / s;

		} else {

			s = 2.0 * Math.sqrt( 1.0 + m33 - m11 - m22 );

			this._w = ( m21 - m12 ) / s;
			this._x = ( m13 + m31 ) / s;
			this._y = ( m23 + m32 ) / s;
			this._z = 0.25 * s;

		}

		this._updateEuler();

		return this;

	},

	inverse: function () {

		this.conjugate().normalize();

		return this;

	},

	conjugate: function () {

		this._x *= -1;
		this._y *= -1;
		this._z *= -1;

		this._updateEuler();

		return this;

	},

	lengthSq: function () {

		return this._x * this._x + this._y * this._y + this._z * this._z + this._w * this._w;

	},

	length: function () {

		return Math.sqrt( this._x * this._x + this._y * this._y + this._z * this._z + this._w * this._w );

	},

	normalize: function () {

		var l = this.length();

		if ( l === 0 ) {

			this._x = 0;
			this._y = 0;
			this._z = 0;
			this._w = 1;

		} else {

			l = 1 / l;

			this._x = this._x * l;
			this._y = this._y * l;
			this._z = this._z * l;
			this._w = this._w * l;

		}

		return this;

	},

	multiply: function ( q ) {

		return this.multiplyQuaternions( this, q );

	},

	multiplyQuaternions: function ( a, b ) {

		// from http://www.euclideanspace.com/maths/algebra/realNormedAlgebra/quaternions/code/index.htm

		var qax = a._x, qay = a._y, qaz = a._z, qaw = a._w;
		var qbx = b._x, qby = b._y, qbz = b._z, qbw = b._w;

		this._x = qax * qbw + qaw * qbx + qay * qbz - qaz * qby;
		this._y = qay * qbw + qaw * qby + qaz * qbx - qax * qbz;
		this._z = qaz * qbw + qaw * qbz + qax * qby - qay * qbx;
		this._w = qaw * qbw - qax * qbx - qay * qby - qaz * qbz;

		this._updateEuler();

		return this;

	},

	slerp: function ( qb, t ) {

		var x = this._x, y = this._y, z = this._z, w = this._w;

		// http://www.euclideanspace.com/maths/algebra/realNormedAlgebra/quaternions/slerp/

		var cosHalfTheta = w * qb._w + x * qb._x + y * qb._y + z * qb._z;

		if ( cosHalfTheta < 0 ) {

			this._w = -qb._w;
			this._x = -qb._x;
			this._y = -qb._y;
			this._z = -qb._z;

			cosHalfTheta = -cosHalfTheta;

		} else {

			this.copy( qb );

		}

		if ( cosHalfTheta >= 1.0 ) {

			this._w = w;
			this._x = x;
			this._y = y;
			this._z = z;

			return this;

		}

		var halfTheta = Math.acos( cosHalfTheta );
		var sinHalfTheta = Math.sqrt( 1.0 - cosHalfTheta * cosHalfTheta );

		if ( Math.abs( sinHalfTheta ) < 0.001 ) {

			this._w = 0.5 * ( w + this._w );
			this._x = 0.5 * ( x + this._x );
			this._y = 0.5 * ( y + this._y );
			this._z = 0.5 * ( z + this._z );

			return this;

		}

		var ratioA = Math.sin( ( 1 - t ) * halfTheta ) / sinHalfTheta,
		ratioB = Math.sin( t * halfTheta ) / sinHalfTheta;

		this._w = ( w * ratioA + this._w * ratioB );
		this._x = ( x * ratioA + this._x * ratioB );
		this._y = ( y * ratioA + this._y * ratioB );
		this._z = ( z * ratioA + this._z * ratioB );

		this._updateEuler();

		return this;

	},

	equals: function ( quaternion ) {

		return ( quaternion._x === this._x ) && ( quaternion._y === this._y ) && ( quaternion._z === this._z ) && ( quaternion._w === this._w );

	},

	fromArray: function ( array ) {

		this._x = array[ 0 ];
		this._y = array[ 1 ];
		this._z = array[ 2 ];
		this._w = array[ 3 ];

		this._updateEuler();

		return this;

	},

	toArray: function () {

		return [ this._x, this._y, this._z, this._w ];

	},

	clone: function () {

		return new Engine.Quaternion( this._x, this._y, this._z, this._w );

	}

};

Engine.Quaternion.slerp = function ( qa, qb, qm, t ) {

	return qm.copy( qa ).slerp( qb, t );

};
Engine.Sphere = function ( center, radius ) {

	this.center = ( center !== undefined ) ? center : new Engine.Vector3();
	this.radius = ( radius !== undefined ) ? radius : 0;

};

Engine.Sphere.prototype = {

	constructor: Engine.Sphere,

	set: function ( center, radius ) {

		this.center.copy( center );
		this.radius = radius;

		return this;
	},


	setFromPoints: function () {

		var box = new Engine.Box3();

		return function ( points, optionalCenter )  {

			var center = this.center;

			if ( optionalCenter !== undefined ) {

				center.copy( optionalCenter );

			} else {

				box.setFromPoints( points ).center( center );

			}

			var maxRadiusSq = 0;

			for ( var i = 0, il = points.length; i < il; i ++ ) {

				maxRadiusSq = Math.max( maxRadiusSq, center.distanceToSquared( points[ i ] ) );

			}

			this.radius = Math.sqrt( maxRadiusSq );

			return this;			
 		
 		};

	}(),

	copy: function ( sphere ) {

		this.center.copy( sphere.center );
		this.radius = sphere.radius;

		return this;

	},

	empty: function () {

		return ( this.radius <= 0 );

	},

	containsPoint: function ( point ) {

		return ( point.distanceToSquared( this.center ) <= ( this.radius * this.radius ) );

	},

	distanceToPoint: function ( point ) {

		return ( point.distanceTo( this.center ) - this.radius );

	},

	intersectsSphere: function ( sphere ) {

		var radiusSum = this.radius + sphere.radius;

		return sphere.center.distanceToSquared( this.center ) <= ( radiusSum * radiusSum );

	},

	clampPoint: function ( point, optionalTarget ) {

		var deltaLengthSq = this.center.distanceToSquared( point );

		var result = optionalTarget || new Engine.Vector3();
		result.copy( point );

		if ( deltaLengthSq > ( this.radius * this.radius ) ) {

			result.sub( this.center ).normalize();
			result.multiplyScalar( this.radius ).add( this.center );

		}

		return result;

	},

	getBoundingBox: function ( optionalTarget ) {

		var box = optionalTarget || new Engine.Box3();

		box.set( this.center, this.center );
		box.expandByScalar( this.radius );

		return box;

	},

	applyMatrix4: function ( matrix ) {

		this.center.applyMatrix4( matrix );
		this.radius = this.radius * matrix.getMaxScaleOnAxis();

		return this;

	},

	translate: function ( offset ) {

		this.center.add( offset );

		return this;

	},

	equals: function ( sphere ) {

		return sphere.center.equals( this.center ) && ( sphere.radius === this.radius );

	},

	clone: function () {

		return new Engine.Sphere().copy( this );

	}

};
Engine.Spline = function ( points ) {

	this.points = points;

	var c = [], v3 = { x: 0, y: 0, z: 0 },
	point, intPoint, weight, w2, w3,
	pa, pb, pc, pd;

	this.initFromArray = function( a ) {

		this.points = [];

		for ( var i = 0; i < a.length; i++ ) {

			this.points[ i ] = { x: a[ i ][ 0 ], y: a[ i ][ 1 ], z: a[ i ][ 2 ] };

		}

	};

	this.getPoint = function ( k ) {

		point = ( this.points.length - 1 ) * k;
		intPoint = Math.floor( point );
		weight = point - intPoint;

		c[ 0 ] = intPoint === 0 ? intPoint : intPoint - 1;
		c[ 1 ] = intPoint;
		c[ 2 ] = intPoint  > this.points.length - 2 ? this.points.length - 1 : intPoint + 1;
		c[ 3 ] = intPoint  > this.points.length - 3 ? this.points.length - 1 : intPoint + 2;

		pa = this.points[ c[ 0 ] ];
		pb = this.points[ c[ 1 ] ];
		pc = this.points[ c[ 2 ] ];
		pd = this.points[ c[ 3 ] ];

		w2 = weight * weight;
		w3 = weight * w2;

		v3.x = interpolate( pa.x, pb.x, pc.x, pd.x, weight, w2, w3 );
		v3.y = interpolate( pa.y, pb.y, pc.y, pd.y, weight, w2, w3 );
		v3.z = interpolate( pa.z, pb.z, pc.z, pd.z, weight, w2, w3 );

		return v3;

	};

	this.getControlPointsArray = function () {

		var i, p, l = this.points.length,
			coords = [];

		for ( i = 0; i < l; i ++ ) {

			p = this.points[ i ];
			coords[ i ] = [ p.x, p.y, p.z ];

		}

		return coords;

	};

	// approximate length by summing linear segments

	this.getLength = function ( nSubDivisions ) {

		var i, index, nSamples, position,
			point = 0, intPoint = 0, oldIntPoint = 0,
			oldPosition = new Engine.Vector3(),
			tmpVec = new Engine.Vector3(),
			chunkLengths = [],
			totalLength = 0;

		// first point has 0 length

		chunkLengths[ 0 ] = 0;

		if ( !nSubDivisions ) nSubDivisions = 100;

		nSamples = this.points.length * nSubDivisions;

		oldPosition.copy( this.points[ 0 ] );

		for ( i = 1; i < nSamples; i ++ ) {

			index = i / nSamples;

			position = this.getPoint( index );
			tmpVec.copy( position );

			totalLength += tmpVec.distanceTo( oldPosition );

			oldPosition.copy( position );

			point = ( this.points.length - 1 ) * index;
			intPoint = Math.floor( point );

			if ( intPoint != oldIntPoint ) {

				chunkLengths[ intPoint ] = totalLength;
				oldIntPoint = intPoint;

			}

		}

		// last point ends with total length

		chunkLengths[ chunkLengths.length ] = totalLength;

		return { chunks: chunkLengths, total: totalLength };

	};

	this.reparametrizeByArcLength = function ( samplingCoef ) {

		var i, j,
			index, indexCurrent, indexNext,
			realDistance,
			sampling, position,
			newpoints = [],
			tmpVec = new Engine.Vector3(),
			sl = this.getLength();

		newpoints.push( tmpVec.copy( this.points[ 0 ] ).clone() );

		for ( i = 1; i < this.points.length; i++ ) {

			realDistance = sl.chunks[ i ] - sl.chunks[ i - 1 ];

			sampling = Math.ceil( samplingCoef * realDistance / sl.total );

			indexCurrent = ( i - 1 ) / ( this.points.length - 1 );
			indexNext = i / ( this.points.length - 1 );

			for ( j = 1; j < sampling - 1; j++ ) {

				index = indexCurrent + j * ( 1 / sampling ) * ( indexNext - indexCurrent );

				position = this.getPoint( index );
				newpoints.push( tmpVec.copy( position ).clone() );

			}

			newpoints.push( tmpVec.copy( this.points[ i ] ).clone() );

		}

		this.points = newpoints;

	};

	// Catmull-Rom

	function interpolate( p0, p1, p2, p3, t, t2, t3 ) {

		var v0 = ( p2 - p0 ) * 0.5,
			v1 = ( p3 - p1 ) * 0.5;

		return ( 2 * ( p1 - p2 ) + v0 + v1 ) * t3 + ( - 3 * ( p1 - p2 ) - 2 * v0 - v1 ) * t2 + v0 * t + p1;

	};

};
Engine.Screen = function () {

    var vertex = [1.0, 1.0, 0.0, -1.0, 1.0, 0.0,
        1.0, -1.0, 0.0, -1.0, -1.0, 0.0,
        1.0, -1.0, 0.0, -1.0, 1.0, 0.0,

    ];
    var vUv = [1.0, 1.0,
        0.0, 1.0,
        1.0, 0.0,
        0.0, -0.0,
        1.0, 0.0,
        0.0, 1.0
    ];

    this.__webglVertexBuffer = gl.createBuffer();
    gl.bindBuffer(gl.ARRAY_BUFFER, this.__webglVertexBuffer);
    gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(vertex), gl.STATIC_DRAW);
    this.__webglVertexBuffer.itemSize = 3;
    this.__webglVertexBuffer.numItems = vertex.length / 3;

    this.__webglUVBuffer = gl.createBuffer();
    gl.bindBuffer(gl.ARRAY_BUFFER, this.__webglUVBuffer);
    gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(vUv), gl.STATIC_DRAW);
    this.__webglUVBuffer.itemSize = 2;
    this.__webglUVBuffer.numItems = vUv / 2;

};


Engine.Framebuffer = function ( width, height, backgroundColor ) {

	this._clearColor = backgroundColor?new Engine.Color( backgroundColor ):new Engine.Color( 0x000000 );

  	var framebuffer = gl.createFramebuffer();
  	gl.bindFramebuffer(gl.FRAMEBUFFER, framebuffer);

	var texture = gl.createTexture();
	gl.bindTexture(gl.TEXTURE_2D, texture);
	//if no power of 2
	if( ((width&(width-1)) != 0) || ((height&(height-1)) != 0) ){
		gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
		gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
		gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
	}else{
    	gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);
    	gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
	}

	gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, width, height, 0, gl.RGBA, gl.UNSIGNED_BYTE, null);

	var renderbuffer = gl.createRenderbuffer();
	gl.bindRenderbuffer(gl.RENDERBUFFER, renderbuffer);
	gl.renderbufferStorage(gl.RENDERBUFFER, gl.DEPTH_COMPONENT16, width, height);

	gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, texture, 0);
	gl.framebufferRenderbuffer(gl.FRAMEBUFFER, gl.DEPTH_ATTACHMENT, gl.RENDERBUFFER, renderbuffer);

	gl.bindTexture(gl.TEXTURE_2D, null);
	gl.bindRenderbuffer(gl.RENDERBUFFER, null);
	gl.bindFramebuffer(gl.FRAMEBUFFER, null);

	this.texture = texture;
	this.framebuffer = framebuffer;
	this.height = height;
	this.width = width;
};

Engine.Framebuffer.prototype = {

	constructor: Engine.Framebuffer,

	saveInTexture: function() {
		gl.bindTexture(gl.TEXTURE_2D, this.shadowMap);
		gl.bindTexture(gl.TEXTURE_2D, null);
	},

	enableBuffer: function() {
		gl.bindFramebuffer(gl.FRAMEBUFFER, this.framebuffer);
		gl.viewport(0, 0, this.width, this.height);
		gl.clearColor(this._clearColor.r, this._clearColor.g, this._clearColor.b, 1.0);
		gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);
	}

}


Engine.Object3D = function () {

	this.parent = undefined;
	this.children = [];

	this.position = new Engine.Vector3();
	this._rotation = new Engine.Euler();
	this._quaternion = new Engine.Quaternion();
	this.scale = new Engine.Vector3( 1, 1, 1 );

	// keep rotation and quaternion in sync

	this._rotation._quaternion = this.quaternion;
	this._quaternion._euler = this.rotation;

	this.matrix = new Engine.Matrix4();
	this.matrixWorld = new Engine.Matrix4();

	this.matrixAutoUpdate = true;
	this.matrixWorldNeedsUpdate = true;

};


Engine.Object3D.prototype = {

	constructor: Engine.Object3D,
	
	get rotation () { 
		return this._rotation; 
	},

	set rotation ( value ) {
		
		this._rotation = value;
		this._rotation._quaternion = this._quaternion;
		this._quaternion._euler = this._rotation;
		this._rotation._updateQuaternion();

	},

	get quaternion () { 
		return this._quaternion; 
	},
	
	set quaternion ( value ) {
		
		this._quaternion = value;
		this._quaternion._euler = this._rotation;
		this._rotation._quaternion = this._quaternion;
		this._quaternion._updateEuler();

	},

	applyMatrix: function ( matrix ) {

		this.matrix.multiplyMatrices( matrix, this.matrix );

		this.matrix.decompose( this.position, this.quaternion, this.scale );

	},


	getPlanView:  function () {

		var direction = this.getDirection();

		var d = -(direction.x * this.position.x + direction.y * this.position.y + direction.z * this.position.z);
		return {
			a: direction.x,
			b: direction.y,
			c: direction.z,
			d: d
		};

	},

	setRotationFromAxisAngle: function ( axis, angle ) {

		// assumes axis is normalized

		this.quaternion.setFromAxisAngle( axis, angle );

	},

	setRotationFromEuler: function ( euler ) {

		this.quaternion.setFromEuler( euler, true );

	},

	setRotationFromMatrix: function ( m ) {

		// assumes the upper 3x3 of m is a pure rotation matrix (i.e, unscaled)

		this.quaternion.setFromRotationMatrix( m );

	},

	setRotationFromQuaternion: function ( q ) {

		// assumes q is normalized

		this.quaternion.copy( q );

	},

	rotateOnAxis: function() {

		// rotate object on axis in object space
		// axis is assumed to be normalized

		var q1 = new Engine.Quaternion();

		return function ( axis, angle ) {

			q1.setFromAxisAngle( axis, angle );

			this.quaternion.multiply( q1 );

			return this;

		}

	}(),

	rotateX: function () {

		var v1 = new Engine.Vector3( 1, 0, 0 );

		return function ( angle ) {

			return this.rotateOnAxis( v1, angle );

		};

	}(),

	rotateY: function () {

		var v1 = new Engine.Vector3( 0, 1, 0 );

		return function ( angle ) {

			return this.rotateOnAxis( v1, angle );

		};

	}(),

	rotateZ: function () {

		var v1 = new Engine.Vector3( 0, 0, 1 );

		return function ( angle ) {

			return this.rotateOnAxis( v1, angle );

		};

	}(),

	translateOnAxis: function () {

		// translate object by distance along axis in object space
		// axis is assumed to be normalized

		var v1 = new Engine.Vector3();

		return function ( axis, distance ) {

			v1.copy( axis );

			v1.applyQuaternion( this.quaternion );

			this.position.add( v1.multiplyScalar( distance ) );

			return this;

		}

	}(),


	translateX: function () {

		var v1 = new Engine.Vector3( 1, 0, 0 );

		return function ( distance ) {

			return this.translateOnAxis( v1, distance );

		};

	}(),

	translateY: function () {

		var v1 = new Engine.Vector3( 0, 1, 0 );

		return function ( distance ) {

			return this.translateOnAxis( v1, distance );

		};

	}(),

	translateZ: function () {

		var v1 = new Engine.Vector3( 0, 0, 1 );

		return function ( distance ) {

			return this.translateOnAxis( v1, distance );

		};

	}(),

	localToWorld: function ( vector ) {

		return vector.applyMatrix4( this.matrixWorld );

	},

	worldToLocal: function () {

		var m1 = new Engine.Matrix4();

		return function ( vector ) {

			return vector.applyMatrix4( m1.getInverse( this.matrixWorld ) );

		};

	}(),

	lookAt: function () {

		// This routine does not support objects with rotated and/or translated parent(s)

		var m1 = new Engine.Matrix4();
		var up = new Engine.Vector3( 0, 1, 0 );

		return function ( vector ) {

			m1.lookAt( vector, this.position, up );

			this.quaternion.setFromRotationMatrix( m1 );

		};

	}(),


	getDirection: function () {

		return new Engine.Vector3(
			this.matrix.elements[8],
			this.matrix.elements[9],
			this.matrix.elements[10] 
		);

	},

	add: function ( object ) {

		if ( object === this ) {

			console.warn( 'Engine.Object3D.add: An object can\'t be added as a child of itself.' );
			return;

		}

		if ( object instanceof Engine.Object3D ) {

			if ( object.parent !== undefined ) {

				object.parent.remove( object );

			}

			if(!(this instanceof Engine.Scene)){
				object.parent = this;
			}
			this.children.push( object );


			
			// add to scene

			var scene = this;

			while ( scene.parent !== undefined ) {

				scene = scene.parent;

			}

			if ( scene !== undefined && scene instanceof Engine.Scene )  {

				scene.__addObject( object );

			}

		}

	},

	remove: function ( object ) {

		var index = this.children.indexOf( object );

		if ( index !== - 1 ) {

			object.parent = undefined;

			this.children.splice( index, 1 );

			// remove from scene

			var scene = this;

			while ( scene.parent !== undefined ) {

				scene = scene.parent;

			}

			if ( scene !== undefined && scene instanceof Engine.Scene ) {

				scene.__removeObject( object );

			}

		}

	},



	updateMatrix: function () {

		this.matrix.compose( this.position, this.quaternion, this.scale );

		this.matrixWorldNeedsUpdate = true;

	},

	updateMatrixWorld: function ( force ) {

		if ( this.matrixAutoUpdate === true ) this.updateMatrix();

		if ( this.matrixWorldNeedsUpdate === true || force === true ) {

			if ( this.parent === undefined ) {

				this.matrixWorld.copy( this.matrix );

			} else {

				this.matrixWorld.multiplyMatrices( this.parent.matrixWorld, this.matrix );

			}

			this.matrixWorldNeedsUpdate = false;

			force = true;

		}

		// update children

		for ( var i = 0, l = this.children.length; i < l; i ++ ) {

			this.children[ i ].updateMatrixWorld( force );

		}

	},

	clone: function ( object, recursive ) {

		if ( object === undefined ) object = new Engine.Object3D();
		if ( recursive === undefined ) recursive = true;

		object.position.copy( this.position );
		object.quaternion.copy( this.quaternion );
		object.scale.copy( this.scale );

		object.renderDepth = this.renderDepth;

		object.matrix.copy( this.matrix );
		object.matrixWorld.copy( this.matrixWorld );

		object.matrixAutoUpdate = this.matrixAutoUpdate;
		object.matrixWorldNeedsUpdate = this.matrixWorldNeedsUpdate;

		object.visible = this.visible;
		object.castShadow = this.castShadow;
		object.receiveShadow = this.receiveShadow;

		if ( recursive === true ) {

			for ( var i = 0; i < this.children.length; i ++ ) {

				var child = this.children[ i ];
				object.add( child.clone() );

			}

		}

		return object;

	}

};





Engine.Euler = function ( x, y, z, order ) {

	this._x = x || 0;
	this._y = y || 0;
	this._z = z || 0;
	this._order = order || Engine.Euler.DefaultOrder;

};

Engine.Euler.RotationOrders = [ 'XYZ', 'YZX', 'ZXY', 'XZY', 'YXZ', 'ZYX' ];

Engine.Euler.DefaultOrder = 'XYZ';

Engine.Euler.prototype = {

	constructor: Engine.Euler,

	_x: 0, _y: 0, _z: 0, _order: Engine.Euler.DefaultOrder,

	_quaternion: undefined,

	_updateQuaternion: function () {

		if ( this._quaternion !== undefined ) {

			this._quaternion.setFromEuler( this, false );

		}

	},

	get x () {

		return this._x;

	},

	set x ( value ) {

		this._x = value;
		this._updateQuaternion();

	},

	get y () {

		return this._y;

	},

	set y ( value ) {

		this._y = value;
		this._updateQuaternion();

	},

	get z () {

		return this._z;

	},

	set z ( value ) {

		this._z = value;
		this._updateQuaternion();

	},

	get order () {

		return this._order;

	},

	set order ( value ) {

		this._order = value;
		this._updateQuaternion();

	},

	set: function ( x, y, z, order ) {

		this._x = x;
		this._y = y;
		this._z = z;
		this._order = order || this._order;

		this._updateQuaternion();

		return this;

	},

	copy: function ( euler ) {

		this._x = euler._x;
		this._y = euler._y;
		this._z = euler._z;
		this._order = euler._order;

		this._updateQuaternion();

		return this;

	},

	setFromRotationMatrix: function ( m, order ) {

		// assumes the upper 3x3 of m is a pure rotation matrix (i.e, unscaled)

		// clamp, to handle numerical problems

		function clamp( x ) {

			return Math.min( Math.max( x, -1 ), 1 );

		}

		var te = m.elements;
		var m11 = te[0], m12 = te[4], m13 = te[8];
		var m21 = te[1], m22 = te[5], m23 = te[9];
		var m31 = te[2], m32 = te[6], m33 = te[10];

		order = order || this._order;

		if ( order === 'XYZ' ) {

			this._y = Math.asin( clamp( m13 ) );

			if ( Math.abs( m13 ) < 0.99999 ) {

				this._x = Math.atan2( - m23, m33 );
				this._z = Math.atan2( - m12, m11 );

			} else {

				this._x = Math.atan2( m32, m22 );
				this._z = 0;

			}

		} else if ( order === 'YXZ' ) {

			this._x = Math.asin( - clamp( m23 ) );

			if ( Math.abs( m23 ) < 0.99999 ) {

				this._y = Math.atan2( m13, m33 );
				this._z = Math.atan2( m21, m22 );

			} else {

				this._y = Math.atan2( - m31, m11 );
				this._z = 0;

			}

		} else if ( order === 'ZXY' ) {

			this._x = Math.asin( clamp( m32 ) );

			if ( Math.abs( m32 ) < 0.99999 ) {

				this._y = Math.atan2( - m31, m33 );
				this._z = Math.atan2( - m12, m22 );

			} else {

				this._y = 0;
				this._z = Math.atan2( m21, m11 );

			}

		} else if ( order === 'ZYX' ) {

			this._y = Math.asin( - clamp( m31 ) );

			if ( Math.abs( m31 ) < 0.99999 ) {

				this._x = Math.atan2( m32, m33 );
				this._z = Math.atan2( m21, m11 );

			} else {

				this._x = 0;
				this._z = Math.atan2( - m12, m22 );

			}

		} else if ( order === 'YZX' ) {

			this._z = Math.asin( clamp( m21 ) );

			if ( Math.abs( m21 ) < 0.99999 ) {

				this._x = Math.atan2( - m23, m22 );
				this._y = Math.atan2( - m31, m11 );

			} else {

				this._x = 0;
				this._y = Math.atan2( m13, m33 );

			}

		} else if ( order === 'XZY' ) {

			this._z = Math.asin( - clamp( m12 ) );

			if ( Math.abs( m12 ) < 0.99999 ) {

				this._x = Math.atan2( m32, m22 );
				this._y = Math.atan2( m13, m11 );

			} else {

				this._x = Math.atan2( - m23, m33 );
				this._y = 0;

			}

		} else {

			console.warn( 'WARNING: Euler.setFromRotationMatrix() given unsupported order: ' + order )

		}

		this._order = order;

		this._updateQuaternion();

		return this;

	},

	setFromQuaternion: function ( q, order, update ) {

		// q is assumed to be normalized

		// clamp, to handle numerical problems

		function clamp( x ) {

			return Math.min( Math.max( x, -1 ), 1 );

		}

		// http://www.mathworks.com/matlabcentral/fileexchange/20696-function-to-convert-between-dcm-euler-angles-quaternions-and-euler-vectors/content/SpinCalc.m

		var sqx = q.x * q.x;
		var sqy = q.y * q.y;
		var sqz = q.z * q.z;
		var sqw = q.w * q.w;

		order = order || this._order;

		if ( order === 'XYZ' ) {

			this._x = Math.atan2( 2 * ( q.x * q.w - q.y * q.z ), ( sqw - sqx - sqy + sqz ) );
			this._y = Math.asin(  clamp( 2 * ( q.x * q.z + q.y * q.w ) ) );
			this._z = Math.atan2( 2 * ( q.z * q.w - q.x * q.y ), ( sqw + sqx - sqy - sqz ) );

		} else if ( order ===  'YXZ' ) {

			this._x = Math.asin(  clamp( 2 * ( q.x * q.w - q.y * q.z ) ) );
			this._y = Math.atan2( 2 * ( q.x * q.z + q.y * q.w ), ( sqw - sqx - sqy + sqz ) );
			this._z = Math.atan2( 2 * ( q.x * q.y + q.z * q.w ), ( sqw - sqx + sqy - sqz ) );

		} else if ( order === 'ZXY' ) {

			this._x = Math.asin(  clamp( 2 * ( q.x * q.w + q.y * q.z ) ) );
			this._y = Math.atan2( 2 * ( q.y * q.w - q.z * q.x ), ( sqw - sqx - sqy + sqz ) );
			this._z = Math.atan2( 2 * ( q.z * q.w - q.x * q.y ), ( sqw - sqx + sqy - sqz ) );

		} else if ( order === 'ZYX' ) {

			this._x = Math.atan2( 2 * ( q.x * q.w + q.z * q.y ), ( sqw - sqx - sqy + sqz ) );
			this._y = Math.asin(  clamp( 2 * ( q.y * q.w - q.x * q.z ) ) );
			this._z = Math.atan2( 2 * ( q.x * q.y + q.z * q.w ), ( sqw + sqx - sqy - sqz ) );

		} else if ( order === 'YZX' ) {

			this._x = Math.atan2( 2 * ( q.x * q.w - q.z * q.y ), ( sqw - sqx + sqy - sqz ) );
			this._y = Math.atan2( 2 * ( q.y * q.w - q.x * q.z ), ( sqw + sqx - sqy - sqz ) );
			this._z = Math.asin(  clamp( 2 * ( q.x * q.y + q.z * q.w ) ) );

		} else if ( order === 'XZY' ) {

			this._x = Math.atan2( 2 * ( q.x * q.w + q.y * q.z ), ( sqw - sqx + sqy - sqz ) );
			this._y = Math.atan2( 2 * ( q.x * q.z + q.y * q.w ), ( sqw + sqx - sqy - sqz ) );
			this._z = Math.asin(  clamp( 2 * ( q.z * q.w - q.x * q.y ) ) );

		} else {

			console.warn( 'WARNING: Euler.setFromQuaternion() given unsupported order: ' + order )

		}

		this._order = order;

		if ( update !== false ) this._updateQuaternion();

		return this;

	},

	reorder: function () {

		// WARNING: this discards revolution information -bhouston

		var q = new Engine.Quaternion();

		return function ( newOrder ) {

			q.setFromEuler( this );
			this.setFromQuaternion( q, newOrder );

		};


	}(),

	fromArray: function ( array ) {

		this._x = array[ 0 ];
		this._y = array[ 1 ];
		this._z = array[ 2 ];
		if ( array[ 3 ] !== undefined ) this._order = array[ 3 ];

		this._updateQuaternion();

		return this;

	},

	toArray: function () {

		return [ this._x, this._y, this._z, this._order ];

	},

	equals: function ( euler ) {

		return ( euler._x === this._x ) && ( euler._y === this._y ) && ( euler._z === this._z ) && ( euler._order === this._order );

	},

	clone: function () {

		return new Engine.Euler( this._x, this._y, this._z, this._order );

	}

};



Engine.Geometry = function () {

	this.ready = false;

	this.faces = [];

	this.vVertices = [];
	this.vNormals = [];
	this.vUvs = [];

	this.morphTargets = [];

	this.boundingBox = null;
	this.boundingSphere = null;

};

Engine.Geometry.prototype = {

	constructor: Engine.Geometry,


	computeVertexNormals: function () {

		var v, vl, f, fl, face, vertices;

		vertices = new Array( this.vVertices.length );

		for ( v = 0, vl = this.vVertices.length; v < vl; v ++ ) {

			vertices[ v ] = new Engine.Vector3();

		}

		var vA, vB, vC, vD;
		var cb = new Engine.Vector3(), ab = new Engine.Vector3(),
			db = new Engine.Vector3(), dc = new Engine.Vector3(), bc = new Engine.Vector3();

		for ( f = 0, fl = this.faces.length; f < fl; f ++ ) {

			face = this.faces[ f ];

			vA = this.vVertices[ face.a ];
			vB = this.vVertices[ face.b ];
			vC = this.vVertices[ face.c ];

			cb.subVectors( vC, vB );
			ab.subVectors( vA, vB );
			cb.cross( ab );

			vertices[ face.a ].add( cb );
			vertices[ face.b ].add( cb );
			vertices[ face.c ].add( cb );

		}

		//Les vertices commun son deja dédoublé. c'est un mauvaise solution

		for ( v = 0, vl = this.vVertices.length; v < vl; v ++ ) {

			vertices[ v ].normalize();

		}

		this.vNormals = vertices;

	},


	computeVertexNormalsForObj: function (vFaces,vVertices) {

		var v, vl, f, fl, vertices;

		vertices = new Array( vVertices.length );

		for ( v = 0, vl = vVertices.length; v < vl; v ++ ) {

			vertices[ v ] = new Engine.Vector3();

		}

		var vA, vB, vC, vD;
		var cb = new Engine.Vector3(), ab = new Engine.Vector3(),
			db = new Engine.Vector3(), dc = new Engine.Vector3(), bc = new Engine.Vector3();

		for ( f = 0, fl = vFaces.length; f < fl; f +=3 ) {

			vA = vVertices[ vFaces[f] ];
			vB = vVertices[ vFaces[f+1] ];
			vC = vVertices[ vFaces[f+2] ];

			cb.subVectors( vC, vB );
			ab.subVectors( vA, vB );
			cb.cross( ab );

			vertices[ vFaces[f] ].add( cb );
			vertices[ vFaces[f+1] ].add( cb );
			vertices[ vFaces[f+2] ].add( cb );

		}

		//Les vertices commun son deja dédoublé. c'est un mauvaise solution

		for ( v = 0, vl = vVertices.length; v < vl; v ++ ) {

			vertices[ v ].normalize();

		}

		return vertices;
 
	},

	computeMorphVertexNormals: function () {

		var i, il;

		for ( i = 0, il = this.morphTargets.length; i < il; i ++ ) {

			var tmpGeo = new Engine.Geometry();

			tmpGeo.faces = this.faces;

			tmpGeo.vVertices = this.morphTargets[ i ].vVertices;

			tmpGeo.computeVertexNormals();

			this.morphTargets[ i ].vNormals = tmpGeo.vNormals;

			delete tmpGeo;

		}

	},


	computeMorphVertexTangents: function () {

		var i, il;

		for ( i = 0, il = this.morphTargets.length; i < il; i ++ ) {

			var tmpGeo = new Engine.Geometry();

			tmpGeo.faces = this.faces;

			tmpGeo.vVertices = this.morphTargets[ i ].vVertices;
			tmpGeo.vUvs = this.vUvs;
			tmpGeo.vNormals = this.morphTargets[ i ].vNormals;

			tmpGeo.computeVertexTangents();

			this.morphTargets[ i ].vTangents = tmpGeo.vTangents;

			delete tmpGeo;

		}

	},

	computeBoundingBox: function () {

		if ( this.boundingBox === null ) {

			this.boundingBox = new Engine.Box3();

		}

		this.boundingBox.setFromPoints( this.vVertices );

	},

	computeVertexTangents: function () {

		var f, fl, v, vl, i, il, vertexIndex,
			face, uv, vA, vB, vC, uvA, uvB, uvC,
			x1, x2, y1, y2, z1, z2,
			s1, s2, t1, t2, r, t, test,
			tan1 = [],
			sdir = new Engine.Vector3(), tdir = new Engine.Vector3(),
			tmp = new Engine.Vector3(), tmp2 = new Engine.Vector3(),
			n = new Engine.Vector3(), w;

		for ( v = 0, vl = this.vVertices.length; v < vl; v ++ ) {

			tan1[ v ] = new Engine.Vector3();

		}

		function handleTriangle( context, a, b, c) {

			vA = context.vVertices[ a ];
			vB = context.vVertices[ b ];
			vC = context.vVertices[ c ];

			uvA = context.vUvs[ a ];
			uvB = context.vUvs[ b ];
			uvC = context.vUvs[ c ];

			x1 = vB.x - vA.x;
			x2 = vC.x - vA.x;
			y1 = vB.y - vA.y;
			y2 = vC.y - vA.y;
			z1 = vB.z - vA.z;
			z2 = vC.z - vA.z;

			s1 = uvB.x - uvA.x;
			s2 = uvC.x - uvA.x;
			t1 = uvB.y - uvA.y;
			t2 = uvC.y - uvA.y;

			r = 1.0 / ( s1 * t2 - s2 * t1 );
			if(r == Infinity || r == -Infinity) r = 0;

			sdir.set( ( t2 * x1 - t1 * x2 ) * r,
					  ( t2 * y1 - t1 * y2 ) * r,
					  ( t2 * z1 - t1 * z2 ) * r );

			tan1[ a ].add( sdir );
			tan1[ b ].add( sdir );
			tan1[ c ].add( sdir );

		}

		for ( f = 0, fl = this.faces.length; f < fl; f ++ ) {

			face = this.faces[ f ];
		
			handleTriangle( this, face.a, face.b, face.c);

		}

		this.vTangents = [];

		for ( k = 0, kl = this.vNormals.length; k < kl; k ++ ) {

			this.vTangents[k] = tan1[ k ];

		}

	},

	merge: function (geometry,matrix) {

		if( (this.vNormals.length==0)!=(geometry.vVertices.length==0) || (this.vUvs.length==0)!=(geometry.vUvs.length==0) || (this.vUvs.length==0)!=(geometry.vUvs.length==0)){
			console.log("merge fail, unassigned attributes !")
			return;
		}

		var offset = this.vVertices.length;
		var newface = [];
		var newvertice = [];
		var newnormal = [];

		if(matrix){
			if(geometry.vNormals.length){
				for(var k=0,kl=geometry.vVertices.length;k<kl;k++){
					newvertice.push(geometry.vVertices[k].clone().applyMatrix4(matrix));
					newnormal.push(geometry.vNormals[k].clone().applyMatrix3of4(matrix));
				}
			}else{
				for(var k=0,kl=geometry.vVertices.length;k<kl;k++){
					newvertice.push(geometry.vVertices[k].clone().applyMatrix4(matrix));
				}
			}

		}else{
			newvertice = geometry.vVertices;
			newnormal = geometry.vNormals;
		}

		this.vVertices = this.vVertices.concat(newvertice);

		if(geometry.vNormals.length)
		this.vNormals = this.vNormals.concat(newnormal);

		if(geometry.vUvs.length)
		this.vUvs = this.vUvs.concat(geometry.vUvs)

		for(var k=0,kl=geometry.faces.length;k<kl;k++)
			newface.push(geometry.faces[k].clone().addOffset(offset));

		this.faces = this.faces.concat(newface);

	}
    
}; 
Engine.CubeGeometry = function ( width, height, depth, widthSegments, heightSegments, depthSegments ) {

	Engine.Geometry.call( this );

	var scope = this;

	this.width = width;
	this.height = height;
	this.depth = depth;

	this.widthSegments = widthSegments || 1;
	this.heightSegments = heightSegments || 1;
	this.depthSegments = depthSegments || 1;

	var width_half = this.width / 2;
	var height_half = this.height / 2;
	var depth_half = this.depth / 2;

	buildPlane( 'z', 'y', - 1, - 1, this.depth, this.height, width_half, 0 ); // px
	buildPlane( 'z', 'y',   1, - 1, this.depth, this.height, - width_half, 1 ); // nx
	buildPlane( 'x', 'z',   1,   1, this.width, this.depth, height_half, 2 ); // py
	buildPlane( 'x', 'z',   1, - 1, this.width, this.depth, - height_half, 3 ); // ny
	buildPlane( 'x', 'y',   1, - 1, this.width, this.height, depth_half, 4 ); // pz
	buildPlane( 'x', 'y', - 1, - 1, this.width, this.height, - depth_half, 5 ); // nz

	this.ready = true;

	function buildPlane( u, v, udir, vdir, width, height, depth ) {

		var w, ix, iy,
		gridX = scope.widthSegments,
		gridY = scope.heightSegments,
		width_half = width / 2,
		height_half = height / 2,
		offset = scope.vVertices.length;

		if ( ( u === 'x' && v === 'y' ) || ( u === 'y' && v === 'x' ) ) {

			w = 'z';

		} else if ( ( u === 'x' && v === 'z' ) || ( u === 'z' && v === 'x' ) ) {

			w = 'y';
			gridY = scope.depthSegments;

		} else if ( ( u === 'z' && v === 'y' ) || ( u === 'y' && v === 'z' ) ) {

			w = 'x';
			gridX = scope.depthSegments;

		}

		var gridX1 = gridX + 1,
		gridY1 = gridY + 1,
		segment_width = width / gridX,
		segment_height = height / gridY,
		normal = new Engine.Vector3();

		normal[ w ] = depth > 0 ? 1 : - 1;

		for ( iy = 0; iy < gridY1; iy ++ ) {

			for ( ix = 0; ix < gridX1; ix ++ ) {

				var vector = new Engine.Vector3();
				vector[ u ] = ( ix * segment_width - width_half ) * udir;
				vector[ v ] = ( iy * segment_height - height_half ) * vdir;
				vector[ w ] = depth;

				scope.vVertices.push( vector );

				scope.vNormals.push( normal.clone());



			}

		}

		for ( iy = 0; iy < gridY; iy++ ) {

			for ( ix = 0; ix < gridX; ix++ ) {

				var a = ix + gridX1 * iy;
				var b = ix + gridX1 * ( iy + 1 );
				var c = ( ix + 1 ) + gridX1 * ( iy + 1 );
				var d = ( ix + 1 ) + gridX1 * iy;

				var uva = new Engine.Vector2( ix / gridX, 1 - iy / gridY );
				var uvb = new Engine.Vector2( ix / gridX, 1 - ( iy + 1 ) / gridY );
				var uvc = new Engine.Vector2( ( ix + 1 ) / gridX, 1 - ( iy + 1 ) / gridY );
				var uvd = new Engine.Vector2( ( ix + 1 ) / gridX, 1 - iy / gridY );

				var face = new Engine.Face3( a + offset, b + offset, d + offset );

				scope.faces.push( face );

				scope.vUvs[a + offset] = uva;
				scope.vUvs[b + offset] = uvb;
				scope.vUvs[d + offset] = uvd;
				

				face = new Engine.Face3( b + offset, c + offset, d + offset );

				scope.faces.push( face );

				scope.vUvs[b + offset] = uvb.clone();
				scope.vUvs[c + offset] = uvc;
				scope.vUvs[d + offset] = uvd.clone();

			}

		}

	}
};

Engine.CubeGeometry.prototype = Object.create( Engine.Geometry.prototype );

Engine.SkyboxGeometry = function () {

	Engine.Geometry.call( this );

	this.vVertices.push( new Engine.Vector3(10,-10,-10) );
	this.vVertices.push( new Engine.Vector3(10,10,-10) );
	this.vVertices.push( new Engine.Vector3(10,10,10) );
	this.vVertices.push( new Engine.Vector3(10,-10,10) );
	this.vVertices.push( new Engine.Vector3(-10,-10,-10) );
	this.vVertices.push( new Engine.Vector3(-10,-10,10) );
	this.vVertices.push( new Engine.Vector3(-10,10,10) );
	this.vVertices.push( new Engine.Vector3(-10,10,-10) );

	this.faces.push( new Engine.Face3( 0, 1, 2 ) );
	this.faces.push( new Engine.Face3( 2, 3, 0 ) );
	this.faces.push( new Engine.Face3( 4, 5, 6 ) );
	this.faces.push( new Engine.Face3( 6, 7, 4 ) );
	this.faces.push( new Engine.Face3( 0, 3, 5 ) );
	this.faces.push( new Engine.Face3( 5, 4, 0 ) );
	this.faces.push( new Engine.Face3( 3, 2, 6 ) );
	this.faces.push( new Engine.Face3( 6, 5, 3 ) );
	this.faces.push( new Engine.Face3( 2, 1, 7 ) );
	this.faces.push( new Engine.Face3( 7, 6, 2 ) );
	this.faces.push( new Engine.Face3( 1, 0, 4 ) );
	this.faces.push( new Engine.Face3( 4, 7, 1 ) );

	this.ready = true;
};

Engine.SkyboxGeometry.prototype = Object.create( Engine.Geometry.prototype );
Engine.CylinderGeometry = function ( radiusTop, radiusBottom, height, radialSegments, heightSegments, openEnded ) {

	Engine.Geometry.call( this );

	this.radiusTop = radiusTop = radiusTop !== undefined ? radiusTop : 20;
	this.radiusBottom = radiusBottom = radiusBottom !== undefined ? radiusBottom : 20;
	this.height = height = height !== undefined ? height : 100;

	this.radialSegments = radialSegments = radialSegments || 8;
	this.heightSegments = heightSegments = heightSegments || 1;

	this.openEnded = openEnded = openEnded !== undefined ? openEnded : false;

	var heightHalf = height / 2;

	var x, y, vertices = [], uvs = [];

	for ( y = 0; y <= heightSegments; y ++ ) {

		var verticesRow = [];
		var uvsRow = [];

		var v = y / heightSegments;
		var radius = v * ( radiusBottom - radiusTop ) + radiusTop;

		for ( x = 0; x <= radialSegments; x ++ ) {

			var u = x / radialSegments;

			var vertex = new Engine.Vector3();
			vertex.x = radius * Math.sin( u * Math.PI * 2 );
			vertex.y = - v * height + heightHalf;
			vertex.z = radius * Math.cos( u * Math.PI * 2 );

			this.vVertices.push( vertex );

			verticesRow.push( this.vVertices.length - 1 );

			this.vUvs.push( new Engine.Vector2( u, 1 - v ) );

		}

		vertices.push( verticesRow );

	}

	var tanTheta = ( radiusBottom - radiusTop ) / height;

	for ( x = 0; x < radialSegments; x ++ ) {

		for ( y = 0; y < heightSegments; y ++ ) {

			var v1 = vertices[ y ][ x ];
			var v2 = vertices[ y + 1 ][ x ];
			var v3 = vertices[ y + 1 ][ x + 1 ];
			var v4 = vertices[ y ][ x + 1 ];

			this.faces.push( new Engine.Face3( v1, v2, v4 ) );

			this.faces.push( new Engine.Face3( v2, v3, v4 ) );

		}

	}

	// top cap

	if ( openEnded === false && radiusTop > 0 ) {

		this.vVertices.push( new Engine.Vector3( 0, heightHalf, 0 ) );

		this.vUvs.push( new Engine.Vector2( 0, 1 ) );

		for ( x = 0; x < radialSegments; x ++ ) {

			var v1 = vertices[ 0 ][ x ];
			var v2 = vertices[ 0 ][ x + 1 ];
			var v3 = this.vVertices.length - 1;

			this.faces.push( new Engine.Face3( v1, v2, v3) );

		}

	}

	// bottom cap

	if ( openEnded === false && radiusBottom > 0 ) {

		this.vVertices.push( new Engine.Vector3( 0, - heightHalf, 0 ) );

		this.vUvs.push( new Engine.Vector2( 0, 1 ) );

		for ( x = 0; x < radialSegments; x ++ ) {

			var v1 = vertices[ y ][ x + 1 ];
			var v2 = vertices[ y ][ x ];
			var v3 = this.vVertices.length - 1;

			this.faces.push( new Engine.Face3( v1, v2, v3 ) );

		}

	}

	this.computeVertexNormals();

	this.ready = true;

}

Engine.CylinderGeometry.prototype = Object.create( Engine.Geometry.prototype );
Engine.PlaneGeometry = function ( width, height, widthSegments, heightSegments ) {

	Engine.Geometry.call( this );

	this.width = width;
	this.height = height;

	this.widthSegments = widthSegments || 1;
	this.heightSegments = heightSegments || 1;

	var ix, iz;
	var width_half = width / 2;
	var height_half = height / 2;

	var gridX = this.widthSegments;
	var gridZ = this.heightSegments;

	var gridX1 = gridX + 1;
	var gridZ1 = gridZ + 1;

	var segment_width = this.width / gridX;
	var segment_height = this.height / gridZ;

	var normal = new Engine.Vector3( 0, 0, 1 );

	for ( iz = 0; iz < gridZ1; iz ++ ) {

		for ( ix = 0; ix < gridX1; ix ++ ) {

			var x = ix * segment_width - width_half;
			var y = iz * segment_height - height_half;

			this.vVertices.push( new Engine.Vector3( x, - y, 0 ) );
			this.vNormals.push( normal.clone());

		}

	}

	this.vUvs = [];

	for ( iz = 0; iz < gridZ; iz ++ ) {

		for ( ix = 0; ix < gridX; ix ++ ) {

			var a = ix + gridX1 * iz;
			var b = ix + gridX1 * ( iz + 1 );
			var c = ( ix + 1 ) + gridX1 * ( iz + 1 );
			var d = ( ix + 1 ) + gridX1 * iz;

			var uva = new Engine.Vector2( ix / gridX, 1 - iz / gridZ );
			var uvb = new Engine.Vector2( ix / gridX, 1 - ( iz + 1 ) / gridZ );
			var uvc = new Engine.Vector2( ( ix + 1 ) / gridX, 1 - ( iz + 1 ) / gridZ );
			var uvd = new Engine.Vector2( ( ix + 1 ) / gridX, 1 - iz / gridZ );

			var face = new Engine.Face3( a, b, d );
			this.faces.push( face );

			this.vUvs[a] = uva;
			this.vUvs[b] = uvb;
			this.vUvs[d ] = uvd;

			face = new Engine.Face3( b, c, d );
			this.faces.push( face );

			this.vUvs[b] = uvb.clone();
			this.vUvs[c] = uvc;
			this.vUvs[d ] =  uvd.clone();

		}

	}

	this.ready = true;

};

Engine.PlaneGeometry.prototype = Object.create( Engine.Geometry.prototype );
Engine.SphereGeometry = function ( radius, widthSegments, heightSegments, phiStart, phiLength, thetaStart, thetaLength ) {

	Engine.Geometry.call( this );

	this.radius = radius = radius || 50;

	this.widthSegments = widthSegments = Math.max( 3, Math.floor( widthSegments ) || 8 );
	this.heightSegments = heightSegments = Math.max( 2, Math.floor( heightSegments ) || 6 );

	this.phiStart = phiStart = phiStart !== undefined ? phiStart : 0;
	this.phiLength = phiLength = phiLength !== undefined ? phiLength : Math.PI * 2;

	this.thetaStart = thetaStart = thetaStart !== undefined ? thetaStart : 0;
	this.thetaLength = thetaLength = thetaLength !== undefined ? thetaLength : Math.PI;

	var x, y, vertices = [], uvs = [];

	for ( y = 0; y <= heightSegments; y ++ ) {

		var verticesRow = [];
		var uvsRow = [];

		for ( x = 0; x <= widthSegments; x ++ ) {

			var u = x / widthSegments;
			var v = y / heightSegments;

			var vertex = new Engine.Vector3();
			vertex.x = - radius * Math.cos( phiStart + u * phiLength ) * Math.sin( thetaStart + v * thetaLength );
			vertex.y = radius * Math.cos( thetaStart + v * thetaLength );
			vertex.z = radius * Math.sin( phiStart + u * phiLength ) * Math.sin( thetaStart + v * thetaLength );

			this.vVertices.push( vertex );

			verticesRow.push( this.vVertices.length - 1 );
			this.vUvs.push( new Engine.Vector2( u, 1 - v ) );

		}

		vertices.push( verticesRow );

	}

	for ( y = 0; y < this.heightSegments; y ++ ) {

		for ( x = 0; x < this.widthSegments; x ++ ) {

			var v1 = vertices[ y ][ x + 1 ];
			var v2 = vertices[ y ][ x ];
			var v3 = vertices[ y + 1 ][ x ];
			var v4 = vertices[ y + 1 ][ x + 1 ];

			if ( Math.abs( this.vVertices[ v1 ].y ) === this.radius ) {

				this.faces.push( new Engine.Face3( v1, v3, v4 ) );

			} else if ( Math.abs( this.vVertices[ v3 ].y ) === this.radius ) {

				this.faces.push( new Engine.Face3( v1, v2, v3 ) );

			} else {

				this.faces.push( new Engine.Face3( v1, v2, v4 ) );

				this.faces.push( new Engine.Face3( v2, v3, v4 ) );

			}

		}

	}

	this.computeVertexNormals();

	this.ready = true;
	
};

Engine.SphereGeometry.prototype = Object.create( Engine.Geometry.prototype );
Engine.TubeGeometry = function( path, segments, radius, radialSegments, closed ) {

	Engine.Geometry.call( this );

	this.path = path;
	this.segments = segments || 64;
	this.radius = radius || 1;
	this.radialSegments = radialSegments || 8;
	this.closed = closed || false;

	this.grid = [];

	var scope = this,

		tangent,
		normal,
		binormal,

		numpoints = this.segments + 1,

		x, y, z,
		tx, ty, tz,
		u, v,

		cx, cy,
		pos, pos2 = new Engine.Vector3(),
		i, j,
		ip, jp,
		a, b, c, d,
		uva, uvb, uvc, uvd;

	var frames = new Engine.TubeGeometry.FrenetFrames( this.path, this.segments, this.closed ),
		tangents = frames.tangents,
		normals = frames.normals,
		binormals = frames.binormals;

	// proxy internals
	this.tangents = tangents;
	this.normals = normals;
	this.binormals = binormals;

	function vert( x, y, z ) {

		return scope.vertices.push( new Engine.Vector3( x, y, z ) ) - 1;

	}


	// consruct the grid

	for ( i = 0; i < numpoints; i++ ) {

		this.grid[ i ] = [];

		u = i / ( numpoints - 1 );

		pos = path.getPointAt( u );

		tangent = tangents[ i ];
		normal = normals[ i ];
		binormal = binormals[ i ];

		for ( j = 0; j < this.radialSegments; j++ ) {

			v = j / this.radialSegments * 2 * Math.PI;

			cx = -this.radius * Math.cos( v );
			cy = this.radius * Math.sin( v );

			pos2.copy( pos );
			pos2.x += cx * normal.x + cy * binormal.x;
			pos2.y += cx * normal.y + cy * binormal.y;
			pos2.z += cx * normal.z + cy * binormal.z;

			this.grid[ i ][ j ] = vert( pos2.x, pos2.y, pos2.z );

		}
	}


	// construct the mesh

	for ( i = 0; i < this.segments; i++ ) {

		for ( j = 0; j < this.radialSegments; j++ ) {

			ip = ( this.closed ) ? (i + 1) % this.segments : i + 1;
			jp = (j + 1) % this.radialSegments;

			a = this.grid[ i ][ j ];		// *** NOT NECESSARILY PLANAR ! ***
			b = this.grid[ ip ][ j ];
			c = this.grid[ ip ][ jp ];
			d = this.grid[ i ][ jp ];

			uva = new Engine.Vector2( i / this.segments, j / this.radialSegments );
			uvb = new Engine.Vector2( ( i + 1 ) / this.segments, j / this.radialSegments );
			uvc = new Engine.Vector2( ( i + 1 ) / this.segments, ( j + 1 ) / this.radialSegments );
			uvd = new Engine.Vector2( i / this.segments, ( j + 1 ) / this.radialSegments );

			this.faces.push( new Engine.Face3( a, b, d ) );
			this.vUvs = this.vUvs.concat( [ uva, uvb, uvd ] );

			this.faces.push( new Engine.Face3( b, c, d ) );
			this.vUvs = this.vUvs.concat( [ uvb.clone(), uvc, uvd.clone() ] );

		}
	}

	this.computeVertexNormals();

	this.ready = true;

};

Engine.TubeGeometry.prototype = Object.create( Engine.Geometry.prototype );



Engine.ObjGeometry = function ( url, params ) {

		Engine.Geometry.call( this );

		this.opti = {
			normals : true,
			uvs : true,
			vertices : true
		}

		if(params){
			if(params.normals!=undefined)this.opti.normals=params.normals;
			if(params.uvs!=undefined)this.opti.uvs=params.uvs;
			if(params.vertices!=undefined)this.opti.vertices=params.vertices;
		}

		this.onLoad = function(){};

		var that = this;

		var loader = new Engine.XHRLoader( );
		
		loader.load( url, function ( text ) {
			 that.parse( text );
			 that.onLoad();

			 if(that.callbackReady&&that.callbackReady.length){

			 	for(var k=0, kl=that.callbackReady.length; k<kl; k++){
			 		that.callbackReady[k]();
			 	}

			 	that.callbackReady = null;
			 }else{
			 	that.ready = true;
			 }

		} );

};

Engine.ObjGeometry.prototype = Object.create( Engine.Geometry.prototype );

Engine.ObjGeometry.prototype.parse = function ( text ) {

		var that = this;

		function vector( x, y, z ) {

			return new Engine.Vector3( parseFloat( x ), parseFloat( y ), parseFloat( z ) );

		}

		function uv( u, v ) {

			return new Engine.Vector2( parseFloat( u ), parseFloat( v ) );

		}

		var vertices = [];
		var normals = [];
		var uvs = [];
		var faceIndexVertices = [];
		var faceIndexNormals = [];
		var faceIndexUvs = [];
		var normalGenerated = false;

		// v float float float

		var vertex_pattern = /v( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)/;

		// vn float float float

		var normal_pattern = /vn( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)/;

		// vt float float

		var uv_pattern = /vt( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)/;

		// f vertex vertex vertex ...

		var face_pattern1 = /f( +-?\d+)( +-?\d+)( +-?\d+)( +-?\d+)?/;

		// f vertex/uv vertex/uv vertex/uv ...

		var face_pattern2 = /f( +(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+))?/;

		// f vertex/uv/normal vertex/uv/normal vertex/uv/normal ...

		var face_pattern3 = /f( +(-?\d+)\/(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+)\/(-?\d+))?/;

		// f vertex//normal vertex//normal vertex//normal ... 

		var face_pattern4 = /f( +(-?\d+)\/\/(-?\d+))( +(-?\d+)\/\/(-?\d+))( +(-?\d+)\/\/(-?\d+))( +(-?\d+)\/\/(-?\d+))?/

		//

		var lines = text.split( '\n' );

		for ( var i = 0; i < lines.length; i ++ ) {

			var line = lines[ i ];
			line = line.trim();

			var result;

			if ( line.length === 0 || line.charAt( 0 ) === '#' ) {

				continue;

			} else if ( ( result = vertex_pattern.exec( line ) ) !== null ) {

				vertices.push( 
						vector(
							result[ 1 ], result[ 2 ], result[ 3 ]
						)
				);

			} else if ( ( result = normal_pattern.exec( line ) ) !== null ) {

				normals.push(
					vector(
						result[ 1 ], result[ 2 ], result[ 3 ]
					)
				);

			} else if ( ( result = uv_pattern.exec( line ) ) !== null ) {

				uvs.push(
					uv(
						result[ 1 ], result[ 2 ]
					)
				);

			} else if ( ( result = face_pattern1.exec( line ) ) !== null ) {

				faceIndexVertices.push(  parseInt(result[ 1 ])-1, parseInt(result[ 2 ])-1, parseInt(result[ 3 ])-1  );

			} else if ( ( result = face_pattern2.exec( line ) ) !== null ) {

				faceIndexVertices.push(  parseInt(result[ 2 ])-1, parseInt(result[ 5 ])-1, parseInt(result[ 8 ])-1  );//faces
				faceIndexUvs.push(  parseInt(result[ 3 ])-1, parseInt(result[ 6 ])-1, parseInt(result[ 9 ])-1  );//uv

			} else if ( ( result = face_pattern3.exec( line ) ) !== null ) {
				faceIndexVertices.push(  parseInt(result[ 2 ])-1, parseInt(result[ 6 ])-1, parseInt(result[ 10 ])-1  );//faces
				faceIndexUvs.push(  parseInt(result[ 3 ])-1, parseInt(result[ 7 ])-1, parseInt(result[ 11 ])-1  );//uv
				faceIndexNormals.push(  parseInt(result[ 4 ])-1, parseInt(result[ 8 ])-1, parseInt(result[ 12 ])-1 );//normal

			} else if ( ( result = face_pattern4.exec( line ) ) !== null ) {

				faceIndexVertices.push( parseInt( result[ 2 ])-1, parseInt(result[ 5 ])-1, parseInt(result[ 8 ])-1 );//faces
				faceIndexNormals.push( parseInt( result[ 3 ])-1, parseInt(result[ 6 ])-1, parseInt(result[ 9 ])-1 );//normal

			}

		}

		if(this.opti.normals && !normals.length){

			normals = this.computeVertexNormalsForObj(faceIndexVertices,vertices);
			faceIndexNormals = faceIndexVertices;

		}


		var index = 0;
		var face = [];


		if(faceIndexUvs.length&&faceIndexNormals.length){

			if(that.opti.normals&&that.opti.uvs){

				for ( i = 0, il = faceIndexVertices.length; i < il; i ++ ) {

					if(!face[i]){

							face[i] = index;

						for ( k = i+1, kl = faceIndexVertices.length; k < kl; k ++ ) {

							if(faceIndexVertices[i]==faceIndexVertices[k] && faceIndexUvs[i]==faceIndexUvs[k] && faceIndexNormals[i]==faceIndexNormals[k]){

								face[k] = index;
							}
						}

						that.vVertices.push(  vertices[ faceIndexVertices[i] ] );
						that.vUvs.push(  uvs[ faceIndexUvs[i]  ] );
						that.vNormals.push( normals[ faceIndexNormals[i] ] );
						index++;

					}
				}

			}else if(that.opti.normals){

				for ( i = 0, il = faceIndexVertices.length; i < il; i ++ ) {

					if(!face[i]){

							face[i] = index;

						for ( k = i+1, kl = faceIndexVertices.length; k < kl; k ++ ) {

							if(faceIndexVertices[i]==faceIndexVertices[k] && faceIndexNormals[i]==faceIndexNormals[k]){

								face[k] = index;
							}
						}

						that.vVertices.push(  vertices[ faceIndexVertices[i] ] );
						that.vUvs.push(  uvs[ faceIndexUvs[i]  ] );
						that.vNormals.push( normals[ faceIndexNormals[i] ] );
						index++;

					}
				}

			}else if(that.opti.uvs){
				for ( i = 0, il = faceIndexVertices.length; i < il; i ++ ) {

					if(!face[i]){

							face[i] = index;

						for ( k = i+1, kl = faceIndexVertices.length; k < kl; k ++ ) {

							if(faceIndexVertices[i]==faceIndexVertices[k] && faceIndexUvs[i]==faceIndexUvs[k] ){

								face[k] = index;
							}
						}

						that.vVertices.push(  vertices[ faceIndexVertices[i] ] );
						that.vUvs.push(  uvs[ faceIndexUvs[i]  ] );
						that.vNormals.push( normals[ faceIndexNormals[i] ] );
						index++;

					}
				}
			}else{

				for ( i = 0, il = faceIndexVertices.length; i < il; i ++ ) {

					if(!face[i]){

							face[i] = index;

						for ( k = i+1, kl = faceIndexVertices.length; k < kl; k ++ ) {

							if(faceIndexVertices[i]==faceIndexVertices[k]){

								face[k] = index;
							}
						}

						that.vVertices.push(  vertices[ faceIndexVertices[i] ] );
						that.vUvs.push(  uvs[ faceIndexUvs[i]  ] );
						that.vNormals.push( normals[ faceIndexNormals[i] ] );
						index++;

					}
				}
			}

		}else if(faceIndexNormals.length){

			if(that.opti.normals){

				for ( i = 0, il = faceIndexVertices.length; i < il; i ++ ) {

					if(!face[i]){

							face[i] = index;

						for ( k = i+1, kl = faceIndexVertices.length; k < kl; k ++ ) {

							if(faceIndexVertices[i]==faceIndexVertices[k] && faceIndexNormals[i]==faceIndexNormals[k]){

								face[k] = index;
							}
						}

						that.vVertices.push(  vertices[ faceIndexVertices[i] ] );
						that.vNormals.push( normals[ faceIndexNormals[i] ] );
						index++;

					}
				}

			}else{

				for ( i = 0, il = faceIndexVertices.length; i < il; i ++ ) {

					if(!face[i]){

							face[i] = index;

						for ( k = i+1, kl = faceIndexVertices.length; k < kl; k ++ ) {

							if(faceIndexVertices[i]==faceIndexVertices[k]){

								face[k] = index;
							}
						}

						that.vVertices.push(  vertices[ faceIndexVertices[i] ] );
						that.vNormals.push( normals[ faceIndexNormals[i] ] );
						index++;

					}
				}
			}

		}else if(faceIndexUvs.length){

			if(that.opti.uvs){

				for ( i = 0, il = faceIndexVertices.length; i < il; i ++ ) {

					if(!face[i]){

							face[i] = index;

						for ( k = i+1, kl = faceIndexVertices.length; k < kl; k ++ ) {

							if(faceIndexVertices[i]==faceIndexVertices[k] && faceIndexUvs[i]==faceIndexUvs[k] ){

								face[k] = index;
							}
						}

						that.vVertices.push(  vertices[ faceIndexVertices[i] ] );
						that.vUvs.push(  uvs[ faceIndexUvs[i]  ] );
						index++;

					}
				}

			}else{
				for ( i = 0, il = faceIndexVertices.length; i < il; i ++ ) {

					if(!face[i]){

							face[i] = index;

						for ( k = i+1, kl = faceIndexVertices.length; k < kl; k ++ ) {

							if(faceIndexVertices[i]==faceIndexVertices[k]){

								face[k] = index;
							}
						}

						that.vVertices.push(  vertices[ faceIndexVertices[i] ] );
						that.vUvs.push(  uvs[ faceIndexUvs[i]  ] );
						index++;

					}
				}

			}

		}else{

			for ( i = 0, il = faceIndexVertices.length; i < il; i ++ ) {

				if(!face[i]){

						face[i] = index;

					for ( k = i+1, kl = faceIndexVertices.length; k < kl; k ++ ) {

						if(faceIndexVertices[i]==faceIndexVertices[k]){

							face[k] = index;
						}
					}

					that.vVertices.push(  vertices[ faceIndexVertices[i] ] );
					index++;

				}
			}
		}



		for ( i = 0, il = face.length; i < il; i+=3 ){

			that.faces.push( new Engine.Face3( face[i], face[i+1], face[i+2] ) );

		}

	}






Engine.ObjMorphGeometry = function ( url, params ) {

		Engine.Geometry.call( this );

		this.opti = {
			normals : true,
			uvs : true,
			vertices : true
		}

		if(params){
			if(params.normals!=undefined)this.opti.normals=params.normals;
			if(params.uvs!=undefined)this.opti.uvs=params.uvs;
			if(params.vertices!=undefined)this.opti.vertices=params.vertices;
		}

		this.onLoad = function(){};

		var that = this;

		var loader = new Engine.XHRLoader( );

		loader.load( url, function ( text ) {
			 
			 that.parse( text );
			  that.onLoad();

			  if(that.callbackReady&&that.callbackReady.length){

			 	for(var k=0, kl=that.callbackReady.length; k<kl; k++){
			 		that.callbackReady[k]();
			 	}

			 	that.callbackReady = null;
			 }else{
			 	that.ready = true;
			 }


		},null,function(e){console.log(e);} );

};

Engine.ObjMorphGeometry.prototype = Object.create( Engine.Geometry.prototype );

Engine.ObjMorphGeometry.prototype.parse = function ( text ) {

		var that = this;

		function vector( x, y, z ) {

			return new Engine.Vector3( parseFloat( x ), parseFloat( y ), parseFloat( z ) );

		}

		function uv( u, v ) {

			return new Engine.Vector2( parseFloat( u ), parseFloat( v ) );

		}
		var morphTargets = []
		//var vVertices = [];
		//var vNormals = [];
		var uvs = [];
		var faceIndexVertices = [];
		var faceIndexNormals = [];
		var faceIndexUvs = [];
		var newTarget = false;
		var indexTarget = -1;

		this.morphTargets = [];

		// v float float float

		var vertex_pattern = /v( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)/;

		// vt float float

		var uv_pattern = /vt( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)/;

		// f vertex vertex vertex ...

		var face_pattern1 = /f( +-?\d+)( +-?\d+)( +-?\d+)( +-?\d+)?/;

		// f vertex/uv vertex/uv vertex/uv ...

		var face_pattern2 = /f( +(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+))?/;

		// f vertex/uv/normal vertex/uv/normal vertex/uv/normal ...

		var face_pattern3 = /f( +(-?\d+)\/(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+)\/(-?\d+))?/;

		// f vertex//normal vertex//normal vertex//normal ... 

		var face_pattern4 = /f( +(-?\d+)\/\/(-?\d+))( +(-?\d+)\/\/(-?\d+))( +(-?\d+)\/\/(-?\d+))( +(-?\d+)\/\/(-?\d+))?/

		//

		var lines = text.split( '\n' );

		for ( var i = 0; i < lines.length; i ++ ) {

			var line = lines[ i ];
			line = line.trim();

			var result;

			if ( line.length === 0 || line.charAt( 0 ) === '#' ) {

				newTarget = true;

				continue;

			} else if ( ( result = vertex_pattern.exec( line ) ) !== null ) {
				//var vVertices = [];
				//var vNormals = [];

				if(newTarget) {
					newTarget = false;
					morphTargets.push({vVertices:[],vNormals:[]});
					this.morphTargets.push({vVertices:[],vNormals:[]});
					indexTarget++;
				}
				morphTargets[indexTarget].vVertices.push( 
						vector(
							result[ 1 ], result[ 2 ], result[ 3 ]
						)
				); 

			}  else if ( ( result = uv_pattern.exec( line ) ) !== null ) {

				uvs.push(
					uv(
						result[ 1 ], result[ 2 ]
					)
				);

			} else if ( ( result = face_pattern1.exec( line ) ) !== null ) {

				faceIndexVertices.push(  parseInt(result[ 1 ])-1, parseInt(result[ 2 ])-1, parseInt(result[ 3 ])-1  );

			} else if ( ( result = face_pattern2.exec( line ) ) !== null ) {

				faceIndexVertices.push(  parseInt(result[ 2 ])-1, parseInt(result[ 5 ])-1, parseInt(result[ 8 ])-1  );//faces
				faceIndexUvs.push(  parseInt(result[ 3 ])-1, parseInt(result[ 6 ])-1, parseInt(result[ 9 ])-1  );//uv

			}

		}

		if(this.opti.normals){

			for(var k=0,kl=morphTargets.length;k<kl;k++){

				morphTargets[k].vNormals = this.computeVertexNormalsForObj(faceIndexVertices,morphTargets[k].vVertices);

			}

		}

		var index = 0;
		var face = [];


		if(faceIndexUvs.length && that.opti.uvs && this.opti.normals){

				for ( var i = 0, il = faceIndexVertices.length; i < il; i ++ ) {

					if(!face[i]){

							face[i] = index;

						for ( var k = i+1, kl = faceIndexVertices.length; k < kl; k ++ ) {

							if(faceIndexVertices[i]==faceIndexVertices[k] && faceIndexUvs[i]==faceIndexUvs[k] ){

								face[k] = index;
							}
						}
						
						for ( var k = 0, kl = this.morphTargets.length; k < kl; k ++ ){
							
							this.morphTargets[k].vVertices.push(  morphTargets[k].vVertices[ faceIndexVertices[i] ] );
							this.morphTargets[k].vNormals.push(  morphTargets[k].vNormals[ faceIndexVertices[i] ] );
						}

						this.vUvs.push(  uvs[ faceIndexUvs[i] ] );

						index++;

					}
				}

		} else if ( this.opti.normals) {

			for ( var i = 0, il = faceIndexVertices.length; i < il; i ++ ) {

				if(!face[i]) {

						face[i] = index;

					for ( var k = i+1, kl = faceIndexVertices.length; k < kl; k ++ ) {

						if(faceIndexVertices[i]==faceIndexVertices[k]){

							face[k] = index;
						}
					}

					for ( var k = 0, kl = this.morphTargets.length; k < kl; k ++ ){
						this.morphTargets[k].vVertices.push(  morphTargets[k].vVertices[ faceIndexVertices[i] ] );
						this.morphTargets[k].vNormals.push(  morphTargets[k].vNormals[ faceIndexVertices[i] ] );
					}


					index++;

				}
			}
		}




		else if(faceIndexUvs.length && that.opti.uvs && !this.opti.normals){

				for ( var i = 0, il = faceIndexVertices.length; i < il; i ++ ) {

					if(!face[i]){

							face[i] = index;

						for ( var k = i+1, kl = faceIndexVertices.length; k < kl; k ++ ) {

							if(faceIndexVertices[i]==faceIndexVertices[k] && faceIndexUvs[i]==faceIndexUvs[k] ){

								face[k] = index;
							}
						}
						
						for ( var k = 0, kl = this.morphTargets.length; k < kl; k ++ ){
							
							this.morphTargets[k].vVertices.push(  morphTargets[k].vVertices[ faceIndexVertices[i] ] );
						}

						this.vUvs.push(  uvs[ faceIndexUvs[i] ] );

						index++;

					}
				}

		} else {

			for ( var i = 0, il = faceIndexVertices.length; i < il; i ++ ) {

				if(!face[i]) {

						face[i] = index;

					for ( var k = i+1, kl = faceIndexVertices.length; k < kl; k ++ ) {

						if(faceIndexVertices[i]==faceIndexVertices[k]){

							face[k] = index;
						}
					}

					for ( var k = 0, kl = this.morphTargets.length; k < kl; k ++ ){
						this.morphTargets[k].vVertices.push(  morphTargets[k].vVertices[ faceIndexVertices[i] ] );
					}


					index++;

				}
			}
		}


		for ( var i = 0, il = face.length; i < il; i+=3 ){

			that.faces.push( new Engine.Face3( face[i], face[i+1], face[i+2] ) );

		}

	}





Engine.Material = function () {

	this.opacity = 1; 

	this.needsUpdate = true;
};

Engine.Material.prototype = {

	constructor: Engine.Material,

	setValues: function ( values ) {

		if ( values === undefined ) return;

		for ( var key in values ) {

			var newValue = values[ key ];

			if ( newValue === undefined ) {

				console.warn( 'Engine.Material: \'' + key + '\' parameter is undefined.' );
				continue;

			}

			if ( key in this ) {

				var currentValue = this[ key ];

				if ( currentValue instanceof Engine.Color ) {

					currentValue.set( newValue );

				} else if ( currentValue instanceof Engine.Vector3 && newValue instanceof Engine.Vector3 ) {

					currentValue.copy( newValue );

				} else {

					this[ key ] = newValue;

				}

			}

		}

	},

	clone: function ( material ) {

		if ( material === undefined ) material = new Engine.Material();

		material.opacity = this.opacity;

		return material;

	},


};
Engine.LineBasicMaterial = function ( parameters ) {

	Engine.Material.call( this );

	this.color = new Engine.Color( 0xffffff );

	this.linewidth = 1;
	this.linecap = 'round';
	this.linejoin = 'round';

	this.fog = true;

	this.setValues( parameters );

};

Engine.LineBasicMaterial.prototype = Object.create( Engine.Material.prototype );
Engine.MeshBasicMaterial = function ( parameters ) {

	Engine.Material.call( this );

	this.color = new Engine.Color( 0xffffff );

	this.emissive = new Engine.Color( 0x000000 );

	this.shininess = 0;

	this.lighting = true;

	this.map = null;

	this.fog = true;

	this.transparent = false;

	this.setValues( parameters );

};

Engine.MeshBasicMaterial.prototype = Object.create( Engine.Material.prototype );
Engine.MeshPhongMaterial = function ( parameters ) {

	Engine.Material.call( this );

	this.color = new Engine.Color( 0xffffff );
	this.emissive = new Engine.Color( 0x000000 );
	this.specular = new Engine.Color( 0x111111 );
	this.shininess = 0;

	this.map = null;

	this.lightMap = null;

	this.normalMap = null;

	this.specularMap = null;

	this.cubeMap = null;
	this.reflectivity = 1;
	this.refractionRatio = 0.98;

	this.fog = true;

	this.lighting = true;

	this.morphTargets = false;
	this.morphNormals = false;

	this.transparent = false;

	this.setValues( parameters );

};

Engine.MeshPhongMaterial.prototype = Object.create( Engine.Material.prototype );
Engine.ParticleSystemMaterial = function ( parameters ) {

	Engine.Material.call( this );

	this.color = new Engine.Color( 0xffffff );

	this.map = null;

	this.size = 1;
	this.sizeAttenuation = true;

	this.transparent = true;

	this.fog = true;

	this.setValues( parameters );

};

Engine.ParticleSystemMaterial.prototype = Object.create( Engine.Material.prototype );


Engine.Texture = function (url /*wrapS, wrapT, magFilter, minFilter, format, anisotropy*/ ) {

	if(url instanceof Engine.Color){

		var can = document.createElement('canvas');
		can.width=8;
		can.height=8; 
		var ctx = can.getContext('2d');
		ctx.fillStyle =  'rgb('+(url.r*255)+','+(url.g*255)+','+(url.b*255)+')';
		ctx.fillRect(0,0 ,8,8 );
		this.image = can;
		this.needsUpdate = true;

	}else{

		this.image = document.createElement( 'img' );

		var that = this;

		this.image.onload = function(){

			that.needsUpdate = true;

			Engine.pendingRequest -=1;
		}

		this.image.onerror = function(){

		}

		Engine.pendingRequest +=1;

		this.image.src = url;

		this.needsUpdate = false;

	}

};
Engine.TextureCube = function (url) {

		var that = this;

		url = url.split(".");

		var axes = ["_X0","_X1","_Y1","_Y0","_Z0","_Z1"];
		var images = [];
		images.loadCount = 0;
		this.images = images;

		for ( var i = 0, il = 6; i < il; ++ i ) {

			var cubeImage = document.createElement( 'img' );
			images[ i ] = cubeImage;

			cubeImage.onload = function () {

				images.loadCount += 1;

				Engine.pendingRequest -=1;

				if ( images.loadCount === 6 ) {

					that.needsUpdate = true;
				}

			};

			cubeImage.onerror = function(){};

			Engine.pendingRequest +=1;

			cubeImage.src = url[0]+axes[ i ]+"."+url[1];

		}

		this.needsUpdate = false;

};


Engine.Object3D = function () {

	this.parent = undefined;
	this.children = [];

	this.position = new Engine.Vector3();
	this._rotation = new Engine.Euler();
	this._quaternion = new Engine.Quaternion();
	this.scale = new Engine.Vector3( 1, 1, 1 );

	// keep rotation and quaternion in sync

	this._rotation._quaternion = this.quaternion;
	this._quaternion._euler = this.rotation;

	this.matrix = new Engine.Matrix4();
	this.matrixWorld = new Engine.Matrix4();

	this.matrixAutoUpdate = true;
	this.matrixWorldNeedsUpdate = true;

};


Engine.Object3D.prototype = {

	constructor: Engine.Object3D,
	
	get rotation () { 
		return this._rotation; 
	},

	set rotation ( value ) {
		
		this._rotation = value;
		this._rotation._quaternion = this._quaternion;
		this._quaternion._euler = this._rotation;
		this._rotation._updateQuaternion();

	},

	get quaternion () { 
		return this._quaternion; 
	},
	
	set quaternion ( value ) {
		
		this._quaternion = value;
		this._quaternion._euler = this._rotation;
		this._rotation._quaternion = this._quaternion;
		this._quaternion._updateEuler();

	},

	applyMatrix: function ( matrix ) {

		this.matrix.multiplyMatrices( matrix, this.matrix );

		this.matrix.decompose( this.position, this.quaternion, this.scale );

	},


	getPlanView:  function () {

		var direction = this.getDirection();

		var d = -(direction.x * this.position.x + direction.y * this.position.y + direction.z * this.position.z);
		return {
			a: direction.x,
			b: direction.y,
			c: direction.z,
			d: d
		};

	},

	setRotationFromAxisAngle: function ( axis, angle ) {

		// assumes axis is normalized

		this.quaternion.setFromAxisAngle( axis, angle );

	},

	setRotationFromEuler: function ( euler ) {

		this.quaternion.setFromEuler( euler, true );

	},

	setRotationFromMatrix: function ( m ) {

		// assumes the upper 3x3 of m is a pure rotation matrix (i.e, unscaled)

		this.quaternion.setFromRotationMatrix( m );

	},

	setRotationFromQuaternion: function ( q ) {

		// assumes q is normalized

		this.quaternion.copy( q );

	},

	rotateOnAxis: function() {

		// rotate object on axis in object space
		// axis is assumed to be normalized

		var q1 = new Engine.Quaternion();

		return function ( axis, angle ) {

			q1.setFromAxisAngle( axis, angle );

			this.quaternion.multiply( q1 );

			return this;

		}

	}(),

	rotateX: function () {

		var v1 = new Engine.Vector3( 1, 0, 0 );

		return function ( angle ) {

			return this.rotateOnAxis( v1, angle );

		};

	}(),

	rotateY: function () {

		var v1 = new Engine.Vector3( 0, 1, 0 );

		return function ( angle ) {

			return this.rotateOnAxis( v1, angle );

		};

	}(),

	rotateZ: function () {

		var v1 = new Engine.Vector3( 0, 0, 1 );

		return function ( angle ) {

			return this.rotateOnAxis( v1, angle );

		};

	}(),

	translateOnAxis: function () {

		// translate object by distance along axis in object space
		// axis is assumed to be normalized

		var v1 = new Engine.Vector3();

		return function ( axis, distance ) {

			v1.copy( axis );

			v1.applyQuaternion( this.quaternion );

			this.position.add( v1.multiplyScalar( distance ) );

			return this;

		}

	}(),


	translateX: function () {

		var v1 = new Engine.Vector3( 1, 0, 0 );

		return function ( distance ) {

			return this.translateOnAxis( v1, distance );

		};

	}(),

	translateY: function () {

		var v1 = new Engine.Vector3( 0, 1, 0 );

		return function ( distance ) {

			return this.translateOnAxis( v1, distance );

		};

	}(),

	translateZ: function () {

		var v1 = new Engine.Vector3( 0, 0, 1 );

		return function ( distance ) {

			return this.translateOnAxis( v1, distance );

		};

	}(),

	localToWorld: function ( vector ) {

		return vector.applyMatrix4( this.matrixWorld );

	},

	worldToLocal: function () {

		var m1 = new Engine.Matrix4();

		return function ( vector ) {

			return vector.applyMatrix4( m1.getInverse( this.matrixWorld ) );

		};

	}(),

	lookAt: function () {

		// This routine does not support objects with rotated and/or translated parent(s)

		var m1 = new Engine.Matrix4();
		var up = new Engine.Vector3( 0, 1, 0 );

		return function ( vector ) {

			m1.lookAt( vector, this.position, up );

			this.quaternion.setFromRotationMatrix( m1 );

		};

	}(),


	getDirection: function () {

		return new Engine.Vector3(
			this.matrixWorld.elements[8],
			this.matrixWorld.elements[9],
			this.matrixWorld.elements[10] 
		);

	},

	add: function ( object ) {

		if ( object === this ) {

			console.warn( 'Engine.Object3D.add: An object can\'t be added as a child of itself.' );
			return;

		}

		if ( object instanceof Engine.Object3D ) {

			if ( object.parent !== undefined ) {

				object.parent.remove( object );

			}

			//if(!(this instanceof Engine.Scene)){
				object.parent = this;
			//}
			this.children.push( object );


			
			// add to scene

			var scene = this;

			while ( scene.parent !== undefined ) {

				scene = scene.parent;

			}

			if ( scene !== undefined && scene instanceof Engine.Scene )  {

				scene.__addObject( object );

			}

		}

	},

	remove: function ( object ) {

		var index = this.children.indexOf( object );

		if ( index !== - 1 ) {

			object.parent = undefined;

			this.children.splice( index, 1 );

			// remove from scene

			var scene = this;

			while ( scene.parent !== undefined ) {

				scene = scene.parent;

			}

			if ( scene !== undefined && scene instanceof Engine.Scene ) {

				scene.__removeObject( object );

			}

		}

	},



	updateMatrix: function () {

		this.matrix.compose( this.position, this.quaternion, this.scale );

		this.matrixWorldNeedsUpdate = true;

	},

	updateMatrixWorld: function ( force ) {

		if ( this.matrixAutoUpdate === true ) this.updateMatrix();

		if ( this.matrixWorldNeedsUpdate === true || force === true ) {

			if ( this.parent === undefined ) {

				this.matrixWorld.copy( this.matrix );

			} else {

				this.matrixWorld.multiplyMatrices( this.parent.matrixWorld, this.matrix );

			}

			this.matrixWorldNeedsUpdate = false;

			force = true;

		}

		// update children

		for ( var i = 0, l = this.children.length; i < l; i ++ ) {

			this.children[ i ].updateMatrixWorld( force );

		}

	},

	clone: function ( object, recursive ) {

		if ( object === undefined ) object = new Engine.Object3D();
		if ( recursive === undefined ) recursive = true;

		object.position.copy( this.position );
		object.quaternion.copy( this.quaternion );
		object.scale.copy( this.scale );

		object.renderDepth = this.renderDepth;

		object.matrix.copy( this.matrix );
		object.matrixWorld.copy( this.matrixWorld );

		object.matrixAutoUpdate = this.matrixAutoUpdate;
		object.matrixWorldNeedsUpdate = this.matrixWorldNeedsUpdate;

		object.visible = this.visible;
		object.castShadow = this.castShadow;
		object.receiveShadow = this.receiveShadow;

		if ( recursive === true ) {

			for ( var i = 0; i < this.children.length; i ++ ) {

				var child = this.children[ i ];
				object.add( child.clone() );

			}

		}

		return object;

	}

};





Engine.Euler = function ( x, y, z, order ) {

	this._x = x || 0;
	this._y = y || 0;
	this._z = z || 0;
	this._order = order || Engine.Euler.DefaultOrder;

};

Engine.Euler.RotationOrders = [ 'XYZ', 'YZX', 'ZXY', 'XZY', 'YXZ', 'ZYX' ];

Engine.Euler.DefaultOrder = 'XYZ';

Engine.Euler.prototype = {

	constructor: Engine.Euler,

	_x: 0, _y: 0, _z: 0, _order: Engine.Euler.DefaultOrder,

	_quaternion: undefined,

	_updateQuaternion: function () {

		if ( this._quaternion !== undefined ) {

			this._quaternion.setFromEuler( this, false );

		}

	},

	get x () {

		return this._x;

	},

	set x ( value ) {

		this._x = value;
		this._updateQuaternion();

	},

	get y () {

		return this._y;

	},

	set y ( value ) {

		this._y = value;
		this._updateQuaternion();

	},

	get z () {

		return this._z;

	},

	set z ( value ) {

		this._z = value;
		this._updateQuaternion();

	},

	get order () {

		return this._order;

	},

	set order ( value ) {

		this._order = value;
		this._updateQuaternion();

	},

	set: function ( x, y, z, order ) {

		this._x = x;
		this._y = y;
		this._z = z;
		this._order = order || this._order;

		this._updateQuaternion();

		return this;

	},

	copy: function ( euler ) {

		this._x = euler._x;
		this._y = euler._y;
		this._z = euler._z;
		this._order = euler._order;

		this._updateQuaternion();

		return this;

	},

	setFromRotationMatrix: function ( m, order ) {

		// assumes the upper 3x3 of m is a pure rotation matrix (i.e, unscaled)

		// clamp, to handle numerical problems

		function clamp( x ) {

			return Math.min( Math.max( x, -1 ), 1 );

		}

		var te = m.elements;
		var m11 = te[0], m12 = te[4], m13 = te[8];
		var m21 = te[1], m22 = te[5], m23 = te[9];
		var m31 = te[2], m32 = te[6], m33 = te[10];

		order = order || this._order;

		if ( order === 'XYZ' ) {

			this._y = Math.asin( clamp( m13 ) );

			if ( Math.abs( m13 ) < 0.99999 ) {

				this._x = Math.atan2( - m23, m33 );
				this._z = Math.atan2( - m12, m11 );

			} else {

				this._x = Math.atan2( m32, m22 );
				this._z = 0;

			}

		} else if ( order === 'YXZ' ) {

			this._x = Math.asin( - clamp( m23 ) );

			if ( Math.abs( m23 ) < 0.99999 ) {

				this._y = Math.atan2( m13, m33 );
				this._z = Math.atan2( m21, m22 );

			} else {

				this._y = Math.atan2( - m31, m11 );
				this._z = 0;

			}

		} else if ( order === 'ZXY' ) {

			this._x = Math.asin( clamp( m32 ) );

			if ( Math.abs( m32 ) < 0.99999 ) {

				this._y = Math.atan2( - m31, m33 );
				this._z = Math.atan2( - m12, m22 );

			} else {

				this._y = 0;
				this._z = Math.atan2( m21, m11 );

			}

		} else if ( order === 'ZYX' ) {

			this._y = Math.asin( - clamp( m31 ) );

			if ( Math.abs( m31 ) < 0.99999 ) {

				this._x = Math.atan2( m32, m33 );
				this._z = Math.atan2( m21, m11 );

			} else {

				this._x = 0;
				this._z = Math.atan2( - m12, m22 );

			}

		} else if ( order === 'YZX' ) {

			this._z = Math.asin( clamp( m21 ) );

			if ( Math.abs( m21 ) < 0.99999 ) {

				this._x = Math.atan2( - m23, m22 );
				this._y = Math.atan2( - m31, m11 );

			} else {

				this._x = 0;
				this._y = Math.atan2( m13, m33 );

			}

		} else if ( order === 'XZY' ) {

			this._z = Math.asin( - clamp( m12 ) );

			if ( Math.abs( m12 ) < 0.99999 ) {

				this._x = Math.atan2( m32, m22 );
				this._y = Math.atan2( m13, m11 );

			} else {

				this._x = Math.atan2( - m23, m33 );
				this._y = 0;

			}

		} else {

			console.warn( 'WARNING: Euler.setFromRotationMatrix() given unsupported order: ' + order )

		}

		this._order = order;

		this._updateQuaternion();

		return this;

	},

	setFromQuaternion: function ( q, order, update ) {

		// q is assumed to be normalized

		// clamp, to handle numerical problems

		function clamp( x ) {

			return Math.min( Math.max( x, -1 ), 1 );

		}

		// http://www.mathworks.com/matlabcentral/fileexchange/20696-function-to-convert-between-dcm-euler-angles-quaternions-and-euler-vectors/content/SpinCalc.m

		var sqx = q.x * q.x;
		var sqy = q.y * q.y;
		var sqz = q.z * q.z;
		var sqw = q.w * q.w;

		order = order || this._order;

		if ( order === 'XYZ' ) {

			this._x = Math.atan2( 2 * ( q.x * q.w - q.y * q.z ), ( sqw - sqx - sqy + sqz ) );
			this._y = Math.asin(  clamp( 2 * ( q.x * q.z + q.y * q.w ) ) );
			this._z = Math.atan2( 2 * ( q.z * q.w - q.x * q.y ), ( sqw + sqx - sqy - sqz ) );

		} else if ( order ===  'YXZ' ) {

			this._x = Math.asin(  clamp( 2 * ( q.x * q.w - q.y * q.z ) ) );
			this._y = Math.atan2( 2 * ( q.x * q.z + q.y * q.w ), ( sqw - sqx - sqy + sqz ) );
			this._z = Math.atan2( 2 * ( q.x * q.y + q.z * q.w ), ( sqw - sqx + sqy - sqz ) );

		} else if ( order === 'ZXY' ) {

			this._x = Math.asin(  clamp( 2 * ( q.x * q.w + q.y * q.z ) ) );
			this._y = Math.atan2( 2 * ( q.y * q.w - q.z * q.x ), ( sqw - sqx - sqy + sqz ) );
			this._z = Math.atan2( 2 * ( q.z * q.w - q.x * q.y ), ( sqw - sqx + sqy - sqz ) );

		} else if ( order === 'ZYX' ) {

			this._x = Math.atan2( 2 * ( q.x * q.w + q.z * q.y ), ( sqw - sqx - sqy + sqz ) );
			this._y = Math.asin(  clamp( 2 * ( q.y * q.w - q.x * q.z ) ) );
			this._z = Math.atan2( 2 * ( q.x * q.y + q.z * q.w ), ( sqw + sqx - sqy - sqz ) );

		} else if ( order === 'YZX' ) {

			this._x = Math.atan2( 2 * ( q.x * q.w - q.z * q.y ), ( sqw - sqx + sqy - sqz ) );
			this._y = Math.atan2( 2 * ( q.y * q.w - q.x * q.z ), ( sqw + sqx - sqy - sqz ) );
			this._z = Math.asin(  clamp( 2 * ( q.x * q.y + q.z * q.w ) ) );

		} else if ( order === 'XZY' ) {

			this._x = Math.atan2( 2 * ( q.x * q.w + q.y * q.z ), ( sqw - sqx + sqy - sqz ) );
			this._y = Math.atan2( 2 * ( q.x * q.z + q.y * q.w ), ( sqw + sqx - sqy - sqz ) );
			this._z = Math.asin(  clamp( 2 * ( q.z * q.w - q.x * q.y ) ) );

		} else {

			console.warn( 'WARNING: Euler.setFromQuaternion() given unsupported order: ' + order )

		}

		this._order = order;

		if ( update !== false ) this._updateQuaternion();

		return this;

	},

	reorder: function () {

		// WARNING: this discards revolution information -bhouston

		var q = new Engine.Quaternion();

		return function ( newOrder ) {

			q.setFromEuler( this );
			this.setFromQuaternion( q, newOrder );

		};


	}(),

	fromArray: function ( array ) {

		this._x = array[ 0 ];
		this._y = array[ 1 ];
		this._z = array[ 2 ];
		if ( array[ 3 ] !== undefined ) this._order = array[ 3 ];

		this._updateQuaternion();

		return this;

	},

	toArray: function () {

		return [ this._x, this._y, this._z, this._order ];

	},

	equals: function ( euler ) {

		return ( euler._x === this._x ) && ( euler._y === this._y ) && ( euler._z === this._z ) && ( euler._order === this._order );

	},

	clone: function () {

		return new Engine.Euler( this._x, this._y, this._z, this._order );

	}

};




Engine.Mesh = function ( geometry, material, castShadow, receiveShadow, depthOfField ) {

	Engine.Object3D.call( this );

	this.geometry = geometry;
	this.material = material;
	this.visible = true;
	this.castShadow = castShadow||false;
	this.receiveShadow = receiveShadow||false;
	this.depthOfField = depthOfField||false;
	this.clones = [];
	if(!this.geometry.attributes)
	this.geometry.attributes = {};

	var that = this;

		if(geometry.ready){

			this.initMeshBuffers(material);

		}else if(!geometry.callbackReady){

			geometry.callbackReady = [];
			geometry.callbackReady[0] = function(){

				that.initMeshBuffers(material);
			}
		}else {
			geometry.callbackReady.push(function(){

				that.initMeshBuffers(material);
			})
		}

};

Engine.Mesh.prototype = Object.create( Engine.Object3D.prototype );


Engine.Mesh.prototype.clone = function ( object ) {

	if ( object === undefined ) object = new Engine.Mesh( this.geometry, this.material );

	Engine.Object3D.prototype.clone.call( this, object );

	this.visible = object.visible;
	this.castShadow = object.castShadow;
	this.receiveShadow = object.receiveShadow;

	return object;

};

Engine.Mesh.prototype.addClone = function(vx,vy,vz,ax,ay,az,sx,sy,sz) {
	var matrix = new Engine.Matrix4();
	var matrixRotation = new Engine.Matrix4();

	if(sx||sy||sz)
	matrix.scale({x:sx||1,y:sy||1,z:sz||1});
	
	matrix.setPosition({x:vx,y:vy,z:vz});
	if(ax)
	matrix.multiply(matrixRotation.makeRotationX(ax));
	if(ay)
	matrix.multiply(matrixRotation.makeRotationY(ay));
	if(az)
	matrix.multiply(matrixRotation.makeRotationZ(az));

	this.matrix = null;
	this.clones.push(matrix);


	return this;
}

Engine.Mesh.prototype.removeClone = function(index) {
	if(index){

		if(typeof index == "number" )
			this.clones.splice(index,1);
		else{

			for(var i=0,il=this.clones.length;i<il;i++){
				if(this.clones[i]==index){
					this.clones.splice(i,1);
					return;
				}

			}

		}

	}

	else
	this.clones.splice(this.clones.length-1,1);

	return this;
}

Engine.Mesh.prototype.removeAllClones = function(index) {
	
	this.clones = [];

	return this;
}


Engine.Mesh.prototype.getClone = function(index) {
	if(index)
	return this.clones[index];
	else
	return this.clones[this.clones.length-1];

}
		
Engine.Mesh.prototype.initMeshBuffers =  function( material ) {

	var geometry =  this.geometry;

	var attributes = geometry.attributes;

	if(attributes.__webglIndexBuffer)return;
	var faces = geometry.faces;
	var vertices = geometry.vVertices;
	var vertexNormals = geometry.vNormals;
	var vertexUvs = geometry.vUvs;
	var morphTargets = geometry.morphTargets;

	var f,fl;
	var vv, vertexArray = new Float32Array(vertices.length*3), offset_vertex = 0;
	var vn, normalArray = new Float32Array(vertexNormals.length*3), offset_normal = 0;
	var uv, uvArray = new Float32Array(vertexUvs.length*2), offset_uv = 0;
	var vf, faceArray = new Uint16Array(faces.length*3),offset_face=0;
	var vk,offset_morphTarget;
	
	if(vertices.length&&!attributes.__webglVertexBuffer){
	
		for ( f = 0, fl = vertices.length; f < fl; f ++ ) {
	
					vv = vertices[ f ];
					vertexArray[ offset_vertex ]     = vv.x;
					vertexArray[ offset_vertex + 1 ] = vv.y;
					vertexArray[ offset_vertex + 2 ] = vv.z;

					offset_vertex += 3;
	
		}
		attributes.__webglVertexBuffer = gl.createBuffer();
		gl.bindBuffer( gl.ARRAY_BUFFER, attributes.__webglVertexBuffer );
		gl.bufferData( gl.ARRAY_BUFFER, vertexArray, gl.STATIC_DRAW );
		attributes.__webglVertexBuffer.numItems = vertexArray.length/3;
		attributes.__webglVertexBuffer.itemSize = 3;
	
	}

	if(vertexNormals.length&&!attributes.__webglNormalBuffer){

		for ( f = 0, fl = vertexNormals.length; f < fl; f ++ ) {

					vn = vertexNormals[ f ];

					normalArray[ offset_normal ]     = vn.x;
					normalArray[ offset_normal + 1 ] = vn.y;
					normalArray[ offset_normal + 2 ] = vn.z;
					offset_normal += 3;

		}

		attributes.__webglNormalBuffer = gl.createBuffer();
		gl.bindBuffer( gl.ARRAY_BUFFER, attributes.__webglNormalBuffer );
		gl.bufferData( gl.ARRAY_BUFFER, normalArray,  gl.STATIC_DRAW );
		attributes.__webglNormalBuffer.numItems = normalArray.length/3;
		attributes.__webglNormalBuffer.itemSize = 3;

	}
	
	if(vertexUvs.length && !attributes.__webglUVBuffer && material&&(material.map||material.lightMap||material.normalMap||material.cubeMap)) {

		for ( f = 0, fl = vertexUvs.length; f < fl; f ++ ) {

				uv = vertexUvs[ f ];

				uvArray[ offset_uv ]     = uv.x;
				uvArray[ offset_uv + 1 ] = uv.y;
				offset_uv += 2;

		}

		attributes.__webglUVBuffer = gl.createBuffer();
		gl.bindBuffer( gl.ARRAY_BUFFER, attributes.__webglUVBuffer );
		gl.bufferData( gl.ARRAY_BUFFER, uvArray, gl.STATIC_DRAW  );
		attributes.__webglUVBuffer.numItems = uvArray.length/2;
		attributes.__webglUVBuffer.itemSize = 2;
	
	}
	
	if(morphTargets.length) {

		if(!attributes.__webglMorphTargetsBuffers)
		attributes.__webglMorphTargetsBuffers = [];

		if(!attributes.__webglMorphNormalsBuffers)
		attributes.__webglMorphNormalsBuffers = [];

		if(!attributes.__webglMorphTangentsBuffer)
		attributes.__webglMorphTangentsBuffer = [];
		
		if(material.normalMap&&morphTargets[0].vNormals.length&&vertexUvs.length){

			geometry.computeMorphVertexTangents();

		}

		for ( vk = 0, vkl = morphTargets.length; vk < vkl; vk ++ ) {

			offset_morphTarget = 0;

			var verticesMorph =  morphTargets[vk].vVertices;

			vertexArray = new Float32Array(verticesMorph.length*3);
			offset_vertex = 0;

			for ( f = 0, fl = verticesMorph.length; f < fl; f ++ ) {

				vertexArray[ offset_vertex ]     = verticesMorph[ f ].x;
				vertexArray[ offset_vertex + 1 ] = verticesMorph[ f ].y;
				vertexArray[ offset_vertex + 2 ] = verticesMorph[ f ].z;

				offset_vertex += 3;

			}

			attributes.__webglMorphTargetsBuffers[ vk ] = gl.createBuffer();
			gl.bindBuffer( gl.ARRAY_BUFFER, attributes.__webglMorphTargetsBuffers[ vk ] );
			gl.bufferData( gl.ARRAY_BUFFER, vertexArray, gl.STATIC_DRAW );
			attributes.__webglMorphTargetsBuffers[ vk ].numItems = vertexArray.length/3;
			attributes.__webglMorphTargetsBuffers[ vk ].itemSize = 3;



			offset_morphTarget = 0;


			var normalsMorph =  morphTargets[vk].vNormals;

			if(normalsMorph){



				normalArray = new Float32Array(normalsMorph.length*3);
				offset_vertex = 0;

				for ( var f = 0, fl = normalsMorph.length; f < fl; f ++ ) {

					normalArray[ offset_vertex ]     = normalsMorph[ f ].x;
					normalArray[ offset_vertex + 1 ] = normalsMorph[ f ].y;
					normalArray[ offset_vertex + 2 ] = normalsMorph[ f ].z;

					offset_vertex += 3;

				}

				attributes.__webglMorphNormalsBuffers[ vk ] = gl.createBuffer();
				gl.bindBuffer( gl.ARRAY_BUFFER, attributes.__webglMorphNormalsBuffers[ vk ] );
				gl.bufferData( gl.ARRAY_BUFFER, normalArray , gl.STATIC_DRAW );
				attributes.__webglMorphNormalsBuffers[ vk ].numItems = normalArray.length/3;
				attributes.__webglMorphNormalsBuffers[ vk ].itemSize = 3;

				if(material.normalMap){

					var vertexTangents = geometry.morphTargets[vk].vTangents;

					normalArray = new Float32Array(vertexTangents.length*3); offset_normal=0;


					for ( f = 0, fl = vertexTangents.length; f < fl; f ++ ) {

								vn = vertexTangents[ f ];

								normalArray[ offset_normal ]     = vn.x;
								normalArray[ offset_normal + 1 ] = vn.y;
								normalArray[ offset_normal + 2 ] = vn.z;
								offset_normal += 3;

					}

					attributes.__webglMorphTangentsBuffer[ vk ]  = gl.createBuffer();
					gl.bindBuffer( gl.ARRAY_BUFFER, attributes.__webglMorphTangentsBuffer[ vk ] );
					gl.bufferData( gl.ARRAY_BUFFER, normalArray,  gl.STATIC_DRAW );
					attributes.__webglMorphTangentsBuffer.numItems = normalArray.length/3;
					attributes.__webglMorphTangentsBuffer.itemSize = 3;

					
				}


			}



		}

	}

	if(vertexNormals.length&&material.normalMap){

		geometry.computeVertexTangents();
		var vertexTangents = geometry.vTangents;

		normalArray = new Float32Array(vertexTangents.length*3); offset_normal=0;

		for ( f = 0, fl = vertexTangents.length; f < fl; f ++ ) {

					vn = vertexTangents[ f ];

					normalArray[ offset_normal ]     = vn.x;
					normalArray[ offset_normal + 1 ] = vn.y;
					normalArray[ offset_normal + 2 ] = vn.z;
					offset_normal += 3;

		}

		attributes.__webglTangentBuffer = gl.createBuffer();
		gl.bindBuffer( gl.ARRAY_BUFFER, attributes.__webglTangentBuffer );
		gl.bufferData( gl.ARRAY_BUFFER, normalArray,  gl.STATIC_DRAW );
		attributes.__webglTangentBuffer.numItems = normalArray.length/3;
		attributes.__webglTangentBuffer.itemSize = 3;

	}




	if(faces.length){
	
		for ( f = 0, fl = faces.length; f < fl; f ++ ) {
	
					vf = faces[ f ];
					faceArray[ offset_face ]     = vf.a;
					faceArray[ offset_face + 1 ] = vf.b;
					faceArray[ offset_face + 2 ] = vf.c;

					offset_face += 3;
	
		}
		attributes.__webglIndexBuffer = gl.createBuffer();
		gl.bindBuffer( gl.ELEMENT_ARRAY_BUFFER, attributes.__webglIndexBuffer );
		gl.bufferData( gl.ELEMENT_ARRAY_BUFFER, faceArray, gl.STATIC_DRAW );
		attributes.__webglIndexBuffer.numItems = faceArray.length;
		attributes.__webglIndexBuffer.itemSize = 1;
	
	}

}


 

Engine.MeshMorph = function ( geometry, material, castShadow, receiveShadow, depthOfField  ) {
	
	Engine.Mesh.call( this,geometry, material, castShadow, receiveShadow, depthOfField );

	this.keys = [];
	this.keyA = {index:0,step:1000};
	this.keyB = {index:0,step:1000};
	this.duration = 1000;
	this.running = false;
	this.step = 0;
	this.stepProgress = 0;
	this.progress = 0;
	this.loop = 0;
	this.onFinish = function(){};

};

Engine.MeshMorph.prototype = Object.create( Engine.Mesh.prototype );

Engine.MeshMorph.prototype.setAnimation = function(keys,duration,loop){

	if(this.keys != keys ){

		this.keys = keys||[];
		this.duration = duration||1000;
		this.loop = loop||false;

		this.step = 0;
		this.stepProgress = 0;
		this.progress = 0;
		this.keyA = this.keys[0]||0;
		this.keyB = this.keys[1]||0;

		this.running = true;

	}
}

Engine.MeshMorph.prototype.stop = function(){
	this.running = false;
	this.keys = [];
}

Engine.MeshMorph.prototype.nextStep = function(){
	if(this.step+1<this.keys.length){
		this.step++;
		this.keyA = this.keys[this.step];
		if(this.step+1<this.keys.length){
			this.keyB = this.keys[this.step+1];
		}else{
			this.keyB = this.keys[0];
		}
	}else if(this.loop){
		this.stepProgress = 0;
		this.step = 0;
		this.keyA = this.keys[this.step];
		if(this.step+1<this.keys.length){
			this.keyB = this.keys[this.step+1];
		}else{
			this.keyB = this.keys[0];
		}
	}else{
		this.running = false;
		this.onFinish();
	}

}


Engine.MeshMorph.prototype.update = function(t){

	//console.log('this.step : '+this.step)

	if(!this.running)return

	this.stepProgress +=  t/this.duration;

	if(this.stepProgress>this.keyB.step){
		this.nextStep();
	}
	this.progress = (this.stepProgress-this.keyA.step)/(this.keyB.step-this.keyA.step);
	
	if(this.progress>1)this.progress=1;
	 
}



Engine.ParticlesGenerator = function (generatorParams,particlesParams,material,camera, castShadow, receiveShadow, depthOfField) {

	Engine.Object3D.call( this );

	this.material = material;

	this.position = generatorParams.position||new Engine.Vector3(0,0,0);

	this.direction = this.getDirection();

	this.polarDirection = this.direction.cartToPolar();

	this.needUpdate = true;

	this.running = false;

	this.visible = true;

	this.currentTime = 0;

	this.camera = camera;

	this.sorting = camera?true:false;

	this.tempo = 0;

	this.age = 0; //optimisation memoire. eviter les dechets
	this.progress = 0;

	this.castShadow = castShadow||false;
	this.receiveShadow = receiveShadow||false;
	this.depthOfField = depthOfField||false;

	this.particles = [];
	this.particlesPosition = new Float32Array();

	this.gun = {

		type: generatorParams.type||'point',
		radius: generatorParams.radius!=undefined?generatorParams.radius:10,
		width: generatorParams.width!=undefined?generatorParams.width:10,
		height:generatorParams.height!=undefined?generatorParams.height:10,
		depth:generatorParams.depth!=undefined?generatorParams.depth:10,
		initNumber: generatorParams.initNumber||0,
		frequency: particlesParams.frequency||1000,
		numberMax : particlesParams.numberMax||1000,
		numberBystep : generatorParams.numberBystep!=undefined?generatorParams.numberBystep:1
	};

	var pp = particlesParams;

	this.geometry = {

		attributes:{},

		colorBegin: pp.colorBegin? new Engine.Color(pp.colorBegin):new Engine.Color(1,1,1),
		colorEnd: pp.colorEnd? new Engine.Color(pp.colorEnd):new Engine.Color(1,1,1),
		colorVariance: pp.colorVariance||0,

		opacityBegin: pp.opacityBegin!=undefined?pp.opacityBegin:1,
		opacityEnd: pp.opacityEnd!=undefined?pp.opacityEnd:1,
		opacityVariance: pp.opacityVariance||0,
		
		velocityBegin: pp.velocityBegin!=undefined?pp.velocityBegin:1,
		velocityEnd: pp.velocityEnd!=undefined?pp.velocityEnd:1,
		velocityVariance: pp.velocityVariance||0,
		
		sizeBegin: pp.sizeBegin!=undefined?pp.sizeBegin:1,
		sizeEnd: pp.sizeEnd!=undefined?pp.sizeEnd:1,
		sizeVariance: pp.sizeVariance||0,
		
		angleBegin: pp.angleBegin||0,
		angleEnd: pp.angleEnd||0,
		angleVariance: pp.angleVariance||0,
		

		spritSquareVariance: pp.spritSquareVariance!=undefined?parseInt(pp.spritSquareVariance):1,

		spritRange: pp.spritSquareVariance!=undefined?pp.spritSquareVariance*pp.spritSquareVariance-0.01:1,
		
		duration: pp.duration||1000,

		directionVariance : pp.directionVariance||0,

		lighting: pp.lighting||false
	}



	if(generatorParams.type == 'circle'){
		this.randomPosition = this._randomPositionCicle;
	}else if(generatorParams.type == 'rectangle'){
		this.randomPosition =  this._randomPositionRectangle;
	}else{
		this.randomPosition =  this._randomPositionPoint;
	}

	this.initBuffer();

	this.init();
};


Engine.ParticlesGenerator.prototype = Object.create( Engine.Object3D.prototype );


Engine.ParticlesGenerator.prototype.update =  function (dt) {

	var par, g = this.geometry;

	if(this.particles.length<this.gun.numberMax&&this.running){

		this.tempo+= dt;

		if(this.tempo>this.gun.frequency){

			for(var i=0,il=this.gun.numberBystep;i<il;i++){
				this._addParticle();
			}
			this.tempo = this.tempo-this.gun.frequency;
		}

	}else{

		this.tempo = 0;
	}

	var i = this.particles.length;
	while(i--){
	//for(var i=0,il=this.particles.length;i<il;i++){
		par = this.particles[i];
		this.age = Engine.currentTime-par.t0;
		if(this.age>g.duration){
			this._removeParticle(i);
		}
	}



	if(this.sorting) {
		this.updateParticlesPostion();
		this.sortParticles();
		this.updateBufferIndex();

	}else{
		this.updateParticlesPostion();
	}

	if(this.needUpdate){
		this.updateBuffer();
		this.needUpdate = false;
	}


	this.currentTime = Engine.currentTime;

};


Engine.ParticlesGenerator.prototype.init = function () {

	this.particles = [];

	if(this.gun.initNumber) {

		for(var i=0,il=this.gun.initNumber;i<il;i++){
			this._addParticle();
		}
	}

};

Engine.ParticlesGenerator.prototype.setCamera = function (camera) {

	this.camera = camera;

};


Engine.ParticlesGenerator.prototype.start = function() {
	if(!this.running){
		this.running = true;
		this.tempo = Math.max(0,this.gun.frequency-50);
	}
};

Engine.ParticlesGenerator.prototype.stop = function() {

	this.running = false;
		
};


Engine.ParticlesGenerator.prototype.updateParticlesPostion = function() {

	var par, g = this.geometry;

	this.particlesPosition = new Float32Array(this.particles.length*3);

	var i = this.particles.length
	while(i--){
	//for(var i=0,il=this.particles.length;i<il;i++){
		par = this.particles[i];
		this.age = Engine.currentTime-par.t0;

		this.progress = this.age/g.duration;
		this.age *= this.progress*g.velocityEnd+(1.0-this.progress)*g.velocityBegin;

		this.particlesPosition[i*3] = par.x+this.age*par.dx;
		this.particlesPosition[i*3+1] = par.y+this.age*par.dy;
		this.particlesPosition[i*3+2] = par.z+this.age*par.dz;
		
	}
		
};


Engine.ParticlesGenerator.prototype._addParticle =  function () {

	this.direction = this.getDirection();
	
	var particle = new Engine.Particle( this.randomPosition(), this._randomDirection(), this._randomVariance1(), this._randomVariance2(), this._randomVariance3());
	this.particles.push(particle);

	this.needUpdate= true;

};

Engine.ParticlesGenerator.prototype._removeParticle= function (p) {
	this.particles.splice(p,1);
	this.needUpdate= true;

};



Engine.ParticlesGenerator.prototype._randomPositionCicle= function () {

	var radius = Math.random()*this.gun.radius;
	var a1 = Math.random()*6.2831;
	var a2 = Math.randFloatSpread(6.2831);

	var position = this.matrixWorld.getPosition();

	var x = radius*Math.sin(a1)*Math.cos(a2)+position.x;
	var y = radius*Math.cos(a1)+this.position.y;
	var z = radius*Math.sin(a1)*Math.sin(a2)+position.z;

	return  new Engine.Vector3( x, y, z );

};

Engine.ParticlesGenerator.prototype._randomPositionRectangle= function () {

	var position = this.matrixWorld.getPosition();

	var x = this.gun.width*(Math.random()-0.5)+position.x;
	var y = this.gun.height*(Math.random()-0.5)+position.y;
	var z = this.gun.depth*(Math.random()-0.5)+position.z;

	return  new Engine.Vector3( x, y, z );

};

Engine.ParticlesGenerator.prototype._randomPositionPoint = function () {

	return  this.matrixWorld.getPosition();//this.position;

};


Engine.ParticlesGenerator.prototype._randomDirection = function () {

	if(this.geometry.directionVariance){
		
		this.polarDirection = this.direction.cartToPolar();


		var angle1 = this.polarDirection.angle1+this.geometry.directionVariance*(Math.random()-0.5);
		var angle2 = this.polarDirection.angle2+this.geometry.directionVariance*(Math.random()-0.5);

		var direction = new Engine.Vector3()

		return direction.polarToCart(angle1,angle2,1);

	}else{

		return this.direction.clone();

	}

};


Engine.ParticlesGenerator.prototype._randomVariance1 = function() {
	var r = (Math.random()*this.geometry.angleVariance*100)>>0;//variance max 0 - infinite
	r += Math.random()*this.geometry.opacityVariance;//variance max 0 - 1
	return r;
};
Engine.ParticlesGenerator.prototype._randomVariance2 = function() {
	var r = (Math.random()*this.geometry.sizeVariance*100)>>0;//% max 0 - 500%
	r += Math.random()*this.geometry.velocityVariance;//% max 0 - 99%
	return r;
};
Engine.ParticlesGenerator.prototype._randomVariance3 = function() {
	var r = 1;
	r += (Math.random()*this.geometry.spritRange)>>0;
	r += Math.random()*this.geometry.colorVariance;//variance max 0 - infinite
	return r;
};


Engine.ParticlesGenerator.prototype.initBuffer = function() {

	var maximum = this.gun.numberMax;

	var attributes = this.geometry.attributes;

	attributes.__webglVertexBuffer = gl.createBuffer();
	gl.bindBuffer(gl.ARRAY_BUFFER, attributes.__webglVertexBuffer);
	gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(3 * maximum), gl.DYNAMIC_DRAW);
	attributes.__webglVertexBuffer.itemSize = 3;
	attributes.__webglVertexBuffer.numItems = 0;

	attributes.__webglDirectionBuffer = gl.createBuffer();
	gl.bindBuffer(gl.ARRAY_BUFFER, attributes.__webglDirectionBuffer);
	gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(3 * maximum), gl.DYNAMIC_DRAW);
	attributes.__webglDirectionBuffer.itemSize = 3;
	attributes.__webglDirectionBuffer.numItems = 0;

	attributes.__webglRandomBuffer = gl.createBuffer();
	gl.bindBuffer(gl.ARRAY_BUFFER, attributes.__webglRandomBuffer);
	gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(3 * maximum), gl.DYNAMIC_DRAW);
	attributes.__webglRandomBuffer.itemSize = 3;
	attributes.__webglRandomBuffer.numItems = 0;

	attributes.__webglT0Buffer = gl.createBuffer();
	gl.bindBuffer(gl.ARRAY_BUFFER, attributes.__webglT0Buffer);
	gl.bufferData(gl.ARRAY_BUFFER, new Int32Array( maximum), gl.DYNAMIC_DRAW);
	attributes.__webglT0Buffer.itemSize = 1;
	attributes.__webglT0Buffer.numItems = 0;

	if(this.sorting){

		attributes.__webglIndexBuffer = gl.createBuffer();
		gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, attributes.__webglIndexBuffer);
		gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, new Uint16Array( maximum), gl.DYNAMIC_DRAW);
		attributes.__webglIndexBuffer.itemSize = 1;
	}

};


Engine.ParticlesGenerator.prototype.updateBuffer = function() {

	//if(this.particles.length){

		var attributes = this.geometry.attributes;

		var vertexArray =  new Float32Array(this.particles.length*3), offset = 0;
		var directionArray = new Float32Array(this.particles.length*3);
		var randomArray = new Float32Array(this.particles.length*3);
		var t0Array = new Float32Array(this.particles.length);

		for ( k = 0, kl = this.particles.length; k < kl; k ++ ) {

			var particle = this.particles[k];

			vertexArray[ offset ]     = particle.x;
			vertexArray[ offset + 1 ] = particle.y;
			vertexArray[ offset + 2 ] = particle.z;

			directionArray[ offset ]     = particle.dx;
			directionArray[ offset + 1 ] = particle.dy;
			directionArray[ offset + 2 ] = particle.dz;

			randomArray[ offset ]     = particle.random1;
			randomArray[ offset + 1 ] = particle.random2;
			randomArray[ offset + 2 ] = particle.random3;

			offset += 3;
			t0Array[ k ]     = particle.t0;

		}

	gl.bindBuffer(gl.ARRAY_BUFFER, attributes.__webglVertexBuffer);
	gl.bufferSubData(gl.ARRAY_BUFFER, 0, vertexArray);
	attributes.__webglVertexBuffer.numItems = vertexArray.length / 3;

	gl.bindBuffer(gl.ARRAY_BUFFER, attributes.__webglDirectionBuffer);
	gl.bufferSubData(gl.ARRAY_BUFFER, 0, directionArray);
	attributes.__webglDirectionBuffer.numItems = directionArray.length / 3;

	gl.bindBuffer(gl.ARRAY_BUFFER, attributes.__webglRandomBuffer);
	gl.bufferSubData(gl.ARRAY_BUFFER, 0, randomArray);
	attributes.__webglRandomBuffer.numItems = randomArray.length / 3;

	gl.bindBuffer(gl.ARRAY_BUFFER, attributes.__webglT0Buffer);
	gl.bufferSubData(gl.ARRAY_BUFFER, 0,t0Array);
	attributes.__webglT0Buffer.numItems = t0Array.length ;

	//}

}


Engine.ParticlesGenerator.prototype.updateBufferIndex = function() {

	gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.geometry.attributes.__webglIndexBuffer);
	gl.bufferSubData(gl.ELEMENT_ARRAY_BUFFER, 0, this.indexAttributes);
	this.geometry.attributes.__webglIndexBuffer.numItems = this.indexAttributes.length;

}


Engine.ParticlesGenerator.prototype.sortParticles = function(camera) {

		plan = this.camera.plan;
		this.indexAttributes = new Uint16Array(this.particles.length);
		var sortList = new Array(this.particlesPosition.length/3);
		var distance;
		var a2, b2, c2, x1, y1, z1;
		
		var i = this.particlesPosition.length/3
		//for (var i=0,il=this.particlesPosition.length; i < il ; i++) {
		while(i--){
			a2 = plan.a * plan.a;
			b2 = plan.b * plan.b;
			c2 = plan.c * plan.c;
			x1 = this.particlesPosition[i*3];
			y1 = this.particlesPosition[i*3+1];
			z1 = this.particlesPosition[i*3+2];
			distance = Math.abs(plan.a * x1 + plan.b * y1 + plan.c * z1 + plan.d) / Math.sqrt(a2 + b2 + c2);
			sortList[i] = {
				d: distance,
				k: i
			};
		}
		
		sortList = sortList.sort(function (left, right) {
			return left.d > right.d ? -1 : 1;
		});
		
		for (var i=0,il=this.particles.length; i < il ; i++) {
			this.indexAttributes[i] = sortList[i].k;
		}

		sortList = null;



}
 
Engine.Light = function ( hex ) {

	Engine.Object3D.call( this );

	this.color = new Engine.Color( hex );

};

Engine.Light.prototype = Object.create( Engine.Object3D.prototype );

Engine.Light.prototype.clone = function ( light ) {

	if ( light === undefined ) light = new Engine.Light();

	Engine.Object3D.prototype.clone.call( this, light );

	light.color.copy( this.color );

	return light;

};
Engine.DirectionalLight = function ( color, intensity ) {

	Engine.Light.call( this, color );

	this.position.set( 0, 1, 0 );
	this.target = new Engine.Object3D();

	this.intensity = ( intensity !== undefined ) ? intensity : 1;

};

Engine.DirectionalLight.prototype = Object.create( Engine.Light.prototype );


Engine.DirectionalLight.prototype.clone = function () {

	var light = new Engine.DirectionalLight();

	Engine.Light.prototype.clone.call( this, light );

	light.intensity = this.intensity;

	return light;

};
Engine.AmbientLight = function ( hex ) {

	Engine.Light.call( this, hex );

};

Engine.AmbientLight.prototype = Object.create( Engine.Light.prototype );

Engine.AmbientLight.prototype.clone = function () {

	var light = new Engine.AmbientLight();

	Engine.Light.prototype.clone.call( this, light );

	return light;

};
Engine.PointLight = function ( hex, intensity, distance, position ) {

	Engine.Light.call( this, hex );

	this.intensity = ( intensity !== undefined ) ? intensity : 1;
	this.distance = ( distance !== undefined ) ? distance : 0;
	this.position = !position ? new Engine.Vector3() : new Engine.Vector3().fromArray(position);

};

Engine.PointLight.prototype = Object.create( Engine.Light.prototype );

Engine.PointLight.prototype.clone = function () {

	var light = new Engine.PointLight();

	Engine.Light.prototype.clone.call( this, light );

	light.intensity = this.intensity;
	light.distance = this.distance;

	return light;

};
Engine.Camera = function () {

	Engine.Object3D.call( this );
	
	this.matrixWorldInverse = new Engine.Matrix4();
	this.projectionMatrix = new Engine.Matrix4();
	this.plan = this.getPlanView();

};

Engine.Camera.prototype = Object.create( Engine.Object3D.prototype );


Engine.Camera.prototype.lookAt = function () {

	// This routine does not support cameras with rotated and/or translated parent(s)

	var m1 = new Engine.Matrix4();
	var up = new Engine.Vector3( 0, 1, 0 );

	return function ( vector ) {

		m1.lookAt( this.position, vector, up );

		this.quaternion.setFromRotationMatrix( m1 );
		this.plan = this.getPlanView();

		return this;

	};

}();

Engine.Camera.prototype.clone = function (camera) {

	if ( camera === undefined ) camera = new Engine.Camera();

	Engine.Object3D.prototype.clone.call( this, camera );

	camera.matrixWorldInverse.copy( this.matrixWorldInverse );
	camera.projectionMatrix.copy( this.projectionMatrix );

	return camera;
};




Engine.OrthographicCamera = function ( left, right, top, bottom, near, far, type ) {

	Engine.Camera.call( this );

	this.type = type||"mainCamera";

	this.left = left;
	this.right = right;
	this.top = top;
	this.bottom = bottom;

	this.near = ( near !== undefined ) ? near : 0.1;
	this.far = ( far !== undefined ) ? far : 2000;

	this.updateProjectionMatrix();

};

Engine.OrthographicCamera.prototype = Object.create( Engine.Camera.prototype );

Engine.OrthographicCamera.prototype.updateProjectionMatrix = function () {
	
	this.projectionMatrix.makeOrthographic( this.left, this.right, this.top, this.bottom, this.near, this.far );

};

Engine.OrthographicCamera.prototype.clone = function () {

	var camera = new Engine.OrthographicCamera();

	Engine.Camera.prototype.clone.call( this, camera );

	camera.left = this.left;
	camera.right = this.right;
	camera.top = this.top;
	camera.bottom = this.bottom;
	
	camera.near = this.near;
	camera.far = this.far;

	return camera;
};


Engine.PerspectiveCamera = function ( fov, aspect, near, far, type, position ) {

	Engine.Camera.call( this );

	this.type = type||"mainCamera";

	this.fov = fov !== undefined ? fov : 50;
	this.aspect = aspect ? aspect : gl.viewWidth/gl.viewHeight;
	this.near = near !== undefined ? near : 0.1;
	this.far = far !== undefined ? far : 2000;

	this.position = !position ? new Engine.Vector3() : new  Engine.Vector3().fromArray(position);

	this.updateProjectionMatrix();

};

Engine.PerspectiveCamera.prototype = Object.create( Engine.Camera.prototype );

Engine.PerspectiveCamera.prototype.updateProjectionMatrix = function () {

	if ( this.fullWidth ) {

		var aspect = this.fullWidth / this.fullHeight;
		var top = Math.tan( Math.degToRad( this.fov * 0.5 ) ) * this.near;
		var bottom = -top;
		var left = aspect * bottom;
		var right = aspect * top;
		var width = Math.abs( right - left );
		var height = Math.abs( top - bottom );

		this.projectionMatrix.makeFrustum(
			left + this.x * width / this.fullWidth,
			left + ( this.x + this.width ) * width / this.fullWidth,
			top - ( this.y + this.height ) * height / this.fullHeight,
			top - this.y * height / this.fullHeight,
			this.near,
			this.far
		);

	} else {

		this.projectionMatrix.makePerspective( this.fov, this.aspect, this.near, this.far );

	}

};

Engine.PerspectiveCamera.prototype.clone = function () {

	var camera = new Engine.PerspectiveCamera();

	Engine.Camera.prototype.clone.call( this, camera );

	camera.fov = this.fov;
	camera.aspect = this.aspect;
	camera.near = this.near;
	camera.far = this.far;

	return camera;
};



Engine.PerspectiveCamera.prototype.vectorClick = function(winX, winY) {

	var x = ( winX /gl.viewWidth ) * 2 - 1;
	var y = - ( winY / gl.viewHeight ) * 2 + 1;

	var vector = new Engine.Vector3(x,y,0.5);

	var viewProjectionMatrix = new Engine.Matrix4();
	var projectionMatrixInverse = new Engine.Matrix4();

	this.matrixWorld.setPosition(this.position); //force update of position

	projectionMatrixInverse.getInverse( this.projectionMatrix );
	viewProjectionMatrix.multiplyMatrices( this.matrixWorld, projectionMatrixInverse );

	vector.applyProjection( viewProjectionMatrix );

	
	return vector.sub(this.position).normalize();

};


Engine.WebGLRenderer = function ( width,height,backgroundColor ) {

	var _canvas = document.createElement( 'canvas' );

	this.domElement = _canvas;

	this._clearColor = backgroundColor?new Engine.Color( backgroundColor ) : new Engine.Color( 0x000000 );
	this._clearViewPort = {};

	this.clearColorEnable = true;

	this._shadowEnable = false;


	try {
		gl = _canvas.getContext( 'webgl', {antialias:true} ) || _canvas.getContext( 'experimental-webgl', {antialias:true} );
		if ( gl === null ) {throw 'Error creating WebGL context.';}
	} catch ( error ) { 
		console.error( error );
	}

	var glExtensionTextureFloat = gl.getExtension( 'OES_texture_float' );
	var glExtensionStandardDerivatives = gl.getExtension( 'OES_standard_derivatives' );
	var glExtensionTextureFilterAnisotropic = gl.getExtension( 'EXT_texture_filter_anisotropic' ) || gl.getExtension( 'MOZ_EXT_texture_filter_anisotropic' ) || gl.getExtension( 'WEBKIT_EXT_texture_filter_anisotropic' );

	if ( ! glExtensionTextureFloat ) {
		console.log( 'Engine.WebGLRenderer: Float textures not supported.' );
	}
	if ( ! glExtensionStandardDerivatives ) {
		console.log( 'Engine.WebGLRenderer: Standard derivatives not supported.' );
	}
	if ( ! glExtensionTextureFilterAnisotropic ) {
		console.log( 'Engine.WebGLRenderer: Anisotropic texture filtering not supported.' );
	}

	_canvas.width = width;
	_canvas.height = height;
	_canvas.style.width = width + 'px';
	_canvas.style.height = height + 'px';
	_canvas.style.zIndex = -1;
	_canvas.style.background = '#000000';
	gl.viewWidth = width;
	gl.viewHeight = height;

	this.setViewPort(0,0,gl.viewWidth,gl.viewHeight);

	window.gl = gl;

};


Engine.WebGLRenderer.prototype.SHADER_SOL = "Sol";
Engine.WebGLRenderer.prototype.SHADER_MOR = "Mor";
Engine.WebGLRenderer.prototype.SHADER_SOL_PAR = "SolPar";
Engine.WebGLRenderer.prototype.SHADER_MOR_PAR = "MorPar";
Engine.WebGLRenderer.prototype.SHADER_PAR = "Par";


Engine.WebGLRenderer.prototype.renderBasic = function(scene,framebufferTarget,framebufferSource) {

	gl.enable(gl.DEPTH_TEST);

	var i, il, k, kl, group, object;

	this.setupFramebuffer( framebufferTarget );

	this._updateRenderList(scene);

	this._updateLights(scene);
	
	scene.updateMatrixWorld();



	for(key in scene.renderBasicGroups) {

		group = scene.renderBasicGroups[key];

		group.shader.prepare(scene, scene._lights, framebufferSource?framebufferSource.texture:null);

		k = group.objects1.length;
		while ( k--) {

			object = group.objects1[k];

			if(object.visible){

				group.shader.updateAttributes(object);
				
				group.shader.updateUniforms(object, scene);

				group.shader.draw(object);

			}

		}

	}

	gl.enable(gl.BLEND);
	gl.blendFunc(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA);


	for(key in scene.renderBasicGroups) {

		group = scene.renderBasicGroups[key];

		if(group.transparent){

			group.shader.prepare(scene, scene._lights, framebufferSource?framebufferSource.texture:null);

			for ( k = 0, kl = group.objects2.length; k < kl; k ++ ) {

				object = group.objects2[k];

				if(object.visible){
					
					group.shader.updateUniforms(object, scene);

					group.shader.updateAttributes(object);

					group.shader.draw(object);

				}

			}
		}
	}

	gl.disable(gl.BLEND);

	if(framebufferTarget){ 

		framebufferTarget.saveInTexture();
	}
	

}

Engine.WebGLRenderer.prototype._updateRenderList = function(scene) {

		if( scene.__skyboxAdded){
			this.addSkybox(scene.__skyboxAdded,scene);
		}

		while ( scene.__objectsAdded.length ) {
			this.addObject( scene.__objectsAdded[ 0 ] ,scene);
			scene.__objectsAdded.splice( 0, 1 );
		}

		while ( scene.__objectsRemoved.length ) {
			 this.removeObject( scene.__objectsRemoved[ 0 ] );
			scene.__objectsRemoved.splice( 0, 1 );
		}

}


Engine.WebGLRenderer.prototype.addObject = function( object, scene ) {


	var material = object.material;
	var geometry = object.geometry;
	var that = this;

	if(object instanceof Engine.ParticlesGenerator) {

		this._updateRenderBasicGroup(this.SHADER_PAR, object, scene );
		
		if(object.castShadow||object.depthOfField) {
			this._updateRenderDepthGroup(this.SHADER_PAR, object, scene );
		}

	} else if(geometry.morphTargets.length) {

		this._updateRenderBasicGroup(this.SHADER_MOR, object, scene );
		
		if(object.castShadow||object.depthOfField) {
			this._updateRenderDepthGroup(this.SHADER_MOR, object, scene );
		}

	}else{

		this._updateRenderBasicGroup(this.SHADER_SOL, object, scene );
		
		if(object.castShadow||object.depthOfField) {
			this._updateRenderDepthGroup(this.SHADER_SOL, object, scene );
		}
	}

};



Engine.WebGLRenderer.prototype.removeObject = function( object ) {

	var geometry = object.geometry;

	if(object instanceof Engine.ParticlesGenerator) {
		this._removeInstancesBasicGroup( object );
		if(object.castShadow||object.depthOfField) {
			this._removeInstancesDepthGroup( object );
		}
	} else if(geometry.morphTargets.length){
		this._removeInstancesBasicGroup( object );
		if(object.castShadow||object.depthOfField) {
			this._removeInstancesDepthGroup( object );
		}
	}else{
		this._removeInstancesBasicGroup( object );
		if(object.castShadow||object.depthOfField) {
			this._removeInstancesDepthGroup( object );
		}
	}
	
};


Engine.WebGLRenderer.prototype.addSkybox = function( object, scene ) {

	scene.renderBasicGroups['_solidSkybox'] = {
				shader : new Engine.shaderManager[ "ShaderSkybox" ](),
				objects1 : [object],
				objects2 : []
			};

};

Engine.WebGLRenderer.prototype._updateRenderBasicGroup = function(id, object, scene) {
		
		id  += this._optiShader( object, scene );

		console.log("build Shader"+id);

		var group = scene.renderBasicGroups[id];

		if(!group){
			group = {
				shader : new Engine.shaderManager[ "Shader"+id ](),
				objects1 : [],
				objects2 : []
			}
			 scene.renderBasicGroups[id] = group;
		}
		
		if(!object.material.transparent){
			console.log('_updateRenderBasicGroup')
			group.objects1.push(object);
		}else{
			group.objects2.push(object);
			group.transparent = true;
		}
		object.parentBasicGroup = group;

}
Engine.WebGLRenderer.prototype._removeInstancesBasicGroup = function( object) {

	var group = object.parentBasicGroup;
	var i;
	if(!object.material.transparent){

		i = group.objects1.length;
		while(i--){
			if(group.objects1[i]==object){
				console.log('_removeInstancesBasicGroup')
				group.objects1.splice(i,1);
				break;
			}
			
		}

			
	}else{
		i = group.objects2.length;
		while(i--){
			if(group.objects2[i]==object){
				group.objects2.splice(i,1);
				if(group.objects2.length==0){
					group.transparent = false;
				}
				break;
			}

		}
	}

	delete object;
}

Engine.WebGLRenderer.prototype._removeInstancesDepthGroup =  function( object) {
	
	var group = object.parentDepthGroup;
	var i;
	if(object.castShadow){


		if(!object.material.transparent){
			i = group.objectsForShadow1.length;
			while(i--){
				if(group.objectsForShadow1[i]==object){
					group.objectsForShadow1.splice(i,1);
					break;
				}	
			}
		}else{
			i = group.objectsForShadow2.length;
			while(i--){
				if(group.objectsForShadow2[i]==object){
					group.objectsForShadow2.splice(i,1);
					if(group.objectsForShadow2.length==0){
						group.transparent = false;
					}
					break;
				}
			}
		}
	}
	if(object.depthOfField){
		if(!object.material.transparent){
			i = group.objectsForDepth1.length;
			while(i--){
				if(group.objectsForDepth1[i]==object){
					group.objectsForDepth1.splice(i,1);
					break;
				}
			}
		}else{
			i = group.objectsForDepth2.length;
			while(i--){
				if(group.objectsForDepth2[i]==object){
					group.objectsForDepth2.splice(i,1);
					if(group.objectsForDepth2.length==0){
						group.transparent = false;
					}
					break;
				}
			}
		}
	}

	delete object;

};



Engine.WebGLRenderer.prototype._optiShader = function(object,scene){

		var opti = "",
		light,
		f_lightOmni = false,
		f_lightDirect = false,
		f_light = false,
		f_colorMap = false,
		f_lightMap = false,
		f_normalMap = false,
		f_cubeMap = false,
		f_shadow = false;

		var geo = object.geometry;

		if(object instanceof Engine.ParticlesGenerator){


			if(object.material.map){

				opti+= "Spr";

				if(geo.angleBegin!=0||geo.angleEnd!=0||geo.angleVariance!=0){
					
					opti+= "Rot";

				}

			}

			if(geo.angleVariance!=0||geo.opacityVariance!=0||geo.colorVariance!=0||geo.velocityVariance!=0||geo.sizeVariance!=0||geo.spritSquareVariance>1){

				opti+= "Ran";

			}

			if(geo.lighting){
				for ( i = 0, il = scene.__lights.length; i < il; i ++ ){
					light = scene.__lights[i];
					if(light instanceof Engine.PointLight){
						opti+= "Lom";
					}
				}
			}


		}else{ 

			if( object.material instanceof Engine.LineBasicMaterial ) {

				opti+="Lin";
				return opti;
			}else if( object.material instanceof Engine.ParticleSystemMaterial ) {

				opti+="Par";

				if(object.material.map){
					opti+="Map";
				}else{
					opti+="Col";
				}
				return opti;
			}


			if(object.material.lighting)
			if(geo.vNormals.length||geo.morphTargets[0]&&geo.morphTargets[0].vNormals.length){

				for ( i = 0, il = scene.__lights.length; i < il; i ++ ) {

					light = scene.__lights[i];

					if(light instanceof Engine.PointLight){
						f_lightOmni = true;
						f_light = true;
					}

					if(light instanceof Engine.DirectionalLight){
						f_lightDirect = true;
						f_light = true;

					}

					if(object.receiveShadow){
						f_shadow = true;
					}

				}

			}
			

			if(geo.vUvs.length){

				if(object.material.map){
					f_colorMap = true;
				}

				if(object.material.lightMap&&f_light)
					f_lightMap = true;
				
				if(object.material.normalMap&&f_light)
					f_normalMap = true;
				
				if(object.material.cubeMap&&f_light)
					f_cubeMap = true;

			}


			if(f_lightDirect&&scene.cameraShadow) {

			}
			

			if(f_colorMap||f_lightMap||f_normalMap||f_cubeMap){
				opti+="Map";
			}else{
				opti+="Col";
			}
			if(f_lightOmni){
				opti+="Lom";
			}
			if(f_lightDirect){
				opti+="Ldi";
			}
			if(f_lightMap){
				opti+="Lma";
			}
			if(f_normalMap){
				opti+="Nma";
			}
			if(f_cubeMap){
				opti+="Mma";
			}
			if(f_shadow){
				opti+="Sha";
			}

			if( (f_lightMap||f_normalMap||f_cubeMap) &&!f_colorMap){
				
				object.material.map = new Engine.Texture( object.material.color );

			}
		}

		return opti;

		//Map 1 ok
		//MapLom 2 ok 
		//MapLdi 3 ok
		//MapLomLdi 4 ok
		//MapLomLdiLma 5 ok
		//MapLomLma 6 ok
		//MapLdiLma 7 ok
		//MapLomLdiNma 8 ok
		//MapLdiNma 9 ok
		//MapLomNma 10 ok
		//MapLomLdiLmaNma 11
		//MapLomLmaNma 12
		//MapLdiLmaNma 13

		//MapLomLdiLmaMma 14
		//MapLomLdiLmaNmaMma 15

		//Col 16 ok
		//ColLom 17 ok
		//ColLdi 18 ok
		//ColLomLdi 19 ok

}


Engine.WebGLRenderer.prototype._updateLights = function(scene){

	var pointLight, light, directionalLight;

	if ( scene._lightsNeedUpdate ){

		scene._lights = {
			ambient: [ 0, 0, 0 ],
			point: { length: 0, colors: [], positions: [], distances: [] },
			directional : {length:0, colors: [], direction: [] }
		};

		var lights = scene.__lights;
		var i = lights.length;
		while( i-- ) {

			light = lights[i];

			if( light instanceof Engine.AmbientLight){

				scene._lights.ambient = light.color.toArray();

			}else if( light instanceof Engine.PointLight){

				if(light.intensity==0)continue;

				pointLight =  scene._lights.point

				pointLight.length++;

				pointLight.colors = pointLight.colors.concat(light.color.multiplyScalar(light.intensity).toArray());

				pointLight.positions = pointLight.positions.concat(light.position.toArray());

				pointLight.distances = pointLight.distances.concat(light.distance);

			}else if(light instanceof Engine.DirectionalLight){

				if(light.intensity==0)continue;

				directionalLight =  scene._lights.directional;

				directionalLight.length++;

				directionalLight.colors = directionalLight.colors.concat(light.color.multiplyScalar(light.intensity).toArray());

				var elements = light.matrix.elements;
				
				directionalLight.direction.push(-elements[8],-elements[9],-elements[10]);

			}

		}

		scene._lightsNeedUpdate = false;

	}

}



Engine.WebGLRenderer.prototype.setBackgroundColor = function( r,g,b ) {
	this._clearColor.r = r;
	this._clearColor.g = g;
	this._clearColor.b = b;
}

Engine.WebGLRenderer.prototype.setViewPort = function( x,y,width,height ) {
	this._clearViewPort.x = x
	this._clearViewPort.y = y
	this._clearViewPort.width = width
	this._clearViewPort.height = height
}


Engine.WebGLRenderer.prototype.setupFramebuffer = function( framebuffer ) {

	if(framebuffer){

		framebuffer.enableBuffer(); // rendu effectué dans une zone mémoire allouée

	} else { // rendu effectué sur l'écran

		gl.bindFramebuffer( gl.FRAMEBUFFER, null );

		gl.viewport(this._clearViewPort.x, this._clearViewPort.y, this._clearViewPort.width, this._clearViewPort.height);

		if(this.clearColorEnable){
			gl.clearColor(this._clearColor.r, this._clearColor.g, this._clearColor.b, 1.0);
			gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);
		}else{
			gl.clear(gl.DEPTH_BUFFER_BIT);
		}

	}

	//gl.clearDepth( 1 );
	//gl.clearStencil( 0 );

};


Engine.WebGLRenderer.prototype.renderComposer = function(framebufferTarget,framebufferSource1,framebufferSource2) {

	gl.disable(gl.DEPTH_TEST);

	this.setupFramebuffer( framebufferTarget );

	var postProcessing = this.postProcessing;

	if(!postProcessing.screen){
		postProcessing.screen = new Engine.Screen();
	}

	if(!postProcessing.shaderComposer){
		postProcessing.shaderComposer = new Engine.shaderManager.shaderComposer();
	}

	postProcessing.shaderComposer.prepare( framebufferSource1.texture,framebufferSource2.texture);

	postProcessing.shaderComposer.updateUniforms();

	postProcessing.shaderComposer.updateAttributes(postProcessing.screen);

	postProcessing.shaderComposer.draw();


	if(framebufferTarget){ 

		framebufferTarget.saveInTexture();
	}

}


Engine.WebGLRenderer.prototype.renderDepthField = function(framebufferTarget,framebufferSource1,framebufferSource2,camera) {

	gl.disable(gl.DEPTH_TEST);

	this.setupFramebuffer( framebufferTarget );

	var postProcessing = this.postProcessing;

	if(!postProcessing.screen){
		postProcessing.screen = new Engine.Screen();
	}

	if(!postProcessing.shaderDepthField){
		postProcessing.shaderDepthField = new Engine.shaderManager.shaderDepthField();
	}

	postProcessing.shaderDepthField.prepare( framebufferSource1.texture,framebufferSource2.texture,camera);

	postProcessing.shaderDepthField.updateAttributes(postProcessing.screen);

	postProcessing.shaderDepthField.draw();


	if(framebufferTarget){ 

		framebufferTarget.saveInTexture();
	}

}
Engine.WebGLRenderer.prototype.rendererDepth = function(scene,framebufferTarget) {

	gl.enable(gl.DEPTH_TEST);

	var  k, kl, group, object, texture;

	this.setupFramebuffer( framebufferTarget );

	this._updateRenderList(scene);

	scene.updateMatrixWorld();

	if(framebufferTarget){

		texture = framebufferTarget.texture;
	}

	
	for(key in scene.renderDepthGroups) {

		group = scene.renderDepthGroups[key];
		var shader;
		if(framebufferTarget){
			shader = group.shader;
		}else{
			shader = group.shaderView;
		}

		shader.prepare(scene, scene._lights, texture);

		for ( k = 0, kl = group.objectsForDepth1.length; k < kl; k ++ ) {

			object = group.objectsForDepth1[k];

			if(object.visible){
				
				shader.updateUniforms(object, scene);

				shader.updateAttributes(object);

				shader.draw(object);

			}

		}

	}

	gl.enable(gl.BLEND);
	gl.blendFunc(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA);

	for(key in scene.renderDepthGroups) {

		group = scene.renderDepthGroups[key];

		if(group.transparent){

			group.shader.prepare(scene, scene._lights, texture);

			for ( k = 0, kl = group.objectsForDepth2.length; k < kl; k ++ ) {

				object = group.objectsForDepth2[k];

				if(object.visible){
					
					group.shader.updateUniforms(object, scene);

					group.shader.updateAttributes(object);

					group.shader.draw(object);

				}

			}
		}

	}

	gl.disable(gl.BLEND);


	if(framebufferTarget){ 

		framebufferTarget.saveInTexture();
	}
	
}


Engine.WebGLRenderer.prototype.rendererDepthShadow = function(scene, framebufferDepth) {

	gl.enable(gl.DEPTH_TEST);

	var  k, kl, group, object, texture;

	this.setupFramebuffer( framebufferDepth );

	this._updateRenderList(scene);

	scene.updateMatrixWorld();

	var sceneCamera = scene.camera;
	scene.camera = scene.cameraDepth;

	for(key in scene.renderDepthGroups) {

		group = scene.renderDepthGroups[key];

		group.shader.prepare(scene);

		for ( k = 0, kl = group.objectsForShadow1.length; k < kl; k ++ ) {

			object = group.objectsForShadow1[k];

			if(object.visible){
				
				group.shader.updateUniforms(object, scene);

				group.shader.updateAttributes(object);

				group.shader.draw(object);

			}

		}

	}

	gl.enable(gl.BLEND);
	gl.blendFunc(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA);

	for(key in scene.renderDepthGroups) {

		group = scene.renderDepthGroups[key];

		if(group.transparent){

			group.shader.prepare(scene);

			for ( k = 0, kl = group.objectsForShadow2.length; k < kl; k ++ ) {

				object = group.objectsForShadow2[k];

				if(object.visible){
					
					group.shader.updateUniforms(object, scene);

					group.shader.updateAttributes(object);

					group.shader.draw(object);

				}

			}
		}

	}

	gl.disable(gl.BLEND);



	scene.camera = sceneCamera;

	if(framebufferDepth){
		framebufferDepth.saveInTexture();
	}

}



Engine.WebGLRenderer.prototype._updateRenderDepthGroup = function(id, object, scene) {

		if(id=='Par'){
			var geo = object.geometry;
			if(object.material.map){
				id+= "Spr";
				if(object.geometry.angleBegin!=0||object.geometry.angleEnd!=0||object.geometry.angleVariance!=0){
					id+= "Rot";
				}
			}
			if(geo.angleVariance!=0||geo.opacityVariance!=0||geo.colorVariance!=0||geo.velocityVariance!=0||geo.sizeVariance!=0||geo.spritSquareVariance>1){
				id+= "Ran";
			}
		}
		
		id+="Dep";

		//console.log("build Shader"+id);

		var group = scene.renderDepthGroups[id];

		if(!group){
			group = {
				shader : new Engine.shaderManager[ "Shader"+id ](),
				shaderView : new Engine.shaderManager[ "Shader"+id+"View" ](),
				objectsForShadow1 : [],
				objectsForDepth1 : [],
				objectsForShadow2 : [],
				objectsForDepth2 : []
			}

			scene.renderDepthGroups[id] = group;
		}

		if(object.castShadow){
			if(!object.material.transparent){
				group.objectsForShadow1.push(object);
			}else{
				group.objectsForShadow2.push(object);
				group.transparent = true;
			}
		}
		if(object.depthOfField){
			if(!object.material.transparent){
				group.objectsForDepth1.push(object);
			}else{
				group.objectsForDepth2.push(object);
				group.transparent = true;
			}
		}
		object.parentDepthGroup = group;
}
Engine.WebGLRenderer.prototype.renderLuminance = function(framebufferTarget,framebufferSource) {

	gl.disable(gl.DEPTH_TEST);

	this.setupFramebuffer( framebufferTarget );

	var postProcessing = this.postProcessing;

	if(!postProcessing.screen){
		postProcessing.screen = new Engine.Screen();
	}

	if(!postProcessing.shaderLuminance){
		postProcessing.shaderLuminance = new Engine.shaderManager.shaderLuminance();
	}

	postProcessing.shaderLuminance.prepare( framebufferSource.texture);

	postProcessing.shaderLuminance.updateUniforms();

	postProcessing.shaderLuminance.updateAttributes(postProcessing.screen);

	postProcessing.shaderLuminance.draw();


	if(framebufferTarget){ 

		framebufferTarget.saveInTexture();
	}

}


Engine.WebGLRenderer.prototype.renderGlow = function(framebufferTarget,framebufferSource,loop) {


	//TODO simplifier en une seul passe

	gl.disable(gl.DEPTH_TEST);

	var postProcessing = this.postProcessing;

	if(!postProcessing.screen){postProcessing.screen = new Engine.Screen();}

	if(!postProcessing.shaderGlow){
		postProcessing.shaderGlow = new Engine.shaderManager.shaderGlow({width:framebufferSource.width,height:framebufferSource.height});
	}

	this.setupFramebuffer( framebufferTarget );

	postProcessing.shaderGlow.prepare( framebufferSource.texture );
	postProcessing.shaderGlow.updateAttributes(postProcessing.screen);
	postProcessing.shaderGlow.prepare( framebufferSource.texture );
	postProcessing.shaderGlow.draw(true);


	if(framebufferTarget){ 
		framebufferTarget.saveInTexture();
	}

}
/****************************************************************************************************/
/********************************************EVENT MANAGER*******************************************/
/****************************************************************************************************/

Engine.keys = {
	A:false,
	Q:false,
	W:false,
	Z:false,
	X:false,
	E:false,
	D:false,
	C:false,
	R:false,
	F:false,
	V:false,
	T:false,
	G:false,
	B:false,
	Y:false,
	H:false,
	N:false,
	U:false,
	J:false,
	I:false,
	O:false,
	L:false,
	P:false,
	M:false,
	SPACE:false,
	ALT:false,
	CTRL:false,
	MAJ:false,
	UP:false,
	DOWN:false,
	LEFT:false,
	RIGHT:false,
	N1:false,
	N2:false,
	N3:false,
	N4:false,
	N5:false,
	N6:false,
	N7:false,
	N8:false,
	N9:false,
	N0:false,
	ENTER:false
}

Engine.mouse = {
	move: false,
	downLeft: false,
	downRight: false,
	downMiddle: false,
	wheel : false,
	x : 0,
	y : 0
}

Engine.keys.keys = [];
Engine.keys.keys[65] = "A";
Engine.keys.keys[81] = "Q";
Engine.keys.keys[87] = "W";
Engine.keys.keys[90] = "Z";
Engine.keys.keys[83] = "S";
Engine.keys.keys[88] = "X";
Engine.keys.keys[69] = "E";
Engine.keys.keys[68] = "D";
Engine.keys.keys[67] = "C";
Engine.keys.keys[82] = "R";
Engine.keys.keys[70] = "F";
Engine.keys.keys[86] = "V";
Engine.keys.keys[84] = "T";
Engine.keys.keys[71] = "G";
Engine.keys.keys[66] = "B";
Engine.keys.keys[89] = "Y";
Engine.keys.keys[72] = "H";
Engine.keys.keys[78] = "N";
Engine.keys.keys[85] = "U";
Engine.keys.keys[74] = "J";
Engine.keys.keys[73] = "I";
Engine.keys.keys[75] = "K";
Engine.keys.keys[79] = "O";
Engine.keys.keys[76] = "L";
Engine.keys.keys[80] = "P";
Engine.keys.keys[77] = "M";
Engine.keys.keys[32] = "SPACE";
Engine.keys.keys[18] = "ALT";
Engine.keys.keys[17] = "CTRL";
Engine.keys.keys[16] = "MAJ";
Engine.keys.keys[38] = "UP";
Engine.keys.keys[40] = "DOWN";
Engine.keys.keys[37] = "LEFT";
Engine.keys.keys[39] = "RIGHT";
Engine.keys.keys[49] = "N1";
Engine.keys.keys[50] = "N2";
Engine.keys.keys[51] = "N3";
Engine.keys.keys[52] = "N4";
Engine.keys.keys[53] = "N5";
Engine.keys.keys[54] = "N6";
Engine.keys.keys[55] = "N7";
Engine.keys.keys[56] = "N8";
Engine.keys.keys[57] = "N9";
Engine.keys.keys[48] = "N0";
Engine.keys.keys[13] = "ENTER";
Engine.keys.keys[34] = "PDOWN";
Engine.keys.keys[33] = "PUP";
Engine.keys.keys[35] = "END";
Engine.keys.keys[46] = "SUP";




Engine.prototype.handleKeyDown = function(){};

Engine.prototype.handleKeyUp = function(){};

Engine.prototype.handleWheel = function(){};

Engine.prototype.handleMouseMove = function(){};

Engine.prototype.handleMousedown = function(event){};

Engine.prototype.handleMouseUp = function(event){};

Engine.prototype.initEvents = function(context, node) {

	document.onkeydown = function(event){
		
		var key = Engine.keys.keys[event.keyCode];
		if(key){
			Engine.keys[key] = true;
			context.handleKeyDown();
		}
		if(Engine.keys.UP || Engine.keys.DOWN || Engine.keys.RIGHT || Engine.keys.LEFT ){
			event.preventDefault();
		}
	};

	document.onkeyup = function(event){
		var key = Engine.keys.keys[event.keyCode];
		if(key){
			Engine.keys[key] = false;
			context.handleKeyUp();
		}
		event.preventDefault();
	};

	if(!node) node = document;

	node.onmousewheel = function(event){
		event.preventDefault();
		context.handleWheel(event); 
	};

	node.onwheel = function(event){
		event.preventDefault();
		context.handleWheel(event); 
	};

	node.onmousedown = function(event){
		event.preventDefault();
		context.handleMousedown(event);
		switch(event.button){
			case 0 : Engine.mouse.downRight = true;
			break;
			case 1 : Engine.mouse.downMiddle = true;
			break;
			case 2 : Engine.mouse.downLeft = true;
			break;
		}

		return false;
	};





	node.onmouseout = function(event){
		event.preventDefault();
		context.handleMouseUp(event);
		Engine.mouse.downRight = false;
		Engine.mouse.downMiddle = false;	
		Engine.mouse.downLeft = false;
		return false;
	};

	node.onmouseup = function(event){
		event.preventDefault();
		context.handleMouseUp(event);
		switch(event.button){
			case 0 : Engine.mouse.downRight = false;
			break;
			case 1 : Engine.mouse.downMiddle = false;
			break;
			case 2 : Engine.mouse.downLeft = false;
			break;
		}
		return false;
	};

	var parentElement = gl.canvas.parentElement;

	if(parentElement.localName=="body"){

		node.onmousemove = function(event){
			context.handleMouseMove(event);
			Engine.mouse.x = event.clientX;
			Engine.mouse.y = event.clientY;
		};

	}else{
		node.onmousemove = function(event){
			context.handleMouseMove(event);
			Engine.mouse.x = event.clientX-parentElement.offsetLeft;
			Engine.mouse.y = event.clientY-parentElement.offsetTop;
		};
	}

}

///////////////////////////////////////////////////////////////////////////
//////////////////////////MOUSE CONTROLE///////////////////////////////////
///////////////////////////////////////////////////////////////////////////


function mouseControle (scene) {

	var press = false, uinit = 0, vinit = 0, u = 0, v = 0;
	var polar = scene.camera.position.cartToPolar();
	var uprev = polar.angle1, vprev = polar.angle2, zoom = polar.radius;

	document.onmousedown = function(event){

		press=true;
		uinit=event.clientX;vinit=event.clientY;
		anim=false;

	}

	document.onmouseup = function(event){

		press=false;
		uprev=u;vprev=v;
		anim=true;

	}

	/*document.onmousewheel = function(event){

		zoom+=event.wheelDelta/100;
		zoom = Math.max(zoom,0);
		scene.camera.position.polarToCart(uprev,vprev,zoom);
		scene.camera.lookAt(new Engine.Vector3(0,0,0));

	}*/

	document.onmousemove = function(event){

		if(press){
			u = (event.clientY-vinit)/100+uprev;
			v = (event.clientX-uinit)/100+vprev;
			scene.camera.position.polarToCart(u,v,zoom);
			scene.camera.lookAt(new Engine.Vector3(0,0,0));
		}

	}

}
Engine.Scene = function () {

	Engine.Object3D.call( this );

	this.fog = null;
	this.overrideMaterial = null;

	this.autoUpdate = true; // checked by the renderer
	this.matrixAutoUpdate = false;

	this.__lights = [];

	this.__objectsAdded = [];
	this.__objectsRemoved = [];
	this.__skyboxAdded = null;

	this.camera = null;

	this.cameraDepth = null;

	this.skybox = null;

	this.renderBasicGroups = {};
	this.renderDepthGroups = {};
	this.postProcessing = {};
	this._lightsNeedUpdate = true;

};

Engine.Scene.prototype = Object.create( Engine.Object3D.prototype );



Engine.Scene.prototype.updateMatrixWorld =  function ( force ) {

		if ( this.matrixAutoUpdate === true ) this.updateMatrix();

		if ( this.matrixWorldNeedsUpdate === true || force === true ) {

			this.matrixWorld.copy( this.matrix );

			this.matrixWorldNeedsUpdate = false;

			force = true;

		}

		// update children

		for ( var i = 0, l = this.children.length; i < l; i ++ ) {
			if(this.children[ i ].matrix)
				this.children[ i ].updateMatrixWorld( force );

		}

		if(this.camera){
			
			this.camera.matrixWorldInverse.getInverse( this.camera.matrixWorld );
			
		}
		if(this.cameraDepth){
			
			this.cameraDepth.matrixWorldInverse.getInverse( this.cameraDepth.matrixWorld );
			
		}

};


Engine.Scene.prototype.__addObject = function ( object ) {

	if ( object instanceof Engine.Light ) {

		if ( this.__lights.indexOf( object ) === - 1 ) {

			this.__lights.push( object );

		}

	} else if ( !( object instanceof Engine.Camera  ) ) {


		if(object.geometry instanceof Engine.SkyboxGeometry)	{

			this.__skyboxAdded = object

		}else{

			this.__objectsAdded.push( object );

			// check if previously removed

			var i = this.__objectsRemoved.indexOf( object );

			if ( i !== -1 ) {

				this.__objectsRemoved.splice( i, 1 );

			}
		}

	} else  {

		if(object.type == "mainCamera") {
			this.camera =  object;
		}else if (object.type == "cameraDepth") {			
			this.cameraDepth =  object;
		}	

	}


	for ( var c = 0; c < object.children.length; c ++ ) {

		this.__addObject( object.children[ c ] );

	}

};

Engine.Scene.prototype.__removeObject = function ( object ) {

	if ( object instanceof Engine.Light ) {

		var i = this.__lights.indexOf( object );

		if ( i !== -1 ) {

			this.__lights.splice( i, 1 );

		}

		if ( object.shadowCascadeArray ) {

			for ( var x = 0; x < object.shadowCascadeArray.length; x ++ ) {

				this.__removeObject( object.shadowCascadeArray[ x ] );

			}

		}

	} else if ( !( object instanceof Engine.Camera ) ) {

		this.__objectsRemoved.push( object );

		// check if previously added

		var i = this.__objectsAdded.indexOf( object );

		if ( i !== -1 ) {

			this.__objectsAdded.splice( i, 1 );

		}

	}

	for ( var c = 0; c < object.children.length; c ++ ) {

		this.__removeObject( object.children[ c ] );

	}

};

Engine.Scene.prototype.needUpdateLight = function(){
	this._lightsNeedUpdate = true;
}

Engine.Scene.prototype.clone = function ( object ) {

	if ( object === undefined ) object = new Engine.Scene();

	Engine.Object3D.prototype.clone.call(this, object);

	if ( this.fog !== null ) object.fog = this.fog.clone();
	if ( this.overrideMaterial !== null ) object.overrideMaterial = this.overrideMaterial.clone();

	object.autoUpdate = this.autoUpdate;
	object.matrixAutoUpdate = this.matrixAutoUpdate;

	return object;

};




Math.PI2 =  Math.PI * 2;

Math.clamp =  function ( x, a, b ) {

	return ( x < a ) ? a : ( ( x > b ) ? b : x );

};

Math.clampBottom =  function ( x, a ) {

	return x < a ? a : x;

};

Math.mapLinear =  function ( x, a1, a2, b1, b2 ) {

	return b1 + ( x - a1 ) * ( b2 - b1 ) / ( a2 - a1 );

};
	
// http://en.wikipedia.org/wiki/Smoothstep
Math.smoothstep =  function ( x, min, max ) {

	if ( x <= min ) return 0;
	if ( x >= max ) return 1;

	x = ( x - min )/( max - min );

	return x*x*(3 - 2*x);

};

Math.smootherstep =  function ( x, min, max ) {

	if ( x <= min ) return 0;
	if ( x >= max ) return 1;

	x = ( x - min )/( max - min );

	return x*x*x*(x*(x*6 - 15) + 10);

};

	// Random float from <0, 1> with 16 bits of randomness
	// (standard Math.random() creates repetitive patterns when applied over larger space)

Math.random16 = function () {

	return ( 65280 * this.random() + 255 * this.random() ) / 65535;

};

	// Random integer from <low, high> interval

Math.randInt = function ( low, high ) {

	return low + this.floor( this.random() * ( high - low + 1 ) );

};

	// Random float from <low, high> interval

Math.randFloat =  function ( low, high ) {

	return low + this.random() * ( high - low );

};

	// Random float from <-range/2, range/2> interval

Math.randFloatSpread =  function ( range ) {

	return range * ( 0.5 - this.random() );

};

Math.sign = function ( x ) {

	return ( x < 0 ) ? -1 : ( ( x > 0 ) ? 1 : 0 );

};

Math.degToRad = function() {
	var degreeToRadiansFactor = Math.PI / 180;
	return function ( degrees ) {
		return degrees * degreeToRadiansFactor;
	};

}();


Math.getSmallestSquare = function ( x ) {

	var v = 1;
	var c = 1;
	var result;
	var i = 14;
	while(i--){
		c = c*2;
		v = x/c;

		if(v<1){
			return c/2;
		}

	}

};


Math.getUtmostSquare = function ( x ) {

	var v = 1;
	var c = 1;
	var result;
	var i = 14;
	while(i--){
		c = c*2;
		v = x/c;

		if(v<1){
			return c;
		}

	}

};

Math.getNearestSquare = function ( x ) {

	var v = 1;
	var c = 1;
	var result;
	var i = 14;
	while(i--){
		c = c*2;
		v = x/c;

		if(v<1){
			if(this.abs(x-c)>this.abs(x-c/2))
				return c/2;
			else
				return c;
		}
	}

};

Math.subAngle = function(a1,a2){

	//a1 = a1%(2*this.PI);
	//a2 = a2%(2*this.PI);

	var e = a1-a2;

	if(e > this.PI)
		e-=this.PI2;
	else if(e < -this.PI){
		e+=this.PI2;
	}
	return e;
}


Math.to2KPI = function(a){

	var a = a%Math.PI2;
	if(a<0){
		return a+Math.PI2;
	}else{
		return a;
	}
}




Math.radToDeg = function() {

	var radianToDegreesFactor = 180 / Math.PI;

	return function ( radians ) {
		
		return radians * radianToDegreesFactor;

	};

}();


//	var surface = {x:0,y:0,z:0,dx:0,dy:1,dz:0}
Math.intersectionLinePlane = function ( point, direction, surface) {

	k = -(-(surface.x * surface.dx + surface.y * surface.dy + surface.z * surface.dz) + surface.dx * point.x + surface.dy * point.y + surface.dz * point.z) / (surface.dx * direction.x + surface.dy *
		direction.y + surface.dz * direction.z);

	return new Engine.Vector3(point.x + direction.x * k, point.y + direction.y * k, point.z + direction.z * k )

};

Math.intersectionSegSeg = function ( ax, ay, bx, by, cx, cy, dx, dy) {

	var abx = bx - ax;
	var aby = by - ay;
	var cdx = dx - cx;
	var cdy = dy - cy;

	var d = abx*cdy - aby*cdx;
	if ( d == 0) return false;

	var t1 = -((ax - cx)*cdy - (ay - cy)*cdx)/d; 
	if (t1 < 0 || t1 > 1) return false;
	
	var t2 = ((cx - ax)*aby - (cy - ay)*abx)/d;
	if (t2 < 0 || t2 > 1) return false;

	return true;
	//return new [ax + t2*(bx - ax), ay + t1*(by - ay)];

}


Math.intersectionSegBox = function ( ax, ay, bx, by, xmin, xmax, ymin, ymax) {

	var intersection = false;
	if(this.intersectionSegSeg(ax,ay,bx,by,  xmin,ymin,xmax,ymin ))return true;
	if(this.intersectionSegSeg(ax,ay,bx,by,  xmin,ymax,xmax,ymax ))return true;
	if(this.intersectionSegSeg(ax,ay,bx,by,  xmin,ymin,xmin,ymax ))return true;
	if(this.intersectionSegSeg(ax,ay,bx,by,  xmax,ymin,xmax,ymax ))return true;

	return false;
}



if(!Array.prototype.last) {
    Array.prototype.last = function() {
        return this[this.length - 1];
    }
}


if(!Array.prototype.remove) {
    Array.prototype.remove = function(ele) {
    	var i = this.length;
     	while(i--){
			if(this[i] == ele){
				this.splice(i,1);
				return true;
			}
		}
		return false;
    }
}






Engine.XHRLoader = function ( ) {


};

Engine.XHRLoader.prototype.load = function ( url, onLoad, onProgress, onError ) {

	var request = new XMLHttpRequest();

	if ( onLoad ) {

		request.addEventListener( 'load', function ( event ) {

			if(event.target.status == 200){
				onLoad( event.target.responseText );
				Engine.pendingRequest -=1;
			}else{
				if(onError)onError( event );
			}
			
		}, false );

	}

	if ( onProgress ) {

		request.addEventListener( 'progress', function ( event ) {
			
			onProgress( event );

		}, false );

	}

	if ( onError ) {

		request.addEventListener( 'error', function ( event ) {

			onError( event );

		}, false );

	}

	Engine.pendingRequest +=1;

	request.open( 'GET', url, true );
	request.send( null );

};





var shaderOmnInit = ""+	
	"uniform vec3 pointLightColor[ MAX_POINT_LIGHTS ];"+
	"uniform vec3 pointLightPosition[ MAX_POINT_LIGHTS ];"+
	"uniform float pointLightDistance[ MAX_POINT_LIGHTS ];"+
	"uniform int numberOfOmniLight;";

var shaderOmnLoop = ""+

	"for( int i = 0; i < MAX_POINT_LIGHTS; i ++ ) {"+

	"if(numberOfOmniLight==i){break;}"+

	"vec3 lVector = pointLightPosition[ i ].xyz - vPosition.xyz;"+

	"float lDistance = 1.0;"+
	"if ( pointLightDistance[ i ] > 0.0 )"+
		"lDistance = 1.0 - min( ( length( lVector ) / pointLightDistance[ i ] ), 1.0 );"+
	"lVector = normalize( lVector );"+

	"float dotProduct = dot( normal, lVector );"+
	"float pointDiffuseWeight = max( dotProduct, 0.0 );"+

	"vec3 lightWeight = pointLightColor[ i ] * pointDiffuseWeight * lDistance;"+

	"totalDiffuse  +=  lightWeight;"+

	"if(shininess>0.0){"+

		"vec3 eyeDirection = normalize(vPosition.xyz - viewPosition.xyz);"+
		"vec3 reflectionDirection = reflect(lVector, normal);"+
		"totalSpecular += pow(max(dot(reflectionDirection, eyeDirection), 0.0), shininess)*lightWeight;"+

	"}"+

"}";

var shaderDirInit = ""+	
	"uniform vec3 directionLightColor[ MAX_DIRECTION_LIGHTS ];"+
	"uniform vec3 directionLightVector[ MAX_DIRECTION_LIGHTS ];"+
	"uniform int numberOfDirectionLight;";

var shaderDirLoop = ""+

"for( int i = 0; i < MAX_DIRECTION_LIGHTS; i ++ ) {"+

	"if(numberOfDirectionLight==i){break;}"+

	"float dotProduct = dot( normal, directionLightVector[ i ] );"+
	"float pointDiffuseWeight = max( dotProduct, 0.0 );"+

	"totalDiffuse  += directionLightColor[ i ] * pointDiffuseWeight;"+

	"if(shininess>0.0){"+

		"vec3 eyeDirection = normalize(vPosition.xyz - viewPosition.xyz);"+
		"vec3 reflectionDirection = reflect(-directionLightVector[ i ], normal);"+
		"totalSpecular += pow(max(dot(reflectionDirection, eyeDirection), 0.0), shininess)*directionLightColor[ i ];"+

	"}"+

"}";

var shaderMatInit = ""+
	"uniform float opacity;"+
	"uniform vec3 emissive;"+
	"uniform float shininess;"+
	"uniform vec3 viewPosition;"+
	"uniform vec3 ambientLightColor;";



/************************************************************/
/*                                                          */
/*                      SHADER SOLID                        */
/*                                                          */
/************************************************************/


Engine.shaderManager.ShaderSol = function () {

	var program = gl.createProgram();

	var vertexShader = this.buildVertexShader();

	var fragmentShader = this.buildFragmentShader();

	gl.attachShader(program, vertexShader );

	gl.attachShader(program, fragmentShader);

	gl.linkProgram(program);

	gl.deleteShader( vertexShader );

	gl.deleteShader( fragmentShader );

	this.program = program;

	this.buildUniformsAndAttributes();

}

Engine.shaderManager.ShaderSol.prototype.prepare = function(scene,lights, mapbuffer){

	gl.useProgram(this.program);

	gl.enableVertexAttribArray( this.setAttributVertex );

	this.updateCommonUniforms(scene,lights, mapbuffer);

}

Engine.shaderManager.ShaderSol.prototype.updateUniforms = function(object){

	gl.uniform1f( this.setUniformOpacity, object.material.opacity );

}

Engine.shaderManager.ShaderSol.prototype.updateCommonUniforms = function(scene, light){

	gl.uniformMatrix4fv( this.setUniformPMatrix, false, scene.camera.projectionMatrix.elements );

	gl.uniformMatrix4fv( this.setUniformVVMatrix, false, scene.camera.matrixWorldInverse.elements );
	
}

Engine.shaderManager.ShaderSol.prototype.updateAttributes = function(object){

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglVertexBuffer);

	gl.vertexAttribPointer( this.setAttributVertex, 3, gl.FLOAT, false, 0, 0);

}

Engine.shaderManager.ShaderSol.prototype.buildUniformsAndAttributes = function(){


	this.setAttributVertex = gl.getAttribLocation( this.program, "vertexPosition" );

	this.setUniformVMatrix = gl.getUniformLocation( this.program, "modelMatrix" );

	this.setUniformPMatrix = gl.getUniformLocation( this.program, "projectionMatrix" );

	this.setUniformVVMatrix = gl.getUniformLocation( this.program, "viewMatrix" );

	this.setUniformOpacity = gl.getUniformLocation( this.program, "opacity" );

}

Engine.shaderManager.ShaderSol.prototype.draw = function(object){

  gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, object.geometry.attributes.__webglIndexBuffer);

  var clonesLength = object.clones.length;
  
  if(clonesLength){

  	for(var k=0;k<clonesLength;k++){

		gl.uniformMatrix4fv( this.setUniformVMatrix, false, object.clones[k].elements );
		gl.drawElements(gl.TRIANGLES, object.geometry.attributes.__webglIndexBuffer.numItems, gl.UNSIGNED_SHORT, 0);
	}

  }else{
  	gl.uniformMatrix4fv( this.setUniformVMatrix, false, object.matrixWorld.elements );
  	gl.drawElements(gl.TRIANGLES, object.geometry.attributes.__webglIndexBuffer.numItems, gl.UNSIGNED_SHORT, 0);

  }

}

Engine.shaderManager.ShaderSol.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+

	"void main(void) {"+

		"gl_Position = projectionMatrix * viewMatrix * modelMatrix * vec4(vertexPosition, 1.0);"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}

Engine.shaderManager.ShaderSol.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+
	
	"uniform float opacity;"+

	"void main(void) {"+

		" gl_FragColor = vec4(0.5,0.5,0.5,opacity );"+

	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}
/************************************************************/
/*                                                          */
/*                      SHADER SOLID COLOR                  */
/*                                                          */
/************************************************************/


Engine.shaderManager.ShaderSolCol = function () {
	
	Engine.shaderManager.ShaderSol.call( this );

}

Engine.shaderManager.ShaderSolCol.prototype = Object.create( Engine.shaderManager.ShaderSol.prototype );


Engine.shaderManager.ShaderSolCol.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSol.prototype.updateUniforms.call( this, object );
	
	gl.uniform3fv( this.setUniformDiffuse, object.material.color.toArray());

}

Engine.shaderManager.ShaderSolCol.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderSol.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformDiffuse = gl.getUniformLocation( this.program, "diffuse" );

}


Engine.shaderManager.ShaderSolCol.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+

	"uniform vec3 diffuse;"+

	"uniform float opacity;"+

	"void main(void) {"+

		"gl_FragColor = vec4( diffuse, opacity );"+

	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}
/************************************************************/
/*                                                          */
/*                       COLOR MATERIAL                     */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolColMat = function () {
	
	Engine.shaderManager.ShaderSolCol.call( this );

}

Engine.shaderManager.ShaderSolColMat.prototype = Object.create( Engine.shaderManager.ShaderSolCol.prototype );

Engine.shaderManager.ShaderSolColMat.prototype.prepare = function(scene,lights, mapbuffer){

	gl.useProgram(this.program);
	
	gl.enableVertexAttribArray( this.setAttributVertex );
	
	gl.enableVertexAttribArray( this.setAttributNormal );

	this.updateCommonUniforms(scene,lights, mapbuffer);

}

Engine.shaderManager.ShaderSolColMat.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolCol.prototype.updateUniforms.call( this, object );

	gl.uniform3fv( this.setUniformEmissive, object.material.emissive.toArray() );

	gl.uniform1f( this.setUniformShininess, object.material.shininess );

}

Engine.shaderManager.ShaderSolColMat.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderSolCol.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformViewPosition, scene.camera.position.toArray() );

	gl.uniform3fv( this.setUniformAmbientLightColor, light.ambient );

}

Engine.shaderManager.ShaderSolColMat.prototype.updateAttributes = function(object){
	
	Engine.shaderManager.ShaderSolCol.prototype.updateAttributes.call( this, object );

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglNormalBuffer);

	gl.vertexAttribPointer( this.setAttributNormal, 3, gl.FLOAT, false, 0, 0);
	
}

Engine.shaderManager.ShaderSolColMat.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolCol.prototype.buildUniformsAndAttributes.call( this );

	this.setAttributNormal = gl.getAttribLocation( this.program, "vertexNormal" );

	this.setUniformViewPosition = gl.getUniformLocation( this.program, "viewPosition" );

	this.setUniformAmbientLightColor = gl.getUniformLocation( this.program, "ambientLightColor" );

	this.setUniformEmissive = gl.getUniformLocation( this.program, "emissive" );

	this.setUniformShininess = gl.getUniformLocation( this.program, "shininess" );

}

Engine.shaderManager.ShaderSolColMat.prototype.buildVertexShader = function(inherits){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute vec3 vertexNormal;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+

	"varying vec4 vNormal;"+
	"varying vec4 vPosition;"+

	"void main(void) {"+

		"vNormal = modelMatrix * vec4(vertexNormal, 0);" +
		"vPosition = modelMatrix * vec4(vertexPosition, 1.0);"+
		"gl_Position = projectionMatrix * viewMatrix * vPosition;"+

	"}";

	if(!inherits){

		var vertexShader = gl.createShader(gl.VERTEX_SHADER);
		gl.shaderSource(vertexShader, code);
		gl.compileShader(vertexShader);
		if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

		return vertexShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/*                      SHADER SOLID COLOR LIGHTOMNI        */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolColLom = function () {
	
	Engine.shaderManager.ShaderSolColMat.call( this );

}

Engine.shaderManager.ShaderSolColLom.prototype = Object.create( Engine.shaderManager.ShaderSolColMat.prototype );


Engine.shaderManager.ShaderSolColLom.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderSolColMat.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformPointLightColor, light.point.colors );

	gl.uniform3fv( this.setUniformPointLightPosition, light.point.positions);

	gl.uniform1i( this.setUniformNumberOfOmniLight, light.point.length );

	gl.uniform1fv( this.setUniformpPointLightDistance, light.point.distances );

}

Engine.shaderManager.ShaderSolColLom.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolColMat.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformPointLightColor = gl.getUniformLocation( this.program, "pointLightColor" );

	this.setUniformpPointLightDistance = gl.getUniformLocation( this.program, "pointLightDistance" );

	this.setUniformPointLightPosition = gl.getUniformLocation( this.program, "pointLightPosition" );

	this.setUniformNumberOfOmniLight = gl.getUniformLocation( this.program, "numberOfOmniLight" );

}

Engine.shaderManager.ShaderSolColLom.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_POINT_LIGHTS "+Engine.conf.MAX_POINT_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderOmnInit+

	shaderMatInit+
	"uniform vec3 diffuse;"+

	"varying vec4 vNormal;"+
	"varying vec4 vPosition;"+

	"void main(void) {"+

		"gl_FragColor = vec4( diffuse, opacity );"+

		"vec3 normal = normalize( vNormal.xyz );"+
		
		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderOmnLoop+
		
		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor ) + totalSpecular;"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/*             SHADER SOLID COLOR LIGHTDIR                  */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolColLdi = function () {
	
	Engine.shaderManager.ShaderSolColMat.call( this );

}

Engine.shaderManager.ShaderSolColLdi.prototype = Object.create( Engine.shaderManager.ShaderSolColMat.prototype );

Engine.shaderManager.ShaderSolColLdi.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderSolColMat.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformDirectionLightColor,  light.directional.colors);

	gl.uniform3fv( this.setUniformpDirectionLightVector,  light.directional.direction );

	gl.uniform1i( this.setUniformNumberOfDirectionLight, light.directional.length );

}

Engine.shaderManager.ShaderSolColLdi.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolColMat.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformDirectionLightColor = gl.getUniformLocation( this.program, "directionLightColor" );

	this.setUniformpDirectionLightVector = gl.getUniformLocation( this.program, "directionLightVector" );

	this.setUniformNumberOfDirectionLight = gl.getUniformLocation( this.program, "numberOfDirectionLight" );

}

Engine.shaderManager.ShaderSolColLdi.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_DIRECTION_LIGHTS 3 \n"+

	"precision mediump float;"+

	shaderDirInit+

	shaderMatInit+
	"uniform vec3 diffuse;"+
	
	"varying vec4 vNormal;"+
	"varying vec4 vPosition;"+

	"void main(void) {"+

		"gl_FragColor = vec4( diffuse, opacity );"+

		"vec3 normal = normalize( vNormal.xyz );"+
		
		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderDirLoop+
		
		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor ) + totalSpecular;"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/*             SHADER SOLID COL LIGHTOMI LIGHTDIR           */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolColLomLdi = function () {
	
	Engine.shaderManager.ShaderSolColLom.call( this );

}

Engine.shaderManager.ShaderSolColLomLdi.prototype = Object.create( Engine.shaderManager.ShaderSolColLom.prototype );


Engine.shaderManager.ShaderSolColLomLdi.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderSolColLom.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformDirectionLightColor,  light.directional.colors);

	gl.uniform3fv( this.setUniformpDirectionLightVector,  light.directional.direction );

	gl.uniform1i( this.setUniformNumberOfDirectionLight, light.directional.length );


}

Engine.shaderManager.ShaderSolColLomLdi.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolColLom.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformDirectionLightColor = gl.getUniformLocation( this.program, "directionLightColor" );

	this.setUniformpDirectionLightVector = gl.getUniformLocation( this.program, "directionLightVector" );

	this.setUniformNumberOfDirectionLight = gl.getUniformLocation( this.program, "numberOfDirectionLight" );

}

Engine.shaderManager.ShaderSolColLomLdi.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_POINT_LIGHTS "+Engine.conf.MAX_POINT_LIGHTS+" \n"+
	"#define MAX_DIRECTION_LIGHTS "+Engine.conf.MAX_DIRECTION_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderOmnInit+
	shaderDirInit+
	
	"uniform vec3 diffuse;"+
	shaderMatInit+

	"varying vec4 vNormal;"+
	"varying vec4 vPosition;"+

	"void main(void) {"+

		"gl_FragColor = vec4( diffuse, opacity );"+

		"vec3 normal = normalize( vNormal.xyz );"+
		
		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderOmnLoop+
		shaderDirLoop+

		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor ) + totalSpecular;"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/*                      SHADER SOLID MAP                    */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMap = function () {

	Engine.shaderManager.ShaderSol.call( this );

}

Engine.shaderManager.ShaderSolMap.prototype = Object.create( Engine.shaderManager.ShaderSol.prototype );

Engine.shaderManager.ShaderSolMap.prototype.prepare = function(scene,lights, mapbuffer){

	gl.useProgram(this.program);

	gl.enableVertexAttribArray( this.setAttributVertex );

	gl.enableVertexAttribArray( this.setAttributUv );
	
	this.updateCommonUniforms(scene,lights, mapbuffer);

}

Engine.shaderManager.ShaderSolMap.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSol.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.map,0);

	gl.uniform1i(this.setUniformMapColor, 0);

}

Engine.shaderManager.ShaderSolMap.prototype.updateAttributes = function(object){

	Engine.shaderManager.ShaderSol.prototype.updateAttributes.call( this, object );

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglUVBuffer);

	gl.vertexAttribPointer( this.setAttributUv, 2, gl.FLOAT, false, 0, 0);
	
}

Engine.shaderManager.ShaderSolMap.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderSol.prototype.buildUniformsAndAttributes.call( this );

	this.setAttributUv = gl.getAttribLocation( this.program, "vertexUvs" );

	this.setUniformMapColor = gl.getUniformLocation( this.program, "mapSamplerColor" );

}

Engine.shaderManager.ShaderSolMap.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute vec2 vertexUvs;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+

	"varying vec2 vUv;"+

	"void main(void) {"+

		"vUv = vertexUvs;"+
		"gl_Position = projectionMatrix * viewMatrix * modelMatrix * vec4(vertexPosition, 1.0);"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}


Engine.shaderManager.ShaderSolMap.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+

	"uniform sampler2D mapSamplerColor;"+
	"uniform float opacity;"+
	"varying vec2 vUv;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		" gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}


Engine.shaderManager.ShaderSol.prototype.setTexture = function ( texture, slot ) {

		if(texture.image instanceof HTMLVideoElement){

			if ( texture.image.readyState == texture.image.HAVE_ENOUGH_DATA ) {

				texture.needsUpdate = true;

			}

		}

		if ( texture.needsUpdate ) {

			if ( ! texture.__webglInit ) {

				texture.__webglInit = true;

				texture.__webglTexture = gl.createTexture();
			}

			gl.activeTexture( gl.TEXTURE0 + slot );

			gl.bindTexture( gl.TEXTURE_2D, texture.__webglTexture );

			gl.pixelStorei( gl.UNPACK_FLIP_Y_WEBGL, true );

			//if(texture.image instanceof Uint8Array)
				//gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA,1,1,0, gl.RGBA, gl.UNSIGNED_BYTE, texture.image);
			//else
				gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, texture.image);

			///gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);//Pour afficher des images de toutes dimension
			///gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);//Pour afficher des images de toutes dimension

			//gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR/*gl.NEAREST*/);//En cas de zoom,interpole la couleur entre 2 pixels
			//gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR); //En cas de dézoom,moyenne la couleur entre 2 pixels


    		//demowebgl
    		gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
        	gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR_MIPMAP_NEAREST);
        	gl.generateMipmap(gl.TEXTURE_2D);

			texture.needsUpdate = false;

		} else {

			gl.activeTexture( gl.TEXTURE0 + slot );

			gl.bindTexture( gl.TEXTURE_2D, texture.__webglTexture );

		}

	};
/************************************************************/
/*                                                          */
/*                         MAP MATERIAL                     */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapMat = function () {
	
	Engine.shaderManager.ShaderSolMap.call( this );

}

Engine.shaderManager.ShaderSolMapMat.prototype = Object.create( Engine.shaderManager.ShaderSolMap.prototype );

Engine.shaderManager.ShaderSolMapMat.prototype.prepare = function(scene,lights, mapbuffer){

	gl.useProgram(this.program);
	
	gl.enableVertexAttribArray( this.setAttributVertex );

	gl.enableVertexAttribArray( this.setAttributUv );

	gl.enableVertexAttribArray( this.setAttributNormal );

	this.updateCommonUniforms(scene,lights, mapbuffer);

}

Engine.shaderManager.ShaderSolMapMat.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolMap.prototype.updateUniforms.call( this, object );

	gl.uniform3fv( this.setUniformEmissive, object.material.emissive.toArray() );

	gl.uniform1f( this.setUniformShininess, object.material.shininess );

}

Engine.shaderManager.ShaderSolMapMat.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderSolMap.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformViewPosition, scene.camera.position.toArray() );

	gl.uniform3fv( this.setUniformAmbientLightColor, light.ambient );

}

Engine.shaderManager.ShaderSolMapMat.prototype.updateAttributes = function(object){
	
	Engine.shaderManager.ShaderSolMap.prototype.updateAttributes.call( this, object );

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglNormalBuffer);

	gl.vertexAttribPointer( this.setAttributNormal, 3, gl.FLOAT, false, 0, 0);
	
}

Engine.shaderManager.ShaderSolMapMat.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMap.prototype.buildUniformsAndAttributes.call( this );

	this.setAttributNormal = gl.getAttribLocation( this.program, "vertexNormal" );

	this.setUniformViewPosition = gl.getUniformLocation( this.program, "viewPosition" );

	this.setUniformAmbientLightColor = gl.getUniformLocation( this.program, "ambientLightColor" );

	this.setUniformEmissive = gl.getUniformLocation( this.program, "emissive" );

	this.setUniformShininess = gl.getUniformLocation( this.program, "shininess" );

}

Engine.shaderManager.ShaderSolMapMat.prototype.buildVertexShader = function(inherits){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute vec2 vertexUvs;"+
	"attribute vec3 vertexNormal;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+

	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec4 vNormal;"+

	"void main(void) {"+

		"vUv = vertexUvs;"+
		"vNormal = modelMatrix * vec4(vertexNormal, 0);" +
		"vPosition = modelMatrix * vec4(vertexPosition, 1.0);"+
		"gl_Position = projectionMatrix * viewMatrix * vPosition;"+

	"}";

	if(!inherits){

		var vertexShader = gl.createShader(gl.VERTEX_SHADER);
		gl.shaderSource(vertexShader, code);
		gl.compileShader(vertexShader);
		if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

		return vertexShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/*             SHADER SOLID MAP LIGHTDIR                    */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLdi = function () {
	
	Engine.shaderManager.ShaderSolMapMat.call( this );

}

Engine.shaderManager.ShaderSolMapLdi.prototype = Object.create( Engine.shaderManager.ShaderSolMapMat.prototype );

Engine.shaderManager.ShaderSolMapLdi.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderSolMapMat.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformDirectionLightColor,  light.directional.colors);

	gl.uniform3fv( this.setUniformpDirectionLightVector,  light.directional.direction );

	gl.uniform1i( this.setUniformNumberOfDirectionLight, light.directional.length );


}

Engine.shaderManager.ShaderSolMapLdi.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMapMat.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformDirectionLightColor = gl.getUniformLocation( this.program, "directionLightColor" );

	this.setUniformpDirectionLightVector = gl.getUniformLocation( this.program, "directionLightVector" );

	this.setUniformNumberOfDirectionLight = gl.getUniformLocation( this.program, "numberOfDirectionLight" );

}

Engine.shaderManager.ShaderSolMapLdi.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_DIRECTION_LIGHTS "+Engine.conf.MAX_DIRECTION_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderDirInit+

	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+

	"varying vec4 vNormal;"+
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		" gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 normal = normalize( vNormal.xyz );"+
		
		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderDirLoop+
		
		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor ) + totalSpecular;"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/*                      SHADER SOLID MAP LIGHTOMNI          */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLom = function () {
	
	Engine.shaderManager.ShaderSolMapMat.call( this );

}

Engine.shaderManager.ShaderSolMapLom.prototype = Object.create( Engine.shaderManager.ShaderSolMapMat.prototype );

Engine.shaderManager.ShaderSolMapLom.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderSolMapMat.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformPointLightColor, light.point.colors );

	gl.uniform3fv( this.setUniformPointLightPosition, light.point.positions);

	gl.uniform1i( this.setUniformNumberOfOmniLight, light.point.length );

	gl.uniform1fv( this.setUniformpPointLightDistance, light.point.distances );

}

Engine.shaderManager.ShaderSolMapLom.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMapMat.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformPointLightColor = gl.getUniformLocation( this.program, "pointLightColor" );

	this.setUniformpPointLightDistance = gl.getUniformLocation( this.program, "pointLightDistance" );

	this.setUniformPointLightPosition = gl.getUniformLocation( this.program, "pointLightPosition" );

	this.setUniformNumberOfOmniLight = gl.getUniformLocation( this.program, "numberOfOmniLight" );

}

Engine.shaderManager.ShaderSolMapLom.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_POINT_LIGHTS "+Engine.conf.MAX_POINT_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderOmnInit+

	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+

	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec4 vNormal;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		" gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 normal = normalize( vNormal.xyz );"+
		
		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderOmnLoop+
		
		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor ) + totalSpecular;"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/*             SHADER SOLID MAP LIGHTOMI LIGHTDIR           */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLomLdi = function () {
	
	Engine.shaderManager.ShaderSolMapLom.call( this );

}

Engine.shaderManager.ShaderSolMapLomLdi.prototype = Object.create( Engine.shaderManager.ShaderSolMapLom.prototype );


Engine.shaderManager.ShaderSolMapLomLdi.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderSolMapLom.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformDirectionLightColor,  light.directional.colors);

	gl.uniform3fv( this.setUniformpDirectionLightVector,  light.directional.direction );

	gl.uniform1i( this.setUniformNumberOfDirectionLight, light.directional.length );


}

Engine.shaderManager.ShaderSolMapLomLdi.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMapLom.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformDirectionLightColor = gl.getUniformLocation( this.program, "directionLightColor" );

	this.setUniformpDirectionLightVector = gl.getUniformLocation( this.program, "directionLightVector" );

	this.setUniformNumberOfDirectionLight = gl.getUniformLocation( this.program, "numberOfDirectionLight" );

}

Engine.shaderManager.ShaderSolMapLomLdi.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_POINT_LIGHTS "+Engine.conf.MAX_POINT_LIGHTS+" \n"+
	"#define MAX_DIRECTION_LIGHTS "+Engine.conf.MAX_DIRECTION_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderOmnInit+
	shaderDirInit+
	
	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+

	"varying vec4 vNormal;"+
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		" gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 normal = normalize( vNormal.xyz );"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderOmnLoop+
		shaderDirLoop+

		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular;"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/*       SHADER SOLID MAP LIGHTOMI LIGHTMAP        */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLomLma = function () {
	
	Engine.shaderManager.ShaderSolMapLom.call( this );

}

Engine.shaderManager.ShaderSolMapLomLma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLom.prototype );


Engine.shaderManager.ShaderSolMapLomLma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolMapLom.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.lightMap,1);

	gl.uniform1i(this.setUniformMapLight, 1);

}

Engine.shaderManager.ShaderSolMapLomLma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMapLom.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapLight = gl.getUniformLocation( this.program, "mapSamplerLight" );

}

Engine.shaderManager.ShaderSolMapLomLma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_POINT_LIGHTS "+Engine.conf.MAX_POINT_LIGHTS+" \n"+
	"#define MAX_DIRECTION_LIGHTS "+Engine.conf.MAX_DIRECTION_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderOmnInit+
	
	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+
	"uniform sampler2D mapSamplerLight;"+

	"varying vec4 vNormal;"+
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		"gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 textureLight = texture2D(mapSamplerLight, vec2(vUv.s, vUv.t)).xyz;"+

		"vec3 normal = normalize( vNormal.xyz );"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderOmnLoop+

		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular+textureLight;"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}

/************************************************************/
/*                                                          */
/*       SHADER SOLID MAP LIGHTDIR LIGHTMAP        */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLdiLma = function () {
	
	Engine.shaderManager.ShaderSolMapLdi.call( this );

}

Engine.shaderManager.ShaderSolMapLdiLma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLdi.prototype );


Engine.shaderManager.ShaderSolMapLdiLma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolMapLdi.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.lightMap,1);

	gl.uniform1i(this.setUniformMapLight, 1);

}

Engine.shaderManager.ShaderSolMapLdiLma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMapLdi.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapLight = gl.getUniformLocation( this.program, "mapSamplerLight" );

}

Engine.shaderManager.ShaderSolMapLdiLma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_DIRECTION_LIGHTS 3 \n"+

	"precision mediump float;"+

	shaderDirInit+
	
	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+
	"uniform sampler2D mapSamplerLight;"+

	"varying vec4 vNormal;"+
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		"gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 textureLight = texture2D(mapSamplerLight, vec2(vUv.s, vUv.t)).xyz;"+

		"vec3 normal = normalize( vNormal.xyz );"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderDirLoop+

		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular+textureLight;"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/*       SHADER SOLID MAP LIGHTOMI LIGHTDIR LIGHTMAP        */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLomLdiLma = function () {
	
	Engine.shaderManager.ShaderSolMapLomLdi.call( this );

}

Engine.shaderManager.ShaderSolMapLomLdiLma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLomLdi.prototype );


Engine.shaderManager.ShaderSolMapLomLdiLma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolMapLomLdi.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.lightMap,1);

	gl.uniform1i(this.setUniformMapLight, 1);

}

Engine.shaderManager.ShaderSolMapLomLdiLma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMapLomLdi.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapLight = gl.getUniformLocation( this.program, "mapSamplerLight" );

}

Engine.shaderManager.ShaderSolMapLomLdiLma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_POINT_LIGHTS "+Engine.conf.MAX_POINT_LIGHTS+" \n"+
	"#define MAX_DIRECTION_LIGHTS "+Engine.conf.MAX_DIRECTION_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderOmnInit+
	shaderDirInit+
	
	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+
	"uniform sampler2D mapSamplerLight;"+

	"varying vec4 vNormal;"+
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		"gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 textureLight = texture2D(mapSamplerLight, vec2(vUv.s, vUv.t)).xyz;"+

		"vec3 normal = normalize( vNormal.xyz );"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderOmnLoop+
		shaderDirLoop+

		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular+textureLight;"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}
}
/************************************************************/
/*                                                          */
/*       SHADER SOLID MAP LIGHTOMI LIGHTDIR NORMALMAP       */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLomLdiNma = function () {
	
	Engine.shaderManager.ShaderSolMapLomLdi.call( this );

}

Engine.shaderManager.ShaderSolMapLomLdiNma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLomLdi.prototype );

Engine.shaderManager.ShaderSolMapLomLdiNma.prototype.prepare = function(scene,lights, mapbuffer){

	gl.useProgram(this.program);
	
	gl.enableVertexAttribArray( this.setAttributVertex );

	gl.enableVertexAttribArray( this.setAttributUv );

	gl.enableVertexAttribArray( this.setAttributNormal );

	gl.enableVertexAttribArray( this.setAttributTangent );

	this.updateCommonUniforms(scene,lights, mapbuffer);

}

Engine.shaderManager.ShaderSolMapLomLdiNma.prototype.updateUniforms = function(object){
	
	Engine.shaderManager.ShaderSolMapLomLdi.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.normalMap,2);

	gl.uniform1i(this.setUniformMapNormal, 2);

}

Engine.shaderManager.ShaderSolMapLomLdiNma.prototype.updateAttributes = function(object){

	Engine.shaderManager.ShaderSolMapLomLdi.prototype.updateAttributes.call( this, object );

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglTangentBuffer);

	gl.vertexAttribPointer( this.setAttributTangent, 3, gl.FLOAT, false, 0, 0);

}

Engine.shaderManager.ShaderSolMapLomLdiNma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMapLomLdi.prototype.buildUniformsAndAttributes.call( this );

	this.setAttributTangent = gl.getAttribLocation( this.program, "vertexTangent" );

	this.setUniformMapNormal = gl.getUniformLocation( this.program, "mapSamplerNormal" );

}


Engine.shaderManager.ShaderSolMapLomLdiNma.prototype.buildVertexShader = function(inherits){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute vec2 vertexUvs;"+
	"attribute vec3 vertexNormal;"+
	"attribute vec3 vertexTangent;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+

	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec3 vNormal;"+
	"varying vec3 vTangent;"+
	"varying vec3 vBinormal;"+

	"void main(void) {"+

		"vUv = vertexUvs;"+
		"vec4 normal = modelMatrix * vec4(vertexNormal, 0);" +
		"vNormal = normal.xyz;" +
		"vec4 tangent = modelMatrix * vec4(vertexTangent, 0);" +
		"vTangent = tangent.xyz;;" +
		"vBinormal = normalize( cross( vNormal, vTangent ) );"+

		"vPosition = modelMatrix * vec4(vertexPosition, 1.0);"+
		"gl_Position = projectionMatrix * viewMatrix * vPosition;"+

	"}";

	if(!inherits){

		var vertexShader = gl.createShader(gl.VERTEX_SHADER);
		gl.shaderSource(vertexShader, code);
		gl.compileShader(vertexShader);
		if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

		return vertexShader;

	}else{
		return code;
	}

}


Engine.shaderManager.ShaderSolMapLomLdiNma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_POINT_LIGHTS "+Engine.conf.MAX_POINT_LIGHTS+" \n"+
	"#define MAX_DIRECTION_LIGHTS "+Engine.conf.MAX_DIRECTION_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderOmnInit+
	shaderDirInit+
	
	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+
	"uniform sampler2D mapSamplerNormal;"+

	
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec3 vNormal;"+
	"varying vec3 vTangent;"+
	"varying vec3 vBinormal;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		"gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 textureNormal = texture2D(mapSamplerNormal, vec2(vUv.s, vUv.t)).xyz * 2.0 - 1.0;"+
		"textureNormal = normalize( textureNormal );"+

		"mat3 tbn = mat3( normalize( vTangent ), normalize( vBinormal ), normalize( vNormal ) );"+
		"vec3 normal = tbn * textureNormal;"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderOmnLoop+
		shaderDirLoop+

		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular;"+
		//"gl_FragColor.xyz = textureNormal.xyz;"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}
}
/************************************************************/
/*                                                          */
/*       SHADER SOLID MAP LIGHTDIR NORMALMAP       */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLdiNma = function () {
	
	Engine.shaderManager.ShaderSolMapLdi.call( this );

}

Engine.shaderManager.ShaderSolMapLdiNma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLdi.prototype );

Engine.shaderManager.ShaderSolMapLdiNma.prototype.prepare = Engine.shaderManager.ShaderSolMapLomLdiNma.prototype.prepare


Engine.shaderManager.ShaderSolMapLdiNma.prototype.updateUniforms = function(object){
	
	Engine.shaderManager.ShaderSolMapLdi.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.normalMap,2);

	gl.uniform1i(this.setUniformMapNormal, 2);

}

Engine.shaderManager.ShaderSolMapLdiNma.prototype.updateAttributes = Engine.shaderManager.ShaderSolMapLomLdiNma.prototype.updateAttributes;

Engine.shaderManager.ShaderSolMapLdiNma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMapLdi.prototype.buildUniformsAndAttributes.call( this );

	this.setAttributTangent = gl.getAttribLocation( this.program, "vertexTangent" );

	this.setUniformMapNormal = gl.getUniformLocation( this.program, "mapSamplerNormal" );
}


Engine.shaderManager.ShaderSolMapLdiNma.prototype.buildVertexShader = function(inherits){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute vec2 vertexUvs;"+
	"attribute vec3 vertexNormal;"+
	"attribute vec3 vertexTangent;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+

	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec3 vNormal;"+
	"varying vec3 vTangent;"+
	"varying vec3 vBinormal;"+

	"void main(void) {"+

		"vUv = vertexUvs;"+
		"vec4 normal = modelMatrix * vec4(vertexNormal, 0);" +
		"vNormal = normal.xyz;" +
		"vec4 tangent = modelMatrix * vec4(vertexTangent, 0);" +
		"vTangent = tangent.xyz;;" +
		"vBinormal = normalize( cross( vNormal, vTangent ) );"+

		"vPosition = modelMatrix * vec4(vertexPosition, 1.0);"+
		"gl_Position = projectionMatrix * viewMatrix * vPosition;"+

	"}";

	if(!inherits){

		var vertexShader = gl.createShader(gl.VERTEX_SHADER);
		gl.shaderSource(vertexShader, code);
		gl.compileShader(vertexShader);
		if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

		return vertexShader;

	}else{
		return code;
	}

}


Engine.shaderManager.ShaderSolMapLdiNma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_DIRECTION_LIGHTS 3 \n"+

	"precision mediump float;"+

	shaderDirInit+
	
	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+
	"uniform sampler2D mapSamplerNormal;"+

	
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec3 vNormal;"+
	"varying vec3 vTangent;"+
	"varying vec3 vBinormal;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		"gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 textureNormal = texture2D(mapSamplerNormal, vec2(vUv.s, vUv.t)).xyz * 2.0 - 1.0;"+

		"mat3 tbn = mat3( normalize( vTangent ), normalize( vBinormal ), normalize( vNormal ) );"+
		"vec3 normal = tbn * textureNormal;"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderDirLoop+

		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular;"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/*       SHADER SOLID MAP LIGHTOMI NORMALMAP       */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLomNma = function () {
	
	Engine.shaderManager.ShaderSolMapLom.call( this );

}

Engine.shaderManager.ShaderSolMapLomNma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLom.prototype );

Engine.shaderManager.ShaderSolMapLomNma.prototype.prepare =  Engine.shaderManager.ShaderSolMapLomLdiNma.prototype.prepare


Engine.shaderManager.ShaderSolMapLomNma.prototype.updateUniforms = function(object){
	
	Engine.shaderManager.ShaderSolMapLom.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.normalMap,2);

	gl.uniform1i(this.setUniformMapNormal, 2);

}

Engine.shaderManager.ShaderSolMapLomNma.prototype.updateAttributes = Engine.shaderManager.ShaderSolMapLomLdiNma.prototype.updateAttributes;

Engine.shaderManager.ShaderSolMapLomNma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMapLom.prototype.buildUniformsAndAttributes.call( this );

	this.setAttributTangent = gl.getAttribLocation( this.program, "vertexTangent" );

	this.setUniformMapNormal = gl.getUniformLocation( this.program, "mapSamplerNormal" );

}


Engine.shaderManager.ShaderSolMapLomNma.prototype.buildVertexShader = function(inherits){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute vec2 vertexUvs;"+
	"attribute vec3 vertexNormal;"+
	"attribute vec3 vertexTangent;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+

	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec3 vNormal;"+
	"varying vec3 vTangent;"+
	"varying vec3 vBinormal;"+

	"void main(void) {"+

		"vUv = vertexUvs;"+
		"vec4 normal = modelMatrix * vec4(vertexNormal, 0);" +
		"vNormal = normal.xyz;" +
		"vec4 tangent = modelMatrix * vec4(vertexTangent, 0);" +
		"vTangent = tangent.xyz;;" +
		"vBinormal = normalize( cross( vNormal, vTangent ) );"+

		"vPosition = modelMatrix * vec4(vertexPosition, 1.0);"+
		"gl_Position = projectionMatrix * viewMatrix * vPosition;"+

	"}";

	if(!inherits){

		var vertexShader = gl.createShader(gl.VERTEX_SHADER);
		gl.shaderSource(vertexShader, code);
		gl.compileShader(vertexShader);
		if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

		return vertexShader;

	}else{
		return code;
	}

}


Engine.shaderManager.ShaderSolMapLomNma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_POINT_LIGHTS "+Engine.conf.MAX_POINT_LIGHTS+" \n"+
	"#define MAX_DIRECTION_LIGHTS "+Engine.conf.MAX_DIRECTION_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderOmnInit+
	shaderDirInit+
	
	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+
	"uniform sampler2D mapSamplerNormal;"+

	
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec3 vNormal;"+
	"varying vec3 vTangent;"+
	"varying vec3 vBinormal;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		"gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 textureNormal = texture2D(mapSamplerNormal, vec2(vUv.s, vUv.t)).xyz * 2.0 - 1.0;"+
		"textureNormal = normalize( textureNormal );"+

		"mat3 tbn = mat3( normalize( vTangent ), normalize( vBinormal ), normalize( vNormal ) );"+
		"vec3 normal = tbn * textureNormal;"+
		//"vec3 normal = normalize( vNormal );"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderOmnLoop+
		shaderDirLoop+

		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular;"+
		//"gl_FragColor.xyz = textureNormal.xyz;"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/* SHADER SOLID MAP LIGHTDIR LIGHTMAP NORMALMAP    */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLdiLmaNma = function () {
	
	Engine.shaderManager.ShaderSolMapLdiNma.call( this );

}

Engine.shaderManager.ShaderSolMapLdiLmaNma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLdiNma.prototype );


Engine.shaderManager.ShaderSolMapLdiLmaNma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolMapLdiNma.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.lightMap,1);

	gl.uniform1i(this.setUniformMapLight, 1);

}

Engine.shaderManager.ShaderSolMapLdiLmaNma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMapLdiNma.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapLight = gl.getUniformLocation( this.program, "mapSamplerLight" );

}


Engine.shaderManager.ShaderSolMapLdiLmaNma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_DIRECTION_LIGHTS 3 \n"+

	"precision mediump float;"+

	shaderDirInit+
	
	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+
	"uniform sampler2D mapSamplerLight;"+
	"uniform sampler2D mapSamplerNormal;"+
	
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec3 vNormal;"+
	"varying vec3 vTangent;"+
	"varying vec3 vBinormal;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		"gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 textureLight = texture2D(mapSamplerLight, vec2(vUv.s, vUv.t)).xyz;"+

		"vec3 textureNormal = texture2D(mapSamplerNormal, vec2(vUv.s, vUv.t)).xyz * 2.0 - 1.0;"+
		"textureNormal = normalize( textureNormal );"+

		"mat3 tbn = mat3( normalize( vTangent ), normalize( vBinormal ), normalize( vNormal ) );"+
		"vec3 normal = tbn * textureNormal;"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderDirLoop+

		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular+textureLight;"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}
}
/************************************************************/
/*                                                          */
/* SHADER SOLID MAP LIGHTOMI LIGHTMAP NORMALMAP             */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLomLmaNma = function () {
	
	Engine.shaderManager.ShaderSolMapLomNma.call( this );

}

Engine.shaderManager.ShaderSolMapLomLmaNma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLomNma.prototype );


Engine.shaderManager.ShaderSolMapLomLmaNma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolMapLomNma.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.lightMap,1);

	gl.uniform1i(this.setUniformMapLight, 1);

}

Engine.shaderManager.ShaderSolMapLomLmaNma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMapLomNma.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapLight = gl.getUniformLocation( this.program, "mapSamplerLight" );

}


Engine.shaderManager.ShaderSolMapLomLmaNma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_POINT_LIGHTS "+Engine.conf.MAX_POINT_LIGHTS+" \n"+
	"#define MAX_DIRECTION_LIGHTS "+Engine.conf.MAX_DIRECTION_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderOmnInit+
	
	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+
	"uniform sampler2D mapSamplerLight;"+
	"uniform sampler2D mapSamplerNormal;"+
	
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec3 vNormal;"+
	"varying vec3 vTangent;"+
	"varying vec3 vBinormal;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		"gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 textureLight = texture2D(mapSamplerLight, vec2(vUv.s, vUv.t)).xyz;"+

		"vec3 textureNormal = texture2D(mapSamplerNormal, vec2(vUv.s, vUv.t)).xyz * 2.0 - 1.0;"+
		"textureNormal = normalize( textureNormal );"+

		"mat3 tbn = mat3( normalize( vTangent ), normalize( vBinormal ), normalize( vNormal ) );"+
		"vec3 normal = tbn * textureNormal;"+


		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderOmnLoop+

		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular+textureLight;"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}
}
/************************************************************/
/*                                                          */
/* SHADER SOLID MAP LIGHTOMI LIGHTDIR LIGHTMAP NORMALMAP    */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLomLdiLmaNma = function () {
	
	Engine.shaderManager.ShaderSolMapLomLdiNma.call( this );

}

Engine.shaderManager.ShaderSolMapLomLdiLmaNma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLomLdiNma.prototype );


Engine.shaderManager.ShaderSolMapLomLdiLmaNma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolMapLomLdiNma.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.lightMap,1);

	gl.uniform1i(this.setUniformMapLight, 1);

}

Engine.shaderManager.ShaderSolMapLomLdiLmaNma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMapLomLdiNma.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapLight = gl.getUniformLocation( this.program, "mapSamplerLight" );

}


Engine.shaderManager.ShaderSolMapLomLdiLmaNma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_POINT_LIGHTS "+Engine.conf.MAX_POINT_LIGHTS+" \n"+
	"#define MAX_DIRECTION_LIGHTS "+Engine.conf.MAX_DIRECTION_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderOmnInit+
	shaderDirInit+
	
	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+
	"uniform sampler2D mapSamplerLight;"+
	"uniform sampler2D mapSamplerNormal;"+
	
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec3 vNormal;"+
	"varying vec3 vTangent;"+
	"varying vec3 vBinormal;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		"gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 textureLight = texture2D(mapSamplerLight, vec2(vUv.s, vUv.t)).xyz;"+

		"vec3 textureNormal = texture2D(mapSamplerNormal, vec2(vUv.s, vUv.t)).xyz * 2.0 - 1.0;"+
		"textureNormal = normalize( textureNormal );"+

		"mat3 tbn = mat3( normalize( vTangent ), normalize( vBinormal ), normalize( vNormal ) );"+
		"vec3 normal = tbn * textureNormal;"+


		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderOmnLoop+
		shaderDirLoop+

		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular+textureLight;"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}
}
/************************************************************/
/*                                                          */
/* SHADER SOLID MAP LIGHTOMI LIGHTDIR MIRORMAP              */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLomLdiMma = function () {
	
	Engine.shaderManager.ShaderSolMapLomLdi.call( this );

}

Engine.shaderManager.ShaderSolMapLomLdiMma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLomLdi.prototype );


Engine.shaderManager.ShaderSolMapLomLdiMma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolMapLomLdi.prototype.updateUniforms.call( this, object );

	this.setTextureCube(object.material.cubeMap,3);

	gl.uniform1i(this.setUniformMapCube, 3);

	gl.uniform1f( this.setUniformReflectivity, object.material.reflectivity );

}

Engine.shaderManager.ShaderSolMapLomLdiMma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMapLomLdi.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapCube = gl.getUniformLocation( this.program, "mapSamplerCube" );

	this.setUniformReflectivity = gl.getUniformLocation( this.program, "reflectivity" );

}

Engine.shaderManager.ShaderSol.prototype.setTextureCube = function ( texture, slot ) {

		if ( texture.needsUpdate ) {

			if ( ! texture.__webglInit ) {

				texture.__webglInit = true;

				texture.__webglTexture = gl.createTexture();
			}

			gl.activeTexture( gl.TEXTURE0 + slot );

			gl.bindTexture( gl.TEXTURE_CUBE_MAP, texture.__webglTexture );


			for(var i=0;i<6;i++){

			gl.texImage2D(gl.TEXTURE_CUBE_MAP_POSITIVE_X+i, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, texture.images[i]);
			gl.texParameteri(gl.TEXTURE_CUBE_MAP, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
            gl.texParameteri(gl.TEXTURE_CUBE_MAP, gl.TEXTURE_MIN_FILTER, gl.LINEAR);

			}

			gl.bindTexture(gl.TEXTURE_CUBE_MAP, null);	


			texture.needsUpdate = false;

		} else {

			gl.activeTexture( gl.TEXTURE0 + slot );

			gl.bindTexture( gl.TEXTURE_CUBE_MAP, texture.__webglTexture );

		}

	};



Engine.shaderManager.ShaderSolMapLomLdiMma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_POINT_LIGHTS "+Engine.conf.MAX_POINT_LIGHTS+" \n"+
	"#define MAX_DIRECTION_LIGHTS "+Engine.conf.MAX_DIRECTION_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderOmnInit+
	shaderDirInit+
	
	shaderMatInit+
	"uniform float reflectivity;"+
	"uniform sampler2D mapSamplerColor;"+
	"uniform samplerCube mapSamplerCube;"+

	"varying vec4 vNormal;"+
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		" gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 normal = normalize( vNormal.xyz );"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderOmnLoop+
		shaderDirLoop+

		"vec3 reflectCoord = reflect(vPosition.xyz - viewPosition, normal);" +
		"vec3 reflectColor = textureCube( mapSamplerCube, reflectCoord ).xyz;"+


		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular*reflectivity;"+

		//"gl_FragColor.xyz =  gl_FragColor.xyz*reflectColor;"+

		"gl_FragColor.xyz = mix( gl_FragColor.xyz,  reflectColor,   reflectivity );"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/* SHADER SOLID MAP LIGHTOMI MIRORMAP    */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLomMma = function () {
	
	Engine.shaderManager.ShaderSolMapLdi.call( this );

}

Engine.shaderManager.ShaderSolMapLomMma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLdi.prototype );


Engine.shaderManager.ShaderSolMapLomMma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolMapLdi.prototype.updateUniforms.call( this, object );

	this.setTextureCube(object.material.cubeMap,3);

	gl.uniform1i(this.setUniformMapCube, 3);

	gl.uniform1f( this.setUniformReflectivity, object.material.reflectivity );

}

Engine.shaderManager.ShaderSolMapLomMma.prototype.buildUniformsAndAttributes = Engine.shaderManager.ShaderSolMapLomLdiMma.prototype.buildUniformsAndAttributes


Engine.shaderManager.ShaderSolMapLomMma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_POINT_LIGHTS "+Engine.conf.MAX_POINT_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderOmnInit+
	
	shaderMatInit+
	"uniform float reflectivity;"+
	"uniform sampler2D mapSamplerColor;"+
	"uniform samplerCube mapSamplerCube;"+

	"varying vec4 vNormal;"+
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		" gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 normal = normalize( vNormal.xyz );"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderOmnLoop+

		"vec3 reflectCoord = reflect(vPosition.xyz - viewPosition, normal);" +
		"vec3 reflectColor = textureCube( mapSamplerCube, reflectCoord ).xyz;"+


		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular*reflectivity;"+

		//"gl_FragColor.xyz =  gl_FragColor.xyz*reflectColor;"+

		"gl_FragColor.xyz = mix( gl_FragColor.xyz,  reflectColor,   reflectivity );"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/* SHADER SOLID MAP LIGHTDIR MIRORMAP   				    */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLdiMma = function () {
	
	Engine.shaderManager.ShaderSolMapLdi.call( this );

}

Engine.shaderManager.ShaderSolMapLdiMma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLdi.prototype );


Engine.shaderManager.ShaderSolMapLdiMma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolMapLdi.prototype.updateUniforms.call( this, object );

	this.setTextureCube(object.material.cubeMap,3);

	gl.uniform1i(this.setUniformMapCube, 3);

	gl.uniform1f( this.setUniformReflectivity, object.material.reflectivity );

}

Engine.shaderManager.ShaderSolMapLdiMma.prototype.buildUniformsAndAttributes = Engine.shaderManager.ShaderSolMapLomLdiMma.prototype.buildUniformsAndAttributes

Engine.shaderManager.ShaderSolMapLdiMma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_DIRECTION_LIGHTS 3 \n"+

	"precision mediump float;"+

	shaderDirInit+
	
	shaderMatInit+
	"uniform float reflectivity;"+
	"uniform sampler2D mapSamplerColor;"+
	"uniform samplerCube mapSamplerCube;"+

	"varying vec4 vNormal;"+
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		" gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 normal = normalize( vNormal.xyz );"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderDirLoop+

		"vec3 reflectCoord = reflect(vPosition.xyz - viewPosition, normal);" +
		"vec3 reflectColor = textureCube( mapSamplerCube, reflectCoord ).xyz;"+


		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular*reflectivity;"+

		//"gl_FragColor.xyz =  gl_FragColor.xyz*reflectColor;"+

		"gl_FragColor.xyz = mix( gl_FragColor.xyz,  reflectColor,   reflectivity );"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}
}
/************************************************************/
/*                                                          */
/* SHADER SOLID MAP LIGHTOMI LIGHTDIR NOMALMAP MIRORMAP     */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLomLdiNmaMma = function () {
	
	Engine.shaderManager.ShaderSolMapLomLdiNma.call( this );

}

Engine.shaderManager.ShaderSolMapLomLdiNmaMma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLomLdiNma.prototype );


Engine.shaderManager.ShaderSolMapLomLdiNmaMma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolMapLomLdiNma.prototype.updateUniforms.call( this, object );

	this.setTextureCube(object.material.cubeMap,3);

	gl.uniform1i(this.setUniformMapCube, 3);

	gl.uniform1f( this.setUniformReflectivity, object.material.reflectivity );

}

Engine.shaderManager.ShaderSolMapLomLdiNmaMma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMapLomLdiNma.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapCube = gl.getUniformLocation( this.program, "mapSamplerCube" );

	this.setUniformReflectivity = gl.getUniformLocation( this.program, "reflectivity" );

}


Engine.shaderManager.ShaderSolMapLomLdiNmaMma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_POINT_LIGHTS "+Engine.conf.MAX_POINT_LIGHTS+" \n"+
	"#define MAX_DIRECTION_LIGHTS "+Engine.conf.MAX_DIRECTION_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderOmnInit+
	shaderDirInit+
	
	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+
	"uniform sampler2D mapSamplerNormal;"+
	"uniform float reflectivity;"+
	"uniform samplerCube mapSamplerCube;"+

	
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec3 vNormal;"+
	"varying vec3 vTangent;"+
	"varying vec3 vBinormal;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		"gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 textureNormal = texture2D(mapSamplerNormal, vec2(vUv.s, vUv.t)).xyz * 2.0 - 1.0;"+
		"textureNormal = normalize( textureNormal );"+

		"mat3 tbn = mat3( normalize( vTangent ), normalize( vBinormal ), normalize( vNormal ) );"+
		"vec3 normal = tbn * textureNormal;"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderOmnLoop+
		shaderDirLoop+

		"vec3 reflectCoord = reflect(vPosition.xyz - viewPosition, normal);" +
		"vec3 reflectColor = textureCube( mapSamplerCube, reflectCoord ).xyz;"+


		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular*reflectivity;"+

		//"gl_FragColor.xyz =  gl_FragColor.xyz*reflectColor;"+

		"gl_FragColor.xyz = mix( gl_FragColor.xyz,  reflectColor,   reflectivity );"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/* SHADER SOLID MAP LIGHTOMI  NOMALMAP MIRORMAP             */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLdiNmaMma = function () {
	
	Engine.shaderManager.ShaderSolMapLomLdiNma.call( this );

}

Engine.shaderManager.ShaderSolMapLdiNmaMma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLomLdiNmaMma.prototype );


Engine.shaderManager.ShaderSolMapLdiNmaMma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolMapLdiNma.prototype.updateUniforms.call( this, object );

	this.setTextureCube(object.material.cubeMap,3);

	gl.uniform1i(this.setUniformMapCube, 3);

	gl.uniform1f( this.setUniformReflectivity, object.material.reflectivity );

}


Engine.shaderManager.ShaderSolMapLdiNmaMma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_DIRECTION_LIGHTS 3 \n"+

	"precision mediump float;"+

	shaderDirInit+
	
	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+
	"uniform sampler2D mapSamplerNormal;"+
	"uniform float reflectivity;"+
	"uniform samplerCube mapSamplerCube;"+

	
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec3 vNormal;"+
	"varying vec3 vTangent;"+
	"varying vec3 vBinormal;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		"gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 textureNormal = texture2D(mapSamplerNormal, vec2(vUv.s, vUv.t)).xyz * 2.0 - 1.0;"+
		"textureNormal = normalize( textureNormal );"+

		"mat3 tbn = mat3( normalize( vTangent ), normalize( vBinormal ), normalize( vNormal ) );"+
		"vec3 normal = tbn * textureNormal;"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderDirLoop+

		"vec3 reflectCoord = reflect(vPosition.xyz - viewPosition, normal);" +
		"vec3 reflectColor = textureCube( mapSamplerCube, reflectCoord ).xyz;"+


		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular*reflectivity;"+

		//"gl_FragColor.xyz =  gl_FragColor.xyz*reflectColor;"+

		"gl_FragColor.xyz = mix( gl_FragColor.xyz,  reflectColor,   reflectivity );"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/* SHADER SOLID MAP LIGHTOMI  NOMALMAP MIRORMAP             */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLomNmaMma = function () {
	
	Engine.shaderManager.ShaderSolMapLomLdiNma.call( this );

}

Engine.shaderManager.ShaderSolMapLomNmaMma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLomLdiNmaMma.prototype );


Engine.shaderManager.ShaderSolMapLomNmaMma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolMapLomNma.prototype.updateUniforms.call( this, object );

	this.setTextureCube(object.material.cubeMap,3);

	gl.uniform1i(this.setUniformMapCube, 3);

	gl.uniform1f( this.setUniformReflectivity, object.material.reflectivity );

}


Engine.shaderManager.ShaderSolMapLomNmaMma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_POINT_LIGHTS "+Engine.conf.MAX_POINT_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderOmnInit+
	
	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+
	"uniform sampler2D mapSamplerNormal;"+
	"uniform float reflectivity;"+
	"uniform samplerCube mapSamplerCube;"+

	
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec3 vNormal;"+
	"varying vec3 vTangent;"+
	"varying vec3 vBinormal;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		"gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 textureNormal = texture2D(mapSamplerNormal, vec2(vUv.s, vUv.t)).xyz * 2.0 - 1.0;"+
		"textureNormal = normalize( textureNormal );"+

		"mat3 tbn = mat3( normalize( vTangent ), normalize( vBinormal ), normalize( vNormal ) );"+
		"vec3 normal = tbn * textureNormal;"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderOmnLoop+

		"vec3 reflectCoord = reflect(vPosition.xyz - viewPosition, normal);" +
		"vec3 reflectColor = textureCube( mapSamplerCube, reflectCoord ).xyz;"+


		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular*reflectivity;"+

		//"gl_FragColor.xyz =  gl_FragColor.xyz*reflectColor;"+

		"gl_FragColor.xyz = mix( gl_FragColor.xyz,  reflectColor,   reflectivity );"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/* SHADER SOLID MAP LIGHTOMI LIGHTDIR NOMALMAP MIRORMAP LIGHTMAP*/
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolMapLomLdiLmaNmaMma = function () {
	
	Engine.shaderManager.ShaderSolMapLomLdiLmaNma.call( this );


}

Engine.shaderManager.ShaderSolMapLomLdiLmaNmaMma.prototype = Object.create( Engine.shaderManager.ShaderSolMapLomLdiLmaNma.prototype );


Engine.shaderManager.ShaderSolMapLomLdiLmaNmaMma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolMapLomLdiLmaNma.prototype.updateUniforms.call( this, object );

	this.setTextureCube(object.material.cubeMap,3);

	gl.uniform1i(this.setUniformMapCube, 3);

	gl.uniform1f( this.setUniformReflectivity, object.material.reflectivity );

}

Engine.shaderManager.ShaderSolMapLomLdiLmaNmaMma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderSolMapLomLdiLmaNma.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapCube = gl.getUniformLocation( this.program, "mapSamplerCube" );

	this.setUniformReflectivity = gl.getUniformLocation( this.program, "reflectivity" );

}


Engine.shaderManager.ShaderSolMapLomLdiLmaNmaMma.prototype.buildFragmentShader = function(inherits){

	var code = ""+

	"#define MAX_POINT_LIGHTS "+Engine.conf.MAX_POINT_LIGHTS+" \n"+
	"#define MAX_DIRECTION_LIGHTS "+Engine.conf.MAX_DIRECTION_LIGHTS+" \n"+

	"precision mediump float;"+

	shaderOmnInit+
	shaderDirInit+
	
	shaderMatInit+
	"uniform sampler2D mapSamplerColor;"+
	"uniform sampler2D mapSamplerNormal;"+
	"uniform sampler2D mapSamplerLight;"+
	"uniform samplerCube mapSamplerCube;"+
	"uniform float reflectivity;"+

	
	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec3 vNormal;"+
	"varying vec3 vTangent;"+
	"varying vec3 vBinormal;"+

	"void main(void) {"+

		"vec4 textureColor = texture2D(mapSamplerColor, vec2(vUv.s, vUv.t));"+
		"gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+

		"vec3 textureNormal = texture2D(mapSamplerNormal, vec2(vUv.s, vUv.t)).xyz * 2.0 - 1.0;"+
		"textureNormal = normalize( textureNormal );"+

		"vec3 textureLight = texture2D(mapSamplerLight, vec2(vUv.s, vUv.t)).xyz;"+

		"mat3 tbn = mat3( normalize( vTangent ), normalize( vBinormal ), normalize( vNormal ) );"+
		"vec3 normal = tbn * textureNormal;"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+
		"vec3 totalSpecular = vec3( 0.0 );"+

		shaderOmnLoop+
		shaderDirLoop+

		"vec3 reflectCoord = reflect(vPosition.xyz - viewPosition, normal);" +
		"vec3 reflectColor = textureCube( mapSamplerCube, reflectCoord ).xyz;"+


		"gl_FragColor.xyz = gl_FragColor.xyz * ( emissive + totalDiffuse + ambientLightColor )+totalSpecular*reflectivity+textureLight;"+

		"gl_FragColor.xyz = mix( gl_FragColor.xyz,  reflectColor,   reflectivity );"+

	"}";

	if(!inherits){

		var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
		gl.shaderSource(fragmentShader, code);
		gl.compileShader(fragmentShader);
		if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

		return fragmentShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/*                      SHADER SOLID PARICLES COLOR         */
/*                                                          */
/************************************************************/


Engine.shaderManager.ShaderSolParCol = function () {
	
	Engine.shaderManager.ShaderSolCol.call( this );

}

Engine.shaderManager.ShaderSolParCol.prototype = Object.create( Engine.shaderManager.ShaderSolCol.prototype );


Engine.shaderManager.ShaderSolParCol.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolCol.prototype.updateUniforms.call( this, object );
	
	gl.uniform1f( this.setUniformSize, object.material.size);

}

Engine.shaderManager.ShaderSolParCol.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderSolCol.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformSize = gl.getUniformLocation( this.program, "size" );

}


Engine.shaderManager.ShaderSolParCol.prototype.draw = function(object){

  gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, object.geometry.attributes.__webglIndexBuffer);

  var clonesLength = object.clones.length;

  if(clonesLength){

  	for(var k=0;k<clonesLength;k++){
		gl.uniformMatrix4fv( this.setUniformVMatrix, false, object.clones[k].elements );
		gl.drawElements(gl.POINT, object.geometry.attributes.__webglIndexBuffer.numItems, gl.UNSIGNED_SHORT, 0);
	}

  }else{

  	gl.uniformMatrix4fv( this.setUniformVMatrix, false, object.matrix.elements );
  	gl.drawElements(gl.POINT, object.geometry.attributes.__webglIndexBuffer.numItems, gl.UNSIGNED_SHORT, 0);

  }

}


Engine.shaderManager.ShaderSolParCol.prototype.buildVertexShader = function(inherits){

	var code =  ""+

	"attribute vec3 vertexPosition;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float size;"+

	"void main(void) {"+
		"vec4 mvPosition =  viewMatrix * modelMatrix * vec4(vertexPosition, 1.0);"+
		"gl_PointSize = size * ( 100.0 / length( mvPosition.xyz ) );"+
		"gl_Position = projectionMatrix * mvPosition;"+

	"}";

	if(!inherits){

		var vertexShader = gl.createShader(gl.VERTEX_SHADER);
		gl.shaderSource(vertexShader, code);
		gl.compileShader(vertexShader);
		if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

		return vertexShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/*                      SHADER SOLID PARICLES MAP           */
/*                                                          */
/************************************************************/


Engine.shaderManager.ShaderSolParMap = function () {
	
	Engine.shaderManager.ShaderSolCol.call( this );

}

Engine.shaderManager.ShaderSolParMap.prototype = Object.create( Engine.shaderManager.ShaderSolCol.prototype );


Engine.shaderManager.ShaderSolParMap.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSolCol.prototype.updateUniforms.call( this, object );
	
	gl.uniform1f( this.setUniformSize, object.material.size);

	this.setTexture(object.material.map,0);

	gl.uniform1i(this.setUniformMapColor, 0);

}

Engine.shaderManager.ShaderSolParMap.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderSolCol.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformSize = gl.getUniformLocation( this.program, "size" );

	this.setUniformMapColor = gl.getUniformLocation( this.program, "mapSamplerColor" );

}


Engine.shaderManager.ShaderSolParMap.prototype.draw = Engine.shaderManager.ShaderSolParCol.prototype.draw;


Engine.shaderManager.ShaderSolParMap.prototype.buildVertexShader = Engine.shaderManager.ShaderSolParCol.prototype.buildVertexShader;  


Engine.shaderManager.ShaderSolParMap.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+

	"uniform sampler2D mapSamplerColor;"+
	"uniform float opacity;"+


	"void main(void) {"+
		"vec4 textureColor = texture2D(mapSamplerColor, gl_PointCoord.xy);"+
		" gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+
	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}
/************************************************************/
/*                                                          */
/*                      SHADER SOLID DEPTH                  */
/*                                                          */
/************************************************************/


var codePack = 	""+
				"vec4 pack (float depth){" +
					"const vec4 bias = vec4(1.0 / 255.0, 1.0 / 255.0, 1.0 / 255.0,  0.0);" +
					"float r = depth;" +
					"float g = fract(r * 255.0);" +
					"float b = fract(g * 255.0);" +
					"float a = fract(b * 255.0);" +
					"vec4 color = vec4(r, g, b, a);" +
					"return color - (color.yzww * bias);"+
				"}";


Engine.shaderManager.ShaderSolDep = function () {
	
	Engine.shaderManager.ShaderSol.call( this );

}

Engine.shaderManager.ShaderSolDep.prototype = Object.create( Engine.shaderManager.ShaderSol.prototype );


Engine.shaderManager.ShaderSolDep.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderSol.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform1f(this.setDepthUniform, scene.camera.far);
	
}

Engine.shaderManager.ShaderSolDep.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderSol.prototype.buildUniformsAndAttributes.call( this );

	this.setDepthUniform = gl.getUniformLocation( this.program, "depth" );

}

Engine.shaderManager.ShaderSolDep.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+

	"varying vec3 vPosition;"+

	"void main(void) {"+

		"vec4 position = viewMatrix * modelMatrix * vec4(vertexPosition, 1.0);"+

		"vPosition = position.xyz;"+

		"gl_Position = projectionMatrix * position;"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}


Engine.shaderManager.ShaderSolDep.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+

	"uniform float depth;"+

	"varying vec3 vPosition;"+

	codePack+

	"void main(void) {"+

		"gl_FragColor = pack(length(vPosition) / depth);"+

	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}


Engine.shaderManager.ShaderSolDepView = function () {
	
	Engine.shaderManager.ShaderSolDep.call( this );

}

Engine.shaderManager.ShaderSolDepView.prototype = Object.create( Engine.shaderManager.ShaderSolDep.prototype );

Engine.shaderManager.ShaderSolDepView.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+

	"uniform float depth;"+

	"varying vec3 vPosition;"+

	codePack+

	"void main(void) {"+

		"gl_FragColor.xyz = vec3( length(vPosition) / depth);"+

	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}
/************************************************************/
/*                                                          */
/*                 SHADER SOLID LINES                       */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSolLin = function () {
	
	Engine.shaderManager.ShaderSolCol.call( this );

}

Engine.shaderManager.ShaderSolLin.prototype = Object.create( Engine.shaderManager.ShaderSolCol.prototype );

Engine.shaderManager.ShaderSolLin.prototype.draw = function(object){

  gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, object.geometry.attributes.__webglIndexBuffer);

  var clonesLength = object.clones.length;

  if(clonesLength){

  	for(var k=0;k<clonesLength;k++){
		gl.uniformMatrix4fv( this.setUniformVMatrix, false, object.clones[k].elements );
		gl.drawElements(gl.LINE_STRIP, object.geometry.attributes.__webglIndexBuffer.numItems, gl.UNSIGNED_SHORT, 0);
	}

  }else{

  	gl.uniformMatrix4fv( this.setUniformVMatrix, false, object.matrix.elements );
  	gl.drawElements(gl.LINE_STRIP, object.geometry.attributes.__webglIndexBuffer.numItems, gl.UNSIGNED_SHORT, 0);

  }

}
/************************************************************/
/*                                                          */
/*                      SHADER MORPH                        */
/*                                                          */
/************************************************************/


Engine.shaderManager.ShaderMor = function () {

	Engine.shaderManager.ShaderSol.call( this );

}

Engine.shaderManager.ShaderMor.prototype = Object.create( Engine.shaderManager.ShaderSol.prototype );

Engine.shaderManager.ShaderMor.prototype.prepare = function(scene,lights, mapbuffer){

	gl.useProgram(this.program);

	gl.enableVertexAttribArray( this.setAttributVertex1 );

	gl.enableVertexAttribArray( this.setAttributVertex2 );

	this.updateCommonUniforms(scene,lights, mapbuffer);

}

Engine.shaderManager.ShaderMor.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderSol.prototype.updateUniforms.call( this, object );

	gl.uniform1f( this.setUniformProgress, object.progress );

}

Engine.shaderManager.ShaderMor.prototype.updateAttributes = function(object){

	//attributes.__webglMorphNormalsBuffers[ vk ]

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglMorphTargetsBuffers[ object.keyA.index ]);

	gl.vertexAttribPointer( this.setAttributVertex1, 3, gl.FLOAT, false, 0, 0);

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglMorphTargetsBuffers[ object.keyB.index ]);

	gl.vertexAttribPointer( this.setAttributVertex2, 3, gl.FLOAT, false, 0, 0);
}

Engine.shaderManager.ShaderMor.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderSol.prototype.buildUniformsAndAttributes.call( this );

	this.setAttributVertex1 = gl.getAttribLocation( this.program, "vertexPosition1" );

	this.setAttributVertex2 = gl.getAttribLocation( this.program, "vertexPosition2" );

	this.setUniformProgress = gl.getUniformLocation( this.program, "progress" );
	
}

Engine.shaderManager.ShaderMor.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition1;"+
	"attribute vec3 vertexPosition2;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float progress;"+

	"varying vec4 vPosition;"+

	"void main(void) {"+

		"vPosition = modelMatrix * vec4(vertexPosition1*(1.0-progress)+vertexPosition2*progress, 1.0);"+
		"gl_Position = projectionMatrix * viewMatrix * vPosition;"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}
/************************************************************/
/*                                                          */
/*                      SHADER MORPH COLOR                  */
/*                                                          */
/************************************************************/


Engine.shaderManager.ShaderMorCol = function () {
	
	Engine.shaderManager.ShaderMor.call( this );

}

Engine.shaderManager.ShaderMorCol.prototype = Object.create( Engine.shaderManager.ShaderMor.prototype );


Engine.shaderManager.ShaderMorCol.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMor.prototype.updateUniforms.call( this, object );
	
	gl.uniform3fv( this.setUniformDiffuse, object.material.color.toArray());

}

Engine.shaderManager.ShaderMorCol.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderMor.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformDiffuse = gl.getUniformLocation( this.program, "diffuse" );

}


Engine.shaderManager.ShaderMorCol.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolCol.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/*                       COLOR MATERIAL                     */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorColMat = function () {
	
	Engine.shaderManager.ShaderMorCol.call( this );

}

Engine.shaderManager.ShaderMorColMat.prototype = Object.create( Engine.shaderManager.ShaderMorCol.prototype );

Engine.shaderManager.ShaderMorColMat.prototype.prepare = function(scene,lights, mapbuffer){

	gl.useProgram(this.program);

	gl.enableVertexAttribArray( this.setAttributVertex1 );

	gl.enableVertexAttribArray( this.setAttributVertex2 );

	gl.enableVertexAttribArray( this.setAttributNormal1 );

	gl.enableVertexAttribArray( this.setAttributNormal2 );

	gl.enableVertexAttribArray( this.setAttributUv );

	this.updateCommonUniforms(scene,lights, mapbuffer);

}

Engine.shaderManager.ShaderMorColMat.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorCol.prototype.updateUniforms.call( this, object );

	gl.uniform3fv( this.setUniformEmissive, object.material.emissive.toArray() );

	gl.uniform1f( this.setUniformShininess, object.material.shininess );

}

Engine.shaderManager.ShaderMorColMat.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderMorCol.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformViewPosition, scene.camera.position.toArray() );

	gl.uniform3fv( this.setUniformAmbientLightColor, light.ambient );

}

Engine.shaderManager.ShaderMorColMat.prototype.updateAttributes = function(object){
	
	Engine.shaderManager.ShaderMorCol.prototype.updateAttributes.call( this, object );

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglMorphNormalsBuffers[ object.keyA.index ]);

	gl.vertexAttribPointer( this.setAttributNormal1, 3, gl.FLOAT, false, 0, 0);

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglMorphNormalsBuffers[ object.keyB.index ]);

	gl.vertexAttribPointer( this.setAttributNormal2, 3, gl.FLOAT, false, 0, 0);
	
}

Engine.shaderManager.ShaderMorColMat.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorCol.prototype.buildUniformsAndAttributes.call( this );

	this.setAttributNormal1 = gl.getAttribLocation( this.program, "vertexNormal1" );

	this.setAttributNormal2 = gl.getAttribLocation( this.program, "vertexNormal2" );

	this.setUniformViewPosition = gl.getUniformLocation( this.program, "viewPosition" );

	this.setUniformAmbientLightColor = gl.getUniformLocation( this.program, "ambientLightColor" );

	this.setUniformEmissive = gl.getUniformLocation( this.program, "emissive" );

	this.setUniformShininess = gl.getUniformLocation( this.program, "shininess" );

}

Engine.shaderManager.ShaderMorColMat.prototype.buildVertexShader = function(inherits){


	var code =  ""+

	"attribute vec3 vertexPosition1;"+
	"attribute vec3 vertexPosition2;"+
	"attribute vec3 vertexNormal1;"+
	"attribute vec3 vertexNormal2;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float progress;"+

	"varying vec4 vPosition;"+
	"varying vec4 vNormal;"+

	"void main(void) {"+

		"vNormal = modelMatrix * vec4(vertexNormal1*(1.0-progress)+vertexNormal2*progress, 0.0);"+
		"vPosition = modelMatrix * vec4(vertexPosition1*(1.0-progress)+vertexPosition2*progress, 1.0);"+
		"gl_Position = projectionMatrix * viewMatrix * vPosition;"+

	"}";

	if(!inherits){

		var vertexShader = gl.createShader(gl.VERTEX_SHADER);
		gl.shaderSource(vertexShader, code);
		gl.compileShader(vertexShader);
		if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

		return vertexShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/*                      SHADER MORPH COLOR LIGHTOMNI        */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorColLom = function () {
	
	Engine.shaderManager.ShaderMorColMat.call( this );

}

Engine.shaderManager.ShaderMorColLom.prototype = Object.create( Engine.shaderManager.ShaderMorColMat.prototype );


Engine.shaderManager.ShaderMorColLom.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderMorColMat.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformPointLightColor, light.point.colors );

	gl.uniform3fv( this.setUniformPointLightPosition, light.point.positions);

	gl.uniform1i( this.setUniformNumberOfOmniLight, light.point.length );

	gl.uniform1fv( this.setUniformpPointLightDistance, light.point.distances );

}

Engine.shaderManager.ShaderMorColLom.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorColMat.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformPointLightColor = gl.getUniformLocation( this.program, "pointLightColor" );

	this.setUniformpPointLightDistance = gl.getUniformLocation( this.program, "pointLightDistance" );

	this.setUniformPointLightPosition = gl.getUniformLocation( this.program, "pointLightPosition" );

	this.setUniformNumberOfOmniLight = gl.getUniformLocation( this.program, "numberOfOmniLight" );

}

Engine.shaderManager.ShaderMorColLom.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolColLom.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/*             SHADER MORPH COLOR LIGHTDIR                  */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorColLdi = function () {

	Engine.shaderManager.ShaderMorColMat.call( this );

}

Engine.shaderManager.ShaderMorColLdi.prototype = Object.create( Engine.shaderManager.ShaderMorColMat.prototype );

Engine.shaderManager.ShaderMorColLdi.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderMorColMat.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformDirectionLightColor,  light.directional.colors);

	gl.uniform3fv( this.setUniformpDirectionLightVector,  light.directional.direction );

	gl.uniform1i( this.setUniformNumberOfDirectionLight, light.directional.length );

}

Engine.shaderManager.ShaderMorColLdi.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorColMat.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformDirectionLightColor = gl.getUniformLocation( this.program, "directionLightColor" );

	this.setUniformpDirectionLightVector = gl.getUniformLocation( this.program, "directionLightVector" );

	this.setUniformNumberOfDirectionLight = gl.getUniformLocation( this.program, "numberOfDirectionLight" );

}

Engine.shaderManager.ShaderMorColLdi.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolColLdi.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/*             SHADER MORPH COL LIGHTOMI LIGHTDIR           */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorColLomLdi = function () {
	
	Engine.shaderManager.ShaderMorColLom.call( this );

}

Engine.shaderManager.ShaderMorColLomLdi.prototype = Object.create( Engine.shaderManager.ShaderMorColLom.prototype );


Engine.shaderManager.ShaderMorColLomLdi.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderMorColLom.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformDirectionLightColor,  light.directional.colors);

	gl.uniform3fv( this.setUniformpDirectionLightVector,  light.directional.direction );

	gl.uniform1i( this.setUniformNumberOfDirectionLight, light.directional.length );

}

Engine.shaderManager.ShaderMorColLomLdi.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorColLom.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformDirectionLightColor = gl.getUniformLocation( this.program, "directionLightColor" );

	this.setUniformpDirectionLightVector = gl.getUniformLocation( this.program, "directionLightVector" );

	this.setUniformNumberOfDirectionLight = gl.getUniformLocation( this.program, "numberOfDirectionLight" );

}

Engine.shaderManager.ShaderMorColLomLdi.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolColLomLdi.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/*                      SHADER MORPH MAP                  */
/*                                                          */
/************************************************************/


Engine.shaderManager.ShaderMorMap = function () {
	
	Engine.shaderManager.ShaderMor.call( this );

}

Engine.shaderManager.ShaderMorMap.prototype = Object.create( Engine.shaderManager.ShaderMor.prototype );

Engine.shaderManager.ShaderMorMap.prototype.prepare = function(scene,lights, mapbuffer){

	gl.useProgram(this.program);

	gl.enableVertexAttribArray( this.setAttributVertex1 );

	gl.enableVertexAttribArray( this.setAttributVertex2 );

	gl.enableVertexAttribArray( this.setAttributUv );

	this.updateCommonUniforms(scene,lights, mapbuffer);

}


Engine.shaderManager.ShaderMorMap.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMor.prototype.updateUniforms.call( this, object );
	
	this.setTexture(object.material.map,0);

	gl.uniform1i(this.setUniformMapColor, 0);

}

Engine.shaderManager.ShaderMorMap.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderMor.prototype.buildUniformsAndAttributes.call( this );

	this.setAttributUv = gl.getAttribLocation( this.program, "vertexUvs" );

	this.setUniformMapColor = gl.getUniformLocation( this.program, "mapSamplerColor" );

}

Engine.shaderManager.ShaderMorMap.prototype.updateAttributes = function(object){

	Engine.shaderManager.ShaderMor.prototype.updateAttributes.call( this, object );

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglUVBuffer);

	gl.vertexAttribPointer( this.setAttributUv, 2, gl.FLOAT, false, 0, 0);
	
}

Engine.shaderManager.ShaderMorMap.prototype.setTexture = Engine.shaderManager.ShaderSol.prototype.setTexture

Engine.shaderManager.ShaderMorMap.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition1;"+
	"attribute vec3 vertexPosition2;"+
	"attribute vec2 vertexUvs;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float progress;"+

	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+

	"void main(void) {"+

		"vUv = vertexUvs;"+
		"vPosition = modelMatrix * vec4(vertexPosition1*(1.0-progress)+vertexPosition2*progress, 1.0);"+
		"gl_Position = projectionMatrix * viewMatrix * vPosition;"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}


Engine.shaderManager.ShaderMorMap.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMap.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/*                         MAP MATERIAL                     */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapMat = function () {
	
	Engine.shaderManager.ShaderMorMap.call( this );

}

Engine.shaderManager.ShaderMorMapMat.prototype = Object.create( Engine.shaderManager.ShaderMorMap.prototype );

Engine.shaderManager.ShaderMorMapMat.prototype.prepare = function(scene,lights, mapbuffer){

	gl.useProgram(this.program);

	gl.enableVertexAttribArray( this.setAttributVertex1 );

	gl.enableVertexAttribArray( this.setAttributVertex2 );

	gl.enableVertexAttribArray( this.setAttributNormal1 );

	gl.enableVertexAttribArray( this.setAttributNormal2 );

	gl.enableVertexAttribArray( this.setAttributUv );

	this.updateCommonUniforms(scene,lights, mapbuffer);

}

Engine.shaderManager.ShaderMorMapMat.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorMap.prototype.updateUniforms.call( this, object );

	gl.uniform3fv( this.setUniformEmissive, object.material.emissive.toArray() );

	gl.uniform1f( this.setUniformShininess, object.material.shininess );

}

Engine.shaderManager.ShaderMorMapMat.prototype.updateCommonUniforms = Engine.shaderManager.ShaderMorColMat.prototype.updateCommonUniforms

Engine.shaderManager.ShaderMorMapMat.prototype.updateAttributes = function(object){
	
	Engine.shaderManager.ShaderMorMap.prototype.updateAttributes.call( this, object );

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglMorphNormalsBuffers[ object.keyA.index ]);

	gl.vertexAttribPointer( this.setAttributNormal1, 3, gl.FLOAT, false, 0, 0);

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglMorphNormalsBuffers[ object.keyB.index ]);

	gl.vertexAttribPointer( this.setAttributNormal2, 3, gl.FLOAT, false, 0, 0);
	
}

Engine.shaderManager.ShaderMorMapMat.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorMap.prototype.buildUniformsAndAttributes.call( this );

	this.setAttributNormal1 = gl.getAttribLocation( this.program, "vertexNormal1" );

	this.setAttributNormal2 = gl.getAttribLocation( this.program, "vertexNormal2" );

	this.setUniformViewPosition = gl.getUniformLocation( this.program, "viewPosition" );

	this.setUniformAmbientLightColor = gl.getUniformLocation( this.program, "ambientLightColor" );

	this.setUniformEmissive = gl.getUniformLocation( this.program, "emissive" );

	this.setUniformShininess = gl.getUniformLocation( this.program, "shininess" );

}

Engine.shaderManager.ShaderMorMapMat.prototype.buildVertexShader = function(inherits){

	var code =  ""+

	"attribute vec3 vertexPosition1;"+
	"attribute vec3 vertexPosition2;"+
	"attribute vec3 vertexNormal1;"+
	"attribute vec3 vertexNormal2;"+
	"attribute vec2 vertexUvs;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float progress;"+

	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec4 vNormal;"+


	"void main(void) {"+

		"vUv = vertexUvs;"+
		"vNormal = modelMatrix * vec4(vertexNormal1*(1.0-progress)+vertexNormal2*progress, 0.0);"+
		"vPosition = modelMatrix * vec4(vertexPosition1*(1.0-progress)+vertexPosition2*progress, 1.0);"+
		"gl_Position = projectionMatrix * viewMatrix * vPosition;"+

	"}";

	if(!inherits){

		var vertexShader = gl.createShader(gl.VERTEX_SHADER);
		gl.shaderSource(vertexShader, code);
		gl.compileShader(vertexShader);
		if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

		return vertexShader;

	}else{
		return code;
	}

}
/************************************************************/
/*                                                          */
/*             SHADER MORPH COLOR LIGHTDIR                  */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLdi = function () {

	Engine.shaderManager.ShaderMorMapMat.call( this );

}

Engine.shaderManager.ShaderMorMapLdi.prototype = Object.create( Engine.shaderManager.ShaderMorMapMat.prototype );

Engine.shaderManager.ShaderMorMapLdi.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderMorMapMat.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformDirectionLightColor,  light.directional.colors);

	gl.uniform3fv( this.setUniformpDirectionLightVector,  light.directional.direction );

	gl.uniform1i( this.setUniformNumberOfDirectionLight, light.directional.length );

}

Engine.shaderManager.ShaderMorMapLdi.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorMapMat.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformDirectionLightColor = gl.getUniformLocation( this.program, "directionLightColor" );

	this.setUniformpDirectionLightVector = gl.getUniformLocation( this.program, "directionLightVector" );

	this.setUniformNumberOfDirectionLight = gl.getUniformLocation( this.program, "numberOfDirectionLight" );

}


Engine.shaderManager.ShaderMorMapLdi.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolColLdi.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/*                      SHADER MORPH MAP LIGHTOMNI          */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLom = function () {
	
	Engine.shaderManager.ShaderMorMapMat.call( this );

}

Engine.shaderManager.ShaderMorMapLom.prototype = Object.create( Engine.shaderManager.ShaderMorMapMat.prototype );


Engine.shaderManager.ShaderMorMapLom.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderMorMapMat.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformPointLightColor, light.point.colors );

	gl.uniform3fv( this.setUniformPointLightPosition, light.point.positions);

	gl.uniform1i( this.setUniformNumberOfOmniLight, light.point.length );

	gl.uniform1fv( this.setUniformpPointLightDistance, light.point.distances );

}

Engine.shaderManager.ShaderMorMapLom.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorMapMat.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformPointLightColor = gl.getUniformLocation( this.program, "pointLightColor" );

	this.setUniformpPointLightDistance = gl.getUniformLocation( this.program, "pointLightDistance" );

	this.setUniformPointLightPosition = gl.getUniformLocation( this.program, "pointLightPosition" );

	this.setUniformNumberOfOmniLight = gl.getUniformLocation( this.program, "numberOfOmniLight" );

}

Engine.shaderManager.ShaderMorMapLom.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLom.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/*             SHADER MORPH MAP LIGHTOMI LIGHTDIR           */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLomLdi = function () {
	
	Engine.shaderManager.ShaderMorMapLom.call( this );

}

Engine.shaderManager.ShaderMorMapLomLdi.prototype = Object.create( Engine.shaderManager.ShaderMorMapLom.prototype );


Engine.shaderManager.ShaderMorMapLomLdi.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderMorMapLom.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformDirectionLightColor,  light.directional.colors);

	gl.uniform3fv( this.setUniformpDirectionLightVector,  light.directional.direction );

	gl.uniform1i( this.setUniformNumberOfDirectionLight, light.directional.length );

}

Engine.shaderManager.ShaderMorMapLomLdi.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorMapLom.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformDirectionLightColor = gl.getUniformLocation( this.program, "directionLightColor" );

	this.setUniformpDirectionLightVector = gl.getUniformLocation( this.program, "directionLightVector" );

	this.setUniformNumberOfDirectionLight = gl.getUniformLocation( this.program, "numberOfDirectionLight" );

}

Engine.shaderManager.ShaderMorMapLomLdi.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLomLdi.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/*       SHADER MORPH MAP LIGHTOMI LIGHTMAP        */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLomLma = function () {
	
	Engine.shaderManager.ShaderMorMapLom.call( this );

}

Engine.shaderManager.ShaderMorMapLomLma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLom.prototype );


Engine.shaderManager.ShaderMorMapLomLma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorMapLom.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.lightMap,1);

	gl.uniform1i(this.setUniformMapLight, 1);

}

Engine.shaderManager.ShaderMorMapLomLma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorMapLom.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapLight = gl.getUniformLocation( this.program, "mapSamplerLight" );

}

Engine.shaderManager.ShaderMorMapLomLma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLomLma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/*       SHADER MORPH MAP LIGHTDIR LIGHTMAP                 */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLdiLma = function () {
	
	Engine.shaderManager.ShaderMorMapLdi.call( this );

}

Engine.shaderManager.ShaderMorMapLdiLma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLdi.prototype );


Engine.shaderManager.ShaderMorMapLdiLma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorMapLdi.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.lightMap,1);

	gl.uniform1i(this.setUniformMapLight, 1);

}

Engine.shaderManager.ShaderMorMapLdiLma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorMapLdi.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapLight = gl.getUniformLocation( this.program, "mapSamplerLight" );

}

Engine.shaderManager.ShaderMorMapLdiLma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLdiLma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/*       SHADER MORPH MAP LIGHTOMI LIGHTDIR LIGHTMAP        */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLomLdiLma = function () {
	
	Engine.shaderManager.ShaderMorMapLomLdi.call( this );

}

Engine.shaderManager.ShaderMorMapLomLdiLma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLomLdi.prototype );


Engine.shaderManager.ShaderMorMapLomLdiLma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorMapLomLdi.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.lightMap,1);

	gl.uniform1i(this.setUniformMapLight, 1);

}

Engine.shaderManager.ShaderMorMapLomLdiLma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorMapLomLdi.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapLight = gl.getUniformLocation( this.program, "mapSamplerLight" );

}

Engine.shaderManager.ShaderMorMapLomLdiLma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLomLdiLma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/*       SHADER MORPH MAP LIGHTOMI LIGHTDIR NORMALMAP       */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLomLdiNma = function () {
	
	Engine.shaderManager.ShaderMorMapLomLdi.call( this );

}

Engine.shaderManager.ShaderMorMapLomLdiNma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLomLdi.prototype );

Engine.shaderManager.ShaderMorMapLomLdiNma.prototype.prepare = function(scene,lights, mapbuffer){

	Engine.shaderManager.ShaderMorMapLomLdi.prototype.prepare.call( this, scene,lights );

	gl.enableVertexAttribArray( this.setAttributTangent1 );

	gl.enableVertexAttribArray( this.setAttributTangent2 );

}

Engine.shaderManager.ShaderMorMapLomLdiNma.prototype.updateUniforms = function(object){
	
	Engine.shaderManager.ShaderMorMapLomLdi.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.normalMap,2);

	gl.uniform1i(this.setUniformMapNormal, 2);

}

Engine.shaderManager.ShaderMorMapLomLdiNma.prototype.updateAttributes = function(object){

	Engine.shaderManager.ShaderMorMapLomLdi.prototype.updateAttributes.call( this, object );

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglMorphTangentsBuffer[ object.keyA.index ]);

	gl.vertexAttribPointer( this.setAttributTangent1, 3, gl.FLOAT, false, 0, 0);

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglMorphTangentsBuffer[ object.keyB.index ]);

	gl.vertexAttribPointer( this.setAttributTangent2, 3, gl.FLOAT, false, 0, 0);

}

Engine.shaderManager.ShaderMorMapLomLdiNma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorMapLomLdi.prototype.buildUniformsAndAttributes.call( this );

	this.setAttributTangent1 = gl.getAttribLocation( this.program, "vertexTangent1" );

	this.setAttributTangent2 = gl.getAttribLocation( this.program, "vertexTangent2" );

	this.setUniformMapNormal = gl.getUniformLocation( this.program, "mapSamplerNormal" );

}


Engine.shaderManager.ShaderMorMapLomLdiNma.prototype.buildVertexShader = function(inherits){

	var code =  ""+

	"attribute vec3 vertexPosition1;"+
	"attribute vec3 vertexPosition2;"+
	"attribute vec3 vertexNormal1;"+
	"attribute vec3 vertexNormal2;"+
	"attribute vec3 vertexTangent1;"+
	"attribute vec3 vertexTangent2;"+
	"attribute vec2 vertexUvs;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float progress;"+


	"varying vec4 vPosition;"+
	"varying vec2 vUv;"+
	"varying vec3 vNormal;"+
	"varying vec3 vTangent;"+
	"varying vec3 vBinormal;"+

	"void main(void) {"+

		"vUv = vertexUvs;"+
		"vec4 normal =  modelMatrix * vec4(vertexNormal1*(1.0-progress)+vertexNormal2*progress, 0.0);" +
		"vNormal = normal.xyz;" +
		"vec4 tangent = modelMatrix * vec4(vertexTangent1*(1.0-progress)+vertexTangent2*progress, 0.0);" +
		"vTangent = tangent.xyz;;" +
		"vPosition = modelMatrix * vec4(vertexPosition1*(1.0-progress)+vertexPosition2*progress, 1.0);"+
		"vBinormal = normalize( cross( vNormal, vTangent ) );"+
		"gl_Position = projectionMatrix * viewMatrix * vPosition;"+


	"}";

	if(!inherits){

		var vertexShader = gl.createShader(gl.VERTEX_SHADER);
		gl.shaderSource(vertexShader, code);
		gl.compileShader(vertexShader);
		if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

		return vertexShader;

	}else{
		return code;
	}

}


Engine.shaderManager.ShaderMorMapLomLdiNma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLomLdiNma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/*       SHADER MORPH MAP LIGHTDIR NORMALMAP                */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLdiNma = function () {
	
	Engine.shaderManager.ShaderMorMapLomLdiNma.call( this );

}

Engine.shaderManager.ShaderMorMapLdiNma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLomLdiNma.prototype );

Engine.shaderManager.ShaderMorMapLdiNma.prototype.updateCommonUniforms  = Engine.shaderManager.ShaderMorMapLdi.prototype.updateCommonUniforms;

Engine.shaderManager.ShaderMorMapLdiNma.prototype.updateUniforms = function(object){
	
	Engine.shaderManager.ShaderMorMapLdi.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.normalMap,2);

	gl.uniform1i(this.setUniformMapNormal, 2);

}


Engine.shaderManager.ShaderMorMapLdiNma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLdiNma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/*       SHADER MORPH MAP LIGHTOMI NORMALMAP                */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLomNma = function () {
	
	Engine.shaderManager.ShaderMorMapLomLdiNma.call( this );

}

Engine.shaderManager.ShaderMorMapLomNma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLomLdiNma.prototype );

Engine.shaderManager.ShaderMorMapLomNma.prototype.updateCommonUniforms  = Engine.shaderManager.ShaderMorMapLom.prototype.updateCommonUniforms;

Engine.shaderManager.ShaderMorMapLomNma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorMapLom.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.normalMap,2);

	gl.uniform1i(this.setUniformMapNormal, 2);

}

Engine.shaderManager.ShaderMorMapLomNma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLomNma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/* SHADER MORPH MAP LIGHTDIR LIGHTMAP NORMALMAP    */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLdiLmaNma = function () {
	
	Engine.shaderManager.ShaderMorMapLdiNma.call( this );

}

Engine.shaderManager.ShaderMorMapLdiLmaNma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLdiNma.prototype );


Engine.shaderManager.ShaderMorMapLdiLmaNma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorMapLdiNma.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.lightMap,1);

	gl.uniform1i(this.setUniformMapLight, 1);

}

Engine.shaderManager.ShaderMorMapLdiLmaNma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorMapLdiNma.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapLight = gl.getUniformLocation( this.program, "mapSamplerLight" );

}


Engine.shaderManager.ShaderMorMapLdiLmaNma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLdiLmaNma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/* SHADER MORPH MAP LIGHTOMI LIGHTMAP NORMALMAP             */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLomLmaNma = function () {
	
	Engine.shaderManager.ShaderMorMapLomNma.call( this );

}

Engine.shaderManager.ShaderMorMapLomLmaNma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLomNma.prototype );


Engine.shaderManager.ShaderMorMapLomLmaNma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorMapLomNma.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.lightMap,1);

	gl.uniform1i(this.setUniformMapLight, 1);

}

Engine.shaderManager.ShaderMorMapLomLmaNma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorMapLomNma.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapLight = gl.getUniformLocation( this.program, "mapSamplerLight" );

}

Engine.shaderManager.ShaderMorMapLomLmaNma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLomLmaNma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/* SHADER MORPH MAP LIGHTOMI LIGHTDIR LIGHTMAP NORMALMAP    */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLomLdiLmaNma = function () {
	
	Engine.shaderManager.ShaderMorMapLomLdiNma.call( this );

}

Engine.shaderManager.ShaderMorMapLomLdiLmaNma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLomLdiNma.prototype );


Engine.shaderManager.ShaderMorMapLomLdiLmaNma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorMapLomLdiNma.prototype.updateUniforms.call( this, object );

	this.setTexture(object.material.lightMap,1);

	gl.uniform1i(this.setUniformMapLight, 1);

}

Engine.shaderManager.ShaderMorMapLomLdiLmaNma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorMapLomLdiNma.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapLight = gl.getUniformLocation( this.program, "mapSamplerLight" );

}


Engine.shaderManager.ShaderMorMapLomLdiLmaNma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLomLdiLmaNma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/* SHADER MORPH MAP LIGHTOMI LIGHTDIR MIRORMAP              */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLomLdiMma = function () {
	
	Engine.shaderManager.ShaderMorMapLomLdi.call( this );

}

Engine.shaderManager.ShaderMorMapLomLdiMma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLomLdi.prototype );


Engine.shaderManager.ShaderMorMapLomLdiMma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorMapLomLdi.prototype.updateUniforms.call( this, object );

	this.setTextureCube(object.material.cubeMap,3);

	gl.uniform1i(this.setUniformMapCube, 3);

	gl.uniform1f( this.setUniformReflectivity, object.material.reflectivity );

}
 
Engine.shaderManager.ShaderMorMapLomLdiMma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorMapLomLdi.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapCube = gl.getUniformLocation( this.program, "mapSamplerCube" );

	this.setUniformReflectivity = gl.getUniformLocation( this.program, "reflectivity" );

}


Engine.shaderManager.ShaderMorMapLomLdiMma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLomLdiMma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/* SHADER MORPH MAP LIGHTOMNI MIRORMAP      			    */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLomMma = function () {
	
	Engine.shaderManager.ShaderMorMapLom.call( this );

}

Engine.shaderManager.ShaderMorMapLomMma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLom.prototype );


Engine.shaderManager.ShaderMorMapLomMma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorMapLom.prototype.updateUniforms.call( this, object );

	this.setTextureCube(object.material.cubeMap,3);

	gl.uniform1i(this.setUniformMapCube, 3);

	gl.uniform1f( this.setUniformReflectivity, object.material.reflectivity );

}
 
Engine.shaderManager.ShaderMorMapLomMma.prototype.buildUniformsAndAttributes = Engine.shaderManager.ShaderMorMapLomLdiMma.prototype.buildUniformsAndAttributes;
		

Engine.shaderManager.ShaderMorMapLomMma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLomMma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/* SHADER MORPH MAP LIGHTDIR MIRORMAP      			        */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLdiMma = function () {
	
	Engine.shaderManager.ShaderMorMapLdi.call( this );

}

Engine.shaderManager.ShaderMorMapLdiMma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLdi.prototype );


Engine.shaderManager.ShaderMorMapLdiMma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorMapLdi.prototype.updateUniforms.call( this, object );

	this.setTextureCube(object.material.cubeMap,3);

	gl.uniform1i(this.setUniformMapCube, 3);

	gl.uniform1f( this.setUniformReflectivity, object.material.reflectivity );

}
 
Engine.shaderManager.ShaderMorMapLdiMma.prototype.buildUniformsAndAttributes = Engine.shaderManager.ShaderMorMapLomLdiMma.prototype.buildUniformsAndAttributes;
		

Engine.shaderManager.ShaderMorMapLdiMma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLdiMma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/* SHADER MORPH MAP LIGHTOMI LIGHTDIR NOMALMAP MIRORMAP     */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLomLdiNmaMma = function () {
	
	Engine.shaderManager.ShaderMorMapLomLdiNma.call( this );

}

Engine.shaderManager.ShaderMorMapLomLdiNmaMma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLomLdiNma.prototype );


Engine.shaderManager.ShaderMorMapLomLdiNmaMma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorMapLomLdiNma.prototype.updateUniforms.call( this, object );

	this.setTextureCube(object.material.cubeMap,3);

	gl.uniform1i(this.setUniformMapCube, 3);

	gl.uniform1f( this.setUniformReflectivity, object.material.reflectivity );

}

Engine.shaderManager.ShaderMorMapLomLdiNmaMma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorMapLomLdiNma.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapCube = gl.getUniformLocation( this.program, "mapSamplerCube" );

	this.setUniformReflectivity = gl.getUniformLocation( this.program, "reflectivity" );

}

Engine.shaderManager.ShaderMorMapLomLdiNmaMma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLomLdiNmaMma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/* SHADER MORPH MAP LIGHTDIR  NOMALMAP MIRORMAP             */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLdiNmaMma = function () {
	
	Engine.shaderManager.ShaderMorMapLomLdiNma.call( this );

}

Engine.shaderManager.ShaderMorMapLdiNmaMma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLomLdiNmaMma.prototype );


Engine.shaderManager.ShaderMorMapLdiNmaMma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorMapLdiNma.prototype.updateUniforms.call( this, object );

	this.setTextureCube(object.material.cubeMap,3);

	gl.uniform1i(this.setUniformMapCube, 3);

	gl.uniform1f( this.setUniformReflectivity, object.material.reflectivity );

}


Engine.shaderManager.ShaderMorMapLdiNmaMma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLdiNmaMma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/* SHADER MORPH MAP LIGHTOMI NOMALMAP MIRORMAP             */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLomNmaMma = function () {
	
	Engine.shaderManager.ShaderMorMapLomLdiNma.call( this );

}

Engine.shaderManager.ShaderMorMapLomNmaMma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLomLdiNmaMma.prototype );


Engine.shaderManager.ShaderMorMapLomNmaMma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorMapLomNma.prototype.updateUniforms.call( this, object );

	this.setTextureCube(object.material.cubeMap,3);

	gl.uniform1i(this.setUniformMapCube, 3);

	gl.uniform1f( this.setUniformReflectivity, object.material.reflectivity );

}


Engine.shaderManager.ShaderMorMapLomNmaMma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLomNmaMma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/* SHADER MORPH MAP LIGHTOMI LIGHTDIR NOMALMAP MIRORMAP LIGHTMAP*/
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorMapLomLdiLmaNmaMma = function () {
	
	Engine.shaderManager.ShaderMorMapLomLdiLmaNma.call( this );

}

Engine.shaderManager.ShaderMorMapLomLdiLmaNmaMma.prototype = Object.create( Engine.shaderManager.ShaderMorMapLomLdiLmaNma.prototype );


Engine.shaderManager.ShaderMorMapLomLdiLmaNmaMma.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorMapLomLdiLmaNma.prototype.updateUniforms.call( this, object );

	this.setTextureCube(object.material.cubeMap,3);

	gl.uniform1i(this.setUniformMapCube, 3);

	gl.uniform1f( this.setUniformReflectivity, object.material.reflectivity );

}

Engine.shaderManager.ShaderMorMapLomLdiLmaNmaMma.prototype.buildUniformsAndAttributes = function(){
		
	Engine.shaderManager.ShaderMorMapLomLdiLmaNma.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformMapCube = gl.getUniformLocation( this.program, "mapSamplerCube" );

	this.setUniformReflectivity = gl.getUniformLocation( this.program, "reflectivity" );

}


Engine.shaderManager.ShaderMorMapLomLdiLmaNmaMma.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolMapLomLdiLmaNmaMma.prototype.buildFragmentShader;
/************************************************************/
/*                                                          */
/*                      SHADER MOPRH PARICLES COLOR         */
/*                                                          */
/************************************************************/


Engine.shaderManager.ShaderMorParCol = function () {
	
	Engine.shaderManager.ShaderMorCol.call( this );

}

Engine.shaderManager.ShaderMorParCol.prototype = Object.create( Engine.shaderManager.ShaderMorCol.prototype );


Engine.shaderManager.ShaderMorParCol.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorCol.prototype.updateUniforms.call( this, object );
	
	gl.uniform1f( this.setUniformSize, object.material.size);

}

Engine.shaderManager.ShaderMorParCol.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderMorCol.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformSize = gl.getUniformLocation( this.program, "size" );

}


Engine.shaderManager.ShaderMorParCol.prototype.draw = Engine.shaderManager.ShaderSolParCol.prototype.draw;

Engine.shaderManager.ShaderMorParCol.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition1;"+
	"attribute vec3 vertexPosition2;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float size;"+
	"uniform float progress;"+

	"void main(void) {"+
		"vec4 mvPosition =  viewMatrix * modelMatrix * vec4(vertexPosition1*(1.0-progress)+vertexPosition2*progress, 1.0);"+
		"gl_PointSize = size * ( 100.0 / length( mvPosition.xyz ) );"+
		"gl_Position = projectionMatrix * mvPosition;"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}
/************************************************************/
/*                                                          */
/*                      SHADER MORPH PARICLES MAP           */
/*                                                          */
/************************************************************/


Engine.shaderManager.ShaderMorParMap = function () {
	
	Engine.shaderManager.ShaderMorCol.call( this );

}

Engine.shaderManager.ShaderMorParMap.prototype = Object.create( Engine.shaderManager.ShaderMorCol.prototype );


Engine.shaderManager.ShaderMorParMap.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderMorCol.prototype.updateUniforms.call( this, object );
	
	gl.uniform1f( this.setUniformSize, object.material.size);

	this.setTexture(object.material.map,0);

	gl.uniform1i(this.setUniformMapColor, 0);

}

Engine.shaderManager.ShaderMorParMap.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderMorCol.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformSize = gl.getUniformLocation( this.program, "size" );

	this.setUniformMapColor = gl.getUniformLocation( this.program, "mapSamplerColor" );

}


Engine.shaderManager.ShaderMorParMap.prototype.draw = Engine.shaderManager.ShaderSolParCol.prototype.draw;

Engine.shaderManager.ShaderMorParMap.prototype.buildVertexShader = Engine.shaderManager.ShaderMorParCol.prototype.buildVertexShader;  


Engine.shaderManager.ShaderMorParMap.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+

	"uniform sampler2D mapSamplerColor;"+
	"uniform float opacity;"+


	"void main(void) {"+
		"vec4 textureColor = texture2D(mapSamplerColor, gl_PointCoord.xy);"+
		" gl_FragColor = vec4( textureColor.xyz, opacity*textureColor.a );"+
	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}
/************************************************************/
/*                                                          */
/*                      SHADER MORPH DEPTH                  */
/*                                                          */
/************************************************************/


Engine.shaderManager.ShaderMorDep = function () {
	
	Engine.shaderManager.ShaderMor.call( this );

}

Engine.shaderManager.ShaderMorDep.prototype = Object.create( Engine.shaderManager.ShaderMor.prototype );


Engine.shaderManager.ShaderMorDep.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderMor.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform1f(this.setDepthUniform, scene.camera.far);
	
}


Engine.shaderManager.ShaderMorDep.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderMor.prototype.buildUniformsAndAttributes.call( this );

	this.setDepthUniform = gl.getUniformLocation( this.program, "depth" );

}

Engine.shaderManager.ShaderMorDep.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition1;"+
	"attribute vec3 vertexPosition2;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float progress;"+

	"varying vec3 vPosition;"+

	"void main(void) {"+

		"vec4 position = viewMatrix * modelMatrix * vec4(vertexPosition1*(1.0-progress)+vertexPosition2*progress, 1.0);"+
		"vPosition = position.xyz;"+
		"gl_Position = projectionMatrix * position;"+

	"}";



	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}

Engine.shaderManager.ShaderMorDep.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolDep.prototype.buildFragmentShader;




Engine.shaderManager.ShaderMorDepView = function () {
	
	Engine.shaderManager.ShaderMorDep.call( this );

}

Engine.shaderManager.ShaderMorDepView.prototype = Object.create( Engine.shaderManager.ShaderMorDep.prototype );

Engine.shaderManager.ShaderMorDepView.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+

	"uniform float depth;"+

	"varying vec3 vPosition;"+

	codePack+

	"void main(void) {"+

		"gl_FragColor.xyz = vec3( length(vPosition) / depth);"+

	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}
/************************************************************/
/*                                                          */
/*                 SHADER MORPH LINES                       */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderMorLin = function () {
	
	Engine.shaderManager.ShaderSolCol.call( this );

}

Engine.shaderManager.ShaderMorLin.prototype = Object.create( Engine.shaderManager.ShaderMorCol.prototype );

Engine.shaderManager.ShaderMorLin.prototype.draw = Engine.shaderManager.ShaderSolLin.prototype.draw;
/************************************************************/
/*                                                          */
/*                 SHADER SKYBOX                            */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderSkybox = function () {

	var program = gl.createProgram();

	var vertexShader = this.buildVertexShader();

	var fragmentShader = this.buildFragmentShader();

	gl.attachShader(program, vertexShader );

	gl.attachShader(program, fragmentShader);

	gl.linkProgram(program);

	gl.deleteShader( vertexShader );

	gl.deleteShader( fragmentShader );

	this.program = program;

	this.buildUniformsAndAttributes();

}

Engine.shaderManager.ShaderSkybox.prototype.prepare = function(scene,lights, mapbuffer){

	gl.useProgram(this.program);

	gl.enableVertexAttribArray( this.setAttributVertex );

	this.updateCommonUniforms(scene,lights, mapbuffer);

}

Engine.shaderManager.ShaderSkybox.prototype.updateUniforms = function(object,scene){

	object.matrix.setPosition(scene.camera.position);

	gl.uniformMatrix4fv( this.setUniformVMatrix, false, object.matrix.elements );

	gl.uniform3fv( this.setUniformDiffuse, object.material.color.toArray());

}

Engine.shaderManager.ShaderSkybox.prototype.updateCommonUniforms = function(scene, light){

	gl.uniformMatrix4fv( this.setUniformPMatrix, false, scene.camera.projectionMatrix.elements );

	gl.uniformMatrix4fv( this.setUniformVVMatrix, false, scene.camera.matrixWorldInverse.elements );
	
}

Engine.shaderManager.ShaderSkybox.prototype.updateAttributes = function(object){

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglVertexBuffer);

	gl.vertexAttribPointer( this.setAttributVertex, 3, gl.FLOAT, false, 0, 0);

	this.setTextureCube(object.material.cubeMap,3);

	gl.uniform1i(this.setUniformMapCube, 3);



}

Engine.shaderManager.ShaderSkybox.prototype.buildUniformsAndAttributes = function(){

	this.setAttributVertex = gl.getAttribLocation( this.program, "vertexPosition" );

	this.setUniformVMatrix = gl.getUniformLocation( this.program, "modelMatrix" );

	this.setUniformPMatrix = gl.getUniformLocation( this.program, "projectionMatrix" );

	this.setUniformVVMatrix = gl.getUniformLocation( this.program, "viewMatrix" );

	this.setUniformMapCube = gl.getUniformLocation( this.program, "mapSamplerCube" );

	this.setUniformDiffuse = gl.getUniformLocation( this.program, "diffuse" );

}

Engine.shaderManager.ShaderSkybox.prototype.draw = function(object){

  gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, object.geometry.attributes.__webglIndexBuffer);
  gl.drawElements(gl.TRIANGLES, object.geometry.attributes.__webglIndexBuffer.numItems, gl.UNSIGNED_SHORT, 0);

  gl.clear(gl.DEPTH_BUFFER_BIT);

}

Engine.shaderManager.ShaderSkybox.prototype.setTextureCube = Engine.shaderManager.ShaderSol.prototype.setTextureCube;


Engine.shaderManager.ShaderSkybox.prototype.buildVertexShader = function(inherits){

	var code =  ""+

	"attribute vec3 vertexPosition;"+

	"uniform mat4 modelMatrix;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+

	"varying vec3 vUv;"+

	"void main(void) {"+

		"vUv = normalize (vertexPosition);"+
		"gl_Position = projectionMatrix * viewMatrix * modelMatrix * vec4(vertexPosition, 1.0);"+

	"}";

	if(!inherits){

		var vertexShader = gl.createShader(gl.VERTEX_SHADER);
		gl.shaderSource(vertexShader, code);
		gl.compileShader(vertexShader);
		if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

		return vertexShader;

	}else{
		return code;
	}

}


Engine.shaderManager.ShaderSkybox.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+

	"uniform samplerCube mapSamplerCube;"+
	"uniform vec3 diffuse;"+
	"varying vec3 vUv;"+

	"void main(void) {"+

		"gl_FragColor = textureCube( mapSamplerCube, vUv );"+
		"gl_FragColor.xyz *= diffuse;"+


	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}
//ShaderPar  Spr Rot Ran  Lom


/************************************************************/
/*                                                          */
/*        SHADER PARTICLES                                  */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderPar = function () {

	var program = gl.createProgram();

	var vertexShader = this.buildVertexShader();

	var fragmentShader = this.buildFragmentShader();

	gl.attachShader(program, vertexShader );

	gl.attachShader(program, fragmentShader);

	gl.linkProgram(program);

	gl.deleteShader( vertexShader );

	gl.deleteShader( fragmentShader );

	this.program = program;

	this.buildUniformsAndAttributes();

}

Engine.shaderManager.ShaderPar.prototype.prepare = function(scene,lights, mapbuffer){

	gl.useProgram(this.program);

	gl.enableVertexAttribArray( this.setAttributParticles );
	gl.enableVertexAttribArray( this.setAttributT0 );
	gl.enableVertexAttribArray( this.setAttributDirection );

	this.updateCommonUniforms(scene,lights, mapbuffer);

}

Engine.shaderManager.ShaderPar.prototype.updateUniforms = function(object){

	gl.uniform3fv( this.setUniformColorBegin, object.geometry.colorBegin.toArray() );
	gl.uniform3fv( this.setUniformColorEnd, object.geometry.colorEnd.toArray() );

	gl.uniform1f( this.setUniformOpacityBegin, object.geometry.opacityBegin );
	gl.uniform1f( this.setUniformOpacityEnd, object.geometry.opacityEnd );

	gl.uniform1f( this.setUniformVelocityBegin, object.geometry.velocityBegin );
	gl.uniform1f( this.setUniformVelocityEnd, object.geometry.velocityEnd );

	gl.uniform1f( this.setUniformSizeBegin, object.geometry.sizeBegin );
	gl.uniform1f( this.setUniformSizeEnd, object.geometry.sizeEnd );

	gl.uniform1f( this.setUniformDuration, object.geometry.duration );
	
	gl.uniform1f( this.setUniformT1, object.currentTime);

}

Engine.shaderManager.ShaderPar.prototype.updateCommonUniforms = function(scene, light){

	gl.uniformMatrix4fv( this.setUniformPMatrix, false, scene.camera.projectionMatrix.elements );

	gl.uniformMatrix4fv( this.setUniformVVMatrix, false, scene.camera.matrixWorldInverse.elements );

}

Engine.shaderManager.ShaderPar.prototype.updateAttributes = function(object){

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglVertexBuffer);

	gl.vertexAttribPointer( this.setAttributParticles, 3, gl.FLOAT, false, 0, 0);

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglT0Buffer);

	gl.vertexAttribPointer( this.setAttributT0, 1, gl.FLOAT, false, 0, 0);

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglDirectionBuffer);

	gl.vertexAttribPointer( this.setAttributDirection, 3, gl.FLOAT, false, 0, 0);

}

Engine.shaderManager.ShaderPar.prototype.buildUniformsAndAttributes = function(){

	this.setAttributParticles = gl.getAttribLocation( this.program, "vertexPosition" );
	this.setAttributT0 = gl.getAttribLocation( this.program, "t0" );
	this.setAttributDirection = gl.getAttribLocation( this.program, "vertexDirection" );

	this.setUniformPMatrix = gl.getUniformLocation( this.program, "projectionMatrix" );

	this.setUniformVVMatrix = gl.getUniformLocation( this.program, "viewMatrix" );

	this.setUniformColorBegin = gl.getUniformLocation( this.program, "colorA" );
	this.setUniformColorEnd = gl.getUniformLocation( this.program, "colorB" );

	this.setUniformOpacityBegin = gl.getUniformLocation( this.program, "opacityA" );
	this.setUniformOpacityEnd = gl.getUniformLocation( this.program, "opacityB" );

	this.setUniformVelocityBegin = gl.getUniformLocation( this.program, "velocityA" );
	this.setUniformVelocityEnd = gl.getUniformLocation( this.program, "velocityB" );

	this.setUniformSizeBegin = gl.getUniformLocation( this.program, "sizeA" );
	this.setUniformSizeEnd = gl.getUniformLocation( this.program, "sizeB" );

	this.setUniformDuration = gl.getUniformLocation( this.program, "duration" );

	this.setUniformT1 = gl.getUniformLocation( this.program, "t1" );

}

Engine.shaderManager.ShaderPar.prototype.draw = function(object){

	if(object.sorting){
		gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, object.geometry.attributes.__webglIndexBuffer);
		gl.drawElements(gl.POINTS, object.geometry.attributes.__webglIndexBuffer.numItems, gl.UNSIGNED_SHORT, 0);
	}else{
		gl.drawArrays(gl.POINTS, 0, object.geometry.attributes.__webglVertexBuffer.numItems);
	}
	

}

Engine.shaderManager.ShaderPar.prototype.setTexture = Engine.shaderManager.ShaderSol.prototype.setTexture;

Engine.shaderManager.ShaderPar.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute float t0;"+
	"attribute vec3 vertexDirection;"+

	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float sizeA;"+
	"uniform float sizeB;"+
	"uniform vec3 colorA;"+
	"uniform vec3 colorB;"+
	"uniform float velocityA;"+
	"uniform float velocityB;"+
	"uniform float opacityA;"+
	"uniform float opacityB;"+
	"uniform float duration;"+
	"uniform float t1;"+


	"varying vec3 color;"+
	"varying float opacity;"+
	

	"void main(void) {"+

		"float age = t1-t0;"+
		"float progress = min(age/duration,1.0);"+
		"float size = progress*sizeB+(1.0-progress)*sizeA;"+
		"float velocity = progress*velocityB+(1.0-progress)*velocityA;"+
		"color = progress*colorB+(1.0-progress)*colorA;"+
		"opacity = progress*opacityB+(1.0-progress)*opacityA;"+

		"vec3 point = vertexPosition+vertexDirection*velocity*age;"+

		"vec4 position =  viewMatrix  * vec4(point, 1.0);"+

		"gl_PointSize = size * ( 100.0 / length( position.xyz ) );"+

		"gl_Position = projectionMatrix * position;"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}

Engine.shaderManager.ShaderPar.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+

	"varying vec3 color;"+
	"varying float opacity;"+
	
	"void main(void) {"+

		" gl_FragColor = vec4(color,opacity );"+

	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}

/************************************************************/
/*                                                          */
/*        SHADER PARTICLES SPRITE 				            */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderParSpr = function () {
	Engine.shaderManager.ShaderPar.call( this );
}

Engine.shaderManager.ShaderParSpr.prototype = Object.create( Engine.shaderManager.ShaderPar.prototype );


Engine.shaderManager.ShaderParSpr.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderPar.prototype.updateUniforms.call( this, object );
		
	this.setTexture(object.material.map,0);

	gl.uniform1i(this.setUniformSprite, 0);

}

Engine.shaderManager.ShaderParSpr.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderPar.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformSprite = gl.getUniformLocation( this.program, "sprite" );

}

Engine.shaderManager.ShaderParSpr.prototype.setTexture = Engine.shaderManager.ShaderSol.prototype.setTexture;

Engine.shaderManager.ShaderParSpr.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+

	"uniform sampler2D sprite;"+

	"varying vec3 color;"+
	"varying float opacity;"+
	
	"void main(void) {"+

		"vec4 textureColor = texture2D(sprite, gl_PointCoord.xy );"+

		" gl_FragColor = vec4(color*textureColor.xyz,opacity*textureColor.a );"+

	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}


/************************************************************/
/*                                                          */
/*        SHADER PARTICLES SPRITE ROTATE		            */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderParSprRot = function () {
	Engine.shaderManager.ShaderParSpr.call( this );
}

Engine.shaderManager.ShaderParSprRot.prototype = Object.create( Engine.shaderManager.ShaderParSpr.prototype );

Engine.shaderManager.ShaderParSprRot.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderParSpr.prototype.updateUniforms.call( this, object );

	gl.uniform1f( this.setUniformAngleBegin, object.geometry.angleBegin );
	gl.uniform1f( this.setUniformAngleEnd, object.geometry.angleEnd );
		

}

Engine.shaderManager.ShaderParSprRot.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderParSpr.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformAngleBegin = gl.getUniformLocation( this.program, "angleA" );
	this.setUniformAngleEnd = gl.getUniformLocation( this.program, "angleB" );

}

Engine.shaderManager.ShaderParSprRot.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute float t0;"+
	"attribute vec3 vertexDirection;"+

	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float sizeA;"+
	"uniform float sizeB;"+
	"uniform vec3 colorA;"+
	"uniform vec3 colorB;"+
	"uniform float velocityA;"+
	"uniform float velocityB;"+
	"uniform float opacityA;"+
	"uniform float opacityB;"+
	"uniform float angleA;"+
	"uniform float angleB;"+
	"uniform float duration;"+
	"uniform float t1;"+


	"varying vec3 color;"+
	"varying float opacity;"+
	"varying float angle;"+
	

	"void main(void) {"+

		"float age = t1-t0;"+
		"float progress = min(age/duration,1.0);"+
		"float size = progress*sizeB+(1.0-progress)*sizeA;"+
		"float velocity = progress*velocityB+(1.0-progress)*velocityA;"+
		"color = progress*colorB+(1.0-progress)*colorA;"+
		"opacity = progress*opacityB+(1.0-progress)*opacityA;"+
		"angle = progress*angleB+(1.0-progress)*angleA;"+


		"vec3 point = vertexPosition+vertexDirection*velocity*age;"+

		"vec4 position =  viewMatrix  * vec4(point, 1.0);"+

		"gl_PointSize = size * ( 100.0 / length( position.xyz ) );"+

		"gl_Position = projectionMatrix * position;"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}


Engine.shaderManager.ShaderParSprRot.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+

	"uniform sampler2D sprite;"+

	"varying vec3 color;"+
	"varying float opacity;"+
	"varying float angle;"+
	
	"void main(void) {"+

		"float X = (gl_PointCoord.x-0.5);"+
		"float Y = (gl_PointCoord.y-0.5);"+

		"vec4 textureColor = texture2D(sprite, vec2(  (cos(angle)*X-sin(angle)*Y)   , (sin(angle)*X+cos(angle)*Y) ) + vec2(0.5,0.5) );"+

		" gl_FragColor = vec4(color*textureColor.xyz,opacity*textureColor.a );"+

	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}



/************************************************************/
/*                                                          */
/*        SHADER PARTICLES SPRITE ROTATION RANDOM           */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderParSprRotRan = function () {
	Engine.shaderManager.ShaderParSprRot.call( this );
}

Engine.shaderManager.ShaderParSprRotRan.prototype = Object.create( Engine.shaderManager.ShaderParSprRot.prototype );


Engine.shaderManager.ShaderParSprRotRan.prototype.prepare = function(scene,lights, mapbuffer){

	gl.useProgram(this.program);

	gl.enableVertexAttribArray( this.setAttributParticles );
	gl.enableVertexAttribArray( this.setAttributT0 );
	gl.enableVertexAttribArray( this.setAttributDirection );
	gl.enableVertexAttribArray( this.setAttributRandom );

	this.updateCommonUniforms(scene,lights, mapbuffer);

}

Engine.shaderManager.ShaderParSprRotRan.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderParSprRot.prototype.updateUniforms.call( this, object );

	gl.uniform1f( this.setUniformSpritSquarePower, object.geometry.spritSquareVariance );
}


Engine.shaderManager.ShaderParSprRotRan.prototype.updateAttributes = function(object){

	Engine.shaderManager.ShaderParSprRot.prototype.updateAttributes.call( this, object );

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglRandomBuffer);

	gl.vertexAttribPointer( this.setAttributRandom, 3, gl.FLOAT, false, 0, 0);

}

Engine.shaderManager.ShaderParSprRotRan.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderParSprRot.prototype.buildUniformsAndAttributes.call( this );

	this.setAttributRandom = gl.getAttribLocation( this.program, "vertexRandom" );

	this.setUniformSpritSquarePower = gl.getUniformLocation( this.program, "spritSquarePower" );

}


Engine.shaderManager.ShaderParSprRotRan.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute float t0;"+
	"attribute vec3 vertexDirection;"+
	"attribute vec3 vertexRandom;"+

	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float sizeA;"+
	"uniform float sizeB;"+
	"uniform vec3 colorA;"+
	"uniform vec3 colorB;"+
	"uniform float velocityA;"+
	"uniform float velocityB;"+
	"uniform float opacityA;"+
	"uniform float opacityB;"+
	"uniform float angleA;"+
	"uniform float angleB;"+
	"uniform float duration;"+
	"uniform float t1;"+


	"varying vec3 color;"+
	"varying float opacity;"+
	"varying float angle;"+
	"varying float indexSprite;"+
	

	"void main(void) {"+

		"float random1=floor(vertexRandom.x);"+//angle
		"float random2=vertexRandom.x-random1;"+//opacity

		"float random3=floor(vertexRandom.y);"+//size
		"float random4=vertexRandom.y-random3;"+//volocity

		"float random5=floor(vertexRandom.z);"+//sprite
		"float random6=vertexRandom.z-random5;"+//color

		"float age = t1-t0;"+
		"float progress = min(age/duration,1.0);"+
		"float size = progress*sizeB+(1.0-progress)*sizeA;"+
		"float velocity = progress*velocityB+(1.0-progress)*velocityA;"+
		"color = progress*colorB+(1.0-progress)*colorA;"+
		"opacity = progress*opacityB+(1.0-progress)*opacityA;"+
		"angle = progress*angleB+(1.0-progress)*angleA;"+

		
		"angle += random1/100.0;"+
		"opacity += random2;"+

		"size *= (1.0+random3/100.0);"+
		"velocity *= (1.0+random4);"+

		"indexSprite = random5;"+
		"color += random6;"+


		"vec3 point = vertexPosition+vertexDirection*velocity*age;"+

		"vec4 position =  viewMatrix  * vec4(point, 1.0);"+

		"gl_PointSize = size * ( 100.0 / length( position.xyz ) );"+

		"gl_Position = projectionMatrix * position;"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}

Engine.shaderManager.ShaderParSprRotRan.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+

	"uniform sampler2D sprite;"+
	"uniform float spritSquarePower;"+//3,2,1

	"varying vec3 color;"+
	"varying float opacity;"+
	"varying float angle;"+
	"varying float indexSprite;"+
	
	"void main(void) {"+

		"float offsetX = floor(indexSprite/spritSquarePower);"+
		"float offsetY = mod((indexSprite-1.0),spritSquarePower);"+
		"float X = (gl_PointCoord.x-0.5)/spritSquarePower;"+
		"float Y = (gl_PointCoord.y-0.5)/spritSquarePower;"+

		"vec4 textureColor = texture2D(sprite, vec2(  (cos(angle)*X-sin(angle)*Y)   , (sin(angle)*X+cos(angle)*Y) ) + vec2(offsetX+0.5,offsetY+0.5)/spritSquarePower );"+

		" gl_FragColor = vec4(color*textureColor.xyz,opacity*textureColor.a );"+
		//" gl_FragColor = vec4(color*textureColor.xyz,1.0 );"+

	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;
 
}


/************************************************************/
/*                                                          */
/*        SHADER PARTICLES SPRITE RANDOM           */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderParSprRan = function () {
	Engine.shaderManager.ShaderParSpr.call( this );
}

Engine.shaderManager.ShaderParSprRan.prototype = Object.create( Engine.shaderManager.ShaderParSpr.prototype );


Engine.shaderManager.ShaderParSprRan.prototype.prepare = function(scene,lights, mapbuffer){

	gl.useProgram(this.program);

	gl.enableVertexAttribArray( this.setAttributParticles );
	gl.enableVertexAttribArray( this.setAttributT0 );
	gl.enableVertexAttribArray( this.setAttributDirection );
	gl.enableVertexAttribArray( this.setAttributRandom );

	this.updateCommonUniforms(scene,lights, mapbuffer);

}

Engine.shaderManager.ShaderParSprRan.prototype.updateUniforms = function(object){

	Engine.shaderManager.ShaderParSpr.prototype.updateUniforms.call( this, object );

	gl.uniform1f( this.setUniformSpritSquarePower, object.geometry.spritSquareVariance );
}


Engine.shaderManager.ShaderParSprRan.prototype.updateAttributes = function(object){

	Engine.shaderManager.ShaderParSpr.prototype.updateAttributes.call( this, object );

	gl.bindBuffer( gl.ARRAY_BUFFER, object.geometry.attributes.__webglRandomBuffer);

	gl.vertexAttribPointer( this.setAttributRandom, 3, gl.FLOAT, false, 0, 0);

}

Engine.shaderManager.ShaderParSprRan.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderParSpr.prototype.buildUniformsAndAttributes.call( this );

	this.setAttributRandom = gl.getAttribLocation( this.program, "vertexRandom" );

	this.setUniformSpritSquarePower = gl.getUniformLocation( this.program, "spritSquarePower" );

}


Engine.shaderManager.ShaderParSprRan.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute float t0;"+
	"attribute vec3 vertexDirection;"+
	"attribute vec3 vertexRandom;"+

	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float sizeA;"+
	"uniform float sizeB;"+
	"uniform vec3 colorA;"+
	"uniform vec3 colorB;"+
	"uniform float velocityA;"+
	"uniform float velocityB;"+
	"uniform float opacityA;"+
	"uniform float opacityB;"+
	"uniform float duration;"+
	"uniform float t1;"+


	"varying vec3 color;"+
	"varying float opacity;"+
	"varying float indexSprite;"+
	

	"void main(void) {"+

		"float random2=vertexRandom.x-floor(vertexRandom.x);"+//opacity

		"float random3=floor(vertexRandom.y);"+//size
		"float random4=vertexRandom.y-random3;"+//volocity

		"float random5=floor(vertexRandom.z);"+//sprite
		"float random6=vertexRandom.z-random5;"+//color

		"float age = t1-t0;"+
		"float progress = min(age/duration,1.0);"+
		"float size = progress*sizeB+(1.0-progress)*sizeA;"+
		"float velocity = progress*velocityB+(1.0-progress)*velocityA;"+
		"color = progress*colorB+(1.0-progress)*colorA;"+
		"opacity = progress*opacityB+(1.0-progress)*opacityA;"+

		
		"opacity += random2;"+

		"size *= (1.0+random3/100.0);"+
		"velocity *= (1.0+random4);"+

		"indexSprite = random5;"+
		"color += random6;"+


		"vec3 point = vertexPosition+vertexDirection*velocity*age;"+

		"vec4 position =  viewMatrix  * vec4(point, 1.0);"+

		"gl_PointSize = size * ( 100.0 / length( position.xyz ) );"+

		"gl_Position = projectionMatrix * position;"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}

Engine.shaderManager.ShaderParSprRan.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+

	"uniform sampler2D sprite;"+
	"uniform float spritSquarePower;"+//3,2,1

	"varying vec3 color;"+
	"varying float opacity;"+
	"varying float indexSprite;"+
	
	"void main(void) {"+

		"float offsetX = floor(indexSprite/spritSquarePower);"+
		"float offsetY = mod((indexSprite-1.0),spritSquarePower);"+
		"float X = (gl_PointCoord.x)/spritSquarePower;"+
		"float Y = (gl_PointCoord.y)/spritSquarePower;"+

		"vec4 textureColor = texture2D(sprite, vec2( X , Y)+ vec2(offsetX ,offsetY)/spritSquarePower );"+

		" gl_FragColor = vec4(color*textureColor.xyz,opacity*textureColor.a );"+
		//" gl_FragColor = vec4(color*textureColor.xyz,1.0 );"+

	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;
 
}
 



/************************************************************/
/*                                                          */
/*     SHADER PARTICLES SPRITE ROTATION RANDOM LIGHTOMNI    */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderParSprRotRanLom = function () {
	Engine.shaderManager.ShaderParSprRotRan.call( this );
}

Engine.shaderManager.ShaderParSprRotRanLom.prototype = Object.create( Engine.shaderManager.ShaderParSprRotRan.prototype );

Engine.shaderManager.ShaderParSprRotRanLom.prototype.updateCommonUniforms = function(scene, light){

	Engine.shaderManager.ShaderParSprRotRan.prototype.updateCommonUniforms.call( this, scene, light );

	gl.uniform3fv( this.setUniformPointLightColor, light.point.colors );

	gl.uniform3fv( this.setUniformPointLightPosition, light.point.positions);

	gl.uniform1i( this.setUniformNumberOfOmniLight, light.point.length );

	gl.uniform1f( this.setUniformpPointLightDistance, light.point.distances );

	gl.uniform3fv( this.setUniformAmbientLightColor, light.ambient );

}

Engine.shaderManager.ShaderParSprRotRanLom.prototype.buildUniformsAndAttributes = function(){

	Engine.shaderManager.ShaderParSprRotRan.prototype.buildUniformsAndAttributes.call( this );

	this.setUniformPointLightColor = gl.getUniformLocation( this.program, "pointLightColor" );

	this.setUniformpPointLightDistance = gl.getUniformLocation( this.program, "pointLightDistance" );

	this.setUniformPointLightPosition = gl.getUniformLocation( this.program, "pointLightPosition" );

	this.setUniformNumberOfOmniLight = gl.getUniformLocation( this.program, "numberOfOmniLight" );

	this.setUniformAmbientLightColor = gl.getUniformLocation( this.program, "ambientLightColor" );

}

Engine.shaderManager.ShaderParSprRotRanLom.prototype.buildVertexShader = function(){

	var code =  ""+

	"#define MAX_POINT_LIGHTS 10 \n"+

	"attribute vec3 vertexPosition;"+
	"attribute float t0;"+
	"attribute vec3 vertexDirection;"+
	"attribute vec3 vertexRandom;"+

	"uniform vec3 pointLightColor[ MAX_POINT_LIGHTS ];"+
	"uniform vec3 pointLightPosition[ MAX_POINT_LIGHTS ];"+
	"uniform float pointLightDistance[ MAX_POINT_LIGHTS ];"+

	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float sizeA;"+
	"uniform float sizeB;"+
	"uniform vec3 colorA;"+
	"uniform vec3 colorB;"+
	"uniform float velocityA;"+
	"uniform float velocityB;"+
	"uniform float opacityA;"+
	"uniform float opacityB;"+
	"uniform float angleA;"+
	"uniform float angleB;"+
	"uniform float duration;"+
	"uniform float t1;"+
	"uniform int numberOfOmniLight;"+
	"uniform vec3 ambientLightColor;"+


	"varying vec3 color;"+
	"varying float opacity;"+
	"varying float angle;"+
	"varying float indexSprite;"+
	

	"void main(void) {"+

		"float random1=floor(vertexRandom.x);"+//angle
		"float random2=vertexRandom.x-random1;"+//opacity

		"float random3=floor(vertexRandom.y);"+//size
		"float random4=vertexRandom.y-random3;"+//volocity

		"float random5=floor(vertexRandom.z);"+//sprite
		"float random6=vertexRandom.z-random5;"+//color

		"float age = t1-t0;"+
		"float progress = min(age/duration,1.0);"+
		"float size = progress*sizeB+(1.0-progress)*sizeA;"+
		"float velocity = progress*velocityB+(1.0-progress)*velocityA;"+
		"color = progress*colorB+(1.0-progress)*colorA;"+
		"opacity = progress*opacityB+(1.0-progress)*opacityA;"+
		"angle = progress*angleB+(1.0-progress)*angleA;"+

		"angle += random1/100.0;"+
		"opacity += random2;"+

		"size *= (1.0+random3/100.0);"+
		"velocity *= (1.0+random4);"+

		"indexSprite = random5;"+
		"color += random6;"+

		"vec3 point = vertexPosition+vertexDirection*velocity*age;"+

		"vec3 totalDiffuse  = vec3( 0.0 );"+

		"for( int i = 0; i < MAX_POINT_LIGHTS; i ++ ) {"+

			"if(numberOfOmniLight==i){break;}"+

			"vec3 lVector = pointLightPosition[ i ].xyz - point;"+

			"float lDistance = 1.0;"+
			"if ( pointLightDistance[ i ] > 0.0 )"+
				"lDistance = 1.0 - min( ( length( lVector ) / pointLightDistance[ i ] ), 1.0 );"+

			"totalDiffuse += pointLightColor[ i ] * lDistance;"+

		"}"+

		"color *= vec3(totalDiffuse+ambientLightColor);"+

		"vec4 position =  viewMatrix  * vec4(point, 1.0);"+

		"gl_PointSize = size * ( 100.0 / length( position.xyz ) );"+

		"gl_Position = projectionMatrix * position;"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}

/************************************************************/
/*                                                          */
/*                      SHADER PARTICLES DEPTH              */
/*                                                          */
/************************************************************/

Engine.shaderManager.ShaderParDep = function () {
	Engine.shaderManager.ShaderPar.call( this );
}

Engine.shaderManager.ShaderParDep.prototype = Object.create( Engine.shaderManager.ShaderPar.prototype );

Engine.shaderManager.ShaderParDep.prototype.updateCommonUniforms = function(scene, light){
	Engine.shaderManager.ShaderPar.prototype.updateCommonUniforms.call( this, scene, light );
	gl.uniform1f(this.setDepthUniform, scene.camera.far);
}

Engine.shaderManager.ShaderParDep.prototype.buildUniformsAndAttributes = function(){
	Engine.shaderManager.ShaderPar.prototype.buildUniformsAndAttributes.call( this );
	this.setDepthUniform = gl.getUniformLocation( this.program, "depth" );
}

Engine.shaderManager.ShaderParDep.prototype.buildVertexShader = function(){

	var code =  ""+
	"attribute vec3 vertexPosition;"+
	"attribute float t0;"+
	"attribute vec3 vertexDirection;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float sizeA;"+
	"uniform float sizeB;"+
	"uniform vec3 colorA;"+
	"uniform vec3 colorB;"+
	"uniform float velocityA;"+
	"uniform float velocityB;"+
	"uniform float opacityA;"+
	"uniform float opacityB;"+
	"uniform float duration;"+
	"uniform float t1;"+

	"varying vec3 vPosition;"+
	
	"void main(void) {"+
		"float age = t1-t0;"+
		"float progress = min(age/duration,1.0);"+
		"float size = progress*sizeB+(1.0-progress)*sizeA;"+
		"float velocity = progress*velocityB+(1.0-progress)*velocityA;"+
		"vec3 point = vertexPosition+vertexDirection*velocity*age;"+
		"vec4 position =  viewMatrix  * vec4(point, 1.0);"+
		"vPosition = position.xyz;"+
		"gl_PointSize = size * ( 100.0 / length( position.xyz ) );"+
		"gl_Position = projectionMatrix * position;"+
	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}
	return vertexShader;
}

Engine.shaderManager.ShaderParDep.prototype.buildFragmentShader = Engine.shaderManager.ShaderSolDep.prototype.buildFragmentShader;
Engine.shaderManager.ShaderParSprDep = function () {
	Engine.shaderManager.ShaderParSpr.call( this );
}

Engine.shaderManager.ShaderParSprDep.prototype = Object.create( Engine.shaderManager.ShaderParSpr.prototype );

Engine.shaderManager.ShaderParSprDep.prototype.buildVertexShader = Engine.shaderManager.ShaderParDep.prototype.buildVertexShader;

Engine.shaderManager.ShaderParSprDep.prototype.updateCommonUniforms = function(scene, light){
	Engine.shaderManager.ShaderParSpr.prototype.updateCommonUniforms.call( this, scene, light );
	gl.uniform1f(this.setDepthUniform, scene.camera.far);
}

Engine.shaderManager.ShaderParSprDep.prototype.buildUniformsAndAttributes = function(){
	Engine.shaderManager.ShaderParSpr.prototype.buildUniformsAndAttributes.call( this );
	this.setDepthUniform = gl.getUniformLocation( this.program, "depth" );
}

Engine.shaderManager.ShaderParSprDep.prototype.buildFragmentShader = function(){

	var code = ""+
	"precision mediump float;"+
	"uniform sampler2D sprite;"+
	"uniform float depth;"+
	"varying vec3 vPosition;"+
	codePack+
	"void main(void) {"+
		"gl_FragColor = pack(length(vPosition) / depth);"+
		"gl_FragColor.a = texture2D(sprite, gl_PointCoord.xy ).a;"+
	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}
	return fragmentShader;
}
Engine.shaderManager.ShaderParSprRotDep = function () {Engine.shaderManager.ShaderParSprRot.call( this );}
Engine.shaderManager.ShaderParSprRotDep.prototype = Object.create( Engine.shaderManager.ShaderParSprRot.prototype );
Engine.shaderManager.ShaderParSprRotDep.prototype.updateCommonUniforms = function(scene, light){
	Engine.shaderManager.ShaderParSprRot.prototype.updateCommonUniforms.call( this, scene, light );
	gl.uniform1f(this.setDepthUniform, scene.camera.far);
}
Engine.shaderManager.ShaderParSprRotDep.prototype.buildUniformsAndAttributes = function(){
	Engine.shaderManager.ShaderParSprRot.prototype.buildUniformsAndAttributes.call( this );
	this.setDepthUniform = gl.getUniformLocation( this.program, "depth" );
}
Engine.shaderManager.ShaderParSprRotDep.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute float t0;"+
	"attribute vec3 vertexDirection;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float sizeA;"+
	"uniform float sizeB;"+
	"uniform vec3 colorA;"+
	"uniform vec3 colorB;"+
	"uniform float velocityA;"+
	"uniform float velocityB;"+
	"uniform float opacityA;"+
	"uniform float opacityB;"+
	"uniform float angleA;"+
	"uniform float angleB;"+
	"uniform float duration;"+
	"uniform float t1;"+
	"varying float angle;"+
	"varying vec3 vPosition;"+
	
	"void main(void) {"+

		"float age = t1-t0;"+
		"float progress = min(age/duration,1.0);"+
		"float size = progress*sizeB+(1.0-progress)*sizeA;"+
		"float velocity = progress*velocityB+(1.0-progress)*velocityA;"+
		"angle = progress*angleB+(1.0-progress)*angleA;"+
		"vec3 point = vertexPosition+vertexDirection*velocity*age;"+
		"vec4 position =  viewMatrix  * vec4(point, 1.0);"+
		"vPosition = position.xyz;"+
		"gl_PointSize = size * ( 100.0 / length( position.xyz ) );"+
		"gl_Position = projectionMatrix * position;"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}
	return vertexShader;
}

Engine.shaderManager.ShaderParSprRotDep.prototype.buildFragmentShader = function(){

	var code = ""+
	"precision mediump float;"+
	"uniform sampler2D sprite;"+
	"uniform float depth;"+
	"varying vec3 vPosition;"+
	"varying float angle;"+
	codePack+
	"void main(void) {"+
		"float X = (gl_PointCoord.x-0.5);"+
		"float Y = (gl_PointCoord.y-0.5);"+
		"vec4 textureColor = texture2D(sprite, vec2(  (cos(angle)*X-sin(angle)*Y)   , (sin(angle)*X+cos(angle)*Y) ) + vec2(0.5,0.5) );"+
		"gl_FragColor = pack(length(vPosition) / depth);"+
		"gl_FragColor.a = textureColor.a;"+
	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}
	return fragmentShader;
}
Engine.shaderManager.ShaderParSprRotRanDep = function () {Engine.shaderManager.ShaderParSprRotRan.call( this );}
Engine.shaderManager.ShaderParSprRotRanDep.prototype = Object.create( Engine.shaderManager.ShaderParSprRotRan.prototype );
Engine.shaderManager.ShaderParSprRotRanDep.prototype.updateCommonUniforms = function(scene, light){
	Engine.shaderManager.ShaderParSprRotRan.prototype.updateCommonUniforms.call( this, scene, light );
	gl.uniform1f(this.setDepthUniform, scene.camera.far);
}
Engine.shaderManager.ShaderParSprRotRanDep.prototype.buildUniformsAndAttributes = function(){
	Engine.shaderManager.ShaderParSprRotRan.prototype.buildUniformsAndAttributes.call( this );
	this.setDepthUniform = gl.getUniformLocation( this.program, "depth" );
}

Engine.shaderManager.ShaderParSprRotRanDep.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute float t0;"+
	"attribute vec3 vertexDirection;"+
	"attribute vec3 vertexRandom;"+
	"uniform mat4 viewMatrix;"+
	"uniform mat4 projectionMatrix;"+
	"uniform float sizeA;"+
	"uniform float sizeB;"+
	"uniform vec3 colorA;"+
	"uniform vec3 colorB;"+
	"uniform float velocityA;"+
	"uniform float velocityB;"+
	"uniform float opacityA;"+
	"uniform float opacityB;"+
	"uniform float angleA;"+
	"uniform float angleB;"+
	"uniform float duration;"+
	"uniform float t1;"+

	"varying float angle;"+
	"varying float indexSprite;"+
	"varying vec3 vPosition;"+
	
	"void main(void) {"+
		"float random1=floor(vertexRandom.x);"+//angle
		"float random3=floor(vertexRandom.y);"+//size
		"float random4=vertexRandom.y-random3;"+//volocity
		"float random5=floor(vertexRandom.z);"+//sprite
		"float age = t1-t0;"+
		"float progress = min(age/duration,1.0);"+
		"float size = progress*sizeB+(1.0-progress)*sizeA;"+
		"float velocity = progress*velocityB+(1.0-progress)*velocityA;"+
		"angle = progress*angleB+(1.0-progress)*angleA;"+
		"angle += random1/100.0;"+
		"size *= (1.0+random3/100.0);"+
		"velocity *= (1.0+random4);"+
		"indexSprite = random5;"+
		"vec3 point = vertexPosition+vertexDirection*velocity*age;"+
		"vec4 position =  viewMatrix  * vec4(point, 1.0);"+
		"vPosition = position.xyz;"+
		"gl_PointSize = size * ( 100.0 / length( position.xyz ) );"+
		"gl_Position = projectionMatrix * position;"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}

Engine.shaderManager.ShaderParSprRotRanDep.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+
	"uniform sampler2D sprite;"+
	"uniform float spritSquarePower;"+//3,2,1
	"varying float angle;"+
	"uniform float depth;"+
	"varying float indexSprite;"+
	"varying vec3 vPosition;"+
	codePack+
	"void main(void) {"+
		"float offsetX = floor(indexSprite/spritSquarePower);"+
		"float offsetY = mod((indexSprite-1.0),spritSquarePower);"+
		"float X = (gl_PointCoord.x-0.5)/spritSquarePower;"+
		"float Y = (gl_PointCoord.y-0.5)/spritSquarePower;"+
		"vec4 textureColor = texture2D(sprite, vec2(  (cos(angle)*X-sin(angle)*Y)   , (sin(angle)*X+cos(angle)*Y) ) + vec2(offsetX+0.5,offsetY+0.5)/spritSquarePower );"+
		"gl_FragColor = pack(length(vPosition) / depth);"+
		"gl_FragColor.a = textureColor.a;"+
	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}
/************************************************************/
/*                                                          */
/*                      SHADER SHADOW MAP PLUGIN            */
/*                                                          */
/************************************************************/


var addClassShader = function(namefunction){

		Engine.shaderManager[namefunction+"Sha"] = function () {
			Engine.shaderManager[namefunction].call( this );
		}

		Engine.shaderManager[namefunction+"Sha"].prototype = Object.create( Engine.shaderManager[namefunction].prototype );

		Engine.shaderManager[namefunction+"Sha"].prototype.updateCommonUniforms = function(scene, light, mapbuffer){

			Engine.shaderManager[namefunction].prototype.updateCommonUniforms.call( this, scene, light );

			gl.uniform1f(this.setDepthUniform, scene.cameraDepth.far);

			gl.uniformMatrix4fv( this.setUniformPShadowMatrix, false, scene.cameraDepth.projectionMatrix.elements );

			gl.uniformMatrix4fv( this.setUniformVShadowMatrix, false, scene.cameraDepth.matrixWorldInverse.elements );

			gl.uniform3fv( this.setSourceUniform, scene.cameraDepth.position.toArray());

			gl.activeTexture( gl.TEXTURE0 + 6 );

			gl.bindTexture( gl.TEXTURE_2D, mapbuffer );

			gl.uniform1i(this.setUniformShadowMap, 6);
			
		}

		Engine.shaderManager[namefunction+"Sha"].prototype.buildUniformsAndAttributes = function(){

			Engine.shaderManager[namefunction].prototype.buildUniformsAndAttributes.call( this );

			this.setDepthUniform = gl.getUniformLocation( this.program, "depth" );

			this.setUniformPShadowMatrix = gl.getUniformLocation( this.program, "projShadowMatrix" );

			this.setUniformVShadowMatrix = gl.getUniformLocation( this.program, "viewShadowMatrix" );

			this.setSourceUniform = gl.getUniformLocation( this.program, "uSource" );


			this.setUniformShadowMap = gl.getUniformLocation( this.program, "shadowMap" );

		}



		Engine.shaderManager[namefunction+"Sha"].prototype.buildVertexShader = function(){
			
			var code = Engine.shaderManager[namefunction].prototype.buildVertexShader.call( this, true );



			var initCodeShadow = "" +
				"uniform mat4 projShadowMatrix;" +
				"uniform mat4 viewShadowMatrix;"+
				"uniform mat4 modelMatrix;"+
				"varying vec4 vPositionByShadow;";

			var mainCodeShadow = ""+
				"vPositionByShadow = mat4(0.5, 0.0, 0.0, 0.0, 0.0, 0.5, 0.0, 0.0, 0.0, 0.0, 0.5, 0.0, 0.5, 0.5, 0.5, 1.0) * projShadowMatrix * viewShadowMatrix * vPosition;"+
				"gl_Position = projectionMatrix * viewMatrix * vPosition;"

			code = code.replace("uniform mat4 modelMatrix;",initCodeShadow);

			code = code.replace("gl_Position = projectionMatrix * viewMatrix * vPosition;",mainCodeShadow);


			var vertexShader = gl.createShader(gl.VERTEX_SHADER);
			gl.shaderSource(vertexShader, code);
			gl.compileShader(vertexShader);
			if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

			return vertexShader;

		}

		Engine.shaderManager[namefunction+"Sha"].prototype.buildFragmentShader = function(){
			
			var code = Engine.shaderManager[namefunction].prototype.buildFragmentShader.call( this, true );

			var initCodeShadow = "" +

				"precision mediump float;" +

				"uniform sampler2D shadowMap;" +
				"uniform float depth;" +
				"uniform vec3 uSource;" +
				"varying vec4 vPositionByShadow;"+
				"float unpack (vec4 color){return dot(color, vec4(1.0, 1.0 / 255.0,1.0 / (255.0 * 255.0),1.0 / (255.0 * 255.0 * 255.0)));}";


			var mainCodeShadow = "" +

			"vec3 totalDiffuse  = vec3( 0.0 );"+
			"vec3 totalSpecular = vec3( 0.0 );"+

			"vec3 projCoords = vPositionByShadow.xyz/vPositionByShadow.w;" +
			"if(projCoords.x>=0.0&&projCoords.x<=1.0&&projCoords.y>=0.0&&projCoords.y<=1.0){" +
				"projCoords.z = length(vPosition.xyz - uSource)/depth;" +
				"float shadowDepth = unpack( texture2D(shadowMap, vec2(projCoords.s, projCoords.t)) );" +

				"if(projCoords.z*0.99 > shadowDepth){" +

					"shadowDepth = unpack( texture2D(shadowMap, vec2(projCoords.s+float(-1)/1024.0  , projCoords.t+float(-1)/1024.0  )) );" +
					"if(projCoords.z*0.99 > shadowDepth)" +
					"totalDiffuse-=vec3(0.01) ;" +
					"shadowDepth = unpack( texture2D(shadowMap, vec2(projCoords.s+float(-1)/1024.0  , projCoords.t+float(0)/1024.0  )) );" +
					"if(projCoords.z*0.99 > shadowDepth)" +
					"totalDiffuse-=vec3(0.01) ;" +
					"shadowDepth = unpack( texture2D(shadowMap, vec2(projCoords.s+float(-1)/1024.0  , projCoords.t+float(1)/1024.0  )) );" +
					"if(projCoords.z*0.99 > shadowDepth)" +
					"totalDiffuse-=vec3(0.01) ;" +
					"shadowDepth = unpack( texture2D(shadowMap, vec2(projCoords.s+float(0)/1024.0  , projCoords.t+float(-1)/1024.0  )) );" +
					"if(projCoords.z*0.99 > shadowDepth)" +
					"totalDiffuse-=vec3(0.01) ;" +
					"shadowDepth = unpack( texture2D(shadowMap, vec2(projCoords.s+float(0)/1024.0  , projCoords.t+float(0)/1024.0  )) );" +
					"if(projCoords.z*0.99 > shadowDepth)" +
					"totalDiffuse-=vec3(0.01) ;" +
					"shadowDepth = unpack( texture2D(shadowMap, vec2(projCoords.s+float(0)/1024.0  , projCoords.t+float(1)/1024.0  )) );" +
					"if(projCoords.z*0.99 > shadowDepth)" +
					"totalDiffuse-=vec3(0.01) ;" +
					"shadowDepth = unpack( texture2D(shadowMap, vec2(projCoords.s+float(1)/1024.0  , projCoords.t+float(-1)/1024.0  )) );" +
					"if(projCoords.z*0.99 > shadowDepth)" +
					"totalDiffuse-=vec3(0.01) ;" +
					"shadowDepth = unpack( texture2D(shadowMap, vec2(projCoords.s+float(1)/1024.0  , projCoords.t+float(0)/1024.0  )) );" +
					"if(projCoords.z*0.99 > shadowDepth)" +
					"totalDiffuse-=vec3(0.01) ;" +
					"shadowDepth = unpack( texture2D(shadowMap, vec2(projCoords.s+float(1)/1024.0  , projCoords.t+float(1)/1024.0  )) );" +
					"if(projCoords.z*0.99 > shadowDepth)" +
					"totalDiffuse-=vec3(0.01) ;" +

				"}"+

			"}";


				code = code.replace("precision mediump float;",initCodeShadow);

				var toReplace = "vec3 totalDiffuse  = vec3( 0.0 );"+"vec3 totalSpecular = vec3( 0.0 );";

				code = code.replace(toReplace,mainCodeShadow);


			var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
			gl.shaderSource(fragmentShader, code);
			gl.compileShader(fragmentShader);
			if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

			return fragmentShader;

		}

}

tabName = [];

for(var key in Engine.shaderManager){
	if(key.indexOf('Lom')!=-1||key.indexOf('Ldir')!=-1)
		tabName.push(key);
}

for(var k=0,kl=tabName.length;k<kl;k++){
	addClassShader(tabName[k]);
}

delete tabName;
delete addClassShader;
/************************************************************/
/*                                                          */
/*                      SHADER COMPOSER                     */
/*                                                          */
/************************************************************/


Engine.shaderManager.shaderComposer = function () {

	var program = gl.createProgram();

	var vertexShader = this.buildVertexShader();

	var fragmentShader = this.buildFragmentShader();

	gl.attachShader(program, vertexShader );

	gl.attachShader(program, fragmentShader);

	gl.linkProgram(program);

	gl.deleteShader( vertexShader );

	gl.deleteShader( fragmentShader );

	this.program = program;

	this.buildUniformsAndAttributes();

}


Engine.shaderManager.shaderComposer.prototype.prepare = function( mapbuffer1, mapbuffer2, mapbuffer3){

	gl.useProgram(this.program);

	gl.enableVertexAttribArray( this.setAttributVertex );
	gl.enableVertexAttribArray( this.setAttributUv );

	gl.activeTexture( gl.TEXTURE0 + 7 );
	gl.bindTexture( gl.TEXTURE_2D, mapbuffer1 );
	gl.uniform1i(this.setUniformMapScreen1, 7);
	gl.activeTexture( gl.TEXTURE0 + 8 );
	gl.bindTexture( gl.TEXTURE_2D, mapbuffer2 );
	gl.uniform1i(this.setUniformMapScreen2, 8);
	gl.activeTexture( gl.TEXTURE0 + 9 );
	gl.bindTexture( gl.TEXTURE_2D, mapbuffer3 );
	gl.uniform1i(this.setUniformMapScreen3, 9);

}

Engine.shaderManager.shaderComposer.prototype.updateUniforms = function(){}

Engine.shaderManager.shaderComposer.prototype.updateAttributes = function(object){

	gl.bindBuffer( gl.ARRAY_BUFFER, object.__webglVertexBuffer);

	gl.vertexAttribPointer( this.setAttributVertex, 3, gl.FLOAT, false, 0, 0);

	gl.bindBuffer( gl.ARRAY_BUFFER, object.__webglUVBuffer);

	gl.vertexAttribPointer( this.setAttributUv, 2, gl.FLOAT, false, 0, 0);

}

Engine.shaderManager.shaderComposer.prototype.buildUniformsAndAttributes = function(){


	this.setAttributVertex = gl.getAttribLocation( this.program, "vertexPosition" );

	this.setAttributUv = gl.getAttribLocation( this.program, "vertexUvs" );

	this.setUniformMapScreen1 = gl.getUniformLocation( this.program, "mapScreen1" );
	this.setUniformMapScreen2 = gl.getUniformLocation( this.program, "mapScreen2" );
	this.setUniformMapScreen3 = gl.getUniformLocation( this.program, "mapScreen3" );

}

Engine.shaderManager.shaderComposer.prototype.draw = function(){

	gl.drawArrays(gl.TRIANGLES, 0, 6);

}

Engine.shaderManager.shaderComposer.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute vec2 vertexUvs;"+
	"varying vec2 vUv;"+

	"void main(void) {"+

		"gl_Position = vec4(vertexPosition, 1.0);"+
		"vUv = vertexUvs;"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}

Engine.shaderManager.shaderComposer.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+

	"uniform sampler2D mapScreen1;"+
	"uniform sampler2D mapScreen2;"+
	"uniform sampler2D mapScreen3;"+
	
	"varying vec2 vUv;"+

	"void main(void) {"+

		"vec3 map1 = texture2D(mapScreen1, vec2(vUv.s, vUv.t)).xyz;" +
		"vec3 map2 = texture2D(mapScreen2, vec2(vUv.s, vUv.t)).xyz;" +

		"gl_FragColor = vec4(map1+map2,1.0);" +

	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}

/************************************************************/
/*                                                          */
/*                      SHADER DEPTH OF FIELD               */
/*                                                          */
/************************************************************/


Engine.shaderManager.shaderDepthField = function (params) {

	var program = gl.createProgram();
 //gl.viewWidth,gl.viewHeight
	var size = gl.viewWidth.toString()+".0"; ;
	var loop = "6";

		

	var vertexShader = this.buildVertexShader();

	var fragmentShader = this.buildFragmentShader(size,loop);

	gl.attachShader(program, vertexShader );

	gl.attachShader(program, fragmentShader);

	gl.linkProgram(program);

	gl.deleteShader( vertexShader );

	gl.deleteShader( fragmentShader );

	this.program = program;

	this.buildUniformsAndAttributes();

}


Engine.shaderManager.shaderDepthField.prototype.prepare = function( mapscreen, mapDepth, camera){

	gl.useProgram(this.program);

	gl.enableVertexAttribArray( this.setAttributVertex );
	gl.enableVertexAttribArray( this.setAttributUv );

	gl.uniform1f(this.setDepthUniform, camera.far);

	gl.activeTexture( gl.TEXTURE0 + 12 );
	gl.bindTexture( gl.TEXTURE_2D, mapscreen );
	gl.uniform1i(this.setUniformMapScreen, 12);
	gl.activeTexture( gl.TEXTURE0 + 13 );
	gl.bindTexture( gl.TEXTURE_2D, mapDepth );
	gl.uniform1i(this.setUniformMapDepth, 13);

}


Engine.shaderManager.shaderDepthField.prototype.updateAttributes = function(object){

	gl.bindBuffer( gl.ARRAY_BUFFER, object.__webglVertexBuffer);

	gl.vertexAttribPointer( this.setAttributVertex, 3, gl.FLOAT, false, 0, 0);

	gl.bindBuffer( gl.ARRAY_BUFFER, object.__webglUVBuffer);

	gl.vertexAttribPointer( this.setAttributUv, 2, gl.FLOAT, false, 0, 0);

}

Engine.shaderManager.shaderDepthField.prototype.buildUniformsAndAttributes = function(){

	this.setAttributVertex = gl.getAttribLocation( this.program, "vertexPosition" );

	this.setAttributUv = gl.getAttribLocation( this.program, "vertexUvs" );

	this.setUniformMapScreen = gl.getUniformLocation( this.program, "mapScreen" );

	this.setUniformMapDepth = gl.getUniformLocation( this.program, "mapDepth" );

	this.setUniformBlurDirection = gl.getUniformLocation( this.program, "direction" );

	this.setDepthUniform = gl.getUniformLocation( this.program, "depth" );

}

Engine.shaderManager.shaderDepthField.prototype.draw = function(bool){
	gl.uniform1i( this.setUniformBlurDirection, bool );
	gl.drawArrays(gl.TRIANGLES, 0, 6);
}

Engine.shaderManager.shaderDepthField.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute vec2 vertexUvs;"+
	"varying vec2 vUv;"+

	"void main(void) {"+

		"gl_Position = vec4(vertexPosition, 1.0);"+
		"vUv = vertexUvs;"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}

Engine.shaderManager.shaderDepthField.prototype.buildFragmentShader = function(){


	var aspect = gl.viewHeight/gl.viewWidth;
	if(gl.viewHeight>=gl.viewWidth)
		aspect+=".0"

	var code = ""+

	"precision mediump float;" +
	"varying vec2 vUv;" +
	"uniform float depth;"+
	"uniform sampler2D mapScreen;" +
	"uniform sampler2D mapDepth;" +
	"uniform int direction;"+
	"float unpack (vec4 color){return dot(color, vec4(1.0, 1.0 / 255.0,1.0 / (255.0 * 255.0),1.0 / (255.0 * 255.0 * 255.0)));}"+
	"void main() {"+
		"float focus = 0.1;"+
		"vec2 aspectcorrect = vec2( 1.0, "+aspect+" );"+
		"float depth1 = unpack(texture2D( mapDepth, vUv ) );"+
		"float factor = depth1 - 0.0;"+
		"vec2 dofblur = vec2 ( clamp( factor * 0.03, -2.9, 2.9 ) );"+
		"vec2 dofblur9 = dofblur * 0.9;"+
		"vec2 dofblur7 = dofblur * 0.7;"+
		"vec4 col = vec4( 0.0 );"+
		"col += texture2D( mapScreen, vUv );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.0,0.4) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.15,0.37) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.29,0.29) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2(-0.37,0.15) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.40,0.0) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.37,-0.15) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.29,-0.29) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2(-0.15,-0.37) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.0,-0.4) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2(-0.15,0.37) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2(-0.29,0.29) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2( 0.37,0.15) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2(-0.4,0.0) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2(-0.37,-0.15) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2(-0.29,-0.29) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.15,-0.37) * aspectcorrect ) * dofblur );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.15,0.37) * aspectcorrect ) * dofblur9 );"+
		"col += texture2D( mapScreen, vUv + ( vec2(-0.37,0.15) * aspectcorrect ) * dofblur9 );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.37,-0.15) * aspectcorrect ) * dofblur9 );"+
		"col += texture2D( mapScreen, vUv + ( vec2(-0.15,-0.37) * aspectcorrect ) * dofblur9 );"+
		"col += texture2D( mapScreen, vUv + ( vec2(-0.15,0.37) * aspectcorrect ) * dofblur9 );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.37,0.15) * aspectcorrect ) * dofblur9 );"+
		"col += texture2D( mapScreen, vUv + ( vec2(-0.37,-0.15) * aspectcorrect ) * dofblur9 );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.15,-0.37) * aspectcorrect ) * dofblur9 );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.29,0.29) * aspectcorrect ) * dofblur7 );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.40,0.0) * aspectcorrect ) * dofblur7 );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.29,-0.29) * aspectcorrect ) * dofblur7 );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.0,-0.4) * aspectcorrect ) * dofblur7 );"+
		"col += texture2D( mapScreen, vUv + ( vec2(-0.29,0.29) * aspectcorrect ) * dofblur7 );"+
		"col += texture2D( mapScreen, vUv + ( vec2(-0.4,0.0) * aspectcorrect ) * dofblur7 );"+
		"col += texture2D( mapScreen, vUv + ( vec2(-0.29,-0.29) * aspectcorrect ) * dofblur7 );"+
		"col += texture2D( mapScreen, vUv + ( vec2(0.0,0.4) * aspectcorrect ) * dofblur7 );"+
		"gl_FragColor = col / 33.0;"+
		"gl_FragColor.a = 1.0;"+
	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}
/************************************************************/
/*                                                          */
/*                      SHADER GLOW                    */
/*                                                          */
/************************************************************/


Engine.shaderManager.shaderGlow = function (params) {

	var program = gl.createProgram();

	var vertexShader = this.buildVertexShader();

	var fragmentShader = this.buildFragmentShader(params);

	gl.attachShader(program, vertexShader );

	gl.attachShader(program, fragmentShader);

	gl.linkProgram(program);

	gl.deleteShader( vertexShader );

	gl.deleteShader( fragmentShader );

	this.program = program;

	this.buildUniformsAndAttributes();

}


Engine.shaderManager.shaderGlow.prototype.prepare = function( mapbuffer){

	gl.useProgram(this.program);

	gl.enableVertexAttribArray( this.setAttributVertex );
	gl.enableVertexAttribArray( this.setAttributUv );

	gl.activeTexture( gl.TEXTURE0+ 10 );

	gl.bindTexture( gl.TEXTURE_2D, mapbuffer );

	gl.uniform1i(this.setUniformMapScreen, 10);

}


Engine.shaderManager.shaderGlow.prototype.updateAttributes = function(object){

	gl.bindBuffer( gl.ARRAY_BUFFER, object.__webglVertexBuffer);

	gl.vertexAttribPointer( this.setAttributVertex, 3, gl.FLOAT, false, 0, 0);

	gl.bindBuffer( gl.ARRAY_BUFFER, object.__webglUVBuffer);

	gl.vertexAttribPointer( this.setAttributUv, 2, gl.FLOAT, false, 0, 0);

}

Engine.shaderManager.shaderGlow.prototype.buildUniformsAndAttributes = function(){

	this.setAttributVertex = gl.getAttribLocation( this.program, "vertexPosition" );

	this.setAttributUv = gl.getAttribLocation( this.program, "vertexUvs" );

	this.setUniformMapScreen = gl.getUniformLocation( this.program, "mapScreen" );


}

Engine.shaderManager.shaderGlow.prototype.draw = function(){
	gl.drawArrays(gl.TRIANGLES, 0, 6);
}

Engine.shaderManager.shaderGlow.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute vec2 vertexUvs;"+
	"varying vec2 vUv;"+

	"void main(void) {"+

		"gl_Position = vec4(vertexPosition, 1.0);"+
		"vUv = vertexUvs;"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}

Engine.shaderManager.shaderGlow.prototype.buildFragmentShader = function(params){

	var aspect = params.height/params.width;

	if(params.height>=params.width)
		aspect+=".0"



	var code = ""+
	"precision mediump float;" +
	"varying vec2 vUv;" +
	"uniform sampler2D mapScreen;" +
	"void main() {"+
		"vec2 aspectcorrect = vec2(1.0,"+aspect+");"+
		"float factor =  0.015;"+
		"vec2 dofblur =  vec2(1.0)*factor;"+
		"vec2 dofblur9 =  vec2(0.9)*factor;"+
		"vec2 dofblur7 =  vec2(0.7)*factor;"+
		"vec4 col = texture2D( mapScreen, vUv );"+

		"col += texture2D( mapScreen, vUv +vec2(0.0,0.4) * dofblur );"+
		"col += texture2D( mapScreen, vUv +vec2(0.15,0.37) * dofblur );"+
		"col += texture2D( mapScreen, vUv +vec2(0.29,0.29) * dofblur );"+
		"col += texture2D( mapScreen, vUv +vec2(-0.37,0.15) * dofblur );"+
		"col += texture2D( mapScreen, vUv +vec2(0.40,0.0) * dofblur );"+
		"col += texture2D( mapScreen, vUv +vec2(0.37,-0.15) * dofblur );"+
		"col += texture2D( mapScreen, vUv +vec2(0.29,-0.29) * dofblur );"+
		"col += texture2D( mapScreen, vUv +vec2(-0.15,-0.37) * dofblur );"+
		"col += texture2D( mapScreen, vUv +vec2(0.0,-0.4) * dofblur );"+
		"col += texture2D( mapScreen, vUv +vec2(-0.15,0.37) * dofblur );"+
		"col += texture2D( mapScreen, vUv +vec2(-0.29,0.29) * dofblur );"+
		"col += texture2D( mapScreen, vUv +vec2( 0.37,0.15) * dofblur );"+
		"col += texture2D( mapScreen, vUv +vec2(-0.4,0.0) * dofblur );"+
		"col += texture2D( mapScreen, vUv +vec2(-0.37,-0.15) * dofblur );"+
		"col += texture2D( mapScreen, vUv +vec2(-0.29,-0.29) * dofblur );"+
		"col += texture2D( mapScreen, vUv +vec2(0.15,-0.37) * dofblur );"+

		"col += texture2D( mapScreen, vUv +vec2(0.15,0.37) * dofblur9 );"+
		"col += texture2D( mapScreen, vUv +vec2(-0.37,0.15) * dofblur9 );"+
		"col += texture2D( mapScreen, vUv +vec2(0.37,-0.15) * dofblur9 );"+
		"col += texture2D( mapScreen, vUv +vec2(-0.15,-0.37) * dofblur9 );"+
		"col += texture2D( mapScreen, vUv +vec2(-0.15,0.37) * dofblur9 );"+
		"col += texture2D( mapScreen, vUv +vec2(0.37,0.15) * dofblur9 );"+
		"col += texture2D( mapScreen, vUv +vec2(-0.37,-0.15) * dofblur9 );"+
		"col += texture2D( mapScreen, vUv +vec2(0.15,-0.37) * dofblur9 );"+

		"col += texture2D( mapScreen, vUv +vec2(0.29,0.29) * dofblur7 );"+
		"col += texture2D( mapScreen, vUv +vec2(0.40,0.0) * dofblur7 );"+
		"col += texture2D( mapScreen, vUv +vec2(0.29,-0.29) * dofblur7 );"+
		"col += texture2D( mapScreen, vUv +vec2(0.0,-0.4) * dofblur7 );"+
		"col += texture2D( mapScreen, vUv +vec2(-0.29,0.29) * dofblur7 );"+
		"col += texture2D( mapScreen, vUv +vec2(-0.4,0.0) * dofblur7 );"+
		"col += texture2D( mapScreen, vUv +vec2(-0.29,-0.29) * dofblur7 );"+
		"col += texture2D( mapScreen, vUv +vec2(0.0,0.4) * dofblur7 );"+

		"gl_FragColor = col / 33.0;"+
		"gl_FragColor.a = 1.0;"+

	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}
/************************************************************/
/*                                                          */
/*                      SHADER LUMINANCE                    */
/*                                                          */
/************************************************************/


Engine.shaderManager.shaderLuminance = function () {

	var program = gl.createProgram();

	var vertexShader = this.buildVertexShader();

	var fragmentShader = this.buildFragmentShader();

	gl.attachShader(program, vertexShader );

	gl.attachShader(program, fragmentShader);

	gl.linkProgram(program);

	gl.deleteShader( vertexShader );

	gl.deleteShader( fragmentShader );

	this.program = program;

	this.buildUniformsAndAttributes();

}


Engine.shaderManager.shaderLuminance.prototype.prepare = function( mapbuffer){

	gl.useProgram(this.program);

	gl.enableVertexAttribArray( this.setAttributVertex );
	gl.enableVertexAttribArray( this.setAttributUv );

	gl.activeTexture( gl.TEXTURE0 + 7 );

	gl.bindTexture( gl.TEXTURE_2D, mapbuffer );

	gl.uniform1i(this.setUniformMapScreen, 7);

}

Engine.shaderManager.shaderLuminance.prototype.updateUniforms = function(){}

Engine.shaderManager.shaderLuminance.prototype.updateAttributes = function(object){

	gl.bindBuffer( gl.ARRAY_BUFFER, object.__webglVertexBuffer);

	gl.vertexAttribPointer( this.setAttributVertex, 3, gl.FLOAT, false, 0, 0);

	gl.bindBuffer( gl.ARRAY_BUFFER, object.__webglUVBuffer);

	gl.vertexAttribPointer( this.setAttributUv, 2, gl.FLOAT, false, 0, 0);

}

Engine.shaderManager.shaderLuminance.prototype.buildUniformsAndAttributes = function(){


	this.setAttributVertex = gl.getAttribLocation( this.program, "vertexPosition" );

	this.setAttributUv = gl.getAttribLocation( this.program, "vertexUvs" );

	this.setUniformMapScreen = gl.getUniformLocation( this.program, "mapScreen" );

}

Engine.shaderManager.shaderLuminance.prototype.draw = function(){

	gl.drawArrays(gl.TRIANGLES, 0, 6);

}

Engine.shaderManager.shaderLuminance.prototype.buildVertexShader = function(){

	var code =  ""+

	"attribute vec3 vertexPosition;"+
	"attribute vec2 vertexUvs;"+
	"varying vec2 vUv;"+

	"void main(void) {"+

		"gl_Position = vec4(vertexPosition, 1.0);"+
		"vUv = vertexUvs;"+

	"}";

	var vertexShader = gl.createShader(gl.VERTEX_SHADER);
	gl.shaderSource(vertexShader, code);
	gl.compileShader(vertexShader);
	if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(vertexShader));}

	return vertexShader;

}

Engine.shaderManager.shaderLuminance.prototype.buildFragmentShader = function(){

	var code = ""+

	"precision mediump float;"+

	"uniform sampler2D mapScreen;"+
	
	"varying vec2 vUv;"+

	"void main(void) {"+

		"vec3 dst = texture2D(mapScreen, vec2(vUv.s, vUv.t)).xyz;  " +

		"if(dst.x>0.9||dst.y>0.9||dst.z>0.9){"+
			"gl_FragColor = vec4(dst.xyz,1.0);" +
		"}else{"+
			"gl_FragColor = vec4(0.0,0.0,0.0,1.0);" +
		"}"+

	"}";

	var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
	gl.shaderSource(fragmentShader, code);
	gl.compileShader(fragmentShader);
	if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {alert(gl.getShaderInfoLog(fragmentShader));}

	return fragmentShader;

}
delete shaderOmnInit
delete shaderOmnLoop
delete shaderDirInit
delete shaderDirLoop
delete shaderMatInit
delete codePack;